@@ -21,7 +21,6 @@ import {
 } from "@gnu-taler/taler-util";
 import {
   BrowserHashNavigationProvider,
-  CommonPreferenceProvider,
   ExchangeApiProvider,
   Loading,
   TranslationProvider,
@@ -56,60 +55,56 @@ export function App(): VNode {
   const baseUrl = getInitialBackendBaseURL(settings.backendBaseURL);
   return (
     <UiSettingsProvider value={settings}>
-      <CommonPreferenceProvider showDebug={false}>
-        <TranslationProvider
-          source={strings}
-          completeness={{
-            es: strings["es"].completeness ?? 0,
-            de: strings["de"].completeness ?? 0,
+      <TranslationProvider
+        source={strings}
+        completeness={{
+          es: strings["es"].completeness ?? 0,
+          de: strings["de"].completeness ?? 0,
+        }}
+      >
+        <ExchangeApiProvider
+          baseUrl={new URL("/", baseUrl)}
+          frameOnError={ExchangeAmlFrame}
+          evictors={{
+            exchange: evictExchangeSwrCache,
           }}
+          preventCompression={preventCompression}
         >
-          <CommonPreferenceProvider showDebug={false}>
-            <ExchangeApiProvider
-              baseUrl={new URL("/", baseUrl)}
-              frameOnError={ExchangeAmlFrame}
-              evictors={{
-                exchange: evictExchangeSwrCache,
-              }}
-              preventCompression={preventCompression}
-            >
-              <SWRConfig
-                value={{
-                  provider: WITH_LOCAL_STORAGE_CACHE
-                    ? localStorageProvider
-                    : undefined,
-                  // normally, do not revalidate
-                  revalidateOnFocus: false,
-                  revalidateOnReconnect: false,
-                  revalidateIfStale: false,
-                  revalidateOnMount: undefined,
-                  focusThrottleInterval: undefined,
+          <SWRConfig
+            value={{
+              provider: WITH_LOCAL_STORAGE_CACHE
+                ? localStorageProvider
+                : undefined,
+              // normally, do not revalidate
+              revalidateOnFocus: false,
+              revalidateOnReconnect: false,
+              revalidateIfStale: false,
+              revalidateOnMount: undefined,
+              focusThrottleInterval: undefined,
 
-                  // normally, do not refresh
-                  refreshInterval: undefined,
-                  dedupingInterval: 2000,
-                  refreshWhenHidden: false,
-                  refreshWhenOffline: false,
+              // normally, do not refresh
+              refreshInterval: undefined,
+              dedupingInterval: 2000,
+              refreshWhenHidden: false,
+              refreshWhenOffline: false,
 
-                  // ignore errors
-                  shouldRetryOnError: false,
-                  errorRetryCount: 0,
-                  errorRetryInterval: undefined,
+              // ignore errors
+              shouldRetryOnError: false,
+              errorRetryCount: 0,
+              errorRetryInterval: undefined,
 
-                  // do not go to loading again if already has data
-                  keepPreviousData: true,
-                }}
-              >
-                <BrowserHashNavigationProvider>
-                  <UiFormsProvider>
-                    <Routing />
-                  </UiFormsProvider>
-                </BrowserHashNavigationProvider>
-              </SWRConfig>
-            </ExchangeApiProvider>
-          </CommonPreferenceProvider>
-        </TranslationProvider>
-      </CommonPreferenceProvider>
+              // do not go to loading again if already has data
+              keepPreviousData: true,
+            }}
+          >
+            <BrowserHashNavigationProvider>
+              <UiFormsProvider>
+                <Routing />
+              </UiFormsProvider>
+            </BrowserHashNavigationProvider>
+          </SWRConfig>
+        </ExchangeApiProvider>
+      </TranslationProvider>
     </UiSettingsProvider>
   );
 }
