@@ -28,13 +28,18 @@ import { useEffect } from "preact/hooks";
 import { HandleAccountNotReady } from "./components/HandleAccountNotReady.js";
 import { ExchangeAmlFrame } from "./ExchangeAmlFrame.js";
 import { useCurrentDecisionRequest } from "./hooks/decision-request.js";
-import { useExpireSessionAfter1hr, useOfficer } from "./hooks/officer.js";
+import {
+  OfficerReady,
+  useExpireSessionAfter1hr,
+  useOfficer,
+} from "./hooks/officer.js";
 import { AccountDetails } from "./pages/AccountDetails.js";
 import {
-  AccountList, HomeIcon,
+  AccountList,
+  HomeIcon,
   PeopleIcon,
   SearchIcon,
-  TransfersIcon
+  TransfersIcon,
 } from "./pages/AccountList.js";
 import { Dashboard } from "./pages/Dashboard.js";
 import { DecisionWizard, WizardSteps } from "./pages/DecisionWizard.js";
@@ -51,7 +56,7 @@ export function Routing(): VNode {
   if (session.state === "ready") {
     return (
       <ExchangeAmlFrame officer={session} NavigationBar={Navigation}>
-        <PrivateRouting />
+        <PrivateRouting officer={session} />
       </ExchangeAmlFrame>
     );
   }
@@ -143,7 +148,7 @@ const privatePages = {
   ),
 };
 
-function PrivateRouting(): VNode {
+function PrivateRouting({ officer }: { officer: OfficerReady }): VNode {
   const { navigateTo } = useNavigationContext();
   const location = useCurrentLocation(privatePages);
   const [, , startNewRequest] = useCurrentDecisionRequest();
@@ -152,7 +157,7 @@ function PrivateRouting(): VNode {
       navigateTo(privatePages.dashboard.url({}));
     }
   }, [location]);
-  useExpireSessionAfter1hr()
+  useExpireSessionAfter1hr();
 
   switch (location.name) {
     case undefined: {
@@ -164,6 +169,7 @@ function PrivateRouting(): VNode {
     case "decide": {
       return (
         <DecisionWizard
+        officer={officer}
           account={location.values.cid}
           formId={
             location.params.formId ? location.params.formId[0] : undefined
@@ -192,6 +198,7 @@ function PrivateRouting(): VNode {
     case "decideWithStep": {
       return (
         <DecisionWizard
+        officer={officer}
           account={location.values.cid}
           formId={
             location.params.formId ? location.params.formId[0] : undefined
@@ -221,6 +228,7 @@ function PrivateRouting(): VNode {
     case "decideNew": {
       return (
         <DecisionWizard
+        officer={officer}
           account={location.values.cid}
           newPayto={decodeCrockFromURI(location.values.payto)}
           formId={
@@ -252,6 +260,7 @@ function PrivateRouting(): VNode {
       return (
         <DecisionWizard
           account={location.values.cid}
+        officer={officer}
           newPayto={decodeCrockFromURI(location.values.payto)}
           formId={
             location.params.formId ? location.params.formId[0] : undefined
