@@ -25,11 +25,11 @@ export function HandleAccountNotReady({
   officer: OfficerNotReady;
 }): VNode {
   if (officer.state === "not-found") {
-    return <CreateAccount />;
+    return <CreateAccount officer={officer}/>;
   }
 
   if (officer.state === "locked") {
-    return <UnlockAccount />;
+    return <UnlockAccount officer={officer}/>;
   }
   assertUnreachable(officer);
 }
