@@ -35,6 +35,68 @@ export const PAGINATED_LIST_SIZE = 10;
 // and use it to know if there are more to request
 export const PAGINATED_LIST_REQUEST = PAGINATED_LIST_SIZE + 1;
 
+/**
+ * @param account
+ * @param args
+ * @returns
+ */
+export function useAmlAccounts({
+  investigated,
+  highRisk,
+  open,
+}: {
+  investigated?: boolean;
+  highRisk?: boolean;
+  open?: boolean;
+} = {}) {
+  const officer = useOfficer();
+  const session = officer.state === "ready" ? officer.account : undefined;
+  const {
+    lib: { exchange: api },
+  } = useExchangeApiContext();
+
+  const [offset, setOffset] = useState<string>();
+
+  async function fetcher([officer, offset, investigation, highRisk, open]: [
+    OfficerAccount,
+    string | undefined,
+    boolean | undefined,
+    boolean | undefined,
+    boolean | undefined,
+  ]) {
+    return await api.getAmlAccounts(officer, {
+      order: "dec",
+      offset,
+      limit: PAGINATED_LIST_REQUEST,
+      investigation,
+      highRisk,
+      open,
+    });
+  }
+
+  const { data, error } = useSWR<
+    TalerExchangeResultByMethod2<"getAmlAccounts">,
+    TalerHttpError
+  >(
+    !session
+      ? undefined
+      : [session, offset, investigated, highRisk, open, "getAmlAccounts"],
+    fetcher,
+  );
+
+  if (error) return error;
+  if (data === undefined) return undefined;
+  if (data.type !== "ok") return data;
+
+  return buildPaginatedResult(
+    data.body.accounts,
+    offset,
+    setOffset,
+    (d) => String(d.rowid),
+    PAGINATED_LIST_REQUEST,
+  );
+}
+
 /**
  * @param account
  * @param args
