@@ -18,9 +18,11 @@ import {
   Codec,
   Duration,
   EddsaPrivP,
+  HttpStatusCode,
   LockedAccount,
   OfficerAccount,
   OfficerId,
+  OperationFail,
   OperationOk,
   buildCodecForObject,
   codecForAbsoluteTime,
@@ -29,7 +31,8 @@ import {
   decodeCrock,
   encodeCrock,
   opFixedSuccess,
-  unlockOfficerAccount,
+  opKnownFailure,
+  unlockOfficerAccount
 } from "@gnu-taler/taler-util";
 import {
   buildStorageKey,
@@ -41,7 +44,7 @@ import { usePreferences } from "./preferences.js";
 
 const DEFAULT_SESSION_DURATION = Duration.fromSpec({
   // seconds: 10,
-  hours: 1
+  hours: 1,
 });
 
 export interface Officer {
@@ -72,16 +75,16 @@ export const codecForOfficer = (): Codec<Officer> =>
 
 export type OfficerState = OfficerNotReady | OfficerReady;
 export type OfficerNotReady = OfficerNotFound | OfficerLocked;
-interface OfficerNotFound {
+export interface OfficerNotFound {
   state: "not-found";
   create: (password: string) => Promise<OperationOk<OfficerId>>;
 }
-interface OfficerLocked {
+export interface OfficerLocked {
   state: "locked";
   forget: () => OperationOk<void>;
-  tryUnlock: (password: string) => Promise<OperationOk<void>>;
+  tryUnlock: (password: string) => Promise<OperationOk<void> | OperationFail<HttpStatusCode.Forbidden>>;
 }
-interface OfficerReady {
+export interface OfficerReady {
   state: "ready";
   account: OfficerAccount;
   forget: () => OperationOk<void>;
@@ -109,7 +112,7 @@ export function useOfficer(): OfficerState {
     return {
       id: accountStorage.value.id as OfficerId,
       signingKey: decodeCrock(accountStorage.value.strKey) as EddsaPrivP,
-      unlocked: accountStorage.value.unlocked
+      unlocked: accountStorage.value.unlocked,
     };
   }, [accountStorage.value?.id, accountStorage.value?.strKey]);
 
@@ -152,14 +155,19 @@ export function useOfficer(): OfficerState {
         return opFixedSuccess(undefined);
       },
       tryUnlock: async (pwd: string) => {
-        const ac = await unlockOfficerAccount(officer.account, pwd);
-        // accountStorage.update(ac);
-        accountStorage.update({
-          id: ac.id,
-          strKey: encodeCrock(ac.signingKey),
-          unlocked: AbsoluteTime.now(),
-        });
-        return opFixedSuccess(undefined);
+        try {
+          const ac = await unlockOfficerAccount(officer.account, pwd);
+          // accountStorage.update(ac);
+          accountStorage.update({
+            id: ac.id,
+            strKey: encodeCrock(ac.signingKey),
+            unlocked: AbsoluteTime.now(),
+          });
+          return opFixedSuccess(undefined);
+        } catch (e) {
+          const d = opKnownFailure(HttpStatusCode.Forbidden);
+          return d;
+        }
       },
     };
   }
