@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2022-2024 Taler Systems S.A.
+ (C) 2022-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -19,7 +19,7 @@ import { h, render } from "preact";
 
 const app = document.getElementById("app");
 if (!app) {
-  console.error("could not found app element")
+  console.error("could not found app element");
 } else {
   render(<App />, app);
 }
