@@ -1,3 +1,6 @@
+import { default as tw_typo } from '@tailwindcss/typography';
+import { default as tw_form } from '@tailwindcss/forms';
+
 /** @type {import('tailwindcss').Config} */
 export default {
   content: {
@@ -10,5 +13,5 @@ export default {
   theme: {
     extend: {},
   },
-  plugins: [require("@tailwindcss/typography"), require("@tailwindcss/forms")],
+  plugins: [tw_typo, tw_form],
 };
