@@ -29,7 +29,10 @@ interface Props {
 }
 
 const VERSION = typeof __VERSION__ !== "undefined" ? __VERSION__ : "dev";
-const GIT_HASH = typeof __GIT_HASH__ !== "undefined" ? __GIT_HASH__ : undefined;
+const GIT_HASH =
+  typeof __GIT_HASH__ !== "undefined"
+    ? __GIT_HASH__.substring(0, 7)
+    : undefined;
 const VERSION_WITH_HASH = GIT_HASH ? `${VERSION}-${GIT_HASH}` : VERSION;
 
 export function Sidebar({ mobile }: Props): VNode {
