@@ -121,13 +121,13 @@ export function WithoutProviderType(props: WithoutType): VNode {
         <div class="container">
           <TextInput
             label="Provider URL"
-            placeholder="https://provider.com"
+            placeholder="https://provider.com/"
             grabFocus
             error={props.errors}
             bind={[props.providerURL, props.setProviderURL]}
           />
         </div>
-        <p class="block">Example: https://kudos.demo.anastasis.lu</p>
+        <p class="block">Example: https://kudos.demo.anastasis.lu/</p>
         {props.testing && <p class="has-text-info">Testing</p>}
 
         <div
