@@ -112,25 +112,27 @@ export function ListPage(data: any): VNode {
                             </tr>
                           );
                         } else {
-                          <tr class="is-link">
-                            <td>{paramName}</td>
-                            <td class="has-text-right">
-                              <p
-                                class={
-                                  value == 0 ? "text-success" : "text-danger"
-                                }
-                              >
-                                {String(value)}
-                              </p>
-                            </td>
-                            <td class="has-text-right">
-                              <p>
-                                {
-                                  //TODO
-                                }
-                              </p>
-                            </td>
-                          </tr>;
+                          return (
+                            <tr class="is-link">
+                              <td>{paramName}</td>
+                              <td class="has-text-right">
+                                <p
+                                  class={
+                                    value == 0 ? "text-success" : "text-danger"
+                                  }
+                                >
+                                  {String(value)}
+                                </p>
+                              </td>
+                              <td class="has-text-right">
+                                <p>
+                                  {
+                                    //TODO
+                                  }
+                                </p>
+                              </td>
+                            </tr>
+                          );
                         }
                       })}
                     </tbody>
