@@ -16,7 +16,6 @@
 
 import {
   LocalNotificationBanner,
-  makeSafeCall,
   urlPattern,
   useBankCoreApiContext,
   useChallengeHandler,
@@ -34,7 +33,7 @@ import {
   TalerErrorCode,
   TokenRequest,
   assertUnreachable,
-  createRFC8959AccessTokenEncoded
+  createRFC8959AccessTokenEncoded,
 } from "@gnu-taler/taler-util";
 import { useEffect } from "preact/hooks";
 import {
@@ -118,7 +117,8 @@ function PublicRounting({
   const { navigateTo } = useNavigationContext();
 
   const { config, lib } = useBankCoreApiContext();
-  const [notification, notifyOnError] = useLocalNotificationBetter();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
+
   const mfa = useChallengeHandler();
 
   useEffect(() => {
@@ -133,50 +133,51 @@ function PublicRounting({
     refreshable: true,
   } as TokenRequest;
 
-  const [doAutomaticLogin, repeatLogin, lastCallingArgs] = mfa.withMfaHandler(
-    ({ challengeIds, onChallengeRequired }) =>
-      makeSafeCall(
-        i18n,
-        (username: string, password: string) =>
-          lib.bank.createAccessToken(
-            username,
-            { type: "basic", password },
-            tokenRequest,
-            { challengeIds },
-          ),
-        (success, username) => {
-          onLoggedUser(
-            username,
-            createRFC8959AccessTokenEncoded(success.body.access_token),
-            AbsoluteTime.fromProtocolTimestamp(success.body.expiration),
-          );
-        },
-        (fail, username) => {
-          switch (fail.case) {
-            case HttpStatusCode.Accepted: {
-              onChallengeRequired(fail.body);
-              return i18n.str`A second factor authentication is required.`;
-            }
-            case HttpStatusCode.Unauthorized:
-              return i18n.str`Wrong credentials for "${username}"`;
-            case TalerErrorCode.GENERIC_FORBIDDEN:
-              return i18n.str`You have no permission to this account.`;
-            case TalerErrorCode.BANK_ACCOUNT_LOCKED:
-              return i18n.str`This account is locked. If you have a active session you can change the password or contact the administrator.`;
-            case HttpStatusCode.NotFound:
-              return i18n.str`Account not found`;
-          }
-        },
+  const login = safeFunctionHandler(
+    (username: string, password: string, challengeIds: string[]) =>
+      lib.bank.createAccessToken(
+        username,
+        { type: "basic", password },
+        tokenRequest,
+        { challengeIds },
       ),
   );
 
-  if (mfa.pendingChallenge && repeatLogin) {
+  login.onSuccess = (success, username) =>
+    onLoggedUser(
+      username,
+      createRFC8959AccessTokenEncoded(success.access_token),
+      AbsoluteTime.fromProtocolTimestamp(success.expiration),
+    );
+
+  login.onFail = (fail, username) => {
+    switch (fail.case) {
+      case HttpStatusCode.Accepted: {
+        mfa.onChallengeRequired(fail.body);
+        return i18n.str`A second factor authentication is required.`;
+      }
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Wrong credentials for "${username}"`;
+      case TalerErrorCode.GENERIC_FORBIDDEN:
+        return i18n.str`You have no permission to this account.`;
+      case TalerErrorCode.BANK_ACCOUNT_LOCKED:
+        return i18n.str`This account is locked. If you have a active session you can change the password or contact the administrator.`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Account not found`;
+    }
+  };
+
+  const repeatLogin = login.lambda((ids: string[]) => {
+    return [login.args![0], login.args![1], ids];
+  });
+
+  if (mfa.pendingChallenge) {
     return (
       <SolveMFAChallenges
         currentChallenge={mfa.pendingChallenge}
         description={i18n.str`New web session`}
         onCancel={mfa.doCancelChallenge}
-        username={lastCallingArgs[0]}
+        username={login.args![0]}
         onCompleted={repeatLogin}
       />
     );
@@ -212,7 +213,9 @@ function PublicRounting({
         <Fragment>
           <LocalNotificationBanner notification={notification} />
           <RegistrationPage
-            onRegistrationSuccesful={notifyOnError(doAutomaticLogin)}
+            onRegistrationSuccesful={(usr, pwd) => {
+              login.withArgs(usr,pwd, []).call()
+            }}
             routeCancel={publicPages.login}
           />
         </Fragment>
