@@ -46,6 +46,6 @@ export function useComponentState({
     status: "ready",
     error: undefined,
     cashouts: result.body.cashouts,
-    routeCashoutDetails,
+    routeCashoutDetails
   };
 }
