@@ -23,23 +23,26 @@ import {
 } from "@gnu-taler/taler-util";
 import {
   Attention,
+  ErrorLoading,
   Loading,
   Time,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { format } from "date-fns";
 import { Fragment, VNode, h } from "preact";
-import { useConversionInfo } from "../../hooks/regional.js";
 import { RenderAmount } from "../../pages/PaytoWireTransferForm.js";
-import { ErrorLoadingWithDebug } from "../ErrorLoadingWithDebug.js";
 import { State } from "./index.js";
+import { useConversionInfo } from "../../hooks/regional.js";
+
+
+const TALER_SCREEN_ID = 3;
 
 export function FailedView({ error }: State.Failed) {
   const { i18n } = useTranslationContext();
   switch (error.case) {
     case HttpStatusCode.NotImplemented: {
       return (
-        <Attention type="danger" title={i18n.str`Cashout are disabled`}>
+        <Attention type="danger" title={i18n.str`Cashout is disabled`}>
           <i18n.Translate>
             Cashout should be enable by configuration and the conversion rate
             should be initialized with fee, ratio and rounding mode.
@@ -57,29 +60,6 @@ export function ReadyView({
   routeCashoutDetails,
 }: State.Ready): VNode {
   const { i18n, dateLocale } = useTranslationContext();
-  const resp = useConversionInfo();
-  if (!resp) {
-    return <Loading />;
-  }
-  if (resp instanceof TalerError) {
-    return <ErrorLoadingWithDebug error={resp} />;
-  }
-  if (resp.type === "fail") {
-    switch (resp.case) {
-      case HttpStatusCode.NotImplemented: {
-        return (
-          <Attention type="danger" title={i18n.str`Cashout are disabled`}>
-            <i18n.Translate>
-              Cashout should be enable by configuration and the conversion rate
-              should be initialized with fee, ratio and rounding mode.
-            </i18n.Translate>
-          </Attention>
-        );
-      }
-      default:
-        assertUnreachable(resp.case);
-    }
-  }
 
   if (!cashouts.length) return <div />;
   const txByDate = cashouts.reduce(
@@ -98,6 +78,30 @@ export function ReadyView({
     },
     {} as Record<string, typeof cashouts>,
   );
+  const conversionResp = useConversionInfo();
+  if (!conversionResp) {
+    return <Loading />;
+  } else if (conversionResp instanceof TalerError) {
+    return <ErrorLoading error={conversionResp} />;
+  } else if (conversionResp.type === "fail") {
+    switch (conversionResp.case) {
+      case HttpStatusCode.NotImplemented: {
+        return (
+          <Attention type="danger" title={i18n.str`Cashout is disabled`}>
+            <i18n.Translate>
+              Cashout should be enabled in the configuration, the conversion
+              rate should be initialized with fee(s), rates and a rounding mode.
+            </i18n.Translate>
+          </Attention>
+        );
+      }
+      default:
+        assertUnreachable(conversionResp);
+    }
+  }
+  const { fiat_currency_specification, regional_currency_specification } =
+    conversionResp.body;
+
   return (
     <div class="px-4 mt-4">
       <div class="sm:flex sm:items-center">
@@ -162,41 +166,17 @@ export function ReadyView({
                               )}
                             />
                           </div>
-                          {
-                            //FIXME: implement responsive view
-                          }
-                          {/* <dl class="font-normal sm:hidden">
-                        <dt class="sr-only sm:hidden"><i18n.Translate>Amount</i18n.Translate></dt>
-                        <dd class="mt-1 truncate text-gray-700">
-                          {item.negative ? i18n.str`sent` : i18n.str`received`} {item.amount ? (
-                            <span data-negative={item.negative ? "true" : "false"} class="data-[negative=false]:text-green-600 data-[negative=true]:text-red-600">
-                              <RenderAmount value={item.amount} />
-                            </span>
-                          ) : (
-                            <span style={{ color: "grey" }}>&lt;{i18n.str`invalid value`}&gt;</span>
-                          )}</dd>
-
-                        <dt class="sr-only sm:hidden"><i18n.Translate>Counterpart</i18n.Translate></dt>
-                        <dd class="mt-1 truncate text-gray-500 sm:hidden">
-                          {item.negative ? i18n.str`to` : i18n.str`from`} {item.counterpart}
-                        </dd>
-                        <dd class="mt-1 text-gray-500 sm:hidden" >
-                          <pre class="break-words w-56 whitespace-break-spaces p-2 rounded-md mx-auto my-2 bg-gray-100">
-                            {item.subject}
-                          </pre>
-                        </dd>
-                      </dl> */}
                         </td>
                         <td class="hidden sm:table-cell px-3 py-3.5 text-sm text-red-600 cursor-pointer">
                           <RenderAmount
                             value={Amounts.parseOrThrow(item.amount_debit)}
-                            spec={resp.body.regional_currency_specification}
+                            spec={regional_currency_specification}
                           />
                         </td>
                         <td class="hidden sm:table-cell px-3 py-3.5 text-sm text-green-600 cursor-pointer">
                           <RenderAmount
                             value={Amounts.parseOrThrow(item.amount_credit)}
-                            spec={resp.body.fiat_currency_specification}
+                            spec={fiat_currency_specification!}
                           />
                         </td>
 
