@@ -31,21 +31,8 @@ export function QR({ text }: { text: string }): VNode {
   });
 
   return (
-    <div
-      style={{
-        display: "flex",
-        flexDirection: "column",
-        alignItems: "left",
-      }}
-    >
-      <div
-        style={{
-          width: "100%",
-          marginRight: "auto",
-          marginLeft: "auto",
-        }}
-        ref={divRef}
-      />
+    <div class="flex flex-col ">
+      <div class="mx-auto w-full" ref={divRef} />
     </div>
   );
 }
