@@ -30,28 +30,25 @@ import { codecOptionalDefault } from "@gnu-taler/taler-util";
 
 const TALER_SCREEN_ID = 102;
 
-
 interface Preferences {
   showWithdrawalSuccess: boolean;
   hideDemo: boolean;
   showInstallWallet: boolean;
-  allowsSimplePassword: boolean;
   fastWithdrawalForm: boolean;
 }
 
 export const codecForPreferences = (): Codec<Preferences> =>
   buildCodecForObject<Preferences>()
+    .allowExtra()
     .property("showWithdrawalSuccess", codecForBoolean())
     .property("hideDemo", codecOptionalDefault(codecForBoolean(), false))
     .property("showInstallWallet", codecForBoolean())
-    .property("allowsSimplePassword", codecForBoolean())
     .property("fastWithdrawalForm", codecForBoolean())
     .build("Preferences");
 
 const defaultPreferences: Preferences = {
   showWithdrawalSuccess: true,
   hideDemo: true,
-  allowsSimplePassword: false,
   showInstallWallet: true,
   fastWithdrawalForm: false,
 };
@@ -92,11 +89,7 @@ export function getAllBooleanPreferences(
       "fastWithdrawalForm",
     ];
   }
-  return [
-    "showInstallWallet",
-    "showWithdrawalSuccess",
-    "fastWithdrawalForm",
-  ];
+  return ["showInstallWallet", "showWithdrawalSuccess", "fastWithdrawalForm"];
 }
 
 export function getLabelForPreferences(
@@ -112,8 +105,6 @@ export function getLabelForPreferences(
       return i18n.str`Hide demo hint.`;
     case "showInstallWallet":
       return i18n.str`Show install wallet first`;
-    case "allowsSimplePassword":
-      return i18n.str`Remove password length validation on registration`;
     // case "showDebugInfo":
     //   return i18n.str`Show debug info`;
   }
