@@ -48,7 +48,7 @@ export type TransCalc = {
   credit: AmountJson;
   beforeFee: AmountJson;
 };
-export type TransferCalculation = TransCalc | "amount-is-too-small";
+export type TransferCalculation = TransCalc | undefined;
 type EstimatorFunction = (
   amount: AmountJson,
   fee: AmountJson,
