@@ -20,17 +20,17 @@ import {
   TalerCorebankApi,
   TalerError,
 } from "@gnu-taler/taler-util";
-import { Loading, utils } from "@gnu-taler/web-util/browser";
-import { ErrorLoadingWithDebug } from "../../components/ErrorLoadingWithDebug.js";
+import { ErrorLoading, Loading, RouteDefinition, utils } from "@gnu-taler/web-util/browser";
+import { Fragment, VNode } from "preact";
+
 import { LoginForm } from "../LoginForm.js";
 import { useComponentState } from "./state.js";
 import { InvalidIbanView, ReadyView } from "./views.js";
-import { RouteDefinition } from "@gnu-taler/web-util/browser";
-import { Fragment } from "preact";
+import { IntAmountJson, IntAmounts } from "../regional/CreateCashout.js";
 
 export interface Props {
   account: string;
-  onAuthorizationRequired: () => void;
+
   onOperationCreated: (wopid: string) => void;
   onClose: () => void;
   tab: "charge-wallet" | "wire-transfer" | undefined;
@@ -49,7 +49,6 @@ export interface Props {
     amount?: string;
   }>;
   routeOperationDetails: RouteDefinition<{ wopid: string }>;
-  routeSolveSecondFactor: RouteDefinition;
 }
 
 export type State =
@@ -79,9 +78,9 @@ export namespace State {
     error: undefined;
     account: string;
     tab: "charge-wallet" | "wire-transfer" | undefined;
-    limit: AmountJson;
+    limit: IntAmountJson;
     balance: AmountJson;
-    onAuthorizationRequired: () => void;
+
     onOperationCreated: (wopid: string) => void;
     onClose: () => void;
     routeClose: RouteDefinition;
@@ -99,7 +98,6 @@ export namespace State {
       amount?: string;
     }>;
     routeOperationDetails: RouteDefinition<{ wopid: string }>;
-    routeSolveSecondFactor: RouteDefinition;
   }
 
   export interface InvalidIban {
@@ -126,15 +124,11 @@ const viewMapping: utils.StateViewMap<State> = {
   loading: Loading,
   login: LoginForm,
   "invalid-iban": InvalidIbanView,
-  "loading-error": (d) => {
-    return Fragment({
-      children: [ErrorLoadingWithDebug({ error: d.error }), LoginForm({})],
-    })!;
-  },
+  "loading-error": ErrorLoading,
   ready: ReadyView,
 };
 
-export const AccountPage = utils.compose(
+export const AccountPage: (p: Props) => VNode = utils.compose(
   (p: Props) => useComponentState(p),
   viewMapping,
 );
