@@ -10,6 +10,7 @@ import {
 } from "@gnu-taler/taler-util";
 import {
   Attention,
+  ButtonBetter,
   ErrorLoading,
   InputText,
   InputToggle,
@@ -19,11 +20,11 @@ import {
   RouteDefinition,
   ShowInputErrorLabel,
   useBankCoreApiContext,
-  useLocalNotification,
+  useLocalNotificationBetter,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { Fragment, h, VNode } from "preact";
-import { useState, useEffect } from "preact/hooks";
+import { useEffect, useState } from "preact/hooks";
 
 import {
   FormErrors,
@@ -35,9 +36,7 @@ import {
   revalidateConversionRateClassDetails,
   revalidateConversionRateClassUsers,
   TransferCalculation,
-  useCashinEstimator,
   useCashinEstimatorForClass,
-  useCashoutEstimator,
   useCashoutEstimatorForClass,
   useConversionInfo,
   useConversionRateClassDetails,
@@ -45,11 +44,15 @@ import {
 } from "../hooks/regional.js";
 import { useSessionState } from "../hooks/session.js";
 import { RecursivePartial, undefinedIfEmpty } from "../utils.js";
+import { DescribeConversion } from "./admin/ConversionClassList.js";
 import { doAutoFocus, InputAmount } from "./PaytoWireTransferForm.js";
 import { ConversionForm } from "./regional/ConversionConfig.js";
+import { AccessToken } from "@gnu-taler/taler-util";
+import { TalerErrorCode } from "@gnu-taler/taler-util";
+import { opFixedSuccess } from "@gnu-taler/taler-util";
 import { AmountJson } from "@gnu-taler/taler-util";
-import { DescribeConversion } from "./admin/ConversionClassList.js";
 
+const TALER_SCREEN_ID = 11;
 interface Props {
   classId: number;
   routeCancel: RouteDefinition;
@@ -132,12 +135,8 @@ function Form({
   const { i18n } = useTranslationContext();
   const { state: credentials } = useSessionState();
   const creds = credentials.status !== "loggedIn" ? undefined : credentials;
-  const {
-    lib: { bank },
-    config,
-  } = useBankCoreApiContext();
-  const [notification, notify, handleError] = useLocalNotification();
-
+  const { lib, config } = useBankCoreApiContext();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
   const [section, setSection] = useState<
     "detail" | "cashout" | "cashin" | "users" | "test" | "delete"
   >("detail");
@@ -166,78 +165,179 @@ function Form({
     ),
   );
 
-  async function doDeleteClass() {
-    if (!creds) return;
-    await bank.deleteConversionRateClass(creds.token, classId);
-    onClassDeleted();
-  }
-
-  const doDelete =
+  const deleteClass = safeFunctionHandler(
+    (token: AccessToken) => lib.bank.deleteConversionRateClass(token, classId),
     !creds || section !== "delete" || detailsResult.num_users > 0
       ? undefined
-      : doDeleteClass;
-
-  async function doUpdateClass() {
-    if (!creds) return;
-    if (status.status !== "ok") {
-      console.log("can submit due to form error", status.errors);
-      return;
+      : [creds.token],
+  );
+  deleteClass.onSuccess = onClassDeleted;
+  deleteClass.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unauthorized`;
+      case HttpStatusCode.Forbidden:
+        return i18n.str`Forbidden`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`NotFound`;
+      case HttpStatusCode.NotImplemented:
+        return i18n.str`NotImplemented`;
     }
+  };
 
-    await bank.updateConversionRateClass(creds.token, classId, {
-      name: status.result.name,
-      description: status.result.description,
+  const input: TalerCorebankApi.ConversionRateClassInput | undefined =
+    status.status === "fail"
+      ? undefined
+      : {
+          name: status.result.name,
+          description: status.result.description,
 
-      cashin_fee: status.result.conv.cashin_fee,
-      cashin_min_amount: status.result.conv.cashin_min_amount,
-      cashin_ratio: status.result.conv.cashin_ratio,
-      cashin_rounding_mode: status.result.conv.cashin_rounding_mode,
+          cashin_fee: status.result.conv.cashin_fee,
+          cashin_min_amount: status.result.conv.cashin_min_amount,
+          cashin_ratio: status.result.conv.cashin_ratio,
+          cashin_rounding_mode: status.result.conv.cashin_rounding_mode,
 
-      cashout_fee: status.result.conv.cashout_fee,
-      cashout_min_amount: status.result.conv.cashout_min_amount,
-      cashout_ratio: status.result.conv.cashout_ratio,
-      cashout_rounding_mode: status.result.conv.cashout_rounding_mode,
-    });
+          cashout_fee: status.result.conv.cashout_fee,
+          cashout_min_amount: status.result.conv.cashout_min_amount,
+          cashout_ratio: status.result.conv.cashout_ratio,
+          cashout_rounding_mode: status.result.conv.cashout_rounding_mode,
+        };
+
+  const updateClass = safeFunctionHandler(
+    lib.bank.updateConversionRateClass.bind(lib.bank),
+    !creds || !input ? undefined : [creds.token, classId, input],
+  );
+  updateClass.onSuccess = () => {
     setSection("detail");
-  }
+  };
+  updateClass.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unauthorized`;
+      case HttpStatusCode.Forbidden:
+        return i18n.str`Forbidden`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Not Found`;
+      case HttpStatusCode.NotImplemented:
+        return i18n.str`Not implemented`;
+      case TalerErrorCode.BANK_NAME_REUSE:
+        return i18n.str`The name of the conversion is already used.`;
+    }
+  };
 
-  const doUpdateDetails =
-    !creds ||
-    section !== "detail" ||
-    status.errors?.name ||
-    status.errors?.description ||
-    (status.result.name === initalState.name &&
-      status.result.description === initalState.description)
+  const updateRequest: TalerCorebankApi.ConversionRateClassInput | undefined =
+    status.status === "fail"
       ? undefined
-      : doUpdateClass;
+      : {
+          name: status.result.name,
+          description: status.result.description,
 
-  const doUpdateCashin =
-    !creds ||
-    section !== "cashin" ||
-    status.errors?.conv?.cashin_fee ||
-    status.errors?.conv?.cashin_min_amount ||
-    status.errors?.conv?.cashin_ratio ||
-    status.errors?.conv?.cashin_rounding_mode
-      ? undefined
-      : doUpdateClass;
+          cashin_fee: status.result.conv.cashin_fee,
+          cashin_min_amount: status.result.conv.cashin_min_amount,
+          cashin_ratio: status.result.conv.cashin_ratio,
+          cashin_rounding_mode: status.result.conv.cashin_rounding_mode,
 
-  const doUpdateCashout =
+          cashout_fee: status.result.conv.cashout_fee,
+          cashout_min_amount: status.result.conv.cashout_min_amount,
+          cashout_ratio: status.result.conv.cashout_ratio,
+          cashout_rounding_mode: status.result.conv.cashout_rounding_mode,
+        };
+
+  const updateDetails = updateClass.lambda(
+    (
+      t: AccessToken,
+      id: number,
+      r: TalerCorebankApi.ConversionRateClassInput,
+    ) => [t, id, r],
     !creds ||
-    section !== "cashout" ||
-    // no errors on fields
-    status.errors?.conv?.cashout_fee ||
-    status.errors?.conv?.cashout_min_amount ||
-    status.errors?.conv?.cashout_ratio ||
-    status.errors?.conv?.cashout_rounding_mode ||
-    // at least on field changed
-    (status.result?.conv?.cashout_fee === initalState.conv.cashout_fee &&
-      status.result?.conv?.cashout_min_amount ===
-        initalState.conv.cashout_min_amount &&
-      status.result?.conv?.cashout_ratio === initalState.conv.cashout_ratio &&
-      status.result?.conv?.cashout_rounding_mode ===
-        initalState.conv.cashout_rounding_mode)
+      !updateRequest ||
+      section !== "detail" ||
+      status.errors?.name ||
+      status.errors?.description ||
+      (status.result.name === initalState.name &&
+        status.result.description === initalState.description)
       ? undefined
-      : doUpdateClass;
+      : [creds.token, classId, updateRequest],
+  );
+
+  // const doUpdateDetails1 =
+  //   !creds ||
+  //   section !== "detail" ||
+  //   status.errors?.name ||
+  //   status.errors?.description ||
+  //   (status.result.name === initalState.name &&
+  //     status.result.description === initalState.description)
+  //     ? undefined
+  //     : doUpdateClass2;
+
+  const updateCashin = updateClass.lambda(
+    (
+      t: AccessToken,
+      id: number,
+      r: TalerCorebankApi.ConversionRateClassInput,
+    ) => [t, id, r],
+    !creds ||
+      !updateRequest ||
+      section !== "cashin" ||
+      status.errors?.conv?.cashin_fee ||
+      status.errors?.conv?.cashin_min_amount ||
+      status.errors?.conv?.cashin_ratio ||
+      status.errors?.conv?.cashin_rounding_mode
+      ? undefined
+      : [creds.token, classId, updateRequest],
+  );
+  // const doUpdateCashin1 =
+  //   !creds ||
+  //   section !== "cashin" ||
+  //   status.errors?.conv?.cashin_fee ||
+  //   status.errors?.conv?.cashin_min_amount ||
+  //   status.errors?.conv?.cashin_ratio ||
+  //   status.errors?.conv?.cashin_rounding_mode
+  //     ? undefined
+  //     : doUpdateClass2;
+
+  const updateCashout = updateClass.lambda(
+    (
+      t: AccessToken,
+      id: number,
+      r: TalerCorebankApi.ConversionRateClassInput,
+    ) => [t, id, r],
+    !creds ||
+      !updateRequest ||
+      section !== "cashout" ||
+      // no errors on fields
+      status.errors?.conv?.cashout_fee ||
+      status.errors?.conv?.cashout_min_amount ||
+      status.errors?.conv?.cashout_ratio ||
+      status.errors?.conv?.cashout_rounding_mode ||
+      // at least on field changed
+      (status.result?.conv?.cashout_fee === initalState.conv.cashout_fee &&
+        status.result?.conv?.cashout_min_amount ===
+          initalState.conv.cashout_min_amount &&
+        status.result?.conv?.cashout_ratio === initalState.conv.cashout_ratio &&
+        status.result?.conv?.cashout_rounding_mode ===
+          initalState.conv.cashout_rounding_mode)
+      ? undefined
+      : [creds.token, classId, updateRequest],
+  );
+
+  // const doUpdateCashout1 =
+  //   !creds ||
+  //   section !== "cashout" ||
+  //   // no errors on fields
+  //   status.errors?.conv?.cashout_fee ||
+  //   status.errors?.conv?.cashout_min_amount ||
+  //   status.errors?.conv?.cashout_ratio ||
+  //   status.errors?.conv?.cashout_rounding_mode ||
+  //   // at least on field changed
+  //   (status.result?.conv?.cashout_fee === initalState.conv.cashout_fee &&
+  //     status.result?.conv?.cashout_min_amount ===
+  //       initalState.conv.cashout_min_amount &&
+  //     status.result?.conv?.cashout_ratio === initalState.conv.cashout_ratio &&
+  //     status.result?.conv?.cashout_rounding_mode ===
+  //       initalState.conv.cashout_rounding_mode)
+  //     ? undefined
+  //     : doUpdateClass2;
 
   const default_rate = conversionInfo.conversion_rate;
 
@@ -590,7 +690,9 @@ function Form({
             />
           )}
 
-          {section == "test" && <TestConversionClass classId={classId} />}
+          {section == "test" && (
+            <TestConversionClass classId={classId} info={conversionInfo} />
+          )}
 
           <div class="flex items-center justify-between mt-4 gap-x-6 border-t border-gray-900/10 px-4 py-4">
             <a
@@ -602,54 +704,50 @@ function Form({
             </a>
             {section == "cashin" ? (
               <Fragment>
-                <button
+                <ButtonBetter
                   type="submit"
                   name="update conversion"
                   class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-                  disabled={!doUpdateCashin}
-                  onClick={doUpdateCashin}
+                  onClick={updateCashin}
                 >
                   <i18n.Translate>Update</i18n.Translate>
-                </button>
+                </ButtonBetter>
               </Fragment>
             ) : undefined}
             {section == "cashout" ? (
               <Fragment>
-                <button
+                <ButtonBetter
                   type="submit"
                   name="update conversion"
                   class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-                  disabled={!doUpdateCashout}
-                  onClick={doUpdateCashout}
+                  onClick={updateCashout}
                 >
                   <i18n.Translate>Update</i18n.Translate>
-                </button>
+                </ButtonBetter>
               </Fragment>
             ) : undefined}
             {section == "detail" ? (
               <Fragment>
-                <button
+                <ButtonBetter
                   type="submit"
                   name="update conversion"
                   class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-                  disabled={!doUpdateDetails}
-                  onClick={doUpdateDetails}
+                  onClick={updateDetails}
                 >
                   <i18n.Translate>Update</i18n.Translate>
-                </button>
+                </ButtonBetter>
               </Fragment>
             ) : undefined}
             {section == "delete" ? (
               <Fragment>
-                <button
+                <ButtonBetter
                   type="submit"
                   name="update conversion"
                   class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-red-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-red-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-red-600"
-                  disabled={!doDelete}
-                  onClick={doDelete}
+                  onClick={deleteClass}
                 >
                   <i18n.Translate>Delete</i18n.Translate>
-                </button>
+                </ButtonBetter>
               </Fragment>
             ) : undefined}
           </div>
@@ -775,15 +873,15 @@ export function createFormValidator(
   };
 }
 
-function TestConversionClass({ classId }: { classId: number }): VNode {
+function TestConversionClass({
+  classId,
+  info,
+}: {
+  classId: number;
+  info: TalerBankConversionApi.TalerConversionInfoConfig;
+}): VNode {
   const { i18n } = useTranslationContext();
-  const [notification, notify, handleError] = useLocalNotification();
-
-  const result = useConversionInfo();
-  const info =
-    result && !(result instanceof TalerError) && result.type === "ok"
-      ? result.body
-      : undefined;
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
 
   const { estimateByDebit: calculateCashoutFromDebit } =
     useCashoutEstimatorForClass(classId);
@@ -798,65 +896,57 @@ function TestConversionClass({ classId }: { classId: number }): VNode {
     cashout: TransferCalculation;
   }>();
 
-  useEffect(() => {
-    async function doAsync() {
-      await handleError(async () => {
-        if (!info) return;
-        if (!amount || error) return;
-        const in_amount = Amounts.parseOrThrow(
-          `${info.fiat_currency}:${amount}`,
-        );
-        const in_fee = Amounts.parseOrThrow(info.conversion_rate.cashin_fee);
-        const respCashin = await calculateCashinFromDebit(in_amount, in_fee);
+  const in_amount = !amount
+    ? undefined
+    : Amounts.parseOrThrow(`${info.fiat_currency}:${amount}`);
 
-        const cashin =
-          respCashin.type === "ok"
-            ? respCashin.body
-            : respCashin.case === HttpStatusCode.Conflict
-              ? ("amount-is-too-small" as const)
-              : undefined;
+  const in_fee = Amounts.parseOrThrow(info.conversion_rate.cashin_fee);
+  const out_fee = Amounts.parseOrThrow(info.conversion_rate.cashout_fee);
 
-        if (!cashin || cashin === "amount-is-too-small") {
-          setCalc(undefined); // silent failure
-          return;
-        }
+  const calculate = safeFunctionHandler(
+    async (amount: AmountJson) => {
+      const respCashin = await calculateCashinFromDebit(amount, in_fee);
+      if (respCashin.type === "fail") {
+        return respCashin;
+      }
+      const cashin = respCashin.body;
+      const respCashout = await calculateCashoutFromDebit(
+        cashin.credit,
+        out_fee,
+      );
+      if (respCashout.type === "fail") {
+        return respCashout;
+      }
+      const cashout = respCashout.body;
+      return opFixedSuccess({ cashin, cashout });
+    },
+    !in_amount || !!error ? undefined : [in_amount],
+  );
 
-        const out_fee = Amounts.parseOrThrow(info.conversion_rate.cashout_fee);
-        const respCashout = await calculateCashoutFromDebit(
-          cashin.credit,
-          out_fee,
-        );
-
-        const cashout =
-          respCashout.type === "ok"
-            ? respCashout.body
-            : respCashout.case === HttpStatusCode.Conflict
-              ? ("amount-is-too-small" as const)
-              : undefined;
-
-        if (!cashout) {
-          setCalc(undefined); // silent failure
-          return;
-        }
-        
-        setCalc({ cashin, cashout });
-      });
+  calculate.onSuccess = (resp) => setCalc(resp);
+  calculate.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.BadRequest:
+        return i18n.str`The server didn't understand the request.`;
+      case HttpStatusCode.Conflict:
+        return i18n.str`The amount is too small`;
+      case HttpStatusCode.NotImplemented:
+        return i18n.str`Conversion is not implemented.`;
+      case TalerErrorCode.GENERIC_PARAMETER_MISSING:
+        return i18n.str`At least debit or credit needs to be provided`;
+      case TalerErrorCode.GENERIC_PARAMETER_MALFORMED:
+        return i18n.str`The amount is malfored`;
+      case TalerErrorCode.GENERIC_CURRENCY_MISMATCH:
+        return i18n.str`The currency is not supported`;
     }
-    doAsync();
+  };
+
+  useEffect(() => {
+    calculate.call();
   }, [amount]);
 
-  if (!info) {
-    return <Loading />;
-  }
-
-  const cashinCalc =
-    calculationResult?.cashin === "amount-is-too-small"
-      ? undefined
-      : calculationResult?.cashin;
-  const cashoutCalc =
-    calculationResult?.cashout === "amount-is-too-small"
-      ? undefined
-      : calculationResult?.cashout;
+  const cashinCalc = calculationResult?.cashin;
+  const cashoutCalc = calculationResult?.cashout;
 
   return (
     <Fragment>
