@@ -16,33 +16,28 @@
 
 import {
   Amounts,
+  HostPortPath,
   HttpStatusCode,
+  Paytos,
   TalerCoreBankErrorsByMethod,
   TalerCorebankApi,
   TalerError,
+  TalerUris,
   assertUnreachable,
-  parsePaytoUri,
-  parseWithdrawUri,
-  stringifyWithdrawUri,
 } from "@gnu-taler/taler-util";
-import { utils } from "@gnu-taler/web-util/browser";
+import { useBankCoreApiContext, utils } from "@gnu-taler/web-util/browser";
 import { useEffect, useState } from "preact/hooks";
-import { mutate } from "swr";
-import { useBankCoreApiContext } from "@gnu-taler/web-util/browser";
+import { useSettingsContext } from "../../context/settings.js";
 import { useWithdrawalDetails } from "../../hooks/account.js";
-import { useSessionState } from "../../hooks/session.js";
 import { useBankState } from "../../hooks/bank-state.js";
 import { usePreferences } from "../../hooks/preferences.js";
+import { useSessionState } from "../../hooks/session.js";
 import { Props, State } from "./index.js";
-import { useSettingsContext } from "../../context/settings.js";
 
 export function useComponentState({
-  currency,
   routeClose,
   onAbort,
   focus,
-  routeHere,
-  onAuthorizationRequired,
 }: Props): utils.RecursiveState<State> {
   const [preference] = usePreferences();
   const settings = useSettingsContext();
@@ -50,6 +45,7 @@ export function useComponentState({
   const { state: credentials } = useSessionState();
   const creds = credentials.status !== "loggedIn" ? undefined : credentials;
   const {
+    config,
     lib: { bank },
   } = useBankCoreApiContext();
 
@@ -60,7 +56,7 @@ export function useComponentState({
 
   async function doSilentStart() {
     // FIXME: if amount is not enough use balance
-    const parsedAmount = Amounts.parseOrThrow(`${currency}:${amount}`);
+    const parsedAmount = Amounts.parseOrThrow(`${config.currency}:${amount}`);
     if (!creds) return;
     const params: TalerCorebankApi.BankAccountCreateWithdrawalRequest =
       preference.fastWithdrawalForm
@@ -100,36 +96,11 @@ export function useComponentState({
     };
   }
 
-  const wid = withdrawalOperationId;
-
-  async function doAbort() {
-    if (!creds) return;
-    const resp = await bank.abortWithdrawalById(creds, wid);
-    if (resp.type === "ok") {
-      // updateBankState("currentWithdrawalOperationId", undefined)
-      onAbort();
-    } else {
-      return resp;
-    }
-  }
-
-  async function doConfirm(): Promise<
-    TalerCoreBankErrorsByMethod<"confirmWithdrawalById"> | undefined
-  > {
-    if (!creds) return;
-    const resp = await bank.confirmWithdrawalById(creds, wid);
-    if (resp.type === "ok") {
-      mutate(() => true); //clean withdrawal state
-    } else {
-      return resp;
-    }
-  }
-
-  const uri = stringifyWithdrawUri({
-    bankIntegrationApiBaseUrl: bank.getIntegrationAPI().href,
+  const parsedUri = TalerUris.createTalerWithdraw(
+    bank.getIntegrationAPI().href as HostPortPath,
     withdrawalOperationId,
-  });
-  const parsedUri = parseWithdrawUri(uri);
+  );
+  const uri = TalerUris.toString(parsedUri);
   if (!parsedUri) {
     return {
       status: "invalid-withdrawal",
@@ -209,12 +180,8 @@ export function useComponentState({
         uri: parsedUri,
         routeClose,
         focus,
-        onAbort: !creds
-          ? async () => {
-              onAbort();
-              return undefined;
-            }
-          : doAbort,
+        operationId: withdrawalOperationId,
+        onAbort,
       };
     }
 
@@ -228,9 +195,9 @@ export function useComponentState({
 
     const account = !data.selected_exchange_account
       ? undefined
-      : parsePaytoUri(data.selected_exchange_account);
+      : Paytos.fromString(data.selected_exchange_account);
 
-    if (!account) {
+    if (!account || account.type === "fail" || !account.body.targetType) {
       return {
         status: "invalid-payto",
         error: undefined,
@@ -241,18 +208,16 @@ export function useComponentState({
     return {
       status: "need-confirmation",
       error: undefined,
-      routeHere,
       details: {
-        account,
+        account: account.body,
         reserve: data.selected_reserve_pub,
         username: data.username,
         amount: !data.amount ? undefined : Amounts.parse(data.amount),
       },
-      onAuthorizationRequired,
+
       account: data.username,
-      id: withdrawalOperationId,
-      onAbort: !creds ? undefined : doAbort,
-      onConfirm: !creds ? undefined : doConfirm,
+      operationId: withdrawalOperationId,
+      onAbort,
     };
   };
 }
