@@ -17,6 +17,7 @@
 import {
   Amounts,
   HttpStatusCode,
+  PaytoType,
   TalerErrorCode,
   TalerUris,
   assertUnreachable,
@@ -25,7 +26,6 @@ import {
   Attention,
   ButtonBetter,
   LocalNotificationBanner,
-  makeSafeCall,
   notifyInfo,
   useBankCoreApiContext,
   useChallengeHandler,
@@ -37,14 +37,13 @@ import { Fragment, VNode, h } from "preact";
 import { useEffect } from "preact/hooks";
 import { QR } from "../../components/QR.js";
 import { usePreferences } from "../../hooks/preferences.js";
-import { useSessionState } from "../../hooks/session.js";
+import { LoggedIn, useSessionState } from "../../hooks/session.js";
 import { RenderAmount } from "../PaytoWireTransferForm.js";
 import { SolveMFAChallenges } from "../SolveMFA.js";
 import { ShouldBeSameUser } from "../WithdrawalConfirmationQuestion.js";
 import { State } from "./index.js";
-import { PaytoType } from "@gnu-taler/taler-util";
 
-const TALER_SCREEN_ID = 5;
+const TALER_SCREEN_ID = 6;
 
 export function InvalidPaytoView({ payto }: State.InvalidPayto) {
   return <div>Payto from server is not valid &quot;{payto}&quot;</div>;
@@ -69,7 +68,8 @@ export function NeedConfirmationView({
 }: State.NeedConfirmation) {
   const { i18n } = useTranslationContext();
   const [settings] = usePreferences();
-  const [notification, notifyOnError] = useLocalNotificationBetter();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
+
   const { state: credentials } = useSessionState();
   const creds = credentials.status !== "loggedIn" ? undefined : credentials;
   const mfa = useChallengeHandler();
@@ -82,68 +82,59 @@ export function NeedConfirmationView({
       ? Amounts.zeroOfCurrency(config.currency)
       : Amounts.parseOrThrow(config.wire_transfer_fees);
 
-  const doAbort = !creds
-    ? undefined
-    : notifyOnError(
-        makeSafeCall(
-          i18n,
-          () => bank.abortWithdrawalById(creds, operationId),
-          (suc) => {
-            onAbort();
-          },
-          (fail) => {
-            switch (fail.case) {
-              case HttpStatusCode.Conflict:
-                return i18n.str`The reserve operation has been confirmed previously and can't be aborted`;
-              case HttpStatusCode.BadRequest:
-                return i18n.str`The operation ID is invalid.`;
-              case HttpStatusCode.NotFound:
-                return i18n.str`The operation was not found.`;
-            }
-          },
-        ),
-      );
+  const abort = safeFunctionHandler(
+    (creds: LoggedIn) => bank.abortWithdrawalById(creds, operationId),
+    !creds ? undefined : [creds],
+  );
+  abort.onSuccess = onAbort;
+  abort.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Conflict:
+        return i18n.str`The reserve operation has been confirmed previously and can't be aborted`;
+      case HttpStatusCode.BadRequest:
+        return i18n.str`The operation ID is invalid.`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`The operation was not found.`;
+    }
+  };
 
-  const [doConfirm, repeatConfirm] = !creds
-    ? [undefined, undefined]
-    : mfa.withMfaHandler(({ challengeIds, onChallengeRequired }) =>
-        makeSafeCall(
-          i18n,
-          () =>
-            bank.confirmWithdrawalById(creds, {}, operationId, {
-              challengeIds,
-            }),
-          (suc) => {
-            if (!settings.showWithdrawalSuccess) {
-              notifyInfo(i18n.str`Wire transfer completed!`);
-            }
-          },
-          (fail) => {
-            switch (fail.case) {
-              case TalerErrorCode.BANK_CONFIRM_ABORT_CONFLICT:
-                return i18n.str`The withdrawal has been aborted previously and can't be confirmed`;
-              case TalerErrorCode.BANK_CONFIRM_INCOMPLETE:
-                return i18n.str`The withdrawal operation can't be confirmed before a wallet accepted the transaction.`;
-              case HttpStatusCode.BadRequest:
-                return i18n.str`The operation ID is invalid.`;
-              case HttpStatusCode.NotFound:
-                return i18n.str`The operation was not found.`;
-              case TalerErrorCode.BANK_UNALLOWED_DEBIT:
-                return i18n.str`Your balance is not sufficient for the operation.`;
-              case HttpStatusCode.Accepted: {
-                onChallengeRequired(fail.body);
-                return i18n.str`A second factor authentication is required.`;
-              }
-              case TalerErrorCode.BANK_AMOUNT_DIFFERS:
-                return i18n.str`The starting withdrawal amount and the confirmation amount differs.`;
-              case TalerErrorCode.BANK_AMOUNT_REQUIRED:
-                return i18n.str`The bank requires a bank account which has not been specified yet.`;
-            }
-          },
-        ),
-      );
+  const confirm = safeFunctionHandler(
+    (creds: LoggedIn, challengeIds: string[]) =>
+      bank.confirmWithdrawalById(creds, {}, operationId, { challengeIds }),
+    !creds ? undefined : [creds, []],
+  );
+  confirm.onSuccess = () => {
+    if (!settings.showWithdrawalSuccess) {
+      notifyInfo(i18n.str`Wire transfer completed!`);
+    }
+  };
+  confirm.onFail = (fail) => {
+    switch (fail.case) {
+      case TalerErrorCode.BANK_CONFIRM_ABORT_CONFLICT:
+        return i18n.str`The withdrawal has been aborted previously and can't be confirmed`;
+      case TalerErrorCode.BANK_CONFIRM_INCOMPLETE:
+        return i18n.str`The withdrawal operation can't be confirmed before a wallet accepted the transaction.`;
+      case HttpStatusCode.BadRequest:
+        return i18n.str`The operation ID is invalid.`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`The operation was not found.`;
+      case TalerErrorCode.BANK_UNALLOWED_DEBIT:
+        return i18n.str`Your balance is not sufficient for the operation.`;
+      case HttpStatusCode.Accepted: {
+        mfa.onChallengeRequired(fail.body);
+        return i18n.str`A second factor authentication is required.`;
+      }
+      case TalerErrorCode.BANK_AMOUNT_DIFFERS:
+        return i18n.str`The starting withdrawal amount and the confirmation amount differs.`;
+      case TalerErrorCode.BANK_AMOUNT_REQUIRED:
+        return i18n.str`The bank requires a bank account which has not been specified yet.`;
+    }
+  };
 
-  if (mfa.pendingChallenge && repeatConfirm) {
+  const repeatConfirm = confirm.lambda((ids: string[]) => {
+    return [confirm.args![0], ids];
+  });
+  if (mfa.pendingChallenge) {
     return (
       <SolveMFAChallenges
         currentChallenge={mfa.pendingChallenge}
@@ -357,7 +348,7 @@ export function NeedConfirmationView({
                   type="button"
                   name="cancel"
                   class="text-sm font-semibold leading-6 text-gray-900"
-                  onClick={doAbort}
+                  onClick={abort}
                 >
                   <i18n.Translate>Cancel</i18n.Translate>
                 </ButtonBetter>
@@ -365,7 +356,7 @@ export function NeedConfirmationView({
                   type="submit"
                   name="transfer"
                   class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-                  onClick={!doConfirm ? undefined : notifyOnError(doConfirm)}
+                  onClick={confirm}
                 >
                   <i18n.Translate>Transfer</i18n.Translate>
                 </ButtonBetter>
@@ -518,7 +509,8 @@ export function ReadyView({
 }: State.Ready): VNode {
   const { i18n } = useTranslationContext();
   const walletInegrationApi = useTalerWalletIntegrationAPI();
-  const [notification, withErrorHandler] = useLocalNotificationBetter();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
+
   const { state: credentials } = useSessionState();
   const creds = credentials.status !== "loggedIn" ? undefined : credentials;
   const {
@@ -535,25 +527,21 @@ export function ReadyView({
     walletInegrationApi.publishTalerAction(uri);
   }, []);
 
-  const doAbort = !creds
-    ? undefined
-    : makeSafeCall(
-        i18n,
-        () => bank.abortWithdrawalById(creds, operationId),
-        (suc) => {
-          onAbort();
-        },
-        (fail) => {
-          switch (fail.case) {
-            case HttpStatusCode.Conflict:
-              return i18n.str`The reserve operation has been confirmed previously and can't be aborted`;
-            case HttpStatusCode.BadRequest:
-              return i18n.str`The operation ID is invalid.`;
-            case HttpStatusCode.NotFound:
-              return i18n.str`The operation was not found.`;
-          }
-        },
-      );
+  const abort = safeFunctionHandler(
+    (creds: LoggedIn) => bank.abortWithdrawalById(creds, operationId),
+    !creds ? undefined : [creds],
+  );
+  abort.onSuccess = onAbort;
+  abort.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Conflict:
+        return i18n.str`The reserve operation has been confirmed previously and can't be aborted`;
+      case HttpStatusCode.BadRequest:
+        return i18n.str`The operation ID is invalid.`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`The operation was not found.`;
+    }
+  };
 
   return (
     <Fragment>
@@ -584,15 +572,15 @@ export function ReadyView({
             </p>
           </div>
           <div class="flex items-center justify-between gap-x-6  pt-2 mt-2 ">
-            <button
+            <ButtonBetter
               type="button"
               name="cancel"
               class="text-sm font-semibold leading-6 text-gray-900"
               // class="inline-flex items-center rounded-md bg-white px-3 py-2 text-sm font-semibold text-black shadow-sm "
-              onClick={onAbort}
+              onClick={abort}
             >
               <i18n.Translate>Cancel</i18n.Translate>
-            </button>
+            </ButtonBetter>
 
             <a
               href={talerWithdrawUri}
@@ -622,15 +610,13 @@ export function ReadyView({
           </div>
         </div>
         <div class="flex items-center justify-center gap-x-6 border-t border-gray-900/10 px-4 py-4 sm:px-8">
-          <button
+          <ButtonBetter
             type="button"
-            // class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md  px-3 py-2 text-sm font-semibold text-black shadow-sm hover:bg-red-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-red-600"
             class="text-sm font-semibold leading-6 text-gray-900"
-            // handler={onAbortHandler}
-            onClick={doAbort}
+            onClick={abort}
           >
             <i18n.Translate>Cancel</i18n.Translate>
-          </button>
+          </ButtonBetter>
         </div>
       </div>
     </Fragment>
