@@ -31,5 +31,7 @@ export const USD = tests.createExample(PaytoWireTransferForm, {
     currency: "USD",
     fraction: 0,
     value: 1,
+    negative: false,
+    saturated: false,
   },
 });
