@@ -17,13 +17,14 @@
 import {
   HttpStatusCode,
   TalerUris,
-  WithdrawUriResult
+  WithdrawUriResult,
 } from "@gnu-taler/taler-util";
 import {
   Button,
+  ButtonBetter,
   LocalNotificationBanner,
   useBankCoreApiContext,
-  useLocalNotificationHandler,
+  useLocalNotificationBetter,
   useTalerWalletIntegrationAPI,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
@@ -31,6 +32,7 @@ import { Fragment, h, VNode } from "preact";
 import { useEffect } from "preact/hooks";
 import { QR } from "../components/QR.js";
 import { useSessionState } from "../hooks/session.js";
+import { UserAndToken } from "@gnu-taler/taler-util";
 
 const TALER_SCREEN_ID = 109;
 
@@ -51,29 +53,30 @@ export function QrCodeSection({
     walletInegrationApi.publishTalerAction(withdrawUri);
   }, []);
 
-  const [notification, handleError] = useLocalNotificationHandler();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
 
   const {
     lib: { bank: api },
   } = useBankCoreApiContext();
 
-  const onAbortHandler = !creds ? undefined : handleError(
-    async () => {
-      return api.abortWithdrawalById(creds, withdrawUri.withdrawalOperationId);
-    },
-    onAborted,
-    (fail) => {
-      switch (fail.case) {
-        case HttpStatusCode.BadRequest:
-          return i18n.str`The operation ID is invalid.`;
-        case HttpStatusCode.NotFound:
-          return i18n.str`The operation was not found.`;
-        case HttpStatusCode.Conflict:
-          return i18n.str`The reserve operation has been confirmed previously and can't be aborted`;
-      }
-    },
+  const abort = safeFunctionHandler(
+    (creds: UserAndToken) =>
+      api.abortWithdrawalById(creds, withdrawUri.withdrawalOperationId),
+    !creds ? undefined : [creds],
   );
 
+  abort.onSuccess = onAborted;
+  abort.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.BadRequest:
+        return i18n.str`The operation ID is invalid.`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`The operation was not found.`;
+      case HttpStatusCode.Conflict:
+        return i18n.str`The reserve operation has been confirmed previously and can't be aborted`;
+    }
+  };
+
   return (
     <Fragment>
       <LocalNotificationBanner notification={notification} />
@@ -103,14 +106,14 @@ export function QrCodeSection({
             </p>
           </div>
           <div class="flex items-center justify-between gap-x-6  pt-2 mt-2 ">
-            <Button
+            <ButtonBetter
               type="button"
               name="cancel"
               class="text-sm font-semibold leading-6 text-gray-900"
-              handler={onAbortHandler}
+              onClick={abort}
             >
               <i18n.Translate>Cancel</i18n.Translate>
-            </Button>
+            </ButtonBetter>
             <a
               href={talerWithdrawUri}
               name="withdraw"
@@ -139,14 +142,14 @@ export function QrCodeSection({
           </div>
         </div>
         <div class="flex items-center justify-center gap-x-6 border-t border-gray-900/10 px-4 py-4 sm:px-8">
-          <Button
+          <ButtonBetter
             type="button"
             // class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md  px-3 py-2 text-sm font-semibold text-black shadow-sm hover:bg-red-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-red-600"
             class="text-sm font-semibold leading-6 text-gray-900"
-            handler={onAbortHandler}
+            onClick={abort}
           >
             <i18n.Translate>Cancel</i18n.Translate>
-          </Button>
+          </ButtonBetter>
         </div>
       </div>
     </Fragment>
