@@ -14,37 +14,36 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { parseWithdrawUri, stringifyWithdrawUri } from "@gnu-taler/taler-util";
 import { Attention, useTranslationContext } from "@gnu-taler/web-util/browser";
 import { VNode, h } from "preact";
 import { useBankCoreApiContext } from "@gnu-taler/web-util/browser";
 import { useBankState } from "../hooks/bank-state.js";
 import { RouteDefinition } from "@gnu-taler/web-util/browser";
 import { WithdrawalQRCode } from "./WithdrawalQRCode.js";
+import { HostPortPath } from "@gnu-taler/taler-util";
+import { TalerUris } from "@gnu-taler/taler-util";
+
+const TALER_SCREEN_ID = 115;
 
 export function WithdrawalOperationPage({
   operationId,
-  onAuthorizationRequired,
   onOperationAborted,
   routeClose,
   origin,
-  routeWithdrawalDetails,
 }: {
-  onAuthorizationRequired: () => void;
   operationId: string;
   origin: "from-bank-ui" | "from-wallet-ui";
   onOperationAborted: () => void;
   routeClose: RouteDefinition;
-  routeWithdrawalDetails: RouteDefinition<{ wopid: string }>;
 }): VNode {
   const {
     lib: { bank: api },
   } = useBankCoreApiContext();
-  const uri = stringifyWithdrawUri({
-    bankIntegrationApiBaseUrl: api.getIntegrationAPI().href,
-    withdrawalOperationId: operationId,
-  });
-  const parsedUri = parseWithdrawUri(uri);
+  const parsedUri = TalerUris.createTalerWithdraw(
+    api.getIntegrationAPI().href as HostPortPath,
+    operationId,
+  );
+  const uri = TalerUris.toString(parsedUri);
   const { i18n } = useTranslationContext();
   const [, updateBankState] = useBankState();
 
@@ -63,8 +62,6 @@ export function WithdrawalOperationPage({
     <WithdrawalQRCode
       withdrawUri={parsedUri}
       origin={origin}
-      routeWithdrawalDetails={routeWithdrawalDetails}
-      onAuthorizationRequired={onAuthorizationRequired}
       onOperationAborted={() => {
         updateBankState("currentWithdrawalOperationId", undefined);
         onOperationAborted();
