@@ -13,13 +13,12 @@
  You should have received a copy of the GNU General Public License along with
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
-import { HttpStatusCode, TalerErrorCode } from "@gnu-taler/taler-util";
+import { AccessToken, TalerCorebankApi } from "@gnu-taler/taler-util";
 import {
   ButtonBetter,
   LocalNotificationBanner,
   RouteDefinition,
   ShowInputErrorLabel,
-  makeSafeCall,
   notifyInfo,
   useBankCoreApiContext,
   useChallengeHandler,
@@ -33,6 +32,8 @@ import { undefinedIfEmpty } from "../../utils.js";
 import { doAutoFocus } from "../PaytoWireTransferForm.js";
 import { ProfileNavigation } from "../ProfileNavigation.js";
 import { SolveMFAChallenges } from "../SolveMFA.js";
+import { TalerErrorCode } from "@gnu-taler/taler-util";
+import { HttpStatusCode } from "@gnu-taler/taler-util";
 
 const TALER_SCREEN_ID = 119;
 
@@ -89,63 +90,68 @@ export function UpdateAccountPassword({
         ? i18n.str`Repeated password doesn't match`
         : undefined,
   });
-  const [notification, notifyOnError] = useLocalNotificationBetter();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
   const mfa = useChallengeHandler();
 
-  const request = !password
-    ? undefined
-    : {
-        old_password: current,
-        new_password: password,
-      };
+  const update = safeFunctionHandler(
+    (
+      token: AccessToken,
+      request: TalerCorebankApi.AccountPasswordChange,
+      challengeIds: string[],
+    ) =>
+      api.updatePassword({ username: accountName, token }, request, {
+        challengeIds,
+      }),
+    !password || !token
+      ? undefined
+      : [
+          token,
+          {
+            old_password: current,
+            new_password: password,
+          },
+          [],
+        ],
+  );
 
-  const [doUpdatePassword, repeatUpdatePassword] =
-    !token || !request || !!errors
-      ? [undefined, undefined]
-      : mfa.withMfaHandler(({ challengeIds, onChallengeRequired }) =>
-          makeSafeCall(
-            i18n,
-            () =>
-              api.updatePassword({ username: accountName, token }, request, {
-                challengeIds,
-              }),
-            (success) => {
-              notifyInfo(i18n.str`Password changed`);
-              onUpdateSuccess();
-            },
-            (fail) => {
-              switch (fail.case) {
-                case HttpStatusCode.Unauthorized:
-                  return i18n.str`Not authorized to change the password, maybe the session is invalid.`;
-                case HttpStatusCode.NotFound:
-                  return i18n.str`Account not found`;
-                case TalerErrorCode.BANK_NON_ADMIN_PATCH_MISSING_OLD_PASSWORD:
-                  return i18n.str`You need to provide the old password. If you don't have it contact your account administrator.`;
-                case TalerErrorCode.BANK_PATCH_BAD_OLD_PASSWORD:
-                  return i18n.str`Your current password doesn't match, can't change to a new password.`;
-                case HttpStatusCode.Accepted: {
-                  onChallengeRequired(fail.body);
-                  return i18n.str`A second factor authentication is required.`;
-                }
-                case HttpStatusCode.Forbidden:
-                  return i18n.str`You don't have the rights to change the password.`;
-                case TalerErrorCode.BANK_PASSWORD_TOO_SHORT:
-                  return i18n.str`The password is too short. Can't have less than 8 characters.`;
-                case TalerErrorCode.BANK_PASSWORD_TOO_LONG:
-                  return i18n.str`The password is too long. Can't have more than 64 characters.`;
-              }
-            },
-          ),
-        );
+  update.onSuccess = (success) => {
+    notifyInfo(i18n.str`Password changed`);
+    onUpdateSuccess();
+  };
+  update.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Not authorized to change the password, maybe the session is invalid.`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Account not found`;
+      case TalerErrorCode.BANK_NON_ADMIN_PATCH_MISSING_OLD_PASSWORD:
+        return i18n.str`You need to provide the old password. If you don't have it contact your account administrator.`;
+      case TalerErrorCode.BANK_PATCH_BAD_OLD_PASSWORD:
+        return i18n.str`Your current password doesn't match, can't change to a new password.`;
+      case HttpStatusCode.Accepted: {
+        mfa.onChallengeRequired(fail.body);
+        return i18n.str`A second factor authentication is required.`;
+      }
+      case HttpStatusCode.Forbidden:
+        return i18n.str`You don't have the rights to change the password.`;
+      case TalerErrorCode.BANK_PASSWORD_TOO_SHORT:
+        return i18n.str`The password is too short. Can't have less than 8 characters.`;
+      case TalerErrorCode.BANK_PASSWORD_TOO_LONG:
+        return i18n.str`The password is too long. Can't have more than 64 characters.`;
+    }
+  };
+  const repeatUpdate = update.lambda((ids: string[]) => {
+    return [update.args![0], update.args![1], ids];
+  });
 
-  if (mfa.pendingChallenge && repeatUpdatePassword) {
+  if (mfa.pendingChallenge) {
     return (
       <SolveMFAChallenges
         currentChallenge={mfa.pendingChallenge}
         description={i18n.str`Update account password.`}
         username={accountName}
         onCancel={mfa.doCancelChallenge}
-        onCompleted={repeatUpdatePassword}
+        onCompleted={repeatUpdate}
       />
     );
   }
@@ -292,9 +298,7 @@ export function UpdateAccountPassword({
               type="submit"
               name="change"
               class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-              onClick={
-                !doUpdatePassword ? undefined : notifyOnError(doUpdatePassword)
-              }
+              onClick={update}
             >
               <i18n.Translate>Change</i18n.Translate>
             </ButtonBetter>
