@@ -26,6 +26,7 @@ import {
 } from "@gnu-taler/taler-util";
 import {
   Attention,
+  ButtonBetter,
   ErrorLoading,
   InternationalizationAPI,
   Loading,
@@ -33,7 +34,7 @@ import {
   RouteDefinition,
   ShowInputErrorLabel,
   useBankCoreApiContext,
-  useLocalNotification,
+  useLocalNotificationBetter,
   useTranslationContext,
   utils,
 } from "@gnu-taler/web-util/browser";
@@ -59,6 +60,9 @@ import { InputAmount, RenderAmount } from "../PaytoWireTransferForm.js";
 import { ProfileNavigation } from "../ProfileNavigation.js";
 
 import { DescribeConversion } from "../admin/ConversionClassList.js";
+import { opEmptySuccess } from "@gnu-taler/taler-util";
+import { opFixedSuccess } from "@gnu-taler/taler-util";
+import { TalerErrorCode } from "@gnu-taler/taler-util";
 
 const TALER_SCREEN_ID = 126;
 
@@ -128,7 +132,7 @@ function useComponentState({
       lib: { conversion },
     } = useBankCoreApiContext();
 
-    const [notification, notify, handleError] = useLocalNotification();
+    const [notification, safeFunctionHandler] = useLocalNotificationBetter();
 
     const initalState: FormValues<FormType> = {
       amount: "100",
@@ -164,53 +168,52 @@ function useComponentState({
       cashout: TransferCalculation;
     }>();
 
-    useEffect(() => {
-      async function doAsync() {
-        await handleError(async () => {
-          if (!info) return;
-          if (!form.amount?.value || form.amount.error) return;
-          const in_amount = Amounts.parseOrThrow(
-            `${info.fiat_currency}:${form.amount.value}`,
-          );
-          const in_fee = Amounts.parseOrThrow(info.conversion_rate.cashin_fee);
-          const respCashin = await calculateCashinFromDebit(in_amount, in_fee);
+    const in_amount = !form.amount
+      ? undefined
+      : Amounts.parseOrThrow(`${info.fiat_currency}:${form.amount.value}`);
 
-          const cashin =
-            respCashin.type === "ok"
-              ? respCashin.body
-              : respCashin.case === HttpStatusCode.Conflict
-                ? ("amount-is-too-small" as const)
-                : undefined;
+    const in_fee = Amounts.parseOrThrow(info.conversion_rate.cashin_fee);
+    const out_fee = Amounts.parseOrThrow(info.conversion_rate.cashout_fee);
 
-          if (!cashin || cashin === "amount-is-too-small") {
-            setCalc(undefined);
-            return;
-          }
-          // const out_amount = Amounts.parseOrThrow(`${info.regional_currency}:${form.amount.value}`)
-          const out_fee = Amounts.parseOrThrow(
-            info.conversion_rate.cashout_fee,
-          );
-          const respCashout = await calculateCashoutFromDebit(
-            cashin.credit,
-            out_fee,
-          );
-
-          const cashout =
-            respCashout.type === "ok"
-              ? respCashout.body
-              : respCashout.case === HttpStatusCode.Conflict
-                ? ("amount-is-too-small" as const)
-                : undefined;
-                
-          if (!cashout) {
-            setCalc(undefined); // silent failure
-            return;
-          }
-
-          setCalc({ cashin, cashout });
-        });
+    const calculate = safeFunctionHandler(
+      async (amount: AmountJson) => {
+        const respCashin = await calculateCashinFromDebit(amount, in_fee);
+        if (respCashin.type === "fail") {
+          return respCashin;
+        }
+        const cashin = respCashin.body;
+        const respCashout = await calculateCashoutFromDebit(
+          cashin.credit,
+          out_fee,
+        );
+        if (respCashout.type === "fail") {
+          return respCashout;
+        }
+        const cashout = respCashout.body;
+        return opFixedSuccess({ cashin, cashout });
+      },
+      !in_amount || status.status === "fail" ? undefined : [in_amount],
+    );
+    calculate.onSuccess = (resp) => setCalc(resp);
+    calculate.onFail = (fail) => {
+      switch (fail.case) {
+        case HttpStatusCode.BadRequest:
+          return i18n.str`The server didn't understand the request.`;
+        case HttpStatusCode.Conflict:
+          return i18n.str`The amount is too small`;
+        case HttpStatusCode.NotImplemented:
+          return i18n.str`Conversion is not implemented.`;
+        case TalerErrorCode.GENERIC_PARAMETER_MISSING:
+          return i18n.str`At least debit or credit needs to be provided`;
+        case TalerErrorCode.GENERIC_PARAMETER_MALFORMED:
+          return i18n.str`The amount is malfored`;
+        case TalerErrorCode.GENERIC_CURRENCY_MISMATCH:
+          return i18n.str`The currency is not supported`;
       }
-      doAsync();
+    };
+
+    useEffect(() => {
+      calculate.call();
     }, [
       form.amount?.value,
       form.conv?.cashin_fee?.value,
@@ -220,50 +223,27 @@ function useComponentState({
     const [section, setSection] = useState<"detail" | "cashout" | "cashin">(
       "detail",
     );
-    const cashinCalc =
-      calculationResult?.cashin === "amount-is-too-small"
+    const cashinCalc = calculationResult?.cashin;
+    const cashoutCalc = calculationResult?.cashout;
+
+    const update = safeFunctionHandler(
+      conversion.updateConversionRate.bind(conversion),
+      !creds || status.status === "fail"
         ? undefined
-        : calculationResult?.cashin;
-    const cashoutCalc =
-      calculationResult?.cashout === "amount-is-too-small"
-        ? undefined
-        : calculationResult?.cashout;
-    async function doUpdate() {
-      if (!creds) return;
-      await handleError(async () => {
-        if (status.status === "fail") return;
-        const resp = await conversion.updateConversionRate(
-          creds.token,
-          status.result.conv,
-        );
-        if (resp.type === "ok") {
-          setSection("detail");
-        } else {
-          switch (resp.case) {
-            case HttpStatusCode.Unauthorized: {
-              return notify({
-                type: "error",
-                title: i18n.str`Wrong credentials`,
-                description: resp.detail?.hint as TranslatedString,
-                debug: resp.detail,
-                when: AbsoluteTime.now(),
-              });
-            }
-            case HttpStatusCode.NotImplemented: {
-              return notify({
-                type: "error",
-                title: i18n.str`Conversion is disabled`,
-                description: resp.detail?.hint as TranslatedString,
-                debug: resp.detail,
-                when: AbsoluteTime.now(),
-              });
-            }
-            default:
-              assertUnreachable(resp);
-          }
-        }
-      });
-    }
+        : [creds.token, status.result.conv],
+    );
+
+    update.onSuccess = () => {
+      setSection("detail");
+    };
+    update.onFail = (fail) => {
+      switch (fail.case) {
+        case HttpStatusCode.Unauthorized:
+          return i18n.str`Wrong credentials`;
+        case HttpStatusCode.NotImplemented:
+          return i18n.str`Conversion is disabled`;
+      }
+    };
 
     const in_ratio = Number.parseFloat(info.conversion_rate.cashin_ratio);
     const out_ratio = Number.parseFloat(info.conversion_rate.cashout_ratio);
@@ -603,18 +583,14 @@ function useComponentState({
                 <i18n.Translate>Cancel</i18n.Translate>
               </a>
               {section == "cashin" || section == "cashout" ? (
-                <Fragment>
-                  <button
-                    type="submit"
-                    name="update conversion"
-                    class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-                    onClick={async () => {
-                      doUpdate();
-                    }}
-                  >
-                    <i18n.Translate>Update</i18n.Translate>
-                  </button>
-                </Fragment>
+                <ButtonBetter
+                  type="submit"
+                  name="update conversion"
+                  class="disabled:opacity-50 disabled:cursor-default cursor-pointer rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
+                  onClick={update}
+                >
+                  <i18n.Translate>Update</i18n.Translate>
+                </ButtonBetter>
               ) : (
                 <div />
               )}
