@@ -166,7 +166,7 @@ export function ShowCashoutDetails({ id, routeClose }: Props): VNode {
                   <div class="flex items-center justify-between border-t-2 afu pt-4">
                     <dt class="flex items-center text-gray-600">
                       <span>
-                        <i18n.Translate>Transfered</i18n.Translate>
+                        <i18n.Translate>Transferred</i18n.Translate>
                       </span>
                     </dt>
                     <dd class="text-sm ">
