@@ -23,13 +23,13 @@ import {
 } from "@gnu-taler/taler-util";
 import {
   Attention,
-  Button,
+  ButtonBetter,
   LocalNotificationBanner,
   RouteDefinition,
   ShowInputErrorLabel,
   Time,
   useChallengerApiContext,
-  useLocalNotificationHandler,
+  useLocalNotificationBetter,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
@@ -96,7 +96,7 @@ export function AnswerChallenge({
   const { config, lib } = useChallengerApiContext();
   const { i18n } = useTranslationContext();
   const { sent, failed, completed } = useSessionState();
-  const [notification, withErrorHandler] = useLocalNotificationHandler();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
   const [pin, setPin] = useState<string | undefined>();
   const errors = undefinedIfEmpty({
     pin: !pin ? i18n.str`Can't be empty` : undefined,
@@ -117,7 +117,7 @@ export function AnswerChallenge({
       ? AbsoluteTime.never()
       : AbsoluteTime.fromProtocolTimestamp(deadlineTS);
   }, [deadlineTS?.t_s]);
-  
+
   useReloadOnDeadline(deadline);
 
   const lastAddr = !lastStatus?.last_address
@@ -130,79 +130,76 @@ export function AnswerChallenge({
   const unableToChangeAddr = !lastStatus || lastStatus.changes_left < 1;
   const contact = lastStatus?.last_address;
 
-  const onSendAgain =
+  const sendAgain = safeFunctionHandler(
+    lib.challenger.challenge.bind(lib.challenger),
     contact === undefined ||
-    lastStatus == undefined ||
-    lastStatus.pin_transmissions_left === 0 ||
-    !AbsoluteTime.isExpired(deadline)
+      lastStatus === undefined ||
+      lastStatus.pin_transmissions_left === 0 ||
+      !AbsoluteTime.isExpired(deadline)
       ? undefined
-      : withErrorHandler(
-          async () => {
-            return await lib.challenger.challenge(session.nonce, contact);
-          },
-          (ok) => {
-            if (ok.body.type === "completed") {
-              completed(ok.body);
-            } else {
-              sent(ok.body);
-            }
-          },
-          (fail) => {
-            switch (fail.case) {
-              case HttpStatusCode.BadRequest:
-                return i18n.str`The request was not accepted, try reloading the app.`;
-              case HttpStatusCode.NotFound:
-                return i18n.str`Challenge not found.`;
-              case HttpStatusCode.NotAcceptable:
-                return i18n.str`Server templates are missing due to misconfiguration.`;
-              case HttpStatusCode.TooManyRequests:
-                return i18n.str`There have been too many attempts to request challenge transmissions.`;
-              case HttpStatusCode.InternalServerError:
-                return i18n.str`Server is unable to respond due to internal problems.`;
-            }
-          },
-        );
+      : [session.nonce, contact],
+  );
+  sendAgain.onSuccess = (success) => {
+    if (success.type === "completed") {
+      completed(success);
+    } else {
+      sent(success);
+    }
+  };
+  sendAgain.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.BadRequest:
+        return i18n.str`The request was not accepted, try reloading the app.`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Challenge not found.`;
+      case HttpStatusCode.NotAcceptable:
+        return i18n.str`Server templates are missing due to misconfiguration.`;
+      case HttpStatusCode.TooManyRequests:
+        return i18n.str`There have been too many attempts to request challenge transmissions.`;
+      case HttpStatusCode.InternalServerError:
+        return i18n.str`Server is unable to respond due to internal problems.`;
+    }
+  };
 
-  const onCheck =
+  const check = safeFunctionHandler(
+    lib.challenger.solve.bind(lib.challenger),
     errors !== undefined ||
-    lastStatus == undefined ||
-    lastStatus.auth_attempts_left === 0
+      lastStatus == undefined ||
+      lastStatus.auth_attempts_left === 0 ||
+      !pin
       ? undefined
-      : withErrorHandler(
-          async () => {
-            return lib.challenger.solve(session.nonce, { pin: pin! });
-          },
-          (ok) => {
-            if (ok.body.type === "completed") {
-              completed(ok.body);
-            } else {
-              failed(ok.body);
-            }
-            onComplete();
-          },
-          (fail) => {
-            switch (fail.case) {
-              case HttpStatusCode.BadRequest:
-                return i18n.str`The request was not accepted, try reloading the app.`;
-              case HttpStatusCode.Forbidden: {
-                revalidateChallengeSession();
-                return i18n.str`Invalid pin.`;
-              }
-              case HttpStatusCode.NotFound:
-                return i18n.str`Challenge not found.`;
-              case HttpStatusCode.NotAcceptable:
-                return i18n.str`Server templates are missing due to misconfiguration.`;
-              case HttpStatusCode.TooManyRequests: {
-                revalidateChallengeSession();
-                return i18n.str`There have been too many attempts to request challenge transmissions.`;
-              }
-              case HttpStatusCode.InternalServerError:
-                return i18n.str`Server is unable to respond due to internal problems.`;
-              default:
-                assertUnreachable(fail);
-            }
-          },
-        );
+      : [session.nonce, { pin }],
+  );
+  check.onSuccess = (success) => {
+    if (success.type === "completed") {
+      completed(success);
+    } else {
+      failed(success);
+    }
+    onComplete();
+  };
+  check.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.BadRequest:
+        return i18n.str`The request was not accepted, try reloading the app.`;
+      case HttpStatusCode.Forbidden: {
+        revalidateChallengeSession();
+        return i18n.str`Invalid pin.`;
+      }
+      case HttpStatusCode.NotFound:
+        return i18n.str`Challenge not found.`;
+      case HttpStatusCode.NotAcceptable:
+        return i18n.str`Server templates are missing due to misconfiguration.`;
+      case HttpStatusCode.TooManyRequests: {
+        revalidateChallengeSession();
+        return i18n.str`There have been too many attempts to request challenge transmissions.`;
+      }
+      case HttpStatusCode.InternalServerError:
+        return i18n.str`Server is unable to respond due to internal problems.`;
+      default:
+        assertUnreachable(fail);
+    }
+  };
   const cantTryAnymore = lastStatus?.auth_attempts_left === 0;
 
   function LastContactSent(): VNode {
@@ -259,14 +256,13 @@ export function AnswerChallenge({
           )}
         </div>
         <div>
-          <Button
+          <ButtonBetter
             type="submit"
-            disabled={!onSendAgain}
             class="block w-full disabled:bg-gray-300 rounded-md bg-indigo-600 px-3.5 py-2.5 text-center text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-            handler={onSendAgain}
+            onClick={sendAgain}
           >
             <i18n.Translate>Send new code</i18n.Translate>
-          </Button>
+          </ButtonBetter>
           {lastStatus === undefined ? undefined : (
             <p class="mt-2 text-sm leading-6 text-gray-400">
               {lastStatus.pin_transmissions_left < 1 ? (
@@ -381,14 +377,13 @@ export function AnswerChallenge({
           </div>
 
           <div class="mt-10">
-            <Button
+            <ButtonBetter
               type="submit"
               class="block w-full disabled:bg-gray-300 rounded-md bg-indigo-600 px-3.5 py-2.5 text-center text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-              disabled={!onCheck}
-              handler={onCheck}
+              onClick={check}
             >
               <i18n.Translate>Check</i18n.Translate>
-            </Button>
+            </ButtonBetter>
           </div>
         </form>
 
