@@ -25,7 +25,7 @@ import {
 } from "@gnu-taler/taler-util";
 import {
   Attention,
-  Button,
+  ButtonBetter,
   countryNameList,
   ErrorLoading,
   FormDesign,
@@ -34,16 +34,14 @@ import {
   RouteDefinition,
   useChallengerApiContext,
   useForm,
-  useLocalNotificationHandler,
+  useLocalNotificationBetter,
   useTranslationContext
 } from "@gnu-taler/web-util/browser";
 import { Fragment, h, VNode } from "preact";
 import { useState } from "preact/hooks";
 import { useChallengeSession } from "../hooks/challenge.js";
 import { SessionId, useSessionState } from "../hooks/session.js";
-import {
-  getAddressDescriptionFromAddrType
-} from "./AnswerChallenge.js";
+import { getAddressDescriptionFromAddrType } from "./AnswerChallenge.js";
 
 type Props = {
   onSendSuccesful: () => void;
@@ -62,7 +60,7 @@ export function AskChallenge({
   const { lib, config } = useChallengerApiContext();
 
   const { i18n } = useTranslationContext();
-  const [notification, withErrorHandler] = useLocalNotificationHandler();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
   // const [address, setEmail] = useState<string | undefined>();
   const [addrIndex, setAddrIndex] = useState<number | undefined>();
 
@@ -161,43 +159,34 @@ export function AskChallenge({
       ? undefined
       : (form.status.result as Record<string, string>);
 
-  const onSend =
-    form.status.errors || !contact
-      ? undefined
-      : withErrorHandler(
-          async () => {
-            const info = lastStatus.fix_address
-              ? lastStatus.last_address!
-              : contact;
+  const info = lastStatus.fix_address ? lastStatus.last_address! : contact;
 
-            return lib.challenger.challenge(session.nonce, info);
-          },
-          (ok) => {
-            if (ok.body.type === "completed") {
-              completed(ok.body);
-            } else {
-              // if (remember) {
-              //   saveAddress(config.address_type, contact);
-              // }
-              sent(ok.body);
-            }
-            onSendSuccesful();
-          },
-          (fail) => {
-            switch (fail.case) {
-              case HttpStatusCode.BadRequest:
-                return i18n.str`The request was not accepted, try reloading the app.`;
-              case HttpStatusCode.NotFound:
-                return i18n.str`Challenge not found.`;
-              case HttpStatusCode.NotAcceptable:
-                return i18n.str`Server templates are missing due to misconfiguration.`;
-              case HttpStatusCode.TooManyRequests:
-                return i18n.str`There have been too many attempts to request challenge transmissions.`;
-              case HttpStatusCode.InternalServerError:
-                return i18n.str`Server is unable to respond due to internal problems.`;
-            }
-          },
-        );
+  const send = safeFunctionHandler(
+    lib.challenger.challenge.bind(lib.challenger),
+    form.status.errors || !info ? undefined : [session.nonce, info],
+  );
+  send.onSuccess = (ok) => {
+    if (ok.type === "completed") {
+      completed(ok);
+    } else {
+      sent(ok);
+    }
+    onSendSuccesful();
+  };
+  send.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.BadRequest:
+        return i18n.str`The request was not accepted, try reloading the app.`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Challenge not found.`;
+      case HttpStatusCode.NotAcceptable:
+        return i18n.str`Server templates are missing due to misconfiguration.`;
+      case HttpStatusCode.TooManyRequests:
+        return i18n.str`There have been too many attempts to request challenge transmissions.`;
+      case HttpStatusCode.InternalServerError:
+        return i18n.str`Server is unable to respond due to internal problems.`;
+    }
+  };
 
   return (
     <Fragment>
@@ -383,7 +372,7 @@ export function AskChallenge({
           <FormUI
             design={design}
             model={form.model}
-            onSubmit={onSend?.onClick}
+            onSubmit={send.call}
           />
         </div>
 
@@ -439,11 +428,10 @@ export function AskChallenge({
         <div class="mx-auto mt-4 max-w-xl ">
           {!prevAddr ? (
             <div class="mt-10">
-              <Button
+              <ButtonBetter
                 type="submit"
-                disabled={!onSend}
                 class="block w-full disabled:bg-gray-300 rounded-md bg-indigo-600 px-3.5 py-2.5 text-center text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-                handler={onSend}
+                onClick={send}
               >
                 {(function (): TranslatedString {
                   switch (config.address_type) {
@@ -456,15 +444,14 @@ export function AskChallenge({
                       return i18n.str`Send SMS`;
                   }
                 })()}
-              </Button>
+              </ButtonBetter>
             </div>
           ) : (
             <div class="mt-10">
-              <Button
+              <ButtonBetter
                 type="submit"
-                disabled={!onSend}
                 class="block w-full disabled:bg-gray-300 rounded-md bg-indigo-600 px-3.5 py-2.5 text-center text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-                handler={onSend}
+                onClick={send}
               >
                 {(function (): TranslatedString {
                   switch (config.address_type) {
@@ -483,7 +470,7 @@ export function AskChallenge({
                         : i18n.str`Change phone`;
                   }
                 })()}
-              </Button>
+              </ButtonBetter>
             </div>
           )}
         </div>
