@@ -15,17 +15,17 @@
  */
 import {
   HttpStatusCode,
-  createClientSecretAccessToken,
   createRFC8959AccessTokenEncoded,
   encodeCrock,
   randomBytes,
 } from "@gnu-taler/taler-util";
 import {
   Button,
+  ButtonBetter,
   LocalNotificationBanner,
   ShowInputErrorLabel,
   useChallengerApiContext,
-  useLocalNotificationHandler,
+  useLocalNotificationBetter,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
@@ -33,6 +33,7 @@ import { useState } from "preact/hooks";
 import { safeToURL } from "../Routing.js";
 import { useSessionState } from "../hooks/session.js";
 import { doAutoFocus, undefinedIfEmpty } from "./AnswerChallenge.js";
+import { AccessToken } from "@gnu-taler/taler-util";
 
 type Props = {
   clientId: string;
@@ -49,51 +50,45 @@ export function Setup({
   onCreated,
 }: Props): VNode {
   const { i18n } = useTranslationContext();
-  const [notification, withErrorHandler] = useLocalNotificationHandler();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
   const { lib } = useChallengerApiContext();
   const { start } = useSessionState();
   const [password, setPassword] = useState<string | undefined>(secret);
   const [url, setUrl] = useState<string | undefined>(redirectURL?.href);
 
   const errors = undefinedIfEmpty({
-    password: !password ? i18n.str`required` : undefined,
+    password: !password ? i18n.str`Required` : undefined,
     url: !url
-      ? i18n.str`required`
+      ? i18n.str`Required`
       : !safeToURL(url)
-        ? i18n.str`invalid format`
+        ? i18n.str`Invalid format`
         : undefined,
   });
 
-  const onStart =
+  const doStart = safeFunctionHandler(
+    (token: AccessToken, url) => lib.challenger.setup(clientId, token),
     !!errors || password === undefined || url === undefined
       ? undefined
-      : withErrorHandler(
-          async () => {
-            return lib.challenger.setup(
-              clientId,
-              createRFC8959AccessTokenEncoded(password),
-            );
-          },
-          (ok) => {
-            start();
+      : [createRFC8959AccessTokenEncoded(password), url],
+  );
+  doStart.onSuccess = (ok, token, redirect_uri) => {
+    start();
+    const redirect = new URL(window.location.href);
+    redirect.searchParams.set("client_id", clientId);
+    redirect.searchParams.set("redirect_uri", redirect_uri);
+    redirect.searchParams.set("state", encodeCrock(randomBytes(32)));
+    redirect.searchParams.set("nonce", ok.nonce);
+    redirect.hash = "";
+    window.location.href = redirect.href;
+    onCreated();
+  };
 
-            const redirect = new URL(window.location.href)
-            redirect.searchParams.set("client_id",clientId)
-            redirect.searchParams.set("redirect_uri",url)
-            redirect.searchParams.set("state",encodeCrock(randomBytes(32)))
-            redirect.searchParams.set("nonce",ok.body.nonce)
-            redirect.hash = ""
-            window.location.href = redirect.href
-
-            onCreated();
-          },
-          (fail) => {
-            switch (fail.case) {
-              case HttpStatusCode.NotFound:
-                return i18n.str`Client doesn't exist.`;
-            }
-          },
-        );
+  doStart.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.NotFound:
+        return i18n.str`The server doesn't know about this client. Either the URL or the secret is wrong.`;
+    }
+  };
 
   return (
     <Fragment>
@@ -173,14 +168,13 @@ export function Setup({
           </div>
         </form>
         <div class="mt-10">
-          <Button
+          <ButtonBetter
             type="submit"
-            disabled={!onStart}
             class="block w-full disabled:bg-gray-300 rounded-md bg-indigo-600 px-3.5 py-2.5 text-center text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
-            handler={onStart}
+            onClick={doStart}
           >
             <i18n.Translate>Start</i18n.Translate>
-          </Button>
+          </ButtonBetter>
         </div>
       </div>
     </Fragment>
