@@ -210,6 +210,7 @@ function furthestKey(
   if (!forward && cmpResult === -1) {
     return key1;
   }
+  return undefined;
 }
 
 export interface AccessStats {
@@ -1448,7 +1449,7 @@ export class MemoryBackend implements Backend {
     }
   }
 
-  rollback(btx: DatabaseTransaction): void {
+  async rollback(btx: DatabaseTransaction): Promise<void> {
     if (this.enableTracing) {
       console.log(`TRACING: rollback`);
     }
