@@ -8,10 +8,10 @@ import {
   IDBRequest,
   IDBTransaction,
 } from "../idbtypes.js";
-import { initTestIndexedDB , useTestIndexedDb } from "../testingdb.js";
+import { useTestIndexedDb } from "../testingdb.js";
 import { compareKeys } from "../util/cmp.js";
 
-export { initTestIndexedDB, useTestIndexedDb } from "../testingdb.js"
+export { initTestIndexedDB, useTestIndexedDb } from "../testingdb.js";
 
 export function createdb(
   t: ExecutionContext<unknown>,
@@ -409,20 +409,18 @@ export function format_value(val: any, seen?: any): string {
       if (val === null) {
         return "null";
       }
-
-    /* falls through */
-    default:
-      try {
-        return typeof val + ' "' + truncate(String(val), 1000) + '"';
-      } catch (e) {
-        return (
-          "[stringifying object threw " +
-          String(e) +
-          " with type " +
-          String(typeof e) +
-          "]"
-        );
-      }
+      break;
+  }
+  try {
+    return typeof val + ' "' + truncate(String(val), 1000) + '"';
+  } catch (e) {
+    return (
+      "[stringifying object threw " +
+      String(e) +
+      " with type " +
+      String(typeof e) +
+      "]"
+    );
   }
 }
 
