@@ -24,7 +24,7 @@ import {
 import {
   AcceptTermOfServiceContext,
   Attention,
-  Button,
+  ButtonBetter,
   ErrorsSummary,
   FormMetadata,
   FormUI,
@@ -34,8 +34,8 @@ import {
   preloadedForms,
   useAsyncAsHook,
   useExchangeApiContext,
-  useLocalNotificationHandler,
-  useTranslationContext,
+  useLocalNotificationBetter,
+  useTranslationContext
 } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useFormMeta } from "../../../web-util/src/hooks/useForm.js";
@@ -115,7 +115,7 @@ function ShowForm({
   onComplete: () => void;
 }): VNode {
   const { lib } = useExchangeApiContext();
-  const [notification, withErrorHandler] = useLocalNotificationHandler();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
   const [preferences] = usePreferences();
   const { i18n } = useTranslationContext();
 
@@ -125,33 +125,29 @@ function ShowForm({
     design,
   } = useFormMeta<FormType>(theForm, formContext, {});
   const validatedForm = status.status !== "ok" ? undefined : status.result;
+  if (validatedForm) {
+    validatedForm[TalerFormAttributes.FORM_ID] = theForm.id;
+    validatedForm[TalerFormAttributes.FORM_VERSION] = theForm.version;
+    validatedForm[TalerFormAttributes.FORM_CONTEXT] = formContext;
+  }
 
-  const submitHandler =
-    validatedForm === undefined
-      ? undefined
-      : withErrorHandler(
-          async () => {
-            validatedForm[TalerFormAttributes.FORM_ID] = theForm.id;
-            validatedForm[TalerFormAttributes.FORM_VERSION] = theForm.version;
-            validatedForm[TalerFormAttributes.FORM_CONTEXT] = formContext;
-            return lib.exchange.uploadKycForm(reqId, validatedForm);
-          },
-          (res) => {
-            onComplete();
-          },
-          (fail) => {
-            switch (fail.case) {
-              case HttpStatusCode.PayloadTooLarge:
-                return i18n.str`The form is too big for the server, try uploading smaller files.`;
-              case HttpStatusCode.NotFound:
-                return i18n.str`The account was not found`;
-              case HttpStatusCode.Conflict:
-                return i18n.str`Officer disabled or more recent decision was already submitted.`;
-              default:
-                assertUnreachable(fail);
-            }
-          },
-        );
+  const submit = safeFunctionHandler(
+    lib.exchange.uploadKycForm.bind(lib.exchange),
+    !validatedForm ? undefined : [reqId, validatedForm],
+  );
+  submit.onSuccess = onComplete;
+  submit.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.PayloadTooLarge:
+        return i18n.str`The form is too big for the server, try uploading smaller files.`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`The account was not found`;
+      case HttpStatusCode.Conflict:
+        return i18n.str`Officer disabled or more recent decision was already submitted.`;
+      default:
+        assertUnreachable(fail);
+    }
+  };
 
   return (
     <div class="rounded-lg bg-white px-5 py-6 shadow m-4">
@@ -181,14 +177,13 @@ function ShowForm({
         >
           <i18n.Translate>Cancel</i18n.Translate>
         </button>
-        <Button
+        <ButtonBetter
           type="submit"
-          handler={submitHandler}
-          disabled={!submitHandler}
           class="disabled:opacity-50 disabled:cursor-default rounded-md bg-indigo-600 px-3 py-2 text-sm font-semibold text-white shadow-sm hover:bg-indigo-500 focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-indigo-600"
+          onClick={submit}
         >
           <i18n.Translate>Submit</i18n.Translate>
-        </Button>
+        </ButtonBetter>
       </div>
       {!status.errors ? undefined : <ErrorsSummary errors={status.errors} />}
     </div>
