@@ -14,20 +14,20 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 import {
-  AbsoluteTime,
   AccessToken,
   HttpStatusCode,
   KycRequirementInformation,
   TalerError,
-  assertUnreachable,
+  assertUnreachable
 } from "@gnu-taler/taler-util";
 import {
   Attention,
+  ButtonBetter,
   ErrorLoading,
   Loading,
   LocalNotificationBanner,
   useExchangeApiContext,
-  useLocalNotificationHandler,
+  useLocalNotificationBetter,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
@@ -168,9 +168,26 @@ export function Start({ token }: Props): VNode {
 function LinkGenerator({ req }: { req: KycRequirementInformation }): VNode {
   const { i18n } = useTranslationContext();
 
-  const [notification, , notify] = useLocalNotificationHandler();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
   const { lib } = useExchangeApiContext();
 
+  const start = safeFunctionHandler(async (id: string) => {
+    return lib.exchange.startExternalKycProcess(id);
+  },[req.id!]);
+  start.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.NotFound:
+        return i18n.str`not found`;
+      case HttpStatusCode.Conflict:
+        return i18n.str`conflict`;
+      case HttpStatusCode.PayloadTooLarge:
+        return i18n.str`payload is too large`;
+    }
+  };
+  start.onSuccess = (success) => {
+    setRedirectUrl(success.redirect_url);
+    return i18n.str`Link generated, you can proceed.`;
+  };
   const [redirectUrl, setRedirectUrl] = useState<string>();
   const row = (
     <Fragment>
@@ -218,56 +235,12 @@ function LinkGenerator({ req }: { req: KycRequirementInformation }): VNode {
           ) : (
             // href={redirectUrl}
             <p class="text-sm font-semibold leading-6 text-gray-900">
-              <button
-                onClick={async () => {
-                  const res = await lib.exchange.startExternalKycProcess(
-                    req.id!,
-                  );
-                  if (res.type === "ok") {
-                    setRedirectUrl(res.body.redirect_url);
-                    notify({
-                      type: "info",
-                      title: i18n.str`Link generated, you can proceed.`,
-                      when: AbsoluteTime.now(),
-                    });
-                  } else {
-                    switch (res.case) {
-                      case HttpStatusCode.NotFound: {
-                        notify({
-                          type: "error",
-                          title: i18n.str`could not create redirect url`,
-                          description: i18n.str`not found`,
-                          when: AbsoluteTime.now(),
-                        });
-                        break;
-                      }
-                      case HttpStatusCode.Conflict: {
-                        notify({
-                          type: "error",
-                          title: i18n.str`could not create redirect url`,
-                          description: i18n.str`conflict`,
-                          when: AbsoluteTime.now(),
-                        });
-                        break;
-                      }
-                      case HttpStatusCode.PayloadTooLarge: {
-                        notify({
-                          type: "error",
-                          title: i18n.str`could not create redirect url`,
-                          description: i18n.str`payload too large`,
-                          when: AbsoluteTime.now(),
-                        });
-                        break;
-                      }
-                    }
-                  }
-                }}
-              >
+              <ButtonBetter onClick={start}>
                 <span class="absolute inset-x-0 -top-px bottom-0"></span>
                 <i18n.Translate context="KYC_REQUIREMENT_LINK_DESCRIPTION">
                   {req.description}
                 </i18n.Translate>
-              </button>
+              </ButtonBetter>
             </p>
           )}
         </div>
