@@ -27,6 +27,7 @@ import "../css/pure-min.css";
 import "../css/style.css";
 import { MerchantBackend } from "../declaration";
 import { Page, InfoBox, TableExpanded, TableSimple } from "../styled";
+import { TIME_DATE_FORMAT } from "../utils";
 
 /**
  * This page creates a payment request QR code
@@ -56,6 +57,7 @@ function Head({ order_summary }: { order_summary?: string }): VNode {
     <Fragment>
       <meta charSet="UTF-8" />
       <meta name="viewport" content="width=device-width, initial-scale=1.0" />
+      <meta name="taler-support" content="uri" />
       <noscript>
         <meta http-equiv="refresh" content="1" />
       </noscript>
@@ -79,7 +81,7 @@ function Location({
   location: MerchantBackend.Location | undefined;
   btr?: boolean;
 }) {
-  //FIXME: mustache strings show be constructed in a way that ends in the final output of the html but is not present in the
+  //FIXME: mustache strings will be constructed in a way that ends in the final output of the html but is not present in the
   // javascript code, otherwise when mustache render engine run over the html it will also replace string in the javascript code
   // that is made to run when the browser has javascript enable leading into undefined behavior.
   // that's why in the next fields we are using concatenations to build the mustache placeholder.
@@ -196,9 +198,9 @@ export function ShowOrderDetails({
               {contract_terms?.timestamp
                 ? contract_terms?.timestamp.t_s != "never"
                   ? format(
-                      contract_terms?.timestamp.t_s,
-                      "dd MMM yyyy HH:mm:ss",
-                    )
+                    contract_terms?.timestamp.t_s * 1000,
+                    TIME_DATE_FORMAT,
+                  )
                   : "never"
                 : `{{ contract_terms.timestamp_str }}`}{" "}
             </dd>
@@ -256,9 +258,9 @@ export function ShowOrderDetails({
                             {p.delivery_date
                               ? p.delivery_date.t_s != "never"
                                 ? format(
-                                    p.delivery_date.t_s,
-                                    "dd MMM yyyy HH:mm:ss",
-                                  )
+                                  p.delivery_date.t_s,
+                                  TIME_DATE_FORMAT,
+                                )
                                 : "never"
                               : `{{ delivery_date_str }}`}{" "}
                           </dd>
@@ -306,9 +308,9 @@ export function ShowOrderDetails({
                     {contract_terms?.delivery_date
                       ? contract_terms?.delivery_date.t_s != "never"
                         ? format(
-                            contract_terms?.delivery_date.t_s,
-                            "dd MMM yyyy HH:mm:ss",
-                          )
+                          contract_terms?.delivery_date.t_s,
+                          TIME_DATE_FORMAT,
+                        )
                         : "never"
                       : `{{ contract_terms.delivery_date_str }}`}{" "}
                   </dd>
@@ -336,48 +338,47 @@ export function ShowOrderDetails({
         <section>
           <h2>Full payment information</h2>
           <TableExpanded>
-            <dt>Amount paid:</dt>
-            <dd>{contract_terms?.amount || `{{ contract_terms.amount }}`}</dd>
-            <dt>Wire transfer method:</dt>
-            <dd>
-              {contract_terms?.wire_method ||
-                `{{ contract_terms.wire_method }}`}
-            </dd>
-            <dt>Payment deadline:</dt>
-            <dd>
-              {contract_terms?.pay_deadline
-                ? contract_terms?.pay_deadline.t_s != "never"
-                  ? format(
-                      contract_terms?.pay_deadline.t_s,
-                      "dd MMM yyyy HH:mm:ss",
-                    )
-                  : "never"
-                : `{{ contract_terms.pay_deadline_str }}`}{" "}
-            </dd>
             <dt>Exchange transfer deadline:</dt>
+            {btr && `{{` + `#contract_terms.wire_transfer_deadline_str}}`}
             <dd>
               {contract_terms?.wire_transfer_deadline
                 ? contract_terms?.wire_transfer_deadline.t_s != "never"
                   ? format(
-                      contract_terms?.wire_transfer_deadline.t_s,
-                      "dd MMM yyyy HH:mm:ss",
-                    )
+                    contract_terms?.wire_transfer_deadline.t_s * 1000,
+                    TIME_DATE_FORMAT,
+                  )
                   : "never"
                 : `{{ contract_terms.wire_transfer_deadline_str }}`}{" "}
             </dd>
+            {btr && `{{` + `/contract_terms.wire_transfer_deadline_str}}`}
+
+            {btr && `{{` + `^contract_terms.wire_transfer_deadline_str}}`}
+            <dd>
+              Wire transfer settled.
+            </dd>
+            {btr && `{{` + `/contract_terms.wire_transfer_deadline_str}}`}
+
+            {btr && `{{` + `#contract_terms.max_fee}}`}
             <dt>Maximum deposit fee:</dt>
             <dd>{contract_terms?.max_fee || `{{ contract_terms.max_fee }}`}</dd>
+            {btr && `{{` + `/contract_terms.max_fee}}`}
+
+            {btr && `{{` + `#contract_terms.max_wire_fee}}`}
             <dt>Maximum wire fee:</dt>
             <dd>
               {contract_terms?.max_wire_fee ||
                 `{{ contract_terms.max_wire_fee }}`}
             </dd>
+            {btr && `{{` + `/contract_terms.max_wire_fee}}`}
+
+            {btr && `{{` + `#contract_terms.wire_fee_amortization}}`}
             <dt>Wire fee amortization:</dt>
             <dd>
               {contract_terms?.wire_fee_amortization ||
                 `{{ contract_terms.wire_fee_amortization }}`}{" "}
               transactions
             </dd>
+            {btr && `{{` + `/contract_terms.wire_fee_amortization}}`}
           </TableExpanded>
         </section>
 
@@ -389,9 +390,9 @@ export function ShowOrderDetails({
               {contract_terms?.refund_deadline
                 ? contract_terms?.refund_deadline.t_s != "never"
                   ? format(
-                      contract_terms?.refund_deadline.t_s,
-                      "dd MMM yyyy HH:mm:ss",
-                    )
+                    contract_terms?.refund_deadline.t_s * 1000,
+                    TIME_DATE_FORMAT,
+                  )
                   : "never"
                 : `{{ contract_terms.refund_deadline_str }}`}{" "}
             </dd>
@@ -404,11 +405,11 @@ export function ShowOrderDetails({
                   {contract_terms?.auto_refund
                     ? contract_terms?.auto_refund.d_us != "forever"
                       ? formatDuration(
-                          intervalToDuration({
-                            start: 0,
-                            end: contract_terms?.auto_refund.d_us,
-                          }),
-                        )
+                        intervalToDuration({
+                          start: 0,
+                          end: contract_terms?.auto_refund.d_us,
+                        }),
+                      )
                       : "forever"
                     : `{{ contract_terms.auto_refund_str }}`}{" "}
                 </dd>
@@ -539,7 +540,7 @@ export function mount(): void {
     let contractTerms: MerchantBackend.ContractTerms | undefined;
     try {
       contractTerms = JSON.parse((window as any).contractTermsStr);
-    } catch {}
+    } catch { }
 
     render(
       <ShowOrderDetails
