@@ -29,7 +29,6 @@ import {
 } from "@gnu-taler/taler-util";
 import {
   BrowserHashNavigationProvider,
-  CommonPreferenceProvider,
   ConfigResultFail,
   MerchantApiProvider,
   TalerWalletIntegrationBrowserProvider,
@@ -44,10 +43,12 @@ import { Loading } from "./components/exception/loading.js";
 import { NotificationCard } from "./components/menu/index.js";
 import { SessionContextProvider } from "./context/session.js";
 import { SettingsProvider } from "./context/settings.js";
+import { revalidateInstanceAccessTokens } from "./hooks/access-tokens.js";
 import {
   revalidateBankAccountDetails,
   revalidateInstanceBankAccounts,
 } from "./hooks/bank.js";
+import { revalidateInstanceCategories } from "./hooks/category.js";
 import {
   revalidateBackendInstances,
   revalidateInstanceDetails,
@@ -73,7 +74,6 @@ import {
   revalidateTokenFamilies,
   revalidateTokenFamilyDetails,
 } from "./hooks/tokenfamily.js";
-import { revalidateInstanceTransfers } from "./hooks/transfer.js";
 import {
   revalidateInstanceWebhooks,
   revalidateWebhookDetails,
@@ -84,8 +84,12 @@ import {
   buildDefaultBackendBaseURL,
   fetchSettings,
 } from "./settings.js";
-import { revalidateInstanceCategories } from "./hooks/category.js";
-import { revalidateInstanceAccessTokens } from "./hooks/access-tokens.js";
+import {
+  revalidateInstanceConfirmedTransfers,
+  revalidateInstanceIncomingTransfers,
+} from "./hooks/transfer.js";
+
+const TALER_SCREEN_ID = 2;
 const WITH_LOCAL_STORAGE_CACHE = false;
 
 export function Application(): VNode {
@@ -98,65 +102,63 @@ export function Application(): VNode {
   const baseUrl = getInitialBackendBaseURL(settings.backendBaseURL);
   return (
     <SettingsProvider value={settings}>
-      <CommonPreferenceProvider showDebug={false}>
-        <TranslationProvider
-          source={strings}
-          completeness={{
-            uk: strings["uk"].completeness,
-            tr: strings["tr"].completeness,
-            // ru: strings["ru"].completeness,
-            sv: strings["sv"].completeness,
-            it: strings["it"].completeness,
-            fr: strings["fr"].completeness,
-            es: strings["es"].completeness,
-            de: strings["de"].completeness,
+      <TranslationProvider
+        source={strings}
+        completeness={{
+          uk: strings["uk"].completeness,
+          tr: strings["tr"].completeness,
+          // ru: strings["ru"].completeness,
+          sv: strings["sv"].completeness,
+          it: strings["it"].completeness,
+          fr: strings["fr"].completeness,
+          es: strings["es"].completeness,
+          de: strings["de"].completeness,
+        }}
+      >
+        <MerchantApiProvider
+          baseUrl={new URL("./", baseUrl)}
+          frameOnError={OnConfigError}
+          evictors={{
+            management: swrCacheEvictor,
           }}
         >
-          <MerchantApiProvider
-            baseUrl={new URL("./", baseUrl)}
-            frameOnError={OnConfigError}
-            evictors={{
-              management: swrCacheEvictor,
-            }}
-          >
-            <SessionContextProvider>
-              <SWRConfig
-                value={{
-                  provider: WITH_LOCAL_STORAGE_CACHE
-                    ? localStorageProvider
-                    : undefined,
-                  // normally, do not revalidate
-                  revalidateOnFocus: false,
-                  revalidateOnReconnect: false,
-                  revalidateIfStale: false,
-                  revalidateOnMount: undefined,
-                  focusThrottleInterval: undefined,
+          <SessionContextProvider>
+            <SWRConfig
+              value={{
+                provider: WITH_LOCAL_STORAGE_CACHE
+                  ? localStorageProvider
+                  : undefined,
+                // normally, do not revalidate
+                revalidateOnFocus: false,
+                revalidateOnReconnect: false,
+                revalidateIfStale: false,
+                revalidateOnMount: undefined,
+                focusThrottleInterval: undefined,
 
-                  // normally, do not refresh
-                  refreshInterval: undefined,
-                  dedupingInterval: 2000,
-                  refreshWhenHidden: false,
-                  refreshWhenOffline: false,
+                // normally, do not refresh
+                refreshInterval: undefined,
+                dedupingInterval: 2000,
+                refreshWhenHidden: false,
+                refreshWhenOffline: false,
 
-                  // ignore errors
-                  shouldRetryOnError: false,
-                  errorRetryCount: 0,
-                  errorRetryInterval: undefined,
+                // ignore errors
+                shouldRetryOnError: false,
+                errorRetryCount: 0,
+                errorRetryInterval: undefined,
 
-                  // do not go to loading again if already has data
-                  keepPreviousData: true,
-                }}
-              >
-                <TalerWalletIntegrationBrowserProvider>
-                  <BrowserHashNavigationProvider>
-                    <Routing />
-                  </BrowserHashNavigationProvider>
-                </TalerWalletIntegrationBrowserProvider>
-              </SWRConfig>
-            </SessionContextProvider>
-          </MerchantApiProvider>
-        </TranslationProvider>
-      </CommonPreferenceProvider>
+                // do not go to loading again if already has data
+                keepPreviousData: true,
+              }}
+            >
+              <TalerWalletIntegrationBrowserProvider>
+                <BrowserHashNavigationProvider>
+                  <Routing />
+                </BrowserHashNavigationProvider>
+              </TalerWalletIntegrationBrowserProvider>
+            </SWRConfig>
+          </SessionContextProvider>
+        </MerchantApiProvider>
+      </TranslationProvider>
     </SettingsProvider>
   );
 }
@@ -208,9 +210,7 @@ function localStorageProvider(): Map<unknown, unknown> {
 function OnConfigError({
   state,
 }: {
-  state:
-    | ConfigResultFail<TalerMerchantApi.MerchantVersionResponse>
-    | undefined;
+  state: ConfigResultFail<TalerMerchantApi.MerchantVersionResponse> | undefined;
 }): VNode {
   const { i18n } = useTranslationContext();
   if (!state) {
@@ -326,11 +326,17 @@ const swrCacheEvictor = new (class
         return;
       }
       case TalerMerchantInstanceCacheEviction.CREATE_TRANSFER: {
-        await Promise.all([revalidateInstanceTransfers()]);
+        await Promise.all([
+          revalidateInstanceConfirmedTransfers(),
+          revalidateInstanceIncomingTransfers(),
+        ]);
         return;
       }
       case TalerMerchantInstanceCacheEviction.DELETE_TRANSFER: {
-        await Promise.all([revalidateInstanceTransfers()]);
+        await Promise.all([
+          revalidateInstanceConfirmedTransfers(),
+          revalidateInstanceIncomingTransfers(),
+        ]);
         return;
       }
       case TalerMerchantInstanceCacheEviction.CREATE_DEVICE: {
