@@ -72,7 +72,6 @@ import TemplateUsePage from "./paths/instance/templates/use/index.js";
 import TokenFamilyCreatePage from "./paths/instance/tokenfamilies/create/index.js";
 import TokenFamilyListPage from "./paths/instance/tokenfamilies/list/index.js";
 import TokenFamilyUpdatePage from "./paths/instance/tokenfamilies/update/index.js";
-import TransferCreatePage from "./paths/instance/transfers/create/index.js";
 import TransferListPage from "./paths/instance/transfers/list/index.js";
 import InstanceUpdatePage, {
   AdminUpdate as InstanceAdminUpdatePage,
@@ -83,9 +82,12 @@ import WebhookListPage from "./paths/instance/webhooks/list/index.js";
 import WebhookUpdatePage from "./paths/instance/webhooks/update/index.js";
 import { LoginPage } from "./paths/login/index.js";
 import { NewAccount } from "./paths/newAccount/index.js";
-import { Settings } from "./paths/settings/index.js";
-import { Notification } from "./utils/types.js";
 import { ResetAccount } from "./paths/resetAccount/index.js";
+import { Settings } from "./paths/settings/index.js";
+import Statistics from "./paths/instance/statistics/list/index.js";
+import { Notification } from "./utils/types.js";
+
+const TALER_SCREEN_ID = 3;
 
 export enum InstancePaths {
   error = "/error",
@@ -114,7 +116,7 @@ export enum InstancePaths {
   kyc = "/kyc",
 
   transfers_list = "/transfers",
-  transfers_new = "/transfer/new",
+  // transfers_new = "/transfer/new",
 
   templates_list = "/templates",
   templates_update = "/templates/:tid/update",
@@ -138,6 +140,7 @@ export enum InstancePaths {
 
   newAccount = "/account/new",
   resetAccount = "/account/reset/:id",
+  statistics = "/statistics",
 }
 
 export enum AdminPaths {
@@ -518,19 +521,6 @@ export function Routing(_p: Props): VNode {
         <Route
           path={InstancePaths.transfers_list}
           component={TransferListPage}
-          onCreate={() => {
-            route(InstancePaths.transfers_new);
-          }}
-        />
-        <Route
-          path={InstancePaths.transfers_new}
-          component={TransferCreatePage}
-          onConfirm={() => {
-            route(InstancePaths.transfers_list);
-          }}
-          onBack={() => {
-            route(InstancePaths.transfers_list);
-          }}
         />
         {/* *
          * Token family pages
@@ -696,6 +686,13 @@ export function Routing(_p: Props): VNode {
           // }}
         />
         <Route path={InstancePaths.interface} component={Settings} />
+        {/**
+         * Statistics pages
+         */}
+        <Route
+          path={InstancePaths.statistics}
+          component={Statistics}
+        />
         {/**
          * Example pages
          */}
@@ -759,6 +756,7 @@ function BankAccountBanner(): VNode {
             <div class="buttons is-right">
               <button
                 class="button"
+                type="button"
                 onClick={() => updatePref("hideMissingAccountUntil", tomorrow)}
               >
                 <i18n.Translate>Hide for today</i18n.Translate>
@@ -810,6 +808,7 @@ function KycBanner(): VNode {
             <div class="buttons is-right">
               <button
                 class="button"
+                type="button"
                 onClick={() => updatePref("hideKycUntil", tomorrow)}
               >
                 <i18n.Translate>Hide for today</i18n.Translate>
