@@ -25,9 +25,20 @@ export function QR({ text }: { text: string }): VNode {
     qr.addData(text);
     qr.make();
     if (divRef.current) {
-      divRef.current.innerHTML = qr.createSvgTag({
+      const image = qr.createSvgTag({
         scalable: true,
       });
+      const imageURL = `data:image/svg+xml,${encodeURIComponent(image)}`;
+      divRef.current.innerHTML = `<img src=${JSON.stringify(
+        imageURL,
+      )} alt=${JSON.stringify(
+        `QR Code containing the data ${text
+          .replace(/&/g, "&amp;")
+          .replace(/</g, "&lt;")
+          .replace(/>/g, "&gt;")
+          .replace(/"/g, "&quot;")
+          .replace(/'/g, "&#039;")}`,
+      )} />`;
     }
   });
 
