@@ -24,6 +24,8 @@ import { useState } from "preact/hooks";
 import { InputProps, useField } from "./useField.js";
 import { DropdownList } from "./InputSearchOnList.js";
 
+const TALER_SCREEN_ID = 6;
+
 export interface Props<T> extends InputProps<T> {
   isValid?: (e: any) => boolean;
   getSuggestion?: (e: any) => Promise<{ id: string; description: string }[]>;
@@ -105,8 +107,7 @@ export function InputArray<T>({
             </p>
             {getSuggestion ? undefined : (
               <p class="control">
-                <button
-                  class="button is-info has-tooltip-left"
+                <button type="button"                  class="button is-info has-tooltip-left"
                   disabled={!currentValue}
                   onClick={(): void => {
                     const v = fromStr(currentValue);
