@@ -29,6 +29,8 @@ import {
   usePreference,
 } from "../../hooks/preference.js";
 
+const TALER_SCREEN_ID = 7;
+
 export interface Props<T> extends InputProps<T> {
   readonly?: boolean;
   expand?: boolean;
@@ -136,6 +138,7 @@ export function InputDate<T>({
             }
           >
             <button
+              type="button"
               class="button is-info mr-3"
               onClick={() => onChange(undefined as any)}
             >
@@ -146,6 +149,7 @@ export function InputDate<T>({
         {withTimestampSupport && (
           <span data-tooltip={i18n.str`Change value to never`}>
             <button
+              type="button"
               class="button is-info"
               onClick={() => onChange({ t_s: "never" } as any)}
             >
