@@ -27,6 +27,9 @@ import { SimpleModal } from "../modal/index.js";
 import { DurationPicker } from "../picker/DurationPicker.js";
 import { InputProps, useField } from "./useField.js";
 
+
+const TALER_SCREEN_ID = 8;
+
 export interface Props<T> extends InputProps<T> {
   expand?: boolean;
   readonly?: boolean;
@@ -162,6 +165,7 @@ export function InputDuration<T>({
         {withForever && (
           <span data-tooltip={i18n.str`Change value to never`}>
             <button
+              type="button"
               class="button is-info mr-3"
               onClick={() => onChange({ d_ms: "forever" } as any)}
             >
@@ -172,6 +176,7 @@ export function InputDuration<T>({
         {!readonly && !withoutClear && (
           <span data-tooltip={i18n.str`Change value to empty`}>
             <button
+              type="button"
               class="button is-info "
               onClick={() => onChange(undefined as any)}
             >
