@@ -61,8 +61,7 @@ export function InputGroup<T>({
           )}
         </p>
         {!fixed && (
-          <button
-            class="card-header-icon"
+          <button type="button"            class="card-header-icon"
             aria-label="more options"
             onClick={(): void => setActive(!active)}
           >
