@@ -23,6 +23,8 @@ import { Fragment, h, VNode } from "preact";
 import { useState } from "preact/hooks";
 import { InputProps, useField } from "./useField.js";
 
+const TALER_SCREEN_ID = 13;
+
 export type Props<T> = InputProps<T>;
 
 const TokenStatus = ({ prev, post }: any) => {
@@ -75,8 +77,7 @@ export function InputSecured<T>({
           {!active ? (
             <Fragment>
               <div class="field has-addons">
-                <button
-                  class="button"
+                <button type="button"                  class="button"
                   onClick={(): void => {
                     setActive(!active);
                   }}
@@ -113,8 +114,7 @@ export function InputSecured<T>({
                   {help}
                 </div>
                 <div class="control">
-                  <button
-                    class="button is-info"
+                  <button type="button"                    class="button is-info"
                     disabled={fromStr(newValue) === value}
                     onClick={(): void => {
                       onChange(fromStr(newValue));
@@ -146,8 +146,7 @@ export function InputSecured<T>({
             <div class="level" style={{ width: "100%" }}>
               <div class="level-right is-flex-grow-1">
                 <div class="level-item">
-                  <button
-                    class="button is-danger"
+                  <button type="button"                    class="button is-danger"
                     disabled={null === value || undefined === value}
                     onClick={(): void => {
                       onChange(null!);
@@ -164,8 +163,7 @@ export function InputSecured<T>({
                   </button>
                 </div>
                 <div class="level-item">
-                  <button
-                    class="button "
+                  <button type="button"                    class="button "
                     onClick={(): void => {
                       onChange(initial!);
                       setActive(!active);
