@@ -29,6 +29,8 @@ import { InputLocation } from "./InputLocation.js";
 import { InputNumber } from "./InputNumber.js";
 import { InputProps, useField } from "./useField.js";
 
+const TALER_SCREEN_ID = 14;
+
 export interface Props<T> extends InputProps<T> {
   alreadyExist?: boolean;
 }
@@ -94,8 +96,7 @@ export function InputStock<T>({
           <div class="field-body is-flex-grow-3">
             <div class="field has-addons">
               {!alreadyExist ? (
-                <button
-                  class="button"
+                <button type="button" class="button"
                   data-tooltip={i18n.str`Click here to configure the product's stock. If left as is, the backend will not control stock.`}
                   onClick={(): void => {
                     valueHandler({
@@ -110,8 +111,7 @@ export function InputStock<T>({
                   </span>
                 </button>
               ) : (
-                <button
-                  class="button"
+                <button type="button" class="button"
                   data-tooltip={i18n.str`This product has been configured without stock control`}
                   disabled
                 >
@@ -184,8 +184,7 @@ export function InputStock<T>({
                 name="current"
                 label={i18n.str`Current`}
                 side={
-                  <button
-                    class="button is-danger"
+                  <button type="button" class="button is-danger"
                     data-tooltip={i18n.str`Remove stock control for this product`}
                     onClick={(): void => {
                       valueHandler(undefined as any);
