@@ -47,7 +47,7 @@ export function InputToggle<T>({
   fromBoolean = defaultFromBoolean,
   toBoolean = defaultToBoolean,
 }: Props<keyof T>): VNode {
-  const { error, value, onChange } = useField<T>(name);
+  const { error, value, onChange, required } = useField<T>(name);
 
   const onCheckboxClick = (): void => {
     const c = toBoolean(value);
@@ -95,6 +95,11 @@ export function InputToggle<T>({
                 style={{ cursor: readonly ? "default" : undefined }}
               ></div>
             </label>
+            {required && (
+              <span class="icon has-text-danger is-right">
+                <i class="mdi mdi-alert" />
+              </span>
+            )}
             <p>{help}</p>
           </p>
           {error && (
