@@ -22,6 +22,7 @@
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useSessionContext } from "../../context/session.js";
+import { UIElement } from "../../hooks/preference.js";
 import { Entity } from "../../paths/admin/create/CreatePage.js";
 import { Input } from "../form/Input.js";
 import { InputDuration } from "../form/InputDuration.js";
@@ -30,14 +31,16 @@ import { InputImage } from "../form/InputImage.js";
 import { InputLocation } from "../form/InputLocation.js";
 import { InputToggle } from "../form/InputToggle.js";
 import { InputWithAddon } from "../form/InputWithAddon.js";
+import { FragmentPersonaFlag } from "../menu/SideBar.js";
+
+
+const TALER_SCREEN_ID = 16;
 
 export function DefaultInstanceFormFields({
   readonlyId,
   showId,
-  showLessFields,
 }: {
   readonlyId?: boolean;
-  showLessFields?: boolean;
   showId: boolean;
 }): VNode {
   const { i18n } = useTranslationContext();
@@ -49,7 +52,7 @@ export function DefaultInstanceFormFields({
           name="id"
           addonBefore={new URL("instances/", state.backendUrl.href).href}
           readonly={readonlyId}
-          label={i18n.str`Identifier`}
+          label={i18n.str`Username`}
           tooltip={i18n.str`Name of the instance in URLs. The 'admin' instance is special in that it is used to administer other instances.`}
         />
       )}
@@ -60,69 +63,64 @@ export function DefaultInstanceFormFields({
         tooltip={i18n.str`Legal name of the business represented by this instance.`}
       />
 
-      {showLessFields ? undefined : (
-        <Fragment>
-          <Input<Entity>
-            name="email"
-            label={i18n.str`Email`}
-            tooltip={i18n.str`Contact email`}
-          />
+      <Input<Entity>
+        name="email"
+        label={i18n.str`Email`}
+        tooltip={i18n.str`Contact email`}
+      />
 
-          <Input<Entity>
-            name="phone_number"
-            label={i18n.str`Phone number`}
-            tooltip={i18n.str`Contact phone`}
-          />
+      <Input<Entity>
+        name="phone_number"
+        label={i18n.str`Phone number`}
+        tooltip={i18n.str`Contact phone`}
+      />
+      <FragmentPersonaFlag point={UIElement.option_advanceInstanceSettings}>
+        <Input<Entity>
+          name="website"
+          label={i18n.str`Website URL`}
+          tooltip={i18n.str`URL.`}
+        />
 
-          <Input<Entity>
-            name="website"
-            label={i18n.str`Website URL`}
-            tooltip={i18n.str`URL.`}
-          />
+        <InputImage<Entity>
+          name="logo"
+          label={i18n.str`Logo`}
+          tooltip={i18n.str`Logo image.`}
+        />
 
-          <InputImage<Entity>
-            name="logo"
-            label={i18n.str`Logo`}
-            tooltip={i18n.str`Logo image.`}
-          />
+        <InputGroup
+          name="address"
+          label={i18n.str`Address`}
+          tooltip={i18n.str`Physical location of the merchant.`}
+        >
+          <InputLocation name="address" />
+        </InputGroup>
 
-          <InputGroup
-            name="address"
-            label={i18n.str`Address`}
-            tooltip={i18n.str`Physical location of the merchant.`}
-          >
-            <InputLocation name="address" />
-          </InputGroup>
+        <InputGroup
+          name="jurisdiction"
+          label={i18n.str`Jurisdiction`}
+          tooltip={i18n.str`Jurisdiction for legal disputes with the merchant.`}
+        >
+          <InputLocation name="jurisdiction" />
+        </InputGroup>
 
-          <InputGroup
-            name="jurisdiction"
-            label={i18n.str`Jurisdiction`}
-            tooltip={i18n.str`Jurisdiction for legal disputes with the merchant.`}
-          >
-            <InputLocation name="jurisdiction" />
-          </InputGroup>
+        <InputToggle<Entity>
+          name="use_stefan"
+          label={i18n.str`Pay transaction fee`}
+          tooltip={i18n.str`Cover the transaction cost or pass it on to the user.`}
+        />
 
-          <InputToggle<Entity>
-            name="use_stefan"
-            label={i18n.str`Pay transaction fee`}
-            tooltip={i18n.str`Cover the transaction cost or pass it on to the user.`}
-          />
+        <InputDuration<Entity>
+          name="default_pay_delay"
+          label={i18n.str`Default payment delay`}
+          tooltip={i18n.str`Time customers have to pay an order before the offer expires by default.`}
+        />
 
-          <InputDuration<Entity>
-            name="default_pay_delay"
-            label={i18n.str`Default payment delay`}
-            withForever
-            tooltip={i18n.str`Time customers have to pay an order before the offer expires by default.`}
-          />
-
-          <InputDuration<Entity>
-            name="default_wire_transfer_delay"
-            label={i18n.str`Default wire transfer delay`}
-            tooltip={i18n.str`Maximum time an exchange is allowed to delay wiring funds to the merchant, enabling it to aggregate smaller payments into larger wire transfers and reducing wire fees.`}
-            withForever
-          />
-        </Fragment>
-      )}
+        <InputDuration<Entity>
+          name="default_wire_transfer_delay"
+          label={i18n.str`Default wire transfer delay`}
+          tooltip={i18n.str`Maximum time an exchange is allowed to delay wiring funds to the merchant, enabling it to aggregate smaller payments into larger wire transfers and reducing wire fees.`}
+        />
+      </FragmentPersonaFlag>
     </Fragment>
   );
 }
