@@ -52,8 +52,7 @@ export function LangSelector(): VNode {
   return (
     <div class="dropdown is-active ">
       <div class="dropdown-trigger">
-        <button
-          class="button has-tooltip-left"
+        <button type="button"          class="button has-tooltip-left"
           data-tooltip="change language selection"
           aria-haspopup="true"
           aria-controls="dropdown-menu"
