@@ -53,8 +53,6 @@ function getInstanceTitle(path: string, id: string): string {
       return `${id}: Update category`;
     case InstancePaths.transfers_list:
       return `${id}: Wire transfers`;
-    case InstancePaths.transfers_new:
-      return `${id}: New transfer`;
     case InstancePaths.webhooks_list:
       return `${id}: Webhooks`;
     case InstancePaths.webhooks_new:
@@ -76,7 +74,7 @@ function getInstanceTitle(path: string, id: string): string {
     case InstancePaths.templates_use:
       return `${id}: Use template`;
     case InstancePaths.interface:
-      return `${id}: Interface`;
+      return `${id}: Personalization`;
     case InstancePaths.token_family_list:
       return `${id}: Token families`;
     case InstancePaths.token_family_new:
