@@ -20,27 +20,31 @@
  */
 
 import {
-  PaytoString,
-  PaytoUri,
-  stringifyPaytoUri,
-  TranslatedString,
+  Paytos,
+  PaytoType,
+  TranslatedString
 } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import {
+  ButtonBetterBulma,
+  SafeHandlerTemplate,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
 import { ComponentChildren, Fragment, h, VNode } from "preact";
 import { useEffect, useRef, useState } from "preact/hooks";
+import { doAutoFocus } from "../../../../web-util/src/components/utils.js";
 import { DEFAULT_REQUEST_TIMEOUT } from "../../utils/constants.js";
 import { Spinner } from "../exception/loading.js";
-import { FormErrors } from "../form/FormProvider.js";
+
+const TALER_SCREEN_ID = 18;
 
 interface Props {
   active?: boolean;
   description?: string;
   onCancel?: () => void;
-  onConfirm?: () => void;
+  confirm?: SafeHandlerTemplate<any, any>;
   label?: string;
   children?: ComponentChildren;
   danger?: boolean;
-  disabled?: boolean;
   /**
    * sometimes we want to prevent the user to close the dialog by error when clicking outside the box
    *
@@ -53,17 +57,16 @@ export function ConfirmModal({
   active,
   description,
   onCancel,
-  onConfirm,
+  confirm,
   children,
   danger,
-  disabled,
   label = "Confirm",
-  noCancelButton
+  noCancelButton,
 }: Props): VNode {
   const { i18n } = useTranslationContext();
   return (
     <div class={active ? "modal is-active" : "modal"}>
-      <div class="modal-background " onClick={onCancel ?? onConfirm} />
+      <div class="modal-background " onClick={onCancel} />
       <div class="modal-card" style={{ maxWidth: 700 }}>
         <header class="modal-card-head">
           {!description ? null : (
@@ -73,46 +76,46 @@ export function ConfirmModal({
           )}
           <button
             class="delete "
+            type="button"
             aria-label="close"
-            onClick={onCancel ?? onConfirm}
+            onClick={onCancel}
           />
         </header>
         <section class="modal-card-body">{children}</section>
         <footer class="modal-card-foot">
           <div class="buttons is-right" style={{ width: "100%" }}>
-            {onConfirm ? (
+            {confirm ? (
               <Fragment>
                 {onCancel && !noCancelButton ? (
-                  <button class="button " onClick={onCancel}>
+                  <button class="button " type="button" onClick={onCancel}>
                     <i18n.Translate>Cancel</i18n.Translate>
                   </button>
                 ) : undefined}
 
-                <button
+                <ButtonBetterBulma
+                  type="submit"
                   class={danger ? "button is-danger " : "button is-info "}
-                  disabled={disabled}
-                  onClick={onConfirm}
+                  onClick={confirm}
                 >
                   <i18n.Translate>{label}</i18n.Translate>
-                </button>
+                </ButtonBetterBulma>
               </Fragment>
-            ) : (
-              (noCancelButton ? undefined :
-                <button class="button " onClick={onCancel}>
-                  <i18n.Translate>Close</i18n.Translate>
-                </button>
-              )
+            ) : noCancelButton ? undefined : (
+              <button class="button " ref={doAutoFocus} onClick={onCancel}>
+                <i18n.Translate>Close</i18n.Translate>
+              </button>
             )}
           </div>
         </footer>
       </div>
-      {noCancelButton ? undefined :
+      {noCancelButton ? undefined : (
         <button
+          type="button"
           class="modal-close is-large "
           aria-label="close"
           onClick={onCancel}
         />
-      }
+      )}
     </div>
   );
 }
@@ -121,9 +124,8 @@ export function ContinueModal({
   active,
   description,
   onCancel,
-  onConfirm,
+  confirm,
   children,
-  disabled,
 }: Props): VNode {
   const { i18n } = useTranslationContext();
   return (
@@ -137,17 +139,19 @@ export function ContinueModal({
         <section class="modal-card-body">{children}</section>
         <footer class="modal-card-foot">
           <div class="buttons is-right" style={{ width: "100%" }}>
-            <button
+            <ButtonBetterBulma
+              ref={doAutoFocus}
+              type="submit"
               class="button is-success "
-              disabled={disabled}
-              onClick={onConfirm}
+              onClick={confirm}
             >
               <i18n.Translate>Continue</i18n.Translate>
-            </button>
+            </ButtonBetterBulma>
           </div>
         </footer>
       </div>
       <button
+        type="button"
         class="modal-close is-large "
         aria-label="close"
         onClick={onCancel}
@@ -173,6 +177,8 @@ export function SimpleModal({
         class="modal-close is-large "
         aria-label="close"
         onClick={onCancel}
+        type="submit"
+        ref={doAutoFocus}
       />
     </div>
   );
@@ -182,7 +188,7 @@ export function ClearConfirmModal({
   description,
   onCancel,
   onClear,
-  onConfirm,
+  confirm,
   children,
 }: Props & { onClear?: () => void }): VNode {
   const { i18n } = useTranslationContext();
@@ -198,6 +204,7 @@ export function ClearConfirmModal({
         <footer class="modal-card-foot">
           {onClear && (
             <button
+              type="button"
               class="button is-danger"
               onClick={onClear}
               disabled={onClear === undefined}
@@ -206,20 +213,22 @@ export function ClearConfirmModal({
             </button>
           )}
           <div class="buttons is-right" style={{ width: "100%" }}>
-            <button class="button " onClick={onCancel}>
+            <button class="button " type="button" onClick={onCancel}>
               <i18n.Translate>Cancel</i18n.Translate>
             </button>
-            <button
+            <ButtonBetterBulma
               class="button is-info"
-              onClick={onConfirm}
-              disabled={onConfirm === undefined}
+              type="submit"
+              onClick={confirm}
+              ref={doAutoFocus}
             >
               <i18n.Translate>Confirm</i18n.Translate>
-            </button>
+            </ButtonBetterBulma>
           </div>
         </footer>
       </div>
       <button
+        type="button"
         class="modal-close is-large "
         aria-label="close"
         onClick={onCancel}
@@ -230,9 +239,9 @@ export function ClearConfirmModal({
 
 interface CompareAccountsModalProps {
   onCancel: () => void;
-  onConfirm: (account: PaytoString) => void;
-  formPayto: PaytoUri | undefined;
-  testPayto: PaytoUri;
+  confirm: SafeHandlerTemplate<any, any>;
+  formPayto: Paytos.URI | undefined;
+  testPayto: Paytos.URI;
 }
 
 function getHostFromHostPath(s: string | undefined) {
@@ -259,7 +268,7 @@ function getAccountIdFromHostPath(s: string | undefined) {
 
 export function CompareAccountsModal({
   onCancel,
-  onConfirm,
+  confirm,
   formPayto,
   testPayto,
 }: CompareAccountsModalProps): VNode {
@@ -270,11 +279,12 @@ export function CompareAccountsModal({
       description={i18n.str`Comparing account details`}
       active
       onCancel={onCancel}
-      onConfirm={() => onConfirm(stringifyPaytoUri(testPayto))}
+      confirm={confirm}
     >
       <p>
         <i18n.Translate>
-          The connection to the account info URL was successful, but the reported account information does not match the details in the form.
+          The connection to the account info URL was successful, but the
+          reported account information does not match the details in the form.
         </i18n.Translate>
       </p>
       <div class="table-container">
@@ -305,8 +315,8 @@ export function CompareAccountsModal({
                 <td>
                   <i18n.Translate>IBAN</i18n.Translate>
                 </td>
-                <td>{formPayto?.targetPath ?? "--"}</td>
-                <td>{testPayto.targetPath}</td>
+                <td>{formPayto?.displayName ?? "--"}</td>
+                <td>{testPayto.iban}</td>
               </tr>
             )}
             {testPayto.targetType === "bitcoin" && (
@@ -314,8 +324,8 @@ export function CompareAccountsModal({
                 <td>
                   <i18n.Translate>Address</i18n.Translate>
                 </td>
-                <td>{formPayto?.targetPath ?? "--"}</td>
-                <td>{testPayto.targetPath}</td>
+                <td>{formPayto?.displayName ?? "--"}</td>
+                <td>{testPayto.address}</td>
               </tr>
             )}
             {testPayto.targetType === "x-taler-bank" && (
@@ -324,17 +334,17 @@ export function CompareAccountsModal({
                   <td>
                     <i18n.Translate>Host</i18n.Translate>
                   </td>
-                  <td>{getHostFromHostPath(formPayto?.targetPath) ?? "--"}</td>
-                  <td>{getHostFromHostPath(testPayto.targetPath)}</td>
+                  <td>{getHostFromHostPath(formPayto?.fullPath) ?? "--"}</td>
+                  <td>{getHostFromHostPath(testPayto.fullPath)}</td>
                 </tr>
                 <tr>
                   <td>
                     <i18n.Translate>Account ID</i18n.Translate>
                   </td>
                   <td>
-                    {getAccountIdFromHostPath(formPayto?.targetPath) ?? "--"}
+                    {getAccountIdFromHostPath(formPayto?.fullPath) ?? "--"}
                   </td>
-                  <td>{getAccountIdFromHostPath(testPayto.targetPath)}</td>
+                  <td>{getAccountIdFromHostPath(testPayto.fullPath)}</td>
                 </tr>
               </Fragment>
             )}
@@ -348,7 +358,9 @@ export function CompareAccountsModal({
             {!!testPayto.params["receiver-postal-code"] && (
               <tr>
                 <td>
-                  <i18n.Translate>The postal code of the account owner's address</i18n.Translate>
+                  <i18n.Translate>
+                    The postal code of the account owner's address
+                  </i18n.Translate>
                 </td>
                 <td>{formPayto?.params["receiver-postal-code"] ?? "--"}</td>
                 <td>{testPayto.params["receiver-postal-code"]}</td>
@@ -372,8 +384,8 @@ export function CompareAccountsModal({
 
 interface ValidateBankAccountModalProps {
   onCancel: () => void;
-  origin: PaytoUri;
-  targets: PaytoUri[];
+  origin: Paytos.URI;
+  targets: Paytos.URI[];
 }
 export function ValidBankAccount({
   onCancel,
@@ -384,43 +396,37 @@ export function ValidBankAccount({
   const payto = targets[0];
   const subject = payto.params["message"];
 
-  const accountPart = !payto.isKnown ? (
-    <Fragment>
-      <Row name={i18n.str`Account`} value={payto.targetPath} />
-    </Fragment>
-  ) : payto.targetType === "x-taler-bank" ? (
-    <Fragment>
-      <Row name={i18n.str`Bank host`} value={payto.host} />
-      <Row name={i18n.str`Bank account`} value={payto.account} />
-    </Fragment>
-  ) : payto.targetType === "iban" ? (
-    <Fragment>
-      {payto.bic !== undefined ? (
-        <Row name={i18n.str`BIC`} value={payto.bic} />
-      ) : undefined}
-      <Row name={i18n.str`IBAN`} value={payto.iban} />
-    </Fragment>
-  ) : undefined;
+  const accountPart =
+    payto.targetType === PaytoType.TalerBank ? (
+      <Fragment>
+        <Row name={i18n.str`Bank host`} value={payto.host} />
+        <Row name={i18n.str`Bank account`} value={payto.account} />
+      </Fragment>
+    ) : payto.targetType === PaytoType.IBAN ? (
+      <Fragment>
+        {payto.bic !== undefined ? (
+          <Row name={i18n.str`BIC`} value={payto.bic} />
+        ) : undefined}
+        <Row name={i18n.str`IBAN`} value={payto.iban} />
+      </Fragment>
+    ) : undefined;
 
   const receiverName =
     payto.params["receiver-name"] || payto.params["receiver"] || undefined;
-  const receiverPostalCode =
-    payto.params["receiver-postal-code"] || undefined;
-  const receiverTown =
-    payto.params["receiver-town"] || undefined;
+  const receiverPostalCode = payto.params["receiver-postal-code"] || undefined;
+  const receiverTown = payto.params["receiver-town"] || undefined;
 
-  const from = !origin.isKnown
-    ? origin.targetPath
-    : origin.targetType === "iban"
+  const from =
+    origin.targetType === PaytoType.IBAN
       ? origin.iban
-      : origin.targetType === "taler-reserve" ||
-        origin.targetType === "taler-reserve-http"
+      : origin.targetType === PaytoType.TalerReserve ||
+          origin.targetType === PaytoType.TalerReserveHttp
         ? origin.reservePub
-        : origin.targetType === "bitcoin"
+        : origin.targetType === PaytoType.Bitcoin
           ? `${origin.address.substring(0, 8)}...`
-          : origin.targetType === "ethereum"
+          : origin.targetType === PaytoType.Ethereum
             ? `${origin.address.substring(0, 8)}...`
-            : origin.account;
+            : origin.displayName;
 
   return (
     <ConfirmModal
@@ -428,11 +434,13 @@ export function ValidBankAccount({
       description={i18n.str`Validate bank account: ${from}`}
       active
       onCancel={onCancel}
-    // onConfirm={onConfirm}
+      // onConfirm={onConfirm}
     >
       <p style={{ paddingTop: 0 }}>
         <i18n.Translate>
-          In order to prove that you are the beneficial owner of the bank account, you are required to wire a small amount to a specified bank account with the subject given hereinafter.
+          In order to prove that you are the beneficial owner of the bank
+          account, you are required to wire a small amount to a specified bank
+          account with the subject below.
         </i18n.Translate>
       </p>
       <div class="table-container">
@@ -443,18 +451,9 @@ export function ValidBankAccount({
                 <i18n.Translate>Step 1:</i18n.Translate>
                 &nbsp;
                 <i18n.Translate>
-                  Copy this string and paste it into the 'Subject' or 'Purpose' field in your preferred banking app or online banking website.
-                </i18n.Translate>
-              </td>
-            </tr>
-            <Row name={i18n.str`Subject`} value={subject} literal />
-
-            <tr>
-              <td colSpan={3}>
-                <i18n.Translate>Step 2:</i18n.Translate>
-                &nbsp;
-                <i18n.Translate>
-                  Copy and paste this IBAN and the legal name of the beneficial owner into the respective fields in your preferred banking app or online banking website.
+                  Copy and paste this IBAN and the legal name of the beneficial
+                  owner into the respective fields in your preferred banking app
+                  or online banking website.
                 </i18n.Translate>
               </td>
             </tr>
@@ -462,8 +461,22 @@ export function ValidBankAccount({
             {receiverName ? (
               <Row name={i18n.str`Receiver name`} value={receiverName} />
             ) : undefined}
+            <tr>
+              <td colSpan={3}>
+                <i18n.Translate>Step 2:</i18n.Translate>
+                &nbsp;
+                <i18n.Translate>
+                  Copy this string and paste it into the 'Subject' or 'Purpose'
+                  field in your preferred banking app or online banking website.
+                </i18n.Translate>
+              </td>
+            </tr>
+            <Row name={i18n.str`Subject`} value={subject} literal />
             {receiverPostalCode ? (
-              <Row name={i18n.str`Receiver postal code`} value={receiverPostalCode} />
+              <Row
+                name={i18n.str`Receiver postal code`}
+                value={receiverPostalCode}
+              />
             ) : undefined}
             {receiverTown ? (
               <Row name={i18n.str`Receiver town`} value={receiverTown} />
@@ -474,7 +487,8 @@ export function ValidBankAccount({
                 <i18n.Translate>Step 3:</i18n.Translate>
                 &nbsp;
                 <i18n.Translate>
-                  Select the smallest possible amount for a wire transfer in your preferred banking app or online banking website.
+                  Select the smallest possible amount for a wire transfer in
+                  your preferred banking app or online banking website.
                 </i18n.Translate>
               </td>
             </tr>
@@ -487,24 +501,26 @@ export function ValidBankAccount({
                 />
               }
             /> */}
-
             <tr>
               <td colSpan={3}>
                 {/* <WarningBox style={{ margin: 0 }}> */}
-                <b>
+                <b >
                   <i18n.Translate>
-                    Make sure ALL data is correct, especially the subject, and that you are choosing the bank account from which you definitely want to make the wire transfer.
-                    You can use the copy buttons (<CopyIcon />) to avoid typos or make use of the "payto://" URI below to copy just one value.
+                    Make sure ALL data is correct, especially the subject, and
+                    that you are choosing the bank account from which you
+                    definitely want to make the wire transfer. You can use the
+                    copy buttons (<CopyIcon />) to avoid typos or make use of
+                    the "payto://" URI below to copy just one value.
                   </i18n.Translate>
                 </b>
                 {/* </WarningBox> */}
               </td>
             </tr>
-
             <tr>
-              <td colSpan={2} width="100%" style={{ wordBreak: "break-all" }}>
+              <td colSpan={2} width="100%">
                 <i18n.Translate>
-                  As an alternative, in case that your bank already supports the 'PayTo URI' standard, you can use this{" "}
+                  As an alternative, in case that your bank already supports the
+                  'PayTo URI' standard, you can use this{" "}
                   <a
                     target="_blank"
                     rel="noreferrer"
@@ -517,7 +533,7 @@ export function ValidBankAccount({
                 </i18n.Translate>
               </td>
               <td>
-                <CopyButton getContent={() => stringifyPaytoUri(payto)} />
+                <CopyButton getContent={() => Paytos.toFullString(payto)} />
               </td>
             </tr>
           </tbody>
@@ -625,276 +641,6 @@ export const CopiedIcon = (): VNode => (
   </svg>
 );
 
-interface DeleteModalProps {
-  element: { id: string; name: string };
-  onCancel: () => void;
-  onConfirm: (id: string) => void;
-}
-
-export function DeleteModal({
-  element,
-  onCancel,
-  onConfirm,
-}: DeleteModalProps): VNode {
-  const { i18n } = useTranslationContext();
-  return (
-    <ConfirmModal
-      label={`Delete instance`}
-      description={`Delete the instance "${element.name}"`}
-      danger
-      active
-      onCancel={onCancel}
-      onConfirm={() => onConfirm(element.id)}
-    >
-      <p>
-        <i18n.Translate>
-          If you delete the instance named <b>&quot;{element.name}&quot;</b>{" "}
-          (ID: <b>{element.id}</b>), the merchant will no longer be able to process orders and refunds
-        </i18n.Translate>
-      </p>
-      <p>
-        <i18n.Translate>
-          This action deletes the instance's private key, but preserves all transaction data.
-          You can still access the transaction data after having deleted the instance.
-        </i18n.Translate>
-      </p>
-      <p class="warning">
-        <i18n.Translate>
-          Deleting an instance{" "}
-          <b>
-            <i18n.Translate>This cannot be undone!</i18n.Translate>
-          </b>
-        </i18n.Translate>
-      </p>
-    </ConfirmModal>
-  );
-}
-
-export function PurgeModal({
-  element,
-  onCancel,
-  onConfirm,
-}: DeleteModalProps): VNode {
-  const { i18n } = useTranslationContext();
-  return (
-    <ConfirmModal
-      label={`Purge the instance`}
-      description={`Purge the instance "${element.name}"`}
-      danger
-      active
-      onCancel={onCancel}
-      onConfirm={() => onConfirm(element.id)}
-    >
-      <p>
-        <i18n.Translate>
-          If you purge the instance named <b>&quot;{element.name}&quot;</b> (ID:{" "}
-          <b>{element.id}</b>), you will also delete all of its transaction data!
-        </i18n.Translate>
-      </p>
-      <p>
-        <i18n.Translate>
-          The instance will disappear from your list and you will no longer be able to access its data.
-        </i18n.Translate>
-      </p>
-      <p class="warning">
-        <i18n.Translate>
-          Purging an instance{" "}
-          <b>
-            <i18n.Translate>This cannot be undone!</i18n.Translate>
-          </b>
-        </i18n.Translate>
-      </p>
-    </ConfirmModal>
-  );
-}
-
-// interface UpdateTokenModalProps {
-//   oldToken?: string;
-//   onCancel: () => void;
-//   onConfirm: (value: string) => void;
-//   onClear: () => void;
-// }
-
-// //FIXME: merge UpdateTokenModal with SetTokenNewInstanceModal
-// export function UpdateTokenModal({
-//   onCancel,
-//   onClear,
-//   onConfirm,
-//   oldToken,
-// }: UpdateTokenModalProps): VNode {
-//   type State = { old_token: string; new_token: string; repeat_token: string };
-//   const [form, setValue] = useState<Partial<State>>({
-//     old_token: "",
-//     new_token: "",
-//     repeat_token: "",
-//   });
-//   const { i18n } = useTranslationContext();
-
-//   const hasInputTheCorrectOldToken = oldToken && oldToken !== form.old_token;
-//   const errors = undefinedIfEmpty({
-//     old_token: hasInputTheCorrectOldToken
-//       ? i18n.str`This is not matching the current password`
-//       : undefined,
-//     new_token: !form.new_token
-//       ? i18n.str`Required`
-//       : form.new_token === form.old_token
-//         ? i18n.str`The new token cannot be the old token`
-//         : undefined,
-//     repeat_token:
-//       form.new_token !== form.repeat_token
-//         ? i18n.str`This is not matching`
-//         : undefined,
-//   });
-
-//   const hasErrors = errors !== undefined;
-
-//   const { state } = useSessionContext();
-
-//   const text = i18n.str`You are updating the password for the instance with ID ${state.instance}`;
-
-//   return (
-//     <ClearConfirmModal
-//       description={text}
-//       onCancel={onCancel}
-//       onConfirm={!hasErrors ? () => onConfirm(form.new_token!) : undefined}
-//       onClear={!hasInputTheCorrectOldToken && oldToken ? onClear : undefined}
-//     >
-//       <div class="columns">
-//         <div class="column" />
-//         <div class="column is-four-fifths">
-//           <FormProvider errors={errors} object={form} valueHandler={setValue}>
-//             {oldToken && (
-//               <Input<State>
-//                 name="old_token"
-//                 label={i18n.str`Old password`}
-//                 tooltip={i18n.str`The password you are currently using`}
-//                 inputType="password"
-//               />
-//             )}
-//             <Input<State>
-//               name="new_token"
-//               label={i18n.str`New password`}
-//               tooltip={i18n.str`The new password to be used`}
-//               inputType="password"
-//             />
-//             <Input<State>
-//               name="repeat_token"
-//               label={i18n.str`Repeat the password`}
-//               tooltip={i18n.str`Please repeat your new password`}
-//               inputType="password"
-//             />
-//           </FormProvider>
-//           <p>
-//             <i18n.Translate>
-//               Leaving the password void will enable public access to the instance
-//             </i18n.Translate>
-//           </p>
-//         </div>
-//         <div class="column" />
-//       </div>
-//     </ClearConfirmModal>
-//   );
-// }
-
-// export function SetTokenNewInstanceModal({
-//   onCancel,
-//   onClear,
-//   onConfirm,
-// }: UpdateTokenModalProps): VNode {
-//   type State = { old_token: string; new_token: string; repeat_token: string };
-//   const [form, setValue] = useState<Partial<State>>({
-//     new_token: "",
-//     repeat_token: "",
-//   });
-//   const { i18n } = useTranslationContext();
-
-//   const errors = undefinedIfEmpty({
-//     new_token: !form.new_token
-//       ? i18n.str`Required`
-//       : form.new_token === form.old_token
-//         ? i18n.str`The new password cannot be the old password`
-//         : undefined,
-//     repeat_token:
-//       form.new_token !== form.repeat_token
-//         ? i18n.str`This is not matching`
-//         : undefined,
-//   });
-
-//   const hasErrors = errors !== undefined;
-
-//   return (
-//     <div class="modal is-active">
-//       <div class="modal-background " onClick={onCancel} />
-//       <div class="modal-card">
-//         <header class="modal-card-head">
-//           <p class="modal-card-title">{i18n.str`You are setting the password for the new instance`}</p>
-//           <button class="delete " aria-label="close" onClick={onCancel} />
-//         </header>
-//         <section class="modal-card-body is-main-section">
-//           <div class="columns">
-//             <div class="column" />
-//             <div class="column is-four-fifths">
-//               <FormProvider
-//                 errors={errors}
-//                 object={form}
-//                 valueHandler={setValue}
-//               >
-//                 <Input<State>
-//                   name="new_token"
-//                   label={i18n.str`New password`}
-//                   tooltip={i18n.str`The new password to be used`}
-//                   inputType="password"
-//                 />
-//                 <Input<State>
-//                   name="repeat_token"
-//                   label={i18n.str`Repeat the password`}
-//                   tooltip={i18n.str`Please repeat your new password`}
-//                   inputType="password"
-//                 />
-//               </FormProvider>
-//               <p>
-//                 <i18n.Translate>
-//                   Making use of the external authorization method, no check will be performed by
-//                   the Taler Merchant Backend
-//                 </i18n.Translate>
-//               </p>
-//             </div>
-//             <div class="column" />
-//           </div>
-//         </section>
-//         <footer class="modal-card-foot">
-//           {onClear && (
-//             <button
-//               class="button is-danger"
-//               onClick={onClear}
-//               disabled={onClear === undefined}
-//             >
-//               <i18n.Translate>Enable external authorization</i18n.Translate>
-//             </button>
-//           )}
-//           <div class="buttons is-right" style={{ width: "100%" }}>
-//             <button class="button " onClick={onCancel}>
-//               <i18n.Translate>Cancel</i18n.Translate>
-//             </button>
-//             <button
-//               class="button is-info"
-//               onClick={() => onConfirm(form.new_token!)}
-//               disabled={hasErrors}
-//             >
-//               <i18n.Translate>Set password</i18n.Translate>
-//             </button>
-//           </div>
-//         </footer>
-//       </div>
-//       <button
-//         class="modal-close is-large "
-//         aria-label="close"
-//         onClick={onCancel}
-//       />
-//     </div>
-//   );
-// }
-
 export function LoadingModal({ onCancel }: { onCancel: () => void }): VNode {
   const { i18n } = useTranslationContext();
   return (
@@ -923,6 +669,7 @@ export function LoadingModal({ onCancel }: { onCancel: () => void }): VNode {
         </footer>
       </div>
       <button
+        type="button"
         class="modal-close is-large "
         aria-label="close"
         onClick={onCancel}
