@@ -29,19 +29,22 @@ import { Loading, useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, h } from "preact";
 import { useCallback, useEffect, useState } from "preact/hooks";
 import { useSessionContext } from "../../context/session.js";
+import { useInstanceCategories } from "../../hooks/category.js";
 import { undefinedIfEmpty } from "../../utils/table.js";
+import { ErrorLoadingMerchant } from "../ErrorLoadingMerchant.js";
 import { FormProvider } from "../form/FormProvider.js";
 import { Input } from "../form/Input.js";
+import { InputArray } from "../form/InputArray.js";
 import { InputCurrency } from "../form/InputCurrency.js";
 import { InputImage } from "../form/InputImage.js";
 import { InputNumber } from "../form/InputNumber.js";
 import { InputStock, Stock } from "../form/InputStock.js";
 import { InputTaxes } from "../form/InputTaxes.js";
 import { InputWithAddon } from "../form/InputWithAddon.js";
-import { InputArray } from "../form/InputArray.js";
-import { useInstanceCategories } from "../../hooks/category.js";
-import { ErrorLoadingMerchant } from "../ErrorLoadingMerchant.js";
-import { usePreference } from "../../hooks/preference.js";
+import { FragmentPersonaFlag } from "../menu/SideBar.js";
+import { UIElement } from "../../hooks/preference.js";
+
+const TALER_SCREEN_ID = 23;
 
 type Entity = TalerMerchantApi.ProductDetail & {
   product_id: string;
@@ -56,7 +59,7 @@ interface Props {
 export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
   const { i18n } = useTranslationContext();
   const { state, lib } = useSessionContext();
-  const [preference] = usePreference();
+  // const [preference] = usePreference();
 
   // FIXME: if the category list is big the will bring a lot of info
   // we could find a lazy way to add up on searches
@@ -142,7 +145,16 @@ export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
     const stock = value.stock;
 
     if (!stock) {
-      value.total_stock = -1;
+      // the the input was untouched, then set the same
+      // initial value
+      value.total_stock = initial?.total_stock;
+      value.total_lost = initial?.total_lost;
+      value.next_restock = initial?.next_restock;
+      value.address = initial?.address;
+
+      if (!value.total_stock) {
+        value.total_stock = -1;
+      }
     } else {
       value.total_stock = stock.current;
       value.total_lost = stock.lost;
@@ -190,7 +202,7 @@ export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
         {alreadyExist ? undefined : (
           <InputWithAddon<Entity>
             name="product_id"
-            addonBefore={new URL("product/", state.backendUrl.href).href}
+            // addonBefore={new URL("product/", state.backendUrl.href).href}
             label={i18n.str`ID`}
             addonBeforeLarge={state.backendUrl.href.length > 50}
             expand
@@ -213,12 +225,14 @@ export function ProductForm({ onSubscribe, initial, alreadyExist }: Props) {
           label={i18n.str`Description`}
           tooltip={i18n.str`Product description for customers.`}
         />
-        <InputNumber<Entity>
-          name="minimum_age"
-          label={i18n.str`Age restriction`}
-          tooltip={i18n.str`Is this product restricted for customer below certain age?`}
-          help={i18n.str`Minimum age of the customer`}
-        />
+        <FragmentPersonaFlag point={UIElement.option_ageRestriction}>
+          <InputNumber<Entity>
+            name="minimum_age"
+            label={i18n.str`Age restriction`}
+            tooltip={i18n.str`Is this product restricted for customer below certain age?`}
+            help={i18n.str`Minimum age of the customer`}
+          />
+        </FragmentPersonaFlag>
         <Input<Entity>
           name="unit"
           label={i18n.str`Unit name`}
