@@ -17,8 +17,8 @@
 import {
   AccessToken,
   Codec,
-  TalerMerchantApi,
-  TalerMerchantConfigResponse,
+  MerchantVersionResponse,
+  TalerMerchantManagementHttpClient,
   buildCodecForObject,
   codecForString,
   codecForURL,
@@ -43,7 +43,7 @@ export type SessionState = LoggedIn | LoggedOut;
 interface LoggedIn {
   status: "loggedIn";
 
-  // is this instance admin? usually "default" name
+  // is this instance admin? usually "admin" name
   isAdmin: boolean;
 
   // url where all the request will be made
@@ -90,6 +90,11 @@ function inferInstanceName(url: URL) {
   return !match || !match[1] ? DEFAULT_ADMIN_USERNAME : match[1];
 }
 
+function recalculateUrlForAnotherUser(original: URL, user:string) {
+  const match = INSTANCE_ID_LOOKUP.exec(original.href);
+  return !match || !match[1] ? DEFAULT_ADMIN_USERNAME : match[1];
+}
+
 export const defaultState = (url: URL): SavedSession => {
   return {
     backendUrl: url,
@@ -100,7 +105,7 @@ export const defaultState = (url: URL): SavedSession => {
 
 export interface SessionStateHandler {
   lib: MerchantLib;
-  config: TalerMerchantConfigResponse;
+  config: MerchantVersionResponse;
 
   state: SessionState;
   /**
@@ -115,20 +120,17 @@ export interface SessionStateHandler {
    * from any to loggedIn
    * @param info
    */
-  logIn(token: AccessToken | undefined): void;
+  logIn(username: string, token: AccessToken | undefined): void;
   /**
    * from loggedIn to impersonate
    * @param info
    */
   impersonate(baseUrl: URL): void;
+
+  getInstanceForUsername(username: string): TalerMerchantManagementHttpClient;
 }
 
-const SESSION_STATE_KEY = buildStorageKey(
-  "merchant-session",
-  codecForSessionState(),
-);
-
-export const DEFAULT_ADMIN_USERNAME = "default";
+export const DEFAULT_ADMIN_USERNAME = "admin";
 
 export const INSTANCE_ID_LOOKUP = /\/instances\/([^/]*)\/?$/;
 
@@ -163,7 +165,12 @@ export const SessionContextProvider = ({
   } = useMerchantApiContext();
   const [status, setStatus] = useState<"loggedIn" | "loggedOut">("loggedIn");
   const [currentConfig, setCurrentConfig] =
-    useState<TalerMerchantConfigResponse>();
+    useState<MerchantVersionResponse>();
+  const SESSION_STATE_KEY = buildStorageKey(
+    `merchant-session-${merchantUrl.pathname}`,
+    codecForSessionState(),
+  );
+
   const { value: state, update } = useLocalStorage(
     SESSION_STATE_KEY,
     defaultState(merchantUrl),
@@ -172,7 +179,7 @@ export const SessionContextProvider = ({
   const currentInstance = inferInstanceName(state.backendUrl);
 
   let lib: MerchantLib;
-  let config: TalerMerchantConfigResponse;
+  let config: MerchantVersionResponse;
   const doingImpersonation = state.backendUrl.href !== merchantUrl.href;
   if (doingImpersonation) {
     /**
@@ -201,7 +208,7 @@ export const SessionContextProvider = ({
     state: {
       backendUrl: state.backendUrl,
       token: state.token,
-      impersonated: doingImpersonation,
+      impersonated: false, // doingImpersonation, FIXME: removing impersonation feature for 1.2
       instance: currentInstance,
       isAdmin: currentInstance === DEFAULT_ADMIN_USERNAME,
       status: status,
@@ -238,15 +245,24 @@ export const SessionContextProvider = ({
       setStatus("loggedIn");
       cleanAllCache();
     },
-    logIn(token) {
+    logIn(username, token) {
       cleanAllCache();
       setStatus("loggedIn");
+      let backendUrl: URL;
+      if (currentInstance !== username) {
+        backendUrl = new URL(rootLib.subInstanceApi(username).instance.baseUrl)
+      } else {
+        backendUrl = state.backendUrl
+      }
       update({
-        backendUrl: state.backendUrl,
-        token: token,
+        backendUrl,
+        token,
         prevToken: state.prevToken,
       });
     },
+    getInstanceForUsername(username:string) {
+      return username !== currentInstance ? rootLib.subInstanceApi(username).instance : lib.instance
+    }
   };
 
   return h(Context.Provider, {
