@@ -164,7 +164,7 @@ export function useInstanceKYCDetailsLongPolling() {
         }
       }
       latestRequestTime = new Date().getTime();
-      return lib.instance.getCurrentInstanceKycStatus(state.token, {
+      return lib.instance.getCurrentInstanceKycStatus(token, {
         timeout: DEFAULT_WAIT,
         reason: latestReason,
       });
