@@ -19,7 +19,11 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { AbsoluteTime, AmountString, TalerMerchantApi } from "@gnu-taler/taler-util";
+import {
+  AbsoluteTime,
+  AmountString,
+  TalerMerchantApi,
+} from "@gnu-taler/taler-util";
 import * as tests from "@gnu-taler/web-util/testing";
 import { expect } from "chai";
 import { useInstanceOrders, useOrderDetails } from "./order.js";
@@ -41,7 +45,10 @@ describe("order api interaction with listing", () => {
     env.addRequestExpectation(API_LIST_ORDERS, {
       qparam: { delta: -20, paid: "yes" },
       response: {
-        orders: [{ order_id: "1" }, { order_id: "2" } as TalerMerchantApi.OrderHistoryEntry],
+        orders: [
+          { order_id: "1" },
+          { order_id: "2" } as TalerMerchantApi.OrderHistoryEntry,
+        ],
       },
     });
 
@@ -52,7 +59,7 @@ describe("order api interaction with listing", () => {
     const hookBehavior = await tests.hookBehaveLikeThis(
       () => {
         const query = useInstanceOrders({ paid: true }, newDate);
-        const { lib: api } = useMerchantApiContext()
+        const { lib: api } = useMerchantApiContext();
         return { query, api };
       },
       {},
@@ -74,21 +81,25 @@ describe("order api interaction with listing", () => {
           env.addRequestExpectation(API_CREATE_ORDER, {
             request: {
               order: { amount: "ARS:12" as AmountString, summary: "pay me" },
-              lock_uuids: []
+              lock_uuids: [],
             },
-            response: { order_id: "3" },
+            response: { order_id: "3", pay_deadline: { t_s: "never" } },
           });
 
           env.addRequestExpectation(API_LIST_ORDERS, {
             qparam: { delta: -20, paid: "yes" },
             response: {
-              orders: [{ order_id: "1" }, { order_id: "2" } as any, { order_id: "3" } as any],
+              orders: [
+                { order_id: "1" },
+                { order_id: "2" } as any,
+                { order_id: "3" } as any,
+              ],
             },
           });
 
           api.instance.createOrder(undefined, {
             order: { amount: "ARS:12" as AmountString, summary: "pay me" },
-          })
+          });
         },
         ({ query, api }) => {
           expect(env.assertJustExpectedRequestWereMade()).deep.eq({
@@ -115,11 +126,13 @@ describe("order api interaction with listing", () => {
     env.addRequestExpectation(API_LIST_ORDERS, {
       qparam: { delta: -20, paid: "yes" },
       response: {
-        orders: [{
-          order_id: "1",
-          amount: "EUR:12",
-          refundable: true,
-        } as TalerMerchantApi.OrderHistoryEntry]
+        orders: [
+          {
+            order_id: "1",
+            amount: "EUR:12",
+            refundable: true,
+          } as TalerMerchantApi.OrderHistoryEntry,
+        ],
       },
     });
 
@@ -130,7 +143,7 @@ describe("order api interaction with listing", () => {
     const hookBehavior = await tests.hookBehaveLikeThis(
       () => {
         const query = useInstanceOrders({ paid: true }, newDate);
-        const { lib: api } = useMerchantApiContext()
+        const { lib: api } = useMerchantApiContext();
         return { query, api };
       },
       {},
@@ -167,14 +180,14 @@ describe("order api interaction with listing", () => {
             response: {
               orders: [
                 { order_id: "1", amount: "EUR:12", refundable: false } as any,
-              ]
+              ],
             },
           });
 
           api.instance.addRefund(undefined, "1", {
             reason: "double pay",
             refund: "EUR:1" as AmountString,
-          })
+          });
         },
         ({ query, api }) => {
           expect(env.assertJustExpectedRequestWereMade()).deep.eq({
@@ -208,7 +221,10 @@ describe("order api interaction with listing", () => {
     env.addRequestExpectation(API_LIST_ORDERS, {
       qparam: { delta: -20, paid: "yes" },
       response: {
-        orders: [{ order_id: "1" }, { order_id: "2" } as TalerMerchantApi.OrderHistoryEntry],
+        orders: [
+          { order_id: "1" },
+          { order_id: "2" } as TalerMerchantApi.OrderHistoryEntry,
+        ],
       },
     });
 
@@ -219,7 +235,7 @@ describe("order api interaction with listing", () => {
     const hookBehavior = await tests.hookBehaveLikeThis(
       () => {
         const query = useInstanceOrders({ paid: true }, newDate);
-        const { lib: api } = useMerchantApiContext()
+        const { lib: api } = useMerchantApiContext();
         return { query, api };
       },
       {},
@@ -247,7 +263,7 @@ describe("order api interaction with listing", () => {
             },
           });
 
-          api.instance.deleteOrder(undefined, "1")
+          api.instance.deleteOrder(undefined, "1");
         },
         ({ query, api }) => {
           expect(env.assertJustExpectedRequestWereMade()).deep.eq({
@@ -283,7 +299,7 @@ describe("order api interaction with details", () => {
     const hookBehavior = await tests.hookBehaveLikeThis(
       () => {
         const query = useOrderDetails("1");
-        const { lib: api } = useMerchantApiContext()
+        const { lib: api } = useMerchantApiContext();
         return { query, api };
       },
       {},
@@ -319,7 +335,7 @@ describe("order api interaction with details", () => {
           api.instance.addRefund(undefined, "1", {
             reason: "double pay",
             refund: "EUR:1" as AmountString,
-          })
+          });
         },
         ({ query, api }) => {
           expect(env.assertJustExpectedRequestWereMade()).deep.eq({
@@ -355,7 +371,7 @@ describe("order api interaction with details", () => {
     const hookBehavior = await tests.hookBehaveLikeThis(
       () => {
         const query = useOrderDetails("1");
-        const { lib: api } = useMerchantApiContext()
+        const { lib: api } = useMerchantApiContext();
         return { query, api };
       },
       {},
@@ -388,7 +404,7 @@ describe("order api interaction with details", () => {
 
           api.instance.forgetOrder(undefined, "1", {
             fields: ["$.summary"],
-          })
+          });
         },
         ({ query, api }) => {
           expect(env.assertJustExpectedRequestWereMade()).deep.eq({
@@ -433,8 +449,11 @@ describe("order listing pagination", () => {
     const hookBehavior = await tests.hookBehaveLikeThis(
       () => {
         const date = new Date(12000);
-        const query = useInstanceOrders({ wired: true, date: AbsoluteTime.fromMilliseconds(date.getTime()) }, newDate);
-        const { lib: api } = useMerchantApiContext()
+        const query = useInstanceOrders(
+          { wired: true, date: AbsoluteTime.fromMilliseconds(date.getTime()) },
+          newDate,
+        );
+        const { lib: api } = useMerchantApiContext();
         return { query, api };
       },
       {},
@@ -454,7 +473,6 @@ describe("order listing pagination", () => {
           // });
           // expect(query.isReachingEnd).true;
           // expect(query.isReachingStart).true;
-
         },
       ],
       env.buildTestingContext(),
@@ -495,8 +513,11 @@ describe("order listing pagination", () => {
     const hookBehavior = await tests.hookBehaveLikeThis(
       () => {
         const date = new Date(12000);
-        const query = useInstanceOrders({ wired: true, date: AbsoluteTime.fromMilliseconds(date.getTime()) }, newDate);
-        const { lib: api } = useMerchantApiContext()
+        const query = useInstanceOrders(
+          { wired: true, date: AbsoluteTime.fromMilliseconds(date.getTime()) },
+          newDate,
+        );
+        const { lib: api } = useMerchantApiContext();
         return { query, api };
       },
       {},
