@@ -20,34 +20,52 @@
  */
 
 import {
+  AccessToken,
   Duration,
+  HttpStatusCode,
   MerchantAuthMethod,
+  opEmptySuccess,
+  opFixedSuccess,
   TalerMerchantApi,
-  createRFC8959AccessTokenPlain,
 } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
-import { Fragment, VNode, h } from "preact";
+import {
+  ButtonBetterBulma,
+  LocalNotificationBannerBulma,
+  useChallengeHandler,
+  useLocalNotificationBetter,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
+import { Fragment, h, VNode } from "preact";
 import { useState } from "preact/hooks";
-import { AsyncButton } from "../../../components/exception/AsyncButton.js";
 import {
   FormErrors,
   FormProvider,
 } from "../../../components/form/FormProvider.js";
 import { Input } from "../../../components/form/Input.js";
-import { InputToggle } from "../../../components/form/InputToggle.js";
 import { DefaultInstanceFormFields } from "../../../components/instance/DefaultInstanceFormFields.js";
+import { SolveMFAChallenges } from "../../../components/SolveMFA.js";
+import { useSessionContext } from "../../../context/session.js";
 import { usePreference } from "../../../hooks/preference.js";
-import { EMAIL_REGEX, INSTANCE_ID_REGEX, PHONE_JUST_NUMBERS_REGEX } from "../../../utils/constants.js";
+import {
+  EMAIL_REGEX,
+  INSTANCE_ID_REGEX,
+  PHONE_JUST_NUMBERS_REGEX,
+} from "../../../utils/constants.js";
 import { undefinedIfEmpty } from "../../../utils/table.js";
+import { FOREVER_REFRESHABLE_TOKEN } from "../../login/index.js";
+
+const TALER_SCREEN_ID = 25;
 
 export type Entity = TalerMerchantApi.InstanceConfigurationMessage & {
   auth_token?: string;
   // default_pay_delay: Duration;
   // default_wire_transfer_delay: Duration;
+  password: string;
+  repeat: string;
 };
 
 export interface Props {
-  onCreate: (d: TalerMerchantApi.InstanceConfigurationMessage) => Promise<void>;
+  onConfirm: () => void;
   onBack?: () => void;
   forceId?: string;
 }
@@ -64,13 +82,12 @@ function with_defaults(id?: string): Partial<Entity> {
   };
 }
 
-type TokenForm = { password: string; repeat: string };
+// type TokenForm = {  };
 
-export function CreatePage({ onCreate, onBack, forceId }: Props): VNode {
+export function CreatePage({ onConfirm, onBack, forceId }: Props): VNode {
   const [pref, updatePref] = usePreference();
   const { i18n } = useTranslationContext();
   const [value, valueHandler] = useState(with_defaults(forceId));
-  const [tokenForm, setTokenForm] = useState<Partial<TokenForm>>({});
 
   const errors = undefinedIfEmpty<FormErrors<Entity>>({
     id: !value.id
@@ -79,16 +96,6 @@ export function CreatePage({ onCreate, onBack, forceId }: Props): VNode {
         ? i18n.str`Invalid`
         : undefined,
     name: !value.name ? i18n.str`Required` : undefined,
-    // accounts:
-    //   !value.accounts || !value.accounts.length
-    //     ? i18n.str`Required`
-    //     : undefinedIfEmpty(
-    //       value.accounts.map((p) => {
-    //         return !PAYTO_REGEX.test(p.payto_uri)
-    //           ? i18n.str`Invalid`
-    //           : undefined;
-    //       }),
-    //     ),
     email: !value.email
       ? undefined
       : !EMAIL_REGEX.test(value.email)
@@ -124,67 +131,94 @@ export function CreatePage({ onCreate, onBack, forceId }: Props): VNode {
           ? i18n.str`Max 7 lines`
           : undefined,
     }),
-  });
-
-  const hasErrors = errors !== undefined;
-
-  const tokenFormErrors = undefinedIfEmpty<FormErrors<TokenForm>>({
-    password: !tokenForm.password ? i18n.str`Required` : undefined,
-    repeat: !tokenForm.repeat
+    password: !value.password ? i18n.str`Required` : undefined,
+    repeat: !value.repeat
       ? i18n.str`Required`
-      : tokenForm.repeat !== tokenForm.password
+      : value.repeat !== value.password
         ? i18n.str`Doesn't match`
         : undefined,
   });
 
-  const hasTokenErrors = tokenFormErrors === undefined;
+  const hasErrors = errors !== undefined;
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
+  const { state: session, lib, logIn } = useSessionContext();
+  const mfa = useChallengeHandler();
 
-  const submit = (): Promise<void> => {
-    // use conversion instead of this
-    const newValue = structuredClone(value);
-
-    newValue.auth_token = undefined;
-    newValue.auth = {
+  const data: TalerMerchantApi.InstanceConfigurationMessage = {
+    ...(value as TalerMerchantApi.InstanceConfigurationMessage),
+    auth: {
       method: MerchantAuthMethod.TOKEN,
-      password: tokenForm.password!,
-    };
-    if (!newValue.address) newValue.address = {};
-    if (!newValue.jurisdiction) newValue.jurisdiction = {};
-
-    return onCreate(newValue as TalerMerchantApi.InstanceConfigurationMessage);
+      password: value.password!,
+    },
   };
+  const create = safeFunctionHandler(
+    async (
+      token: AccessToken,
+      data: TalerMerchantApi.InstanceConfigurationMessage,
+      challengeIds: string[],
+    ) => {
+      if (!data.address) data.address = {};
+      if (!data.jurisdiction) data.jurisdiction = {};
+      const instanceResp = await lib.instance.createInstance(token, data, {
+        challengeIds,
+      });
+      if (instanceResp.type === "fail") return instanceResp;
+      if (data.auth.password) {
+        const api =
+          data.id === "admin"
+            ? lib.instance
+            : lib.subInstanceApi(data.id).instance;
+        const tokenResp = await api.createAccessToken(
+          data.id,
+          data.auth.password,
+          FOREVER_REFRESHABLE_TOKEN(i18n.str`Instance created`),
+        );
+        if (tokenResp.type === "fail") return tokenResp;
+        return opFixedSuccess(tokenResp.body);
+      }
+      return opEmptySuccess();
+    },
+    !session.token || hasErrors ? undefined : [session.token, data, []],
+  );
+  create.onSuccess = (success, oldtoken, data) => {
+    if (success) {
+      logIn(data.id, success.access_token);
+    }
+    onConfirm();
+  };
+  create.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Accepted:
+        mfa.onChallengeRequired(fail.body);
+        return i18n.str`Second factor authentication required.`;
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unaouthorized.`;
+      case HttpStatusCode.Conflict:
+        return i18n.str`Conflict.`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Not found.`;
+    }
+  };
+  const retry = create.lambda((ids: string[]) => [
+    create.args![0],
+    create.args![1],
+    ids,
+  ]);
+
+  if (mfa.pendingChallenge) {
+    return (
+      <SolveMFAChallenges
+        currentChallenge={mfa.pendingChallenge}
+        onCompleted={retry}
+        onCancel={mfa.doCancelChallenge}
+      />
+    );
+  }
 
   return (
     <div>
+      <LocalNotificationBannerBulma notification={notification} />
       <section class="section is-main-section">
-        <div class="tabs is-toggle is-fullwidth is-small">
-          <ul>
-            <li
-              class={!pref.advanceInstanceMode ? "is-active" : ""}
-              onClick={() => {
-                updatePref("advanceInstanceMode", false);
-              }}
-            >
-              <a>
-                <span>
-                  <i18n.Translate>Simple</i18n.Translate>
-                </span>
-              </a>
-            </li>
-            <li
-              class={pref.advanceInstanceMode ? "is-active" : ""}
-              onClick={() => {
-                updatePref("advanceInstanceMode", true);
-              }}
-            >
-              <a>
-                <span>
-                  <i18n.Translate>Advanced</i18n.Translate>
-                </span>
-              </a>
-            </li>
-          </ul>
-        </div>{" "}
         <div class="columns">
           <div class="column" />
           <div class="column is-four-fifths">
@@ -196,45 +230,38 @@ export function CreatePage({ onCreate, onBack, forceId }: Props): VNode {
               <DefaultInstanceFormFields
                 readonlyId={!!forceId}
                 showId={!forceId}
-                showLessFields={!pref.advanceInstanceMode}
               />
-            </FormProvider>
-            <FormProvider
-              errors={tokenFormErrors}
-              object={tokenForm}
-              valueHandler={setTokenForm}
-            >
-              <Input<TokenForm>
+              <Input<Entity>
                 name="password"
                 label={i18n.str`New password`}
                 tooltip={i18n.str`Next password to be used`}
                 inputType="password"
               />
-              <Input<TokenForm>
+              <Input<Entity>
                 name="repeat"
                 label={i18n.str`Repeat password`}
                 tooltip={i18n.str`Confirm the same password`}
                 inputType="password"
               />
+              <div class="buttons is-right mt-5">
+                {onBack && (
+                  <button class="button" type="button" onClick={onBack}>
+                    <i18n.Translate>Cancel</i18n.Translate>
+                  </button>
+                )}
+                <ButtonBetterBulma
+                  onClick={create}
+                  type="submit"
+                  data-tooltip={
+                    hasErrors
+                      ? i18n.str`Please complete the marked fields and choose authorization method`
+                      : i18n.str`Confirm operation`
+                  }
+                >
+                  <i18n.Translate>Confirm</i18n.Translate>
+                </ButtonBetterBulma>
+              </div>
             </FormProvider>
-            <div class="buttons is-right mt-5">
-              {onBack && (
-                <button class="button" onClick={onBack}>
-                  <i18n.Translate>Cancel</i18n.Translate>
-                </button>
-              )}
-              <AsyncButton
-                onClick={submit}
-                disabled={hasErrors || !hasTokenErrors}
-                data-tooltip={
-                  hasErrors
-                    ? i18n.str`Please complete the marked fields and choose authorization method`
-                    : i18n.str`Confirm operation`
-                }
-              >
-                <i18n.Translate>Confirm</i18n.Translate>
-              </AsyncButton>
-            </div>
           </div>
           <div class="column" />
         </div>
