@@ -25,6 +25,8 @@ import { VNode, h } from "preact";
 import { StateUpdater, useEffect, useState } from "preact/hooks";
 import { useSessionContext } from "../../../context/session.js";
 
+const TALER_SCREEN_ID = 27;
+
 interface Props {
   instances: TalerMerchantApi.Instance[];
   onUpdate: (id: string) => void;
@@ -99,7 +101,12 @@ export function CardTable({
             class="has-tooltip-left"
             data-tooltip={i18n.str`Add new instance`}
           >
-            <button class="button is-info" type="button" onClick={onCreate}>
+            <button
+              class="button is-info"
+              type="button"
+              accessKey="+"
+              onClick={onCreate}
+            >
               <span class="icon is-small">
                 <i class="mdi mdi-plus mdi-36px" />
               </span>
