@@ -22,22 +22,21 @@
 import {
   HttpStatusCode,
   TalerError,
-  TalerMerchantApi,
-  assertUnreachable,
+  assertUnreachable
 } from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
-import { useState } from "preact/hooks";
 import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
 import { Loading } from "../../../../components/exception/loading.js";
 import { NotificationCard } from "../../../../components/menu/index.js";
 import { useSessionContext } from "../../../../context/session.js";
 import { useInstanceBankAccounts } from "../../../../hooks/bank.js";
-import { Notification } from "../../../../utils/types.js";
 import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { CardTable } from "./Table.js";
 
+const TALER_SCREEN_ID = 34;
+
 interface Props {
   onCreate: () => void;
   onSelect: (id: string) => void;
@@ -45,7 +44,7 @@ interface Props {
 
 export default function ListOtpDevices({ onCreate, onSelect }: Props): VNode {
   const { i18n } = useTranslationContext();
-  const [notif, setNotif] = useState<Notification | undefined>(undefined);
+
   const { state, lib } = useSessionContext();
   const result = useInstanceBankAccounts();
 
@@ -69,7 +68,6 @@ export default function ListOtpDevices({ onCreate, onSelect }: Props): VNode {
 
   return (
     <Fragment>
-      <NotificationCard notification={notif} />
       {result.body.accounts.length < 1 && (
         <NotificationCard
           notification={{
@@ -89,32 +87,6 @@ export default function ListOtpDevices({ onCreate, onSelect }: Props): VNode {
           onSelect={(e) => {
             onSelect(e.h_wire);
           }}
-          onDelete={async (e: TalerMerchantApi.BankAccountEntry) => {
-            return lib.instance
-              .deleteBankAccount(state.token, e.h_wire)
-              .then((resp) => {
-                if (resp.type === "ok") {
-                  setNotif({
-                    message: i18n.str`The bank account details have been successfully deleted.`,
-                    type: "SUCCESS",
-                  });
-                } else {
-                  setNotif({
-                    message: i18n.str`The bank account details could not be deleted.`,
-                    type: "ERROR",
-                    description: resp.detail?.hint,
-                  });
-                }
-              })
-              .catch((error) =>
-                setNotif({
-                  message: i18n.str`The bank account details could not be deleted.`,
-                  type: "ERROR",
-                  description:
-                    error instanceof Error ? error.message : String(error),
-                }),
-              );
-          }}
         />
       </section>
     </Fragment>
