@@ -19,26 +19,33 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { TalerMerchantApi } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import { HttpStatusCode, TalerMerchantApi } from "@gnu-taler/taler-util";
+import {
+  ButtonBetterBulma,
+  LocalNotificationBannerBulma,
+  useLocalNotificationBetter,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
 import { VNode, h } from "preact";
 import { useState } from "preact/hooks";
-import { AsyncButton } from "../../../../components/exception/AsyncButton.js";
 import {
   FormErrors,
   FormProvider,
 } from "../../../../components/form/FormProvider.js";
 import { Input } from "../../../../components/form/Input.js";
+import { useSessionContext } from "../../../../context/session.js";
 import { undefinedIfEmpty } from "../../../../utils/table.js";
 
+const TALER_SCREEN_ID = 37;
+
 type Entity = TalerMerchantApi.CategoryCreateRequest;
 
 interface Props {
-  onCreate: (d: Entity) => Promise<void>;
+  onCreated: () => void;
   onBack?: () => void;
 }
 
-export function CreatePage({ onCreate, onBack }: Props): VNode {
+export function CreatePage({ onCreated, onBack }: Props): VNode {
   const { i18n } = useTranslationContext();
 
   const [state, setState] = useState<Partial<Entity>>({ name_i18n: {} });
@@ -52,14 +59,29 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
   });
 
   const hasErrors = errors !== undefined;
+  const { state: session, lib } = useSessionContext();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
 
-  const submitForm = () => {
-    if (hasErrors) return Promise.reject();
-    return onCreate(state as Entity);
+  const data = !!errors
+    ? undefined
+    : (state as TalerMerchantApi.CategoryCreateRequest);
+  const create = safeFunctionHandler(
+    lib.instance.addCategory.bind(lib.instance),
+    !session.token || !data ? undefined : [session.token, data],
+  );
+  create.onSuccess = onCreated;
+  create.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unauthorized`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Not found`;
+    }
   };
 
   return (
     <div>
+      <LocalNotificationBannerBulma notification={notification} />
       <section class="section is-main-section">
         <div class="columns">
           <div class="column" />
@@ -74,26 +96,25 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
                 label={i18n.str`Name`}
                 tooltip={i18n.str`Category name`}
               />{" "}
+              <div class="buttons is-right mt-5">
+                {onBack && (
+                  <button class="button" type="button" onClick={onBack}>
+                    <i18n.Translate>Cancel</i18n.Translate>
+                  </button>
+                )}
+                <ButtonBetterBulma
+                  data-tooltip={
+                    hasErrors
+                      ? i18n.str`Please complete the marked fields`
+                      : i18n.str`Confirm operation`
+                  }
+                  type="submit"
+                  onClick={create}
+                >
+                  <i18n.Translate>Confirm</i18n.Translate>
+                </ButtonBetterBulma>
+              </div>
             </FormProvider>
-
-            <div class="buttons is-right mt-5">
-              {onBack && (
-                <button class="button" onClick={onBack}>
-                  <i18n.Translate>Cancel</i18n.Translate>
-                </button>
-              )}
-              <AsyncButton
-                disabled={hasErrors}
-                data-tooltip={
-                  hasErrors
-                    ? i18n.str`Please complete the marked fields`
-                    : i18n.str`Confirm operation`
-                }
-                onClick={submitForm}
-              >
-                <i18n.Translate>Confirm</i18n.Translate>
-              </AsyncButton>
-            </div>
           </div>
           <div class="column" />
         </div>
