@@ -25,7 +25,7 @@ import {
   TalerMerchantApi,
   assertUnreachable,
 } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import { LocalNotificationBannerBulma, useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
@@ -46,7 +46,7 @@ interface Props {
 export default function ListCategories({ onCreate, onSelect }: Props): VNode {
   // const [position, setPosition] = useState<string | undefined>(undefined);
   const { i18n } = useTranslationContext();
-  const [notif, setNotif] = useState<Notification | undefined>(undefined);
+
   const { state, lib } = useSessionContext();
   const result = useInstanceCategories();
 
@@ -70,7 +70,6 @@ export default function ListCategories({ onCreate, onSelect }: Props): VNode {
 
   return (
     <Fragment>
-      <NotificationCard notification={notif} />
 
       <section class="section is-main-section">
         <CardTable
@@ -81,31 +80,6 @@ export default function ListCategories({ onCreate, onSelect }: Props): VNode {
           onSelect={(e) => {
             onSelect(String(e.category_id));
           }}
-          onDelete={async (e: TalerMerchantApi.CategoryListEntry) => {
-            return lib.instance
-              .deleteCategory(state.token, String(e.category_id))
-              .then((resp) => {
-                if (resp.type === "ok") {
-                  setNotif({
-                    message: i18n.str`Category delete successfully`,
-                    type: "SUCCESS",
-                  });
-                } else {
-                  setNotif({
-                    message: i18n.str`Could not delete the category`,
-                    type: "ERROR",
-                    description: resp.detail?.hint,
-                  });
-                }
-              })
-              .catch((error) =>
-                setNotif({
-                  message: i18n.str`Could not delete the category`,
-                  type: "ERROR",
-                  description: error instanceof Error ? error.message : String(error),
-                }),
-              );
-          }}
         />
       </section>
     </Fragment>
