@@ -21,6 +21,7 @@
 
 import {
   HttpStatusCode,
+  Paytos,
   TalerError,
   TalerMerchantApi,
   assertUnreachable,
@@ -38,6 +39,8 @@ import {
 import { useInstanceKYCDetailsLongPolling } from "../../../../hooks/instance.js";
 import { ListPage } from "./ListPage.js";
 
+const TALER_SCREEN_ID = 40;
+
 interface Props {
   // onGetInfo: (id: string) => void;
   // onShowInstructions: (id: string) => void;
@@ -124,12 +127,36 @@ function ShowInstructionForKycRedirect({
   const { i18n } = useTranslationContext();
   switch (e.status) {
     case TalerMerchantApi.MerchantAccountKycStatus.KYC_WIRE_REQUIRED:
+      const uri = Paytos.fromString(e.payto_uri);
+      if (uri.type === "fail") {
+        return (
+          <ConfirmModal
+            label={i18n.str`Ok`}
+            description={i18n.str`Wire account is invalid`}
+            active
+            onCancel={onCancel}
+          >
+            <p style={{ paddingTop: 0 }}>
+              <i18n.Translate>
+                The backend service responded with "{e.payto_uri}" which is
+                invalid.
+              </i18n.Translate>
+            </p>
+          </ConfirmModal>
+        );
+      }
+      const tgs = (e.payto_kycauths ?? [])
+        .map((d) => Paytos.fromString(d))
+        .filter((d) => {
+          if (d.type !== "ok") {
+            // FIXME: maybe this need to be shown to the user more promptly
+            console.error("server replied with a wrong payto://", d);
+          }
+          return d.type === "ok";
+        })
+        .map((d) => d.body);
       return (
-        <ValidBankAccount
-          origin={parsePaytoUri(e.payto_uri)!}
-          targets={(e.payto_kycauths ?? []).map((d) => parsePaytoUri(d)!)}
-          onCancel={onCancel}
-        />
+        <ValidBankAccount origin={uri.body} targets={tgs} onCancel={onCancel} />
       );
     case TalerMerchantApi.MerchantAccountKycStatus.NO_EXCHANGE_KEY: {
       return (
