@@ -24,10 +24,16 @@ import {
   AmountString,
   Amounts,
   Duration,
+  HttpStatusCode,
   TalerMerchantApi,
   TalerProtocolDuration,
 } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import {
+  ButtonBetterBulma,
+  LocalNotificationBannerBulma,
+  useLocalNotificationBetter,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
 import { format, isFuture } from "date-fns";
 import { Fragment, VNode, h } from "preact";
 import { useEffect, useState } from "preact/hooks";
@@ -49,12 +55,16 @@ import { NonInventoryProductFrom } from "../../../../components/product/NonInven
 import { ProductList } from "../../../../components/product/ProductList.js";
 import { useSessionContext } from "../../../../context/session.js";
 import { WithId } from "../../../../declaration.js";
-import { usePreference } from "../../../../hooks/preference.js";
+import { UIElement, usePreference } from "../../../../hooks/preference.js";
 import { rate } from "../../../../utils/amount.js";
 import { undefinedIfEmpty } from "../../../../utils/table.js";
+import { FragmentPersonaFlag } from "../../../../components/menu/SideBar.js";
+
+
+const TALER_SCREEN_ID = 42;
 
 export interface Props {
-  onCreate: (d: TalerMerchantApi.PostOrderRequest) => void;
+  onCreated: (id: string) => void;
   onBack?: () => void;
   instanceConfig: InstanceConfig;
   instanceInventory: (TalerMerchantApi.ProductDetail & WithId)[];
@@ -143,12 +153,12 @@ function isInvalidUrl(url: string) {
 }
 
 export function CreatePage({
-  onCreate,
+  onCreated,
   onBack,
   instanceConfig,
   instanceInventory,
 }: Props): VNode {
-  const { config } = useSessionContext();
+  const { config, lib, state: session } = useSessionContext();
   const instance_default = with_defaults(instanceConfig, config.currency);
   const [value, valueHandler] = useState(instance_default);
   const zero = Amounts.zeroOfCurrency(config.currency);
@@ -235,62 +245,80 @@ export function CreatePage({
   });
   const hasErrors = errors !== undefined;
 
-  const submit = (): void => {
-    const order = value;
-    const price = order.pricing?.order_price as AmountString | undefined;
-    const summary = order.pricing?.summary;
-    if (!value.payments) return;
-    if (!value.shipping) return;
-    if (!price || !summary) return;
+  const order = value;
+  const price = order.pricing?.order_price as AmountString | undefined;
+  const summary = order.pricing?.summary;
 
-    const request: TalerMerchantApi.PostOrderRequest = {
-      order: {
-        amount: price,
-        summary: summary,
-        products: productList,
-        extra: undefinedIfEmpty(value.extra),
-        pay_deadline: AbsoluteTime.toProtocolTimestamp(
-          AbsoluteTime.addDuration(
-            AbsoluteTime.now(),
-            value.payments.pay_deadline!,
-          ),
-        ),
-        wire_transfer_deadline: AbsoluteTime.toProtocolTimestamp(
-          AbsoluteTime.addDuration(
-            AbsoluteTime.now(),
-            value.payments.wire_transfer_deadline!,
-          ),
-        ),
-        refund_deadline: AbsoluteTime.toProtocolTimestamp(
-          AbsoluteTime.addDuration(
-            AbsoluteTime.now(),
-            value.payments.refund_deadline!,
-          ),
-        ),
-        auto_refund: value.payments.auto_refund_deadline
-          ? Duration.toTalerProtocolDuration(
-              value.payments.auto_refund_deadline,
-            )
-          : undefined,
-        max_fee: value.payments.max_fee as AmountString,
-        delivery_date: value.shipping.delivery_date
-          ? { t_s: value.shipping.delivery_date.getTime() / 1000 }
-          : undefined,
-        delivery_location: value.shipping.delivery_location,
-        fulfillment_url: value.shipping.fulfillment_url,
-        fulfillment_message: value.shipping.fulfillment_message,
-        minimum_age: value.payments.minimum_age,
-      },
-      inventory_products: inventoryList.map((p) => ({
-        product_id: p.product.id,
-        quantity: p.quantity,
-      })),
-      create_token: value.payments.createToken,
-    };
+  const request: undefined | TalerMerchantApi.PostOrderRequest =
+    !value.payments || !value.shipping || !price || !summary
+      ? undefined
+      : {
+          order: {
+            amount: price,
+            summary: summary,
+            products: productList,
+            extra: undefinedIfEmpty(value.extra),
+            pay_deadline: AbsoluteTime.toProtocolTimestamp(
+              AbsoluteTime.addDuration(
+                AbsoluteTime.now(),
+                value.payments.pay_deadline!,
+              ),
+            ),
+            wire_transfer_deadline: AbsoluteTime.toProtocolTimestamp(
+              AbsoluteTime.addDuration(
+                AbsoluteTime.now(),
+                value.payments.wire_transfer_deadline!,
+              ),
+            ),
+            refund_deadline: AbsoluteTime.toProtocolTimestamp(
+              AbsoluteTime.addDuration(
+                AbsoluteTime.now(),
+                value.payments.refund_deadline!,
+              ),
+            ),
+            auto_refund: value.payments.auto_refund_deadline
+              ? Duration.toTalerProtocolDuration(
+                  value.payments.auto_refund_deadline,
+                )
+              : undefined,
+            max_fee: value.payments.max_fee as AmountString,
+            delivery_date: value.shipping.delivery_date
+              ? { t_s: value.shipping.delivery_date.getTime() / 1000 }
+              : undefined,
+            delivery_location: value.shipping.delivery_location,
+            fulfillment_url: value.shipping.fulfillment_url,
+            fulfillment_message: value.shipping.fulfillment_message,
+            minimum_age: value.payments.minimum_age,
+          },
+          inventory_products: inventoryList.map((p) => ({
+            product_id: p.product.id,
+            quantity: p.quantity,
+          })),
+          create_token: value.payments.createToken,
+        };
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
 
-    onCreate(request);
+  const create = safeFunctionHandler(
+    lib.instance.createOrder.bind(lib.instance),
+    !session.token || !request ? undefined : [session.token, request],
+  );
+  create.onSuccess = (resp) => {
+    onCreated(resp.order_id);
+  };
+  create.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unauthorized`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Not found`;
+      case HttpStatusCode.Conflict:
+        return i18n.str`Conflict`;
+      case HttpStatusCode.Gone:
+        return i18n.str`Product with ID "${fail.body.product_id}" is out of stock.`;
+      case HttpStatusCode.UnavailableForLegalReasons:
+        return i18n.str`No exchange would accept a payment because of KYC requirements.`;
+    }
   };
-
   const addProductToTheInventoryList = (
     product: TalerMerchantApi.ProductDetail & WithId,
     quantity: number,
@@ -388,35 +416,8 @@ export function CreatePage({
 
   return (
     <div>
+      <LocalNotificationBannerBulma notification={notification} />
       <section class="section is-main-section">
-        <div class="tabs is-toggle is-fullwidth is-small">
-          <ul>
-            <li
-              class={!pref.advanceOrderMode ? "is-active" : ""}
-              onClick={() => {
-                updatePrefs("advanceOrderMode", false);
-              }}
-            >
-              <a>
-                <span>
-                  <i18n.Translate>Simple</i18n.Translate>
-                </span>
-              </a>
-            </li>
-            <li
-              class={pref.advanceOrderMode ? "is-active" : ""}
-              onClick={() => {
-                updatePrefs("advanceOrderMode", true);
-              }}
-            >
-              <a>
-                <span>
-                  <i18n.Translate>Advanced</i18n.Translate>
-                </span>
-              </a>
-            </li>
-          </ul>
-        </div>
         <div class="columns">
           <div class="column" />
           <div class="column is-four-fifths">
@@ -442,7 +443,9 @@ export function CreatePage({
                 inventory={instanceInventory}
               />
 
-              {pref.advanceOrderMode && (
+              <FragmentPersonaFlag
+                point={UIElement.option_advanceOrderCreation}
+              >
                 <NonInventoryProductFrom
                   productToEdit={editingProduct}
                   onAddProduct={(p) => {
@@ -450,7 +453,7 @@ export function CreatePage({
                     return addNewProduct(p);
                   }}
                 />
-              )}
+              </FragmentPersonaFlag>
 
               {allProducts.length > 0 && (
                 <ProductList
@@ -513,7 +516,10 @@ export function CreatePage({
                 tooltip={i18n.str`Title of the order to be shown to the customer`}
               />
 
-              {(pref.advanceOrderMode || errors?.shipping) && (
+              <FragmentPersonaFlag
+                point={UIElement.option_advanceOrderCreation}
+                showAnywayIf={errors?.shipping !== undefined}
+              >
                 <InputGroup
                   name="shipping"
                   label={i18n.str`Shipping and fulfillment`}
@@ -542,19 +548,26 @@ export function CreatePage({
                     tooltip={i18n.str`Message shown to the customer after paying for the order.`}
                   />
                 </InputGroup>
-              )}
+              </FragmentPersonaFlag>
 
-              {(pref.advanceOrderMode ||
-                requiresSomeTalerOptions ||
-                errors?.payments) && (
+              <FragmentPersonaFlag
+                point={UIElement.option_advanceOrderCreation}
+                showAnywayIf={
+                  requiresSomeTalerOptions || errors?.payments !== undefined
+                }
+              >
                 <InputGroup
                   name="payments"
                   label={i18n.str`Taler payment options`}
                   tooltip={i18n.str`Override default Taler payment settings for this order`}
                 >
-                  {(pref.advanceOrderMode ||
-                    noDefault_payDeadline ||
-                    errors?.payments?.pay_deadline !== undefined) && (
+                  <FragmentPersonaFlag
+                    point={UIElement.option_advanceOrderCreation}
+                    showAnywayIf={
+                      noDefault_payDeadline ||
+                      errors?.payments?.pay_deadline !== undefined
+                    }
+                  >
                     <InputDuration
                       name="payments.pay_deadline"
                       label={i18n.str`Payment time`}
@@ -567,6 +580,7 @@ export function CreatePage({
                       side={
                         <span>
                           <button
+                            type="button"
                             class="button"
                             onClick={() => {
                               const c = {
@@ -585,9 +599,13 @@ export function CreatePage({
                         </span>
                       }
                     />
-                  )}
-                  {(pref.advanceOrderMode ||
-                    errors?.payments?.refund_deadline !== undefined) && (
+                  </FragmentPersonaFlag>
+                  <FragmentPersonaFlag
+                    point={UIElement.option_advanceOrderCreation}
+                    showAnywayIf={
+                      errors?.payments?.refund_deadline !== undefined
+                    }
+                  >
                     <InputDuration
                       name="payments.refund_deadline"
                       label={i18n.str`Refund time`}
@@ -602,6 +620,7 @@ export function CreatePage({
                       side={
                         <span>
                           <button
+                            type="button"
                             class="button"
                             onClick={() => {
                               valueHandler({
@@ -619,10 +638,15 @@ export function CreatePage({
                         </span>
                       }
                     />
-                  )}
-                  {(pref.advanceOrderMode ||
-                    noDefault_wireDeadline ||
-                    errors?.payments?.wire_transfer_deadline !== undefined) && (
+                  </FragmentPersonaFlag>
+
+                  <FragmentPersonaFlag
+                    point={UIElement.option_advanceOrderCreation}
+                    showAnywayIf={
+                      noDefault_wireDeadline ||
+                      errors?.payments?.wire_transfer_deadline !== undefined
+                    }
+                  >
                     <InputDuration
                       name="payments.wire_transfer_deadline"
                       label={i18n.str`Wire transfer time`}
@@ -637,6 +661,7 @@ export function CreatePage({
                       side={
                         <span>
                           <button
+                            type="button"
                             class="button"
                             onClick={() => {
                               valueHandler({
@@ -655,9 +680,13 @@ export function CreatePage({
                         </span>
                       }
                     />
-                  )}
-                  {(pref.advanceOrderMode ||
-                    errors?.payments?.auto_refund_deadline !== undefined) && (
+                  </FragmentPersonaFlag>
+                  <FragmentPersonaFlag
+                    point={UIElement.option_advanceOrderCreation}
+                    showAnywayIf={
+                      errors?.payments?.auto_refund_deadline !== undefined
+                    }
+                  >
                     <InputDuration
                       name="payments.auto_refund_deadline"
                       label={i18n.str`Auto-refund time`}
@@ -669,26 +698,31 @@ export function CreatePage({
                       tooltip={i18n.str`Time until which the wallet will automatically check for refunds without user interaction.`}
                       withForever
                     />
-                  )}
-
-                  {(pref.advanceOrderMode ||
-                    errors?.payments?.max_fee !== undefined) && (
+                  </FragmentPersonaFlag>
+                  <FragmentPersonaFlag
+                    point={UIElement.option_advanceOrderCreation}
+                    showAnywayIf={errors?.payments?.max_fee !== undefined}
+                  >
                     <InputCurrency
                       name="payments.max_fee"
                       label={i18n.str`Maximum fee`}
                       tooltip={i18n.str`Maximum fees the merchant is willing to cover for this order. Higher deposit fees must be covered in full by the consumer.`}
                     />
-                  )}
-                  {(pref.advanceOrderMode ||
-                    errors?.payments?.createToken !== undefined) && (
+                  </FragmentPersonaFlag>
+                  <FragmentPersonaFlag
+                    point={UIElement.option_advanceOrderCreation}
+                    showAnywayIf={errors?.payments?.createToken !== undefined}
+                  >
                     <InputToggle
                       name="payments.createToken"
                       label={i18n.str`Create token`}
                       tooltip={i18n.str`If the order ID is easy to guess, the token will prevent other users from claiming the order.`}
                     />
-                  )}
-                  {(pref.advanceOrderMode ||
-                    errors?.payments?.minimum_age !== undefined) && (
+                  </FragmentPersonaFlag>
+                  <FragmentPersonaFlag
+                    point={UIElement.option_ageRestriction}
+                    showAnywayIf={errors?.payments?.minimum_age !== undefined}
+                  >
                     <InputNumber
                       name="payments.minimum_age"
                       label={i18n.str`Minimum age required`}
@@ -699,11 +733,16 @@ export function CreatePage({
                           : i18n.str`No product with age restriction in this order`
                       }
                     />
-                  )}
+                  </FragmentPersonaFlag>
                 </InputGroup>
-              )}
+              </FragmentPersonaFlag>
 
-              {(pref.advanceOrderMode || errors?.extra !== undefined) && (
+              <FragmentPersonaFlag
+                point={UIElement.option_advanceOrderCreation}
+                showAnywayIf={
+                  requiresSomeTalerOptions || errors?.extra !== undefined
+                }
+              >
                 <InputGroup
                   name="extra"
                   label={i18n.str`Additional information`}
@@ -719,6 +758,7 @@ export function CreatePage({
                         tooltip={i18n.str`You must enter a value in JavaScript Object Notation (JSON).`}
                         side={
                           <button
+                            type="button"
                             class="button"
                             onClick={(e) => {
                               if (
@@ -733,7 +773,7 @@ export function CreatePage({
                               e.preventDefault();
                             }}
                           >
-                            remove
+                            <i18n.Translate>remove</i18n.Translate>
                           </button>
                         }
                       />
@@ -763,6 +803,7 @@ export function CreatePage({
                       </div>
                     </div>
                     <button
+                      type="button"
                       class="button"
                       onClick={(e) => {
                         setNewField("");
@@ -780,7 +821,7 @@ export function CreatePage({
                     </button>
                   </div>
                 </InputGroup>
-              )}
+              </FragmentPersonaFlag>
             </FormProvider>
 
             <div class="buttons is-right mt-5">
@@ -789,13 +830,13 @@ export function CreatePage({
                   <i18n.Translate>Cancel</i18n.Translate>
                 </button>
               )}
-              <button
+              <ButtonBetterBulma
                 class="button is-success"
-                onClick={submit}
-                disabled={hasErrors}
+                type="submit"
+                onClick={create}
               >
                 <i18n.Translate>Confirm</i18n.Translate>
-              </button>
+              </ButtonBetterBulma>
             </div>
           </div>
           <div class="column" />
