@@ -49,6 +49,9 @@ import { mergeRefunds } from "../../../../utils/amount.js";
 import { RefundModal } from "../list/Table.js";
 import { Event, Timeline } from "./Timeline.js";
 
+
+const TALER_SCREEN_ID = 44;
+
 type Entity = TalerMerchantApi.MerchantOrderStatusResponse;
 type CT = TalerMerchantApi.MerchantContractTermsV0;
 
@@ -56,7 +59,7 @@ export interface Props {
   onBack: () => void;
   selected: Entity;
   id: string;
-  onRefund: (id: string, value: TalerMerchantApi.RefundRequest) => void;
+  onRefunded: () => void;
 }
 
 type Paid = TalerMerchantApi.CheckPaymentPaidResponse & {
@@ -504,7 +507,7 @@ function PaidPage({
                                 : i18n.str`Not refundable`
                             }
                           >
-                            <button
+                            <button type="button" 
                               class="button is-danger"
                               disabled={!refundable}
                               onClick={() => onRefund(id)}
@@ -744,7 +747,7 @@ function UnpaidPage({
   );
 }
 
-export function DetailPage({ id, selected, onRefund, onBack }: Props): VNode {
+export function DetailPage({ id, selected, onRefunded, onBack }: Props): VNode {
   const [showRefund, setShowRefund] = useState<string | undefined>(undefined);
   const { i18n } = useTranslationContext();
   const DetailByStatus = function () {
@@ -773,9 +776,10 @@ export function DetailPage({ id, selected, onRefund, onBack }: Props): VNode {
       {showRefund && (
         <RefundModal
           order={selected}
+          id={id}
           onCancel={() => setShowRefund(undefined)}
-          onConfirm={(value) => {
-            onRefund(showRefund, value);
+          onConfirmed={() => {
+            onRefunded();
             setShowRefund(undefined);
           }}
         />
