@@ -29,6 +29,8 @@ import { dateFormatForSettings, usePreference } from "../../../../hooks/preferen
 import { CardTable } from "./Table.js";
 import { WithId } from "../../../../declaration.js";
 
+const TALER_SCREEN_ID = 47;
+
 export interface ListPageProps {
   onShowAll: () => void;
   onShowNotPaid: () => void;
@@ -36,7 +38,6 @@ export interface ListPageProps {
   onShowRefunded: () => void;
   onShowNotWired: () => void;
   onShowWired: () => void;
-  onCopyURL: (id: string) => void;
   isAllActive: string;
   isPaidActive: string;
   isNotPaidActive: string;
@@ -64,7 +65,6 @@ export function ListPage({
   onSelectOrder,
   onRefundOrder,
   jumpToDate,
-  onCopyURL,
   onShowAll,
   onShowPaid,
   onShowNotPaid,
@@ -212,7 +212,6 @@ export function ListPage({
       <CardTable
         orders={orders}
         onCreate={onCreate}
-        onCopyURL={onCopyURL}
         onSelect={onSelectOrder}
         onRefund={onRefundOrder}
         onLoadMoreAfter={onLoadMoreAfter}
