@@ -25,28 +25,32 @@ import {
   TalerError,
   TalerErrorCode,
   TalerMerchantApi,
-  assertUnreachable,
-  stringifyPayUri,
+  assertUnreachable
 } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import {
+  LocalNotificationBannerBulma,
+  useLocalNotificationBetter,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
 import { VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
 import { Loading } from "../../../../components/exception/loading.js";
 import { JumpToElementById } from "../../../../components/form/JumpToElementById.js";
-import { NotificationCard } from "../../../../components/menu/index.js";
 import { useSessionContext } from "../../../../context/session.js";
 import {
   InstanceOrderFilter,
   useInstanceOrders,
   useOrderDetails,
 } from "../../../../hooks/order.js";
-import { Notification } from "../../../../utils/types.js";
 import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { ListPage } from "./ListPage.js";
 import { RefundModal } from "./Table.js";
 
+
+const TALER_SCREEN_ID = 46;
+
 interface Props {
   onSelect: (id: string) => void;
   onCreate: () => void;
@@ -64,9 +68,8 @@ export default function OrderList({ onCreate, onSelect }: Props): VNode {
   const result = useInstanceOrders(filter, (d) =>
     setFilter({ ...filter, position: d }),
   );
-  const { state, lib } = useSessionContext();
-
-  const [notif, setNotif] = useState<Notification | undefined>(undefined);
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
+  const { state: session, lib } = useSessionContext();
 
   const { i18n } = useTranslationContext();
 
@@ -97,20 +100,40 @@ export default function OrderList({ onCreate, onSelect }: Props): VNode {
   const isWiredActive = filter.wired === true ? "is-active" : "";
   const isAllActive =
     filter.paid === undefined &&
-    filter.refunded === undefined &&
-    filter.wired === undefined
+      filter.refunded === undefined &&
+      filter.wired === undefined
       ? "is-active"
       : "";
 
+  const data = {} as TalerMerchantApi.RefundRequest
+  const refund = safeFunctionHandler(
+    lib.instance.addRefund.bind(lib.instance),
+    !session.token || !orderToBeRefunded ? undefined : [session.token, orderToBeRefunded.order_id, data],
+  );
+  refund.onSuccess = () => {
+    setOrderToBeRefunded(undefined)
+  };
+  refund.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unauthorized.`;
+      case HttpStatusCode.Forbidden:
+        return i18n.str`Forbidden.`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Not found.`;
+      case HttpStatusCode.Conflict:
+        return i18n.str`Conflict.`;
+      case HttpStatusCode.Gone:
+        return i18n.str`Gone.`;
+      case HttpStatusCode.UnavailableForLegalReasons:
+        return i18n.str`UnavailableForLegalReasons.`;
+    }
+  };
   return (
     <section class="section is-main-section">
-      <NotificationCard notification={notif} />
+      <LocalNotificationBannerBulma notification={notification} />
 
       <JumpToElementById
-        testIfExist={async (order) => {
-          const resp = await lib.instance.getOrderDetails(state.token, order);
-          return resp.type === "ok";
-        }}
         onSelect={onSelect}
         description={i18n.str`Jump to order with the given product ID`}
         placeholder={i18n.str`Order id`}
@@ -130,12 +153,6 @@ export default function OrderList({ onCreate, onSelect }: Props): VNode {
         isRefundedActive={isRefundedActive}
         jumpToDate={filter.date}
         onSelectDate={setNewDate}
-        onCopyURL={async (id) => {
-          const resp = await lib.instance.getOrderDetails(state.token, id);
-          if (resp.type === "ok") {
-            copyToClipboard(resp.body.order_status_url);
-          }
-        }}
         onCreate={onCreate}
         onShowAll={() => setFilter({})}
         onShowNotPaid={() => setFilter({ paid: false })}
@@ -149,53 +166,7 @@ export default function OrderList({ onCreate, onSelect }: Props): VNode {
         <RefundModalForTable
           id={orderToBeRefunded.order_id}
           onCancel={() => setOrderToBeRefunded(undefined)}
-          onConfirm={(value) => {
-            lib.instance
-              .addRefund(state.token, orderToBeRefunded.order_id, value)
-              .then((resp) => {
-                if (resp.type === "ok") {
-                  setNotif({
-                    message: i18n.str`Refund created successfully`,
-                    type: "SUCCESS",
-                  });
-                } else {
-                  switch (resp.case) {
-                    case HttpStatusCode.UnavailableForLegalReasons: {
-                      setNotif({
-                        message: i18n.str`Could not create the refund`,
-                        type: "ERROR",
-                        description: i18n.str`There are pending KYC requirements.`,
-                      });
-                      return;
-                    }
-                    case HttpStatusCode.Unauthorized:
-                    case HttpStatusCode.Forbidden:
-                    case HttpStatusCode.NotFound:
-                    case HttpStatusCode.Conflict:
-                    case HttpStatusCode.Gone: {
-                      setNotif({
-                        message: i18n.str`Could not create the refund`,
-                        type: "ERROR",
-                        description: resp.detail?.hint,
-                      });
-                      return;
-                    }
-                    default: {
-                      assertUnreachable(resp);
-                    }
-                  }
-                }
-              })
-              .catch((error) =>
-                setNotif({
-                  message: i18n.str`Could not create the refund`,
-                  type: "ERROR",
-                  description:
-                    error instanceof Error ? error.message : String(error),
-                }),
-              )
-              .then(() => setOrderToBeRefunded(undefined));
-          }}
+          onConfirmed={() => setOrderToBeRefunded(undefined)}
         />
       )}
     </section>
@@ -205,11 +176,11 @@ export default function OrderList({ onCreate, onSelect }: Props): VNode {
 interface RefundProps {
   id: string;
   onCancel: () => void;
-  onConfirm: (m: TalerMerchantApi.RefundRequest) => void;
+  onConfirmed: () => void;
 }
 
-function RefundModalForTable({ id, onConfirm, onCancel }: RefundProps): VNode {
-  const {i18n} = useTranslationContext();
+function RefundModalForTable({ id, onConfirmed, onCancel }: RefundProps): VNode {
+  const { i18n } = useTranslationContext();
   const result = useOrderDetails(id);
 
   if (!result) return <Loading />;
@@ -219,10 +190,10 @@ function RefundModalForTable({ id, onConfirm, onCancel }: RefundProps): VNode {
   if (result.type === "fail") {
     switch (result.case) {
       case TalerErrorCode.MERCHANT_GENERIC_INSTANCE_UNKNOWN: {
-        return <i18n.Translate>Instance unkown</i18n.Translate>
+        return <i18n.Translate>Instance unknown</i18n.Translate>;
       }
       case TalerErrorCode.MERCHANT_GENERIC_ORDER_UNKNOWN: {
-        return <i18n.Translate>Order unkown</i18n.Translate>
+        return <i18n.Translate>Order unknown</i18n.Translate>;
       }
       case HttpStatusCode.Unauthorized: {
         return <LoginPage />;
@@ -235,9 +206,10 @@ function RefundModalForTable({ id, onConfirm, onCancel }: RefundProps): VNode {
 
   return (
     <RefundModal
+      id={id}
       order={result.body}
       onCancel={onCancel}
-      onConfirm={onConfirm}
+      onConfirmed={onConfirmed}
     />
   );
 }
