@@ -20,40 +20,65 @@
  */
 
 import {
+  HttpStatusCode,
   randomRfc3548Base32Key,
   TalerMerchantApi,
 } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import {
+  ButtonBetterBulma,
+  LocalNotificationBannerBulma,
+  useLocalNotificationBetter,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
 import { Fragment, h, VNode } from "preact";
 import { useState } from "preact/hooks";
-import { AsyncButton } from "../../../../components/exception/AsyncButton.js";
 import { FormProvider } from "../../../../components/form/FormProvider.js";
 import { Input } from "../../../../components/form/Input.js";
 import { InputSelector } from "../../../../components/form/InputSelector.js";
 import { InputWithAddon } from "../../../../components/form/InputWithAddon.js";
 import { WithId } from "../../../../declaration.js";
+import { useSessionContext } from "../../../../context/session.js";
+
+const TALER_SCREEN_ID = 52;
 
 type Entity = TalerMerchantApi.OtpDevicePatchDetails & WithId;
 
 interface Props {
-  onUpdate: (d: Entity) => Promise<void>;
+  onUpdated: (d: Entity) => void;
   onBack?: () => void;
   device: Entity;
 }
 const algorithms = [0, 1, 2];
 const algorithmsNames = ["off", "30s 8d TOTP-SHA1", "30s 8d eTOTP-SHA1"];
-export function UpdatePage({ device, onUpdate, onBack }: Props): VNode {
+export function UpdatePage({ device, onUpdated, onBack }: Props): VNode {
   const { i18n } = useTranslationContext();
 
   const [state, setState] = useState<Partial<Entity>>(device);
   const [showKey, setShowKey] = useState(false);
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
+  const { state: session, lib } = useSessionContext();
 
-  const submitForm = () => {
-    return onUpdate(state as Entity);
+  const update = safeFunctionHandler(
+    lib.instance.updateOtpDevice.bind(lib.instance),
+    !session.token ? undefined : [session.token, device.id, state as Entity],
+  );
+  update.onSuccess = (suc, t, id, req) => {
+    onUpdated({ ...req, id: device.id });
+  };
+  update.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unauthorized`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Template id is unknown`;
+      case HttpStatusCode.Conflict:
+        return i18n.str`The provided information is inconsistent with the current state of the template`;
+    }
   };
 
   return (
     <div>
+      <LocalNotificationBannerBulma notification={notification} />
       <section class="section">
         <section class="hero is-hero-bar">
           <div class="hero-body">
@@ -122,6 +147,7 @@ export function UpdatePage({ device, onUpdate, onBack }: Props): VNode {
                       side={
                         state.otp_key === undefined ? (
                           <button
+                            type="button"
                             onClick={() => {
                               setState((s) => ({ ...s, otp_key: "" }));
                             }}
@@ -131,6 +157,7 @@ export function UpdatePage({ device, onUpdate, onBack }: Props): VNode {
                           </button>
                         ) : (
                           <button
+                            type="button"
                             data-tooltip={i18n.str`Generate random secret key`}
                             class="button is-info mr-3"
                             onClick={() => {
@@ -151,17 +178,17 @@ export function UpdatePage({ device, onUpdate, onBack }: Props): VNode {
 
               <div class="buttons is-right mt-5">
                 {onBack && (
-                  <button class="button" onClick={onBack}>
+                  <button class="button" type="button" onClick={onBack}>
                     <i18n.Translate>Cancel</i18n.Translate>
                   </button>
                 )}
-                <AsyncButton
-                  disabled={false}
+                <ButtonBetterBulma
                   data-tooltip={i18n.str`Confirm operation`}
-                  onClick={submitForm}
+                  onClick={update}
+                  type="submit"
                 >
                   <i18n.Translate>Confirm</i18n.Translate>
-                </AsyncButton>
+                </ButtonBetterBulma>
               </div>
             </div>
           </div>
