@@ -19,8 +19,17 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { AmountString, Amounts, TalerMerchantApi } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import {
+  AmountString,
+  Amounts,
+  HttpStatusCode,
+  TalerMerchantApi,
+} from "@gnu-taler/taler-util";
+import {
+  useCommonPreferences,
+  useLocalNotificationBetter,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
 import { format } from "date-fns";
 import { ComponentChildren, Fragment, VNode, h } from "preact";
 import { StateUpdater, useState } from "preact/hooks";
@@ -32,11 +41,16 @@ import {
 } from "../../../../components/form/FormProvider.js";
 import { InputCurrency } from "../../../../components/form/InputCurrency.js";
 import { InputNumber } from "../../../../components/form/InputNumber.js";
+import { useSessionContext } from "../../../../context/session.js";
+import { WithId } from "../../../../declaration.js";
 import {
   dateFormatForSettings,
+  UIElement,
   usePreference,
 } from "../../../../hooks/preference.js";
-import { WithId } from "../../../../declaration.js";
+import { FragmentPersonaFlag } from "../../../../components/menu/SideBar.js";
+
+const TALER_SCREEN_ID = 56;
 
 type Entity = TalerMerchantApi.ProductDetail & WithId;
 
@@ -44,10 +58,6 @@ interface Props {
   instances: Entity[];
   onDelete?: (id: Entity) => void;
   onSelect: (product: Entity) => void;
-  onUpdate?: (
-    id: string,
-    data: TalerMerchantApi.ProductPatchDetail,
-  ) => Promise<void>;
   onCreate?: () => void;
   selected?: boolean;
   onLoadMoreBefore?: () => void;
@@ -58,7 +68,6 @@ export function CardTable({
   instances,
   onCreate,
   onSelect,
-  onUpdate,
   onDelete,
   onLoadMoreAfter,
   onLoadMoreBefore,
@@ -82,7 +91,12 @@ export function CardTable({
               class="has-tooltip-left"
               data-tooltip={i18n.str`Add product to inventory`}
             >
-              <button class="button is-info" type="button" onClick={onCreate}>
+              <button
+                class="button is-info"
+                type="button"
+                onClick={onCreate}
+                accessKey="+"
+              >
                 <span class="icon is-small">
                   <i class="mdi mdi-plus mdi-36px" />
                 </span>
@@ -99,7 +113,6 @@ export function CardTable({
                 instances={instances}
                 onSelect={onSelect}
                 onDelete={onDelete}
-                onUpdate={onUpdate}
                 onLoadMoreAfter={onLoadMoreAfter}
                 onLoadMoreBefore={onLoadMoreBefore}
                 rowSelection={rowSelection}
@@ -118,9 +131,6 @@ interface TableProps {
   rowSelection: string | undefined;
   instances: Entity[];
   onSelect: (id: Entity) => void;
-  onUpdate:
-    | ((id: string, data: TalerMerchantApi.ProductPatchDetail) => Promise<void>)
-    | undefined;
   onDelete: ((id: Entity) => void) | undefined;
   rowSelectionHandler: StateUpdater<string | undefined>;
   onLoadMoreBefore?: () => void;
@@ -132,13 +142,28 @@ function Table({
   rowSelectionHandler,
   instances,
   onSelect,
-  onUpdate,
   onDelete,
   onLoadMoreAfter,
   onLoadMoreBefore,
 }: TableProps): VNode {
   const { i18n } = useTranslationContext();
   const [preference] = usePreference();
+  const { state: session, lib } = useSessionContext();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
+  const update = safeFunctionHandler(
+    lib.instance.updateProduct.bind(lib.instance),
+  );
+  update.onSuccess = () => rowSelectionHandler(undefined);
+  update.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unauthorized.`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Product not found.`;
+      case HttpStatusCode.Conflict:
+        return i18n.str`This change was made without outdated info.`;
+    }
+  };
   return (
     <div class="table-container">
       {onLoadMoreBefore && (
@@ -161,18 +186,14 @@ function Table({
             <th>
               <i18n.Translate>Price per unit</i18n.Translate>
             </th>
-            {preference.developerMode ? (
-              <Fragment>
-                <th>
-                  <i18n.Translate>Taxes</i18n.Translate>
-                </th>
-                <th>
-                  <i18n.Translate>Sales</i18n.Translate>
-                </th>
-              </Fragment>
-            ) : (
-              <Fragment />
-            )}
+            <FragmentPersonaFlag point={UIElement.option_inventoryTaxes}>
+              <th>
+                <i18n.Translate>Taxes</i18n.Translate>
+              </th>
+              <th>
+                <i18n.Translate>Sales</i18n.Translate>
+              </th>
+            </FragmentPersonaFlag>
             <th>
               <i18n.Translate>Stock</i18n.Translate>
             </th>
@@ -211,7 +232,7 @@ function Table({
                 <tr key="info">
                   <td
                     onClick={() =>
-                      onUpdate && rowSelection !== i.id && rowSelectionHandler(i.id)
+                      rowSelection !== i.id && rowSelectionHandler(i.id)
                     }
                     style={{ cursor: "pointer" }}
                   >
@@ -229,7 +250,7 @@ function Table({
                     class="has-tooltip-right"
                     data-tooltip={i.product_name}
                     onClick={() =>
-                      onUpdate && rowSelection !== i.id && rowSelectionHandler(i.id)
+                      rowSelection !== i.id && rowSelectionHandler(i.id)
                     }
                     style={{ cursor: "pointer" }}
                   >
@@ -239,7 +260,7 @@ function Table({
                     class="has-tooltip-right"
                     data-tooltip={i.description}
                     onClick={() =>
-                      onUpdate && rowSelection !== i.id && rowSelectionHandler(i.id)
+                      rowSelection !== i.id && rowSelectionHandler(i.id)
                     }
                     style={{ cursor: "pointer" }}
                   >
@@ -249,37 +270,33 @@ function Table({
                   </td>
                   <td
                     onClick={() =>
-                      onUpdate && rowSelection !== i.id && rowSelectionHandler(i.id)
+                      rowSelection !== i.id && rowSelectionHandler(i.id)
                     }
                     style={{ cursor: "pointer" }}
                   >
                     {isFree ? i18n.str`Free` : `${i.price} / ${i.unit}`}
                   </td>
-                  {preference.developerMode ? (
-                    <Fragment>
-                      <td
-                        onClick={() =>
-                          onUpdate && rowSelection !== i.id && rowSelectionHandler(i.id)
-                        }
-                        style={{ cursor: "pointer" }}
-                      >
-                        {sum(i.taxes)}
-                      </td>
-                      <td
-                        onClick={() =>
-                          onUpdate && rowSelection !== i.id && rowSelectionHandler(i.id)
-                        }
-                        style={{ cursor: "pointer" }}
-                      >
-                        {difference(i.price, sum(i.taxes))}
-                      </td>
-                    </Fragment>
-                  ) : (
-                    <Fragment />
-                  )}
+                  <FragmentPersonaFlag point={UIElement.option_inventoryTaxes}>
+                    <td
+                      onClick={() =>
+                        rowSelection !== i.id && rowSelectionHandler(i.id)
+                      }
+                      style={{ cursor: "pointer" }}
+                    >
+                      {sum(i.taxes)}
+                    </td>
+                    <td
+                      onClick={() =>
+                        rowSelection !== i.id && rowSelectionHandler(i.id)
+                      }
+                      style={{ cursor: "pointer" }}
+                    >
+                      {difference(i.price, sum(i.taxes))}
+                    </td>
+                  </FragmentPersonaFlag>
                   <td
                     onClick={() =>
-                      onUpdate && rowSelection !== i.id && rowSelectionHandler(i.id)
+                      rowSelection !== i.id && rowSelectionHandler(i.id)
                     }
                     style={{ cursor: "pointer" }}
                   >
@@ -287,7 +304,7 @@ function Table({
                   </td>
                   <td
                     onClick={() =>
-                      onUpdate && rowSelection !== i.id && rowSelectionHandler(i.id)
+                      rowSelection !== i.id && rowSelectionHandler(i.id)
                     }
                     style={{ cursor: "pointer" }}
                   >
@@ -332,10 +349,10 @@ function Table({
                       <FastProductUpdateForm
                         product={i}
                         onUpdate={async (prod) => {
-                          if (!onUpdate) return;
-                          return onUpdate(i.id, prod).then(() =>
-                            rowSelectionHandler(undefined),
-                          );
+                          (!session.token
+                            ? update
+                            : update.withArgs(session.token, i.id, prod)
+                          ).call();
                         }}
                         onCancel={() => rowSelectionHandler(undefined)}
                       />
@@ -349,6 +366,7 @@ function Table({
       </table>
       {onLoadMoreAfter && (
         <button
+          type="button"
           class="button is-fullwidth"
           data-tooltip={i18n.str`Load more products after the last one`}
           onClick={onLoadMoreAfter}
@@ -398,7 +416,7 @@ function FastProductWithInfiniteStockUpdateForm({
 
       <div class="buttons is-expanded">
         <div class="buttons is-right mt-5">
-          <button class="button" onClick={onCancel}>
+          <button type="button" class="button" onClick={onCancel}>
             <i18n.Translate>Cancel</i18n.Translate>
           </button>
           <span
@@ -406,6 +424,7 @@ function FastProductWithInfiniteStockUpdateForm({
             data-tooltip={i18n.str`Update product with new price`}
           >
             <button
+              type="button"
               class="button is-info"
               onClick={() =>
                 onUpdate({
@@ -477,7 +496,7 @@ function FastProductWithManagedStockUpdateForm({
       </FormProvider>
 
       <div class="buttons is-right mt-5">
-        <button class="button" onClick={onCancel}>
+        <button type="button" class="button" onClick={onCancel}>
           <i18n.Translate>Cancel</i18n.Translate>
         </button>
         <span
@@ -489,6 +508,7 @@ function FastProductWithManagedStockUpdateForm({
           }
         >
           <button
+            type="button"
             class="button is-info"
             disabled={hasErrors}
             onClick={() =>
