@@ -25,21 +25,25 @@ import {
   TalerMerchantApi,
   assertUnreachable,
 } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import {
+  LocalNotificationBannerBulma,
+  useLocalNotificationBetter,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
 import { VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
 import { Loading } from "../../../../components/exception/loading.js";
 import { JumpToElementById } from "../../../../components/form/JumpToElementById.js";
-import { NotificationCard } from "../../../../components/menu/index.js";
 import { ConfirmModal } from "../../../../components/modal/index.js";
 import { useSessionContext } from "../../../../context/session.js";
+import { WithId } from "../../../../declaration.js";
 import { useInstanceProducts } from "../../../../hooks/product.js";
-import { Notification } from "../../../../utils/types.js";
 import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { CardTable } from "./Table.js";
-import { WithId } from "../../../../declaration.js";
+
+const TALER_SCREEN_ID = 55;
 
 interface Props {
   onCreate: undefined | (() => void);
@@ -47,14 +51,32 @@ interface Props {
 }
 export default function ProductList({ onCreate, onSelect }: Props): VNode {
   const result = useInstanceProducts();
-  const { state, lib } = useSessionContext();
+  const { state: session, lib } = useSessionContext();
   const [deleting, setDeleting] = useState<
     (TalerMerchantApi.ProductDetail & WithId) | null
   >(null);
-  const [notif, setNotif] = useState<Notification | undefined>(undefined);
 
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
   const { i18n } = useTranslationContext();
 
+  const remove = safeFunctionHandler(
+    lib.instance.deleteProduct.bind(lib.instance),
+    !session.token || !deleting ? undefined : [session.token, deleting.id],
+  );
+  remove.onSuccess = (suc, t, id) => {
+    setDeleting(null)
+    return i18n.str`Product (ID: ${id}) has been deleted`
+  }
+  remove.onFail = (fail) =>{
+    switch(fail.case) {
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unauthorized.`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Not found.`;
+      case HttpStatusCode.Conflict:
+        return i18n.str`Conflict.`;
+    }
+  }
   if (!result) return <Loading />;
   if (result instanceof TalerError) {
     return <ErrorLoadingMerchant error={result} />;
@@ -75,13 +97,9 @@ export default function ProductList({ onCreate, onSelect }: Props): VNode {
 
   return (
     <section class="section is-main-section">
-      <NotificationCard notification={notif} />
+      <LocalNotificationBannerBulma notification={notification} />
 
       <JumpToElementById
-        testIfExist={async (id) => {
-          const resp = await lib.instance.getProductDetails(state.token, id);
-          return resp.type === "ok";
-        }}
         onSelect={onSelect}
         description={i18n.str`Jump to product with the given product ID`}
         placeholder={i18n.str`Product id`}
@@ -92,34 +110,6 @@ export default function ProductList({ onCreate, onSelect }: Props): VNode {
         onLoadMoreBefore={result.loadFirst}
         onLoadMoreAfter={result.loadNext}
         onCreate={onCreate}
-        onUpdate={async (id, prod) => {
-          try {
-            const resp = await lib.instance.updateProduct(
-              state.token,
-              id,
-              prod,
-            );
-            if (resp.type === "ok") {
-              setNotif({
-                message: i18n.str`Product updated successfully`,
-                type: "SUCCESS",
-              });
-            } else {
-              setNotif({
-                message: i18n.str`Could not update the product`,
-                type: "ERROR",
-                description: resp.detail?.hint,
-              });
-            }
-          } catch (error) {
-            setNotif({
-              message: i18n.str`Could not update the product`,
-              type: "ERROR",
-              description: error instanceof Error ? error.message : undefined,
-            });
-          }
-          return;
-        }}
         onSelect={(product) => onSelect(product.id)}
         onDelete={(prod: TalerMerchantApi.ProductDetail & WithId) =>
           setDeleting(prod)
@@ -133,33 +123,7 @@ export default function ProductList({ onCreate, onSelect }: Props): VNode {
           danger
           active
           onCancel={() => setDeleting(null)}
-          onConfirm={async (): Promise<void> => {
-            try {
-              const resp = await lib.instance.deleteProduct(
-                state.token,
-                deleting.id,
-              );
-              if (resp.type === "ok") {
-                setNotif({
-                  message: i18n.str`Product "${deleting.description}" (ID: ${deleting.id}) has been deleted`,
-                  type: "SUCCESS",
-                });
-              } else {
-                setNotif({
-                  message: i18n.str`Could not delete the product`,
-                  type: "ERROR",
-                  description: resp.detail?.hint,
-                });
-              }
-            } catch (error) {
-              setNotif({
-                message: i18n.str`Could not delete the product`,
-                type: "ERROR",
-                description: error instanceof Error ? error.message : undefined,
-              });
-            }
-            setDeleting(null);
-          }}
+          confirm={remove}
         >
           <p>
             <i18n.Translate>
