@@ -23,14 +23,19 @@ import {
   AmountString,
   Amounts,
   Duration,
+  HttpStatusCode,
   TalerError,
   TalerMerchantApi,
   TranslatedString,
 } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import {
+  ButtonBetterBulma,
+  LocalNotificationBannerBulma,
+  useLocalNotificationBetter,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
 import { useState } from "preact/hooks";
-import { AsyncButton } from "../../../../components/exception/AsyncButton.js";
 import {
   FormErrors,
   FormProvider,
@@ -43,8 +48,15 @@ import { InputSelector } from "../../../../components/form/InputSelector.js";
 import { InputToggle } from "../../../../components/form/InputToggle.js";
 import { InputWithAddon } from "../../../../components/form/InputWithAddon.js";
 import { TextField } from "../../../../components/form/TextField.js";
+import {
+  ComponentPersonaFlag,
+  FragmentPersonaFlag,
+} from "../../../../components/menu/SideBar.js";
 import { useSessionContext } from "../../../../context/session.js";
 import { useInstanceOtpDevices } from "../../../../hooks/otp.js";
+import { UIElement } from "../../../../hooks/preference.js";
+
+const TALER_SCREEN_ID = 61;
 
 // type Entity = TalerMerchantApi.TemplateAddDetails & { type: Steps };
 type Entity = {
@@ -61,20 +73,22 @@ type Entity = {
 };
 
 interface Props {
-  onCreate: (d: TalerMerchantApi.TemplateAddDetails) => Promise<void>;
+  onCreated: () => void;
   onBack?: () => void;
+  defaultSettingsDuration: Duration;
 }
 
-export function CreatePage({ onCreate, onBack }: Props): VNode {
+export function CreatePage({ defaultSettingsDuration, onCreated, onBack }: Props): VNode {
   const { i18n } = useTranslationContext();
-  const { config, state: session } = useSessionContext();
+  const { config, state: session, lib } = useSessionContext();
   const devices = useInstanceOtpDevices();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
 
   const [state, setState] = useState<Partial<Entity>>({
     minimum_age: 0,
-    pay_duration: {
-      d_ms: 1000 * 60 * 30, //30 min
-    },
+    amount_editable: true,
+    summary_editable: true,
+    pay_duration: defaultSettingsDuration,
   });
 
   function updateState(up: (s: Partial<Entity>) => Partial<Entity>) {
@@ -96,11 +110,16 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
         ? i18n.str`Invalid. Please enter letters and numbers only.`
         : undefined,
     description: !state.description ? i18n.str`Required` : undefined,
+    summary: !state.summary
+      ? state.summary_editable
+        ? undefined
+        : i18n.str`Required`
+      : undefined, // more summary validations? none
     amount: !state.amount
       ? state.amount_editable
         ? undefined
         : i18n.str`Required`
-      : !parsedPrice
+      : !parsedPrice // more summary validations? is valid amount...
         ? i18n.str`Invalid`
         : Amounts.isZero(parsedPrice)
           ? state.amount_editable
@@ -128,37 +147,47 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
 
   const zero = Amounts.stringify(Amounts.zeroOfCurrency(config.currency));
 
-  const submitForm = () => {
-    if (hasErrors) return Promise.reject();
-    const contract_amount = state.amount_editable
-      ? undefined
-      : (state.amount as AmountString);
-    const contract_summary = state.summary_editable ? undefined : state.summary;
-    const template_contract: TalerMerchantApi.TemplateContractDetails = {
-      minimum_age: state.minimum_age!,
-      pay_duration: Duration.toTalerProtocolDuration(state.pay_duration!),
-      amount: contract_amount,
-      summary: contract_summary,
+  const contract_amount = state.amount_editable
+    ? undefined
+    : (state.amount as AmountString);
+  const contract_summary = state.summary_editable ? undefined : state.summary;
+  const template_contract: TalerMerchantApi.TemplateContractDetails = {
+    minimum_age: state.minimum_age!,
+    pay_duration: Duration.toTalerProtocolDuration(state.pay_duration!),
+    amount: contract_amount,
+    summary: contract_summary,
+    currency:
+      cList.length > 1 && state.currency_editable ? undefined : config.currency,
+  };
+  const data: TalerMerchantApi.TemplateAddDetails = {
+    template_id: state.id!,
+    template_description: state.description!,
+    template_contract,
+    editable_defaults: {
+      amount: !state.amount_editable ? undefined : state.amount ?? zero,
+      summary: !state.summary_editable ? undefined : state.summary ?? "",
       currency:
-        cList.length > 1 && state.currency_editable
+        cList.length === 1 || !state.currency_editable
           ? undefined
           : config.currency,
-    };
-    return onCreate({
-      template_id: state.id!,
-      template_description: state.description!,
-      template_contract,
-      editable_defaults: {
-        amount: !state.amount_editable ? undefined : state.amount ?? zero,
-        summary: !state.summary_editable ? undefined : state.summary ?? "",
-        currency:
-          cList.length === 1 || !state.currency_editable
-            ? undefined
-            : config.currency,
-      },
-      otp_id: state.otpId!,
-    });
+    },
+    otp_id: state.otpId!,
   };
+
+  const create = safeFunctionHandler(
+    lib.instance.addTemplate.bind(lib.instance),
+    !session.token ? undefined : [session.token, data],
+  );
+  create.onSuccess = onCreated;
+  create.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unauthorized`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Not found`;
+    }
+  };
+
   const deviceList =
     !devices || devices instanceof TalerError || devices.type === "fail"
       ? []
@@ -172,6 +201,8 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
   );
   return (
     <div>
+      <LocalNotificationBannerBulma notification={notification} />
+
       <section class="section is-main-section">
         <div class="columns">
           <div class="column" />
@@ -194,15 +225,14 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
               />
               <Input<Entity>
                 name="description"
-                label={i18n.str`Description`}
-                help=""
+                label={i18n.str`Template name`}
                 tooltip={i18n.str`Describe what this template stands for`}
               />
 
               <Input<Entity>
                 name="summary"
                 inputType="multiline"
-                label={i18n.str`Summary`}
+                label={i18n.str`Order summary`}
                 tooltip={i18n.str`If specified here, this template will create orders with the same summary`}
               />
               <InputToggle<Entity>
@@ -236,20 +266,29 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
                   </TextField>
                 </Fragment>
               )}
-              <InputNumber<Entity>
-                name="minimum_age"
-                label={i18n.str`Minimum age`}
-                help=""
-                tooltip={i18n.str`Is this contract restricted to some age?`}
-              />
-              <InputDuration<Entity>
-                name="pay_duration"
-                label={i18n.str`Payment timeout`}
-                help=""
-                tooltip={i18n.str`How much time the customer has to complete the payment once the order was created.`}
-              />
+              <FragmentPersonaFlag point={UIElement.option_ageRestriction}>
+                <InputNumber<Entity>
+                  name="minimum_age"
+                  label={i18n.str`Minimum age`}
+                  help=""
+                  tooltip={i18n.str`Is this contract restricted to some age?`}
+                />
+              </FragmentPersonaFlag>
+              <FragmentPersonaFlag
+                point={UIElement.option_paymentTimeoutOnTemplate}
+              >
+                <InputDuration<Entity>
+                  name="pay_duration"
+                  label={i18n.str`Payment timeout`}
+                  help=""
+                  tooltip={i18n.str`How much time the customer has to complete the payment once the order was created.`}
+                />
+              </FragmentPersonaFlag>
+
               {!deviceList.length ? (
-                <TextField
+                <ComponentPersonaFlag
+                  Comp={TextField}
+                  point={UIElement.option_otpDevicesOnTemplate}
                   name="otpId"
                   label={i18n.str`OTP device`}
                   tooltip={i18n.str`Use to verify transactions in offline mode.`}
@@ -258,7 +297,7 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
                   <a href="#/otp-devices/new">
                     <i18n.Translate>Add one first</i18n.Translate>
                   </a>
-                </TextField>
+                </ComponentPersonaFlag>
               ) : (
                 <InputSelector<Entity>
                   name="otpId"
@@ -280,21 +319,21 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
 
             <div class="buttons is-right mt-5">
               {onBack && (
-                <button class="button" onClick={onBack}>
+                <button type="button" class="button" onClick={onBack}>
                   <i18n.Translate>Cancel</i18n.Translate>
                 </button>
               )}
-              <AsyncButton
-                disabled={hasErrors}
+              <ButtonBetterBulma
                 data-tooltip={
                   hasErrors
                     ? i18n.str`Please complete the marked fields`
                     : i18n.str`Confirm operation`
                 }
-                onClick={submitForm}
+                type="submit"
+                onClick={create}
               >
                 <i18n.Translate>Confirm</i18n.Translate>
-              </AsyncButton>
+              </ButtonBetterBulma>
             </div>
           </div>
           <div class="column" />
