@@ -24,6 +24,8 @@ import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { h, VNode } from "preact";
 import { StateUpdater, useState } from "preact/hooks";
 
+const TALER_SCREEN_ID = 63;
+
 type Entity = TalerMerchantApi.TemplateEntry;
 
 interface Props {
@@ -65,7 +67,12 @@ export function CardTable({
             class="has-tooltip-left"
             data-tooltip={i18n.str`Add new templates`}
           >
-            <button class="button is-info" type="button" onClick={onCreate}>
+            <button
+              class="button is-info"
+              type="button"
+              onClick={onCreate}
+              accessKey="+"
+            >
               <span class="icon is-small">
                 <i class="mdi mdi-plus mdi-36px" />
               </span>
@@ -123,6 +130,7 @@ function Table({
     <div class="table-container">
       {onLoadMoreBefore && (
         <button
+          type="button"
           class="button is-fullwidth"
           data-tooltip={i18n.str`Load more templates before the first one`}
           onClick={onLoadMoreBefore}
@@ -161,6 +169,7 @@ function Table({
                 <td class="is-actions-cell right-sticky">
                   <div class="buttons is-right">
                     <button
+                      type="button"
                       class="button is-danger is-small has-tooltip-left"
                       data-tooltip={i18n.str`Delete selected templates from the database`}
                       onClick={() => onDelete(i)}
@@ -168,18 +177,20 @@ function Table({
                       <i18n.Translate>Delete</i18n.Translate>
                     </button>
                     <button
+                      type="button"
                       class="button is-info is-small has-tooltip-left"
                       data-tooltip={i18n.str`Use template to create new order`}
                       onClick={() => onNewOrder(i)}
                     >
-                      <i18n.Translate>Use template</i18n.Translate>
+                      <i18n.Translate>Test</i18n.Translate>
                     </button>
                     <button
+                      type="button"
                       class="button is-info is-small has-tooltip-left"
                       data-tooltip={i18n.str`Generate a QR code for the template.`}
                       onClick={() => onQR(i)}
                     >
-                      QR
+                      <i18n.Translate>Show QR</i18n.Translate>
                     </button>
                   </div>
                 </td>
@@ -190,6 +201,7 @@ function Table({
       </table>
       {onLoadMoreAfter && (
         <button
+          type="button"
           class="button is-fullwidth"
           data-tooltip={i18n.str`Load more templates after the last one`}
           onClick={onLoadMoreAfter}
