@@ -25,21 +25,25 @@ import {
   TalerMerchantApi,
   assertUnreachable,
 } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import {
+  LocalNotificationBannerBulma,
+  useLocalNotificationBetter,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
 import { VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
 import { Loading } from "../../../../components/exception/loading.js";
 import { JumpToElementById } from "../../../../components/form/JumpToElementById.js";
-import { NotificationCard } from "../../../../components/menu/index.js";
 import { ConfirmModal } from "../../../../components/modal/index.js";
 import { useSessionContext } from "../../../../context/session.js";
 import { useInstanceTemplates } from "../../../../hooks/templates.js";
-import { Notification } from "../../../../utils/types.js";
 import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { CardTable } from "./Table.js";
 
+const TALER_SCREEN_ID = 62;
+
 interface Props {
   onCreate: () => void;
   onSelect: (id: string) => void;
@@ -54,8 +58,9 @@ export default function ListTemplates({
   onNewOrder,
 }: Props): VNode {
   const { i18n } = useTranslationContext();
-  const [notif, setNotif] = useState<Notification | undefined>(undefined);
-  const { state, lib } = useSessionContext();
+
+  const { state: session, lib } = useSessionContext();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
   const result = useInstanceTemplates();
   const [deleting, setDeleting] =
     useState<TalerMerchantApi.TemplateEntry | null>(null);
@@ -78,15 +83,29 @@ export default function ListTemplates({
     }
   }
 
+  const remove = safeFunctionHandler(
+    lib.instance.deleteTemplate.bind(lib.instance),
+    !session.token || !deleting
+      ? undefined
+      : [session.token, deleting.template_id],
+  );
+  remove.onSuccess = () => {
+    setDeleting(null);
+  };
+  remove.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unauthorized`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Not found`;
+    }
+  };
+
   return (
     <section class="section is-main-section">
-      <NotificationCard notification={notif} />
+      <LocalNotificationBannerBulma notification={notification} />
 
       <JumpToElementById
-        testIfExist={async (id) => {
-          const resp = await lib.instance.getTemplateDetails(state.token, id);
-          return resp.type === "ok";
-        }}
         onSelect={onSelect}
         description={i18n.str`Jump to template with the given template ID`}
         placeholder={i18n.str`Template identification`}
@@ -121,33 +140,7 @@ export default function ListTemplates({
           danger
           active
           onCancel={() => setDeleting(null)}
-          onConfirm={async (): Promise<void> => {
-            try {
-              const resp = await lib.instance.deleteTemplate(
-                state.token,
-                deleting.template_id,
-              );
-              if (resp.type === "ok") {
-                setNotif({
-                  message: i18n.str`Template "${deleting.template_description}" (ID: ${deleting.template_id}) has been deleted`,
-                  type: "SUCCESS",
-                });
-              } else {
-                setNotif({
-                  message: i18n.str`Failed to delete template`,
-                  type: "ERROR",
-                  description: resp.detail?.hint,
-                });
-              }
-            } catch (error) {
-              setNotif({
-                message: i18n.str`Failed to delete template`,
-                type: "ERROR",
-                description: error instanceof Error ? error.message : undefined,
-              });
-            }
-            setDeleting(null);
-          }}
+          confirm={remove}
         >
           <p>
             <i18n.Translate>
@@ -157,7 +150,9 @@ export default function ListTemplates({
             </i18n.Translate>
           </p>
           <p class="warning">
-            <i18n.Translate>Deleting a template cannot be undone.</i18n.Translate>
+            <i18n.Translate>
+              Deleting a template cannot be undone.
+            </i18n.Translate>
           </p>
         </ConfirmModal>
       )}
