@@ -27,17 +27,13 @@ import {
 } from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, VNode, h } from "preact";
-import { useState } from "preact/hooks";
 import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
 import { Loading } from "../../../../components/exception/loading.js";
-import { NotificationCard } from "../../../../components/menu/index.js";
-import { useSessionContext } from "../../../../context/session.js";
+import { WithId } from "../../../../declaration.js";
 import { useTemplateDetails } from "../../../../hooks/templates.js";
-import { Notification } from "../../../../utils/types.js";
 import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { UpdatePage } from "./UpdatePage.js";
-import { WithId } from "../../../../declaration.js";
 
 export type Entity = TalerMerchantApi.TemplatePatchDetails & WithId;
 
@@ -51,11 +47,7 @@ export default function UpdateTemplate({
   onConfirm,
   onBack,
 }: Props): VNode {
-  const { state, lib } = useSessionContext();
   const result = useTemplateDetails(tid);
-  const [notif, setNotif] = useState<Notification | undefined>(undefined);
-
-  const { i18n } = useTranslationContext();
 
   if (!result) return <Loading />;
   if (result instanceof TalerError) {
@@ -77,36 +69,10 @@ export default function UpdateTemplate({
 
   return (
     <Fragment>
-      <NotificationCard notification={notif} />
       <UpdatePage
         template={{ ...result.body, id: tid }}
         onBack={onBack}
-        onUpdate={(data) => {
-          return lib.instance
-            .updateTemplate(state.token, tid, data)
-            .then((resp) => {
-              if (resp.type === "ok") {
-                setNotif({
-                  message: i18n.str`Template (ID: ${tid}) has been updated`,
-                  type: "SUCCESS",
-                });
-                onConfirm();
-              } else {
-                setNotif({
-                  message: i18n.str`Could not update template`,
-                  type: "ERROR",
-                  description: resp.detail?.hint,
-                });
-              }
-            })
-            .catch((error) => {
-              setNotif({
-                message: i18n.str`Could not update template`,
-                type: "ERROR",
-                description: error instanceof Error ? error.message : String(error),
-              });
-            });
-        }}
+        onUpdated={onConfirm}
       />
     </Fragment>
   );
