@@ -25,16 +25,14 @@ import { StateUpdater, useState } from "preact/hooks";
 import { format } from "date-fns";
 import { TalerMerchantApi } from "@gnu-taler/taler-util";
 
+const TALER_SCREEN_ID = 69;
+
 type Entity = TalerMerchantApi.TokenFamilySummary;
 
 interface Props {
   instances: Entity[];
   onDelete: (tokenFamily: Entity) => void;
   onSelect: (tokenFamily: Entity) => void;
-  onUpdate: (
-    slug: string,
-    data: TalerMerchantApi.TokenFamilyUpdateRequest,
-  ) => Promise<void>;
   onCreate: () => void;
   selected?: boolean;
 }
@@ -43,7 +41,6 @@ export function CardTable({
   instances,
   onCreate,
   onSelect,
-  onUpdate,
   onDelete,
 }: Props): VNode {
   const [rowSelection, rowSelectionHandler] = useState<string | undefined>(
@@ -64,7 +61,12 @@ export function CardTable({
             class="has-tooltip-left"
             data-tooltip={i18n.str`Add token family`}
           >
-            <button class="button is-info" type="button" onClick={onCreate}>
+            <button
+              class="button is-info"
+              type="button"
+              onClick={onCreate}
+              accessKey="+"
+            >
               <span class="icon is-small">
                 <i class="mdi mdi-plus mdi-36px" />
               </span>
@@ -80,7 +82,6 @@ export function CardTable({
                 instances={instances}
                 onSelect={onSelect}
                 onDelete={onDelete}
-                onUpdate={onUpdate}
                 rowSelection={rowSelection}
                 rowSelectionHandler={rowSelectionHandler}
               />
@@ -97,10 +98,6 @@ interface TableProps {
   rowSelection: string | undefined;
   instances: Entity[];
   onSelect: (tokenFamily: Entity) => void;
-  onUpdate: (
-    slug: string,
-    data: TalerMerchantApi.TokenFamilyUpdateRequest,
-  ) => Promise<void>;
   onDelete: (tokenFamily: Entity) => void;
   rowSelectionHandler: StateUpdater<string | undefined>;
 }
@@ -165,7 +162,10 @@ function Table({
                   >
                     {i.valid_after.t_s === "never"
                       ? i18n.str`Never`
-                      : format(new Date(i.valid_after.t_s * 1000), "yyyy/MM/dd hh:mm:ss")}
+                      : format(
+                          new Date(i.valid_after.t_s * 1000),
+                          "yyyy/MM/dd hh:mm:ss",
+                        )}
                   </td>
                   <td
                     onClick={() =>
@@ -175,7 +175,10 @@ function Table({
                   >
                     {i.valid_before.t_s === "never"
                       ? i18n.str`Never`
-                      : format(new Date(i.valid_before.t_s * 1000), "yyyy/MM/dd hh:mm:ss")}
+                      : format(
+                          new Date(i.valid_before.t_s * 1000),
+                          "yyyy/MM/dd hh:mm:ss",
+                        )}
                   </td>
                   <td
                     onClick={() =>
@@ -223,7 +226,6 @@ function Table({
   );
 }
 
-
 function EmptyTable(): VNode {
   const { i18n } = useTranslationContext();
   return (
@@ -235,7 +237,8 @@ function EmptyTable(): VNode {
       </p>
       <p>
         <i18n.Translate>
-          There are no token families yet, add the first one by pressing the + sign.
+          There are no token families yet, add the first one by pressing the +
+          sign.
         </i18n.Translate>
       </p>
     </div>
