@@ -25,20 +25,24 @@ import {
   TalerMerchantApi,
   assertUnreachable,
 } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import {
+  LocalNotificationBannerBulma,
+  useLocalNotificationBetter,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
 import { VNode, h } from "preact";
 import { useState } from "preact/hooks";
 import { ErrorLoadingMerchant } from "../../../../components/ErrorLoadingMerchant.js";
 import { Loading } from "../../../../components/exception/loading.js";
-import { NotificationCard } from "../../../../components/menu/index.js";
 import { ConfirmModal } from "../../../../components/modal/index.js";
 import { useSessionContext } from "../../../../context/session.js";
 import { useInstanceTokenFamilies } from "../../../../hooks/tokenfamily.js";
-import { Notification } from "../../../../utils/types.js";
 import { LoginPage } from "../../../login/index.js";
 import { NotFoundPageOrAdminCreate } from "../../../notfound/index.js";
 import { CardTable } from "./Table.js";
 
+const TALER_SCREEN_ID = 68;
+
 interface Props {
   onUnauthorized: () => VNode;
   onNotFound: () => VNode;
@@ -47,12 +51,13 @@ interface Props {
 }
 export default function TokenFamilyList({ onCreate, onSelect }: Props): VNode {
   const result = useInstanceTokenFamilies();
-  const [notif, setNotif] = useState<Notification | undefined>(undefined);
-  const { lib, state } = useSessionContext();
+
   const [deleting, setDeleting] =
     useState<TalerMerchantApi.TokenFamilySummary | null>(null);
 
   const { i18n } = useTranslationContext();
+  const { state: session, lib } = useSessionContext();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
 
   if (!result) return <Loading />;
   if (result instanceof TalerError) {
@@ -72,41 +77,30 @@ export default function TokenFamilyList({ onCreate, onSelect }: Props): VNode {
     }
   }
 
+  const remove = safeFunctionHandler(
+    lib.instance.deleteTokenFamily.bind(lib.instance),
+    !session.token || !deleting ? undefined : [session.token, deleting.slug],
+  );
+  remove.onSuccess = () => {
+    setDeleting(null);
+    return i18n.str`Token family has been deleted.`;
+  };
+  remove.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unaouthorized`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Not found`;
+    }
+  };
+
   return (
     <section class="section is-main-section">
-      <NotificationCard notification={notif} />
+      <LocalNotificationBannerBulma notification={notification} />
 
       <CardTable
         instances={result.body.token_families}
         onCreate={onCreate}
-        onUpdate={async (slug, fam) => {
-          try {
-            const resp = await lib.instance.updateTokenFamily(
-              state.token,
-              slug,
-              fam,
-            );
-            if (resp.type === "ok") {
-              setNotif({
-                message: i18n.str`Token family updated successfully`,
-                type: "SUCCESS",
-              });
-            } else {
-              setNotif({
-                message: i18n.str`Could not update the token family`,
-                type: "ERROR",
-                description: resp.detail?.hint,
-              });
-            }
-          } catch (error) {
-            setNotif({
-              message: i18n.str`Could not update the token family`,
-              type: "ERROR",
-              description: error instanceof Error ? error.message : undefined,
-            });
-          }
-          return;
-        }}
         onSelect={(tokenFamily) => onSelect(tokenFamily.slug)}
         onDelete={(fam) => setDeleting(fam)}
       />
@@ -118,33 +112,7 @@ export default function TokenFamilyList({ onCreate, onSelect }: Props): VNode {
           danger
           active
           onCancel={() => setDeleting(null)}
-          onConfirm={async (): Promise<void> => {
-            try {
-              const resp = await lib.instance.deleteTokenFamily(
-                state.token,
-                deleting.slug,
-              );
-              if (resp.type === "ok") {
-                setNotif({
-                  message: i18n.str`Token family "${deleting.name}" (SLUG: ${deleting.slug}) has been deleted`,
-                  type: "SUCCESS",
-                });
-              } else {
-                setNotif({
-                  message: i18n.str`Failed to delete token family`,
-                  type: "ERROR",
-                  description: resp.detail?.hint,
-                });
-              }
-            } catch (error) {
-              setNotif({
-                message: i18n.str`Failed to delete token family`,
-                type: "ERROR",
-                description: error instanceof Error ? error.message : undefined,
-              });
-            }
-            setDeleting(null);
-          }}
+          confirm={remove}
         >
           <p>
             <i18n.Translate>
