@@ -19,26 +19,36 @@
  * @author Christian Blättler
  */
 
-import { Duration, TalerMerchantApi } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import { HttpStatusCode, TalerMerchantApi } from "@gnu-taler/taler-util";
+import {
+  ButtonBetterBulma,
+  LocalNotificationBannerBulma,
+  useLocalNotificationBetter,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
 import { h } from "preact";
 import { useState } from "preact/hooks";
-import { AsyncButton } from "../../../../components/exception/AsyncButton.js";
-import { FormErrors, FormProvider } from "../../../../components/form/FormProvider.js";
+import {
+  FormErrors,
+  FormProvider,
+} from "../../../../components/form/FormProvider.js";
 import { Input } from "../../../../components/form/Input.js";
 import { InputDate } from "../../../../components/form/InputDate.js";
 import { InputDuration } from "../../../../components/form/InputDuration.js";
+import { useSessionContext } from "../../../../context/session.js";
 import { undefinedIfEmpty } from "../../../../utils/table.js";
 
+const TALER_SCREEN_ID = 70;
+
 type Entity = TalerMerchantApi.TokenFamilyUpdateRequest;
 
 interface Props {
-  onUpdate: (d: TalerMerchantApi.TokenFamilyUpdateRequest) => Promise<void>;
+  onUpdated: () => void;
   onBack?: () => void;
-  tokenFamily: TalerMerchantApi.TokenFamilyUpdateRequest;
+  tokenFamily: TalerMerchantApi.TokenFamilyDetails;
 }
 
-export function UpdatePage({ onUpdate, onBack, tokenFamily }: Props) {
+export function UpdatePage({ onUpdated, onBack, tokenFamily }: Props) {
   const [value, valueHandler] = useState<Partial<Entity>>(tokenFamily);
   const { i18n } = useTranslationContext();
   const errors = undefinedIfEmpty<FormErrors<Entity>>({
@@ -48,18 +58,30 @@ export function UpdatePage({ onUpdate, onBack, tokenFamily }: Props) {
     valid_before: !value.valid_before ? i18n.str`Required` : undefined,
     duration: !value.duration ? i18n.str`Required` : undefined,
   });
+  const { state: session, lib } = useSessionContext();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
 
   const hasErrors = errors !== undefined;
 
-  const submitForm = () => {
-    if (hasErrors) return Promise.reject();
-
-    return onUpdate(value as Entity);
-  }
-
+  const update = safeFunctionHandler(
+    lib.instance.updateTokenFamily.bind(lib.instance),
+    !session.token || !!errors
+      ? undefined
+      : [session.token, tokenFamily.slug, value as Entity],
+  );
+  update.onSuccess = onUpdated;
+  update.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unaouthorized`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Not found`;
+    }
+  };
 
   return (
     <div>
+      <LocalNotificationBannerBulma notification={notification} />
       <section class="section">
         <section class="hero is-hero-bar">
           <div class="hero-body">
@@ -67,7 +89,9 @@ export function UpdatePage({ onUpdate, onBack, tokenFamily }: Props) {
               <div class="level-left">
                 <div class="level-item">
                   <span class="is-size-4">
-                    <i18n.Translate>Token Family: <b>{tokenFamily.name}</b></i18n.Translate>
+                    <i18n.Translate>
+                      Token Family: <b>{tokenFamily.name}</b>
+                    </i18n.Translate>
                   </span>
                 </div>
               </div>
@@ -120,21 +144,21 @@ export function UpdatePage({ onUpdate, onBack, tokenFamily }: Props) {
 
               <div class="buttons is-right mt-5">
                 {onBack && (
-                  <button class="button" onClick={onBack}>
+                  <button type="button"class="button" onClick={onBack}>
                     <i18n.Translate>Cancel</i18n.Translate>
                   </button>
                 )}
-                <AsyncButton
-                  disabled={hasErrors}
+                <ButtonBetterBulma
                   data-tooltip={
                     hasErrors
                       ? i18n.str`Please complete the marked fields`
                       : i18n.str`Confirm operation`
                   }
-                  onClick={submitForm}
+                  type="submit"
+                  onClick={update}
                 >
                   <i18n.Translate>Confirm</i18n.Translate>
-                </AsyncButton>
+                </ButtonBetterBulma>
               </div>
             </div>
           </div>
