@@ -26,36 +26,38 @@ import { FormProvider } from "../../../../components/form/FormProvider.js";
 import { InputSelector } from "../../../../components/form/InputSelector.js";
 import { CardTable } from "./Table.js";
 
+const TALER_SCREEN_ID = 71;
+
 export interface Props {
   transfers: TalerMerchantApi.TransferDetails[];
   onLoadMoreBefore?: () => void;
   onLoadMoreAfter?: () => void;
-  onShowAll: () => void;
+  // onShowAll: () => void;
   onShowVerified: () => void;
   onShowUnverified: () => void;
   isVerifiedTransfers?: boolean;
   isNonVerifiedTransfers?: boolean;
-  isAllTransfers?: boolean;
+  // isAllTransfers?: boolean;
   accounts: string[];
   onChangePayTo: (p?: string) => void;
   payTo?: string;
-  onCreate: () => void;
-  onDelete: (wid: TalerMerchantApi.TransferDetails) => void;
+  // onCreate: () => void;
+  // onDelete: (wid: TalerMerchantApi.TransferDetails) => void;
 }
 
 export function ListPage({
   payTo,
   onChangePayTo,
   transfers,
-  onCreate,
-  onDelete,
+  // onCreate,
+  // onDelete,
   accounts,
   onLoadMoreBefore,
   onLoadMoreAfter,
-  isAllTransfers,
+  // isAllTransfers,
   isNonVerifiedTransfers,
   isVerifiedTransfers,
-  onShowAll,
+  // onShowAll,
   onShowUnverified,
   onShowVerified,
 }: Props): VNode {
@@ -89,13 +91,13 @@ export function ListPage({
       </div>
       <div class="tabs">
         <ul>
-          <li class={isAllTransfers ? "is-active" : ""}>
+          <li class={isNonVerifiedTransfers ? "is-active" : ""}>
             <div
               class="has-tooltip-right"
-              data-tooltip={i18n.str`Remove all filters`}
+              data-tooltip={i18n.str`Only show wire transfers claimed by the exchange`}
             >
-              <a onClick={onShowAll}>
-                <i18n.Translate>All</i18n.Translate>
+              <a onClick={onShowUnverified}>
+                <i18n.Translate>Incoming</i18n.Translate>
               </a>
             </div>
           </li>
@@ -109,16 +111,6 @@ export function ListPage({
               </a>
             </div>
           </li>
-          <li class={isNonVerifiedTransfers ? "is-active" : ""}>
-            <div
-              class="has-tooltip-right"
-              data-tooltip={i18n.str`Only show wire transfers claimed by the exchange`}
-            >
-              <a onClick={onShowUnverified}>
-                <i18n.Translate>Unverified</i18n.Translate>
-              </a>
-            </div>
-          </li>
         </ul>
       </div>
       <CardTable
@@ -127,8 +119,8 @@ export function ListPage({
           id: String(o.transfer_serial_id),
         }))}
         accounts={accounts}
-        onCreate={onCreate}
-        onDelete={onDelete}
+        // onCreate={onCreate}
+        // onDelete={onDelete}
         onLoadMoreBefore={onLoadMoreBefore}
         onLoadMoreAfter={onLoadMoreAfter}
       />
