@@ -30,12 +30,14 @@ import {
 } from "../../../../hooks/preference.js";
 import { WithId } from "../../../../declaration.js";
 
+const TALER_SCREEN_ID = 72;
+
 type Entity = TalerMerchantApi.TransferDetails & WithId;
 
 interface Props {
   transfers: Entity[];
-  onDelete: (id: Entity) => void;
-  onCreate: () => void;
+  // onDelete: (id: Entity) => void;
+  // onCreate: () => void;
   accounts: string[];
   onLoadMoreBefore?: () => void;
   onLoadMoreAfter?: () => void;
@@ -43,8 +45,8 @@ interface Props {
 
 export function CardTable({
   transfers,
-  onCreate,
-  onDelete,
+  // onCreate,
+  // onDelete,
   onLoadMoreAfter,
   onLoadMoreBefore,
 }: Props): VNode {
@@ -61,18 +63,23 @@ export function CardTable({
           </span>
           <i18n.Translate>Wire transfers</i18n.Translate>
         </p>
-        <div class="card-header-icon" aria-label="more options">
+        {/* <div class="card-header-icon" aria-label="more options">
           <span
             class="has-tooltip-left"
             data-tooltip={i18n.str`Add new transfer`}
           >
-            <button class="button is-info" type="button" onClick={onCreate}>
+            <button
+              type="button"
+              class="button is-info"
+              onClick={onCreate}
+              accessKey="+"
+            >
               <span class="icon is-small">
                 <i class="mdi mdi-plus mdi-36px" />
               </span>
             </button>
           </span>
-        </div>
+        </div> */}
       </header>
       <div class="card-content">
         <div class="b-table has-pagination">
@@ -80,7 +87,7 @@ export function CardTable({
             {transfers.length > 0 ? (
               <Table
                 instances={transfers}
-                onDelete={onDelete}
+                // onDelete={onDelete}
                 rowSelection={rowSelection}
                 rowSelectionHandler={rowSelectionHandler}
                 onLoadMoreAfter={onLoadMoreAfter}
@@ -98,7 +105,7 @@ export function CardTable({
 interface TableProps {
   rowSelection: string[];
   instances: Entity[];
-  onDelete: (id: Entity) => void;
+  // onDelete: (id: Entity) => void;
   rowSelectionHandler: StateUpdater<string[]>;
   onLoadMoreBefore?: () => void;
   onLoadMoreAfter?: () => void;
@@ -107,7 +114,7 @@ interface TableProps {
 function Table({
   instances,
   onLoadMoreAfter,
-  onDelete,
+  // onDelete,
   onLoadMoreBefore,
 }: TableProps): VNode {
   const { i18n } = useTranslationContext();
@@ -116,6 +123,7 @@ function Table({
     <div class="table-container">
       {onLoadMoreBefore && (
         <button
+          type="button"
           class="button is-fullwidth"
           data-tooltip={i18n.str`Load more wire transfers preceding the first one`}
           onClick={onLoadMoreBefore}
@@ -141,7 +149,7 @@ function Table({
             <th>
               <i18n.Translate>Executed on</i18n.Translate>
             </th>
-            <th />
+            {/* <th /> */}
           </tr>
         </thead>
         <tbody>
@@ -162,9 +170,10 @@ function Table({
                         )
                     : i18n.str`unknown`}
                 </td>
-                <td>
+                {/* <td>
                   {i.verified !== true ? (
                     <button
+                      type="button"
                       class="button is-danger is-small has-tooltip-left"
                       data-tooltip={i18n.str`Delete selected transfer from the database`}
                       onClick={() => onDelete(i)}
@@ -172,7 +181,7 @@ function Table({
                       <i18n.Translate>Delete</i18n.Translate>
                     </button>
                   ) : undefined}
-                </td>
+                </td> */}
               </tr>
             );
           })}
@@ -180,6 +189,7 @@ function Table({
       </table>
       {onLoadMoreAfter && (
         <button
+          type="button"
           class="button is-fullwidth"
           data-tooltip={i18n.str`Load more transfers after the last one`}
           onClick={onLoadMoreAfter}
