@@ -19,34 +19,53 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import { assertUnreachable, HttpStatusCode, TalerMerchantApi } from "@gnu-taler/taler-util";
+import {
+  ButtonBetterBulma,
+  LocalNotificationBannerBulma,
+  useLocalNotificationBetter,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
 import { Fragment, h, VNode } from "preact";
 import { useState } from "preact/hooks";
-import { AsyncButton } from "../../../../components/exception/AsyncButton.js";
 import {
   FormErrors,
   FormProvider,
 } from "../../../../components/form/FormProvider.js";
 import { Input } from "../../../../components/form/Input.js";
 import { InputSelector } from "../../../../components/form/InputSelector.js";
-import { assertUnreachable, TalerMerchantApi } from "@gnu-taler/taler-util";
 import { undefinedIfEmpty } from "../../../../utils/table.js";
+import { useSessionContext } from "../../../../context/session.js";
+
+const TALER_SCREEN_ID = 76;
 
 type Entity = TalerMerchantApi.WebhookAddDetails;
 
 interface Props {
-  onCreate: (d: Entity) => Promise<void>;
+  onCreate: () => void;
   onBack?: () => void;
 }
 
-const validType = ["pay", "refund", "order_settled", "category_added", "category_updated", "category_deleted", "inventory_added", "inventory_updated", "inventory_deleted"] as const;
+const validType = [
+  "pay",
+  "refund",
+  "order_settled",
+  "category_added",
+  "category_updated",
+  "category_deleted",
+  "inventory_added",
+  "inventory_updated",
+  "inventory_deleted",
+] as const;
 const validMethod = ["GET", "POST", "PUT", "PATCH", "HEAD"];
 
 export function CreatePage({ onCreate, onBack }: Props): VNode {
   const { i18n } = useTranslationContext();
 
   const [state, setState] = useState<Partial<Entity>>({});
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
 
+  const { state: session, lib } = useSessionContext();
   const errors = undefinedIfEmpty<FormErrors<Entity>>({
     webhook_id: !state.webhook_id ? i18n.str`Required` : undefined,
     event_type: !state.event_type
@@ -69,13 +88,23 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
 
   const hasErrors = errors !== undefined;
 
-  const submitForm = () => {
-    if (hasErrors) return Promise.reject();
-    return onCreate(state as TalerMerchantApi.WebhookAddDetails);
-  };
+  const data = state as TalerMerchantApi.WebhookAddDetails;
+
+  const create = safeFunctionHandler(
+    lib.instance.addWebhook.bind(lib.instance),
+    !session.token || hasErrors ? undefined : [session.token, data],
+  );
+  create.onSuccess = onCreate
+  create.onFail = (fail) =>{
+    switch(fail.case) {
+      case HttpStatusCode.Unauthorized: return i18n.str`Unauthorized.`
+      case HttpStatusCode.NotFound: return i18n.str`Not found.`
+    }
+  }
 
   return (
     <div>
+      <LocalNotificationBannerBulma notification={notification} />
       <section class="section is-main-section">
         <div class="columns">
           <div class="column" />
@@ -104,7 +133,7 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
                     case "refund":
                       return i18n.str`Refund`;
                     case "order_settled":
-                      return i18n.str`Order settled`
+                      return i18n.str`Order settled`;
                     case "category_added":
                       return i18n.str`Category added`;
                     case "category_updated":
@@ -239,21 +268,21 @@ export function CreatePage({ onCreate, onBack }: Props): VNode {
 
             <div class="buttons is-right mt-5">
               {onBack && (
-                <button class="button" onClick={onBack}>
+                <button type="button"class="button" onClick={onBack}>
                   <i18n.Translate>Cancel</i18n.Translate>
                 </button>
               )}
-              <AsyncButton
-                disabled={hasErrors}
+              <ButtonBetterBulma
                 data-tooltip={
                   hasErrors
                     ? i18n.str`Please complete the marked fields`
                     : i18n.str`Confirm operation`
                 }
-                onClick={submitForm}
+                type="submit"
+                onClick={create}
               >
                 <i18n.Translate>Confirm</i18n.Translate>
-              </AsyncButton>
+              </ButtonBetterBulma>
             </div>
           </div>
           <div class="column" />
