@@ -19,16 +19,28 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { TalerMerchantApi } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
-import { h, VNode } from "preact";
+import {
+  AccessToken,
+  HttpStatusCode,
+  TalerMerchantApi,
+} from "@gnu-taler/taler-util";
+import {
+  ButtonBetterBulma,
+  LocalNotificationBannerBulma,
+  SafeHandlerTemplate,
+  useLocalNotificationBetter,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
+import { Fragment, h, VNode } from "preact";
 import { StateUpdater, useState } from "preact/hooks";
+import { useSessionContext } from "../../../../context/session.js";
+
+const TALER_SCREEN_ID = 77;
 
 type Entity = TalerMerchantApi.WebhookEntry;
 
 interface Props {
   webhooks: Entity[];
-  onDelete: (e: Entity) => void;
   onSelect: (e: Entity) => void;
   onCreate: () => void;
   onLoadMoreBefore?: () => void;
@@ -38,7 +50,6 @@ interface Props {
 export function CardTable({
   webhooks,
   onCreate,
-  onDelete,
   onSelect,
   onLoadMoreAfter,
   onLoadMoreBefore,
@@ -47,54 +58,81 @@ export function CardTable({
 
   const { i18n } = useTranslationContext();
 
+  const { state, lib } = useSessionContext();
+
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
+
+  const deleteWebhook = safeFunctionHandler(
+    lib.instance.deleteWebhook.bind(lib.instance),
+  );
+  deleteWebhook.onSuccess = () => i18n.str`Webhook deleted successfully`;
+  deleteWebhook.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unauthorized.`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Not found.`;
+    }
+  };
+  const deleteById = deleteWebhook.lambda((id: string) =>
+    !deleteWebhook.args ? undefined! : [deleteWebhook.args[0], id],
+  );
   return (
-    <div class="card has-table">
-      <header class="card-header">
-        <p class="card-header-title">
-          <span class="icon">
-            <i class="mdi mdi-webhook" />
-          </span>
-          <i18n.Translate>Webhooks</i18n.Translate>
-        </p>
-        <div class="card-header-icon" aria-label="more options">
-          <span
-            class="has-tooltip-left"
-            data-tooltip={i18n.str`Add new webhooks`}
-          >
-            <button class="button is-info" type="button" onClick={onCreate}>
-              <span class="icon is-small">
-                <i class="mdi mdi-plus mdi-36px" />
-              </span>
-            </button>
-          </span>
-        </div>
-      </header>
-      <div class="card-content">
-        <div class="b-table has-pagination">
-          <div class="table-wrapper has-mobile-cards">
-            {webhooks.length > 0 ? (
-              <Table
-                instances={webhooks}
-                onDelete={onDelete}
-                onSelect={onSelect}
-                rowSelection={rowSelection}
-                rowSelectionHandler={rowSelectionHandler}
-                onLoadMoreAfter={onLoadMoreAfter}
-                onLoadMoreBefore={onLoadMoreBefore}
-              />
-            ) : (
-              <EmptyTable />
-            )}
+    <Fragment>
+      <LocalNotificationBannerBulma notification={notification} />
+      <div class="card has-table">
+        <header class="card-header">
+          <p class="card-header-title">
+            <span class="icon">
+              <i class="mdi mdi-webhook" />
+            </span>
+            <i18n.Translate>Webhooks</i18n.Translate>
+          </p>
+          <div class="card-header-icon" aria-label="more options">
+            <span
+              class="has-tooltip-left"
+              data-tooltip={i18n.str`Add new webhooks`}
+            >
+              <button
+                class="button is-info"
+                type="button"
+                onClick={onCreate}
+                accessKey="+"
+              >
+                <span class="icon is-small">
+                  <i class="mdi mdi-plus mdi-36px" />
+                </span>
+              </button>
+            </span>
+          </div>
+        </header>
+        <div class="card-content">
+          <div class="b-table has-pagination">
+            <div class="table-wrapper has-mobile-cards">
+              {webhooks.length > 0 ? (
+                <Table
+                  instances={webhooks}
+                  deleteWebhook={deleteById}
+                  onSelect={onSelect}
+                  rowSelection={rowSelection}
+                  rowSelectionHandler={rowSelectionHandler}
+                  onLoadMoreAfter={onLoadMoreAfter}
+                  onLoadMoreBefore={onLoadMoreBefore}
+                />
+              ) : (
+                <EmptyTable />
+              )}
+            </div>
           </div>
         </div>
       </div>
-    </div>
+    </Fragment>
   );
 }
 interface TableProps {
   rowSelection: string[];
   instances: Entity[];
-  onDelete: (e: Entity) => void;
+  deleteWebhook: SafeHandlerTemplate<[id: string], any>;
   onSelect: (e: Entity) => void;
   rowSelectionHandler: StateUpdater<string[]>;
   onLoadMoreBefore?: () => void;
@@ -104,7 +142,7 @@ interface TableProps {
 function Table({
   instances,
   onLoadMoreAfter,
-  onDelete,
+  deleteWebhook,
   onSelect,
   onLoadMoreBefore,
 }: TableProps): VNode {
@@ -113,6 +151,7 @@ function Table({
     <div class="table-container">
       {onLoadMoreBefore && (
         <button
+          type="button"
           class="button is-fullwidth"
           data-tooltip={i18n.str`Load more webhooks before the first one`}
           onClick={onLoadMoreBefore}
@@ -150,13 +189,13 @@ function Table({
                 </td>
                 <td class="is-actions-cell right-sticky">
                   <div class="buttons is-right">
-                    <button
+                    <ButtonBetterBulma
                       class="button is-danger is-small has-tooltip-left"
                       data-tooltip={i18n.str`Delete selected webhook from the database`}
-                      onClick={() => onDelete(i)}
+                      onClick={deleteWebhook.withArgs(i.webhook_id)}
                     >
                       <i18n.Translate>Delete</i18n.Translate>
-                    </button>
+                    </ButtonBetterBulma>
                   </div>
                 </td>
               </tr>
@@ -166,6 +205,7 @@ function Table({
       </table>
       {onLoadMoreAfter && (
         <button
+          type="button"
           class="button is-fullwidth"
           data-tooltip={i18n.str`Load more webhooks after the last one`}
           onClick={onLoadMoreAfter}
