@@ -19,24 +19,35 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { assertUnreachable, TalerMerchantApi } from "@gnu-taler/taler-util";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
+import {
+  assertUnreachable,
+  HttpStatusCode,
+  TalerMerchantApi,
+} from "@gnu-taler/taler-util";
+import {
+  ButtonBetterBulma,
+  LocalNotificationBannerBulma,
+  useLocalNotificationBetter,
+  useTranslationContext,
+} from "@gnu-taler/web-util/browser";
 import { h, VNode } from "preact";
 import { useState } from "preact/hooks";
-import { AsyncButton } from "../../../../components/exception/AsyncButton.js";
 import {
   FormErrors,
   FormProvider,
 } from "../../../../components/form/FormProvider.js";
 import { Input } from "../../../../components/form/Input.js";
+import { InputSelector } from "../../../../components/form/InputSelector.js";
 import { WithId } from "../../../../declaration.js";
 import { undefinedIfEmpty } from "../../../../utils/table.js";
-import { InputSelector } from "../../../../components/form/InputSelector.js";
+import { useSessionContext } from "../../../../context/session.js";
+
+const TALER_SCREEN_ID = 78;
 
 type Entity = TalerMerchantApi.WebhookPatchDetails & WithId;
 
 interface Props {
-  onUpdate: (d: Entity) => Promise<void>;
+  onConfirm: () => void;
   onBack?: () => void;
   webhook: Entity;
 }
@@ -53,9 +64,12 @@ const validType = [
 ] as const;
 const validMethod = ["GET", "POST", "PUT", "PATCH", "HEAD"];
 
-export function UpdatePage({ webhook, onUpdate, onBack }: Props): VNode {
+export function UpdatePage({ webhook, onConfirm, onBack }: Props): VNode {
   const { i18n } = useTranslationContext();
 
+  const { state: session, lib, logIn } = useSessionContext();
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
+
   const [state, setState] = useState<Partial<Entity>>(webhook);
 
   const errors = undefinedIfEmpty<FormErrors<Entity>>({
@@ -79,13 +93,29 @@ export function UpdatePage({ webhook, onUpdate, onBack }: Props): VNode {
 
   const hasErrors = errors !== undefined;
 
-  const submitForm = () => {
-    if (hasErrors) return Promise.reject();
-    return onUpdate(state as Entity);
+  const data = state as Entity;
+  const update = safeFunctionHandler(
+    lib.instance.updateWebhook.bind(lib.instance),
+    !session.token || !!errors ? undefined : [session.token, webhook.id, data],
+  );
+  update.onSuccess = (success) => {
+    onConfirm();
+    return i18n.str`Webhook updated`;
+  };
+  update.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unauthorized.`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Not found.`;
+      case HttpStatusCode.Conflict:
+        return i18n.str`Conflict.`;
+    }
   };
 
   return (
     <div>
+      <LocalNotificationBannerBulma notification={notification} />
       <section class="section">
         <section class="hero is-hero-bar">
           <div class="hero-body">
@@ -182,21 +212,21 @@ export function UpdatePage({ webhook, onUpdate, onBack }: Props): VNode {
 
               <div class="buttons is-right mt-5">
                 {onBack && (
-                  <button class="button" onClick={onBack}>
+                  <button type="button" class="button" onClick={onBack}>
                     <i18n.Translate>Cancel</i18n.Translate>
                   </button>
                 )}
-                <AsyncButton
-                  disabled={hasErrors}
+                <ButtonBetterBulma
                   data-tooltip={
                     hasErrors
                       ? i18n.str`Please complete the marked fields`
                       : i18n.str`Confirm operation`
                   }
-                  onClick={submitForm}
+                  onClick={update}
+                  type="submit"
                 >
                   <i18n.Translate>Confirm</i18n.Translate>
-                </AsyncButton>
+                </ButtonBetterBulma>
               </div>
             </div>
           </div>
