@@ -20,25 +20,25 @@
  */
 
 import {
-  assertUnreachable,
   Duration,
   HttpStatusCode,
   LoginTokenRequest,
   LoginTokenScope,
-  TalerError,
   TranslatedString,
 } from "@gnu-taler/taler-util";
 import {
+  ButtonBetterBulma,
+  LocalNotificationBannerBulma,
   useChallengeHandler,
+  useLocalNotificationBetter,
   useTranslationContext,
 } from "@gnu-taler/web-util/browser";
 import { Fragment, h, VNode } from "preact";
 import { useState } from "preact/hooks";
-import { AsyncButton } from "../../components/exception/AsyncButton.js";
-import { NotificationCard } from "../../components/menu/index.js";
 import { SolveMFAChallenges } from "../../components/SolveMFA.js";
 import { useSessionContext } from "../../context/session.js";
-import { Notification } from "../../utils/types.js";
+
+const TALER_SCREEN_ID = 79;
 
 interface Props {
   showCreateAccount?: boolean;
@@ -61,76 +61,54 @@ export const FOREVER_REFRESHABLE_TOKEN = (description: TranslatedString) =>
 const VERSION = typeof __VERSION__ !== "undefined" ? __VERSION__ : undefined;
 export function LoginPage({ showCreateAccount }: Props): VNode {
   const [password, setPassword] = useState("");
-  const [notif, setNotif] = useState<Notification | undefined>(undefined);
+
   const { state, logIn, getInstanceForUsername, config } = useSessionContext();
-  const [username, setUsername] = useState(showCreateAccount? "" : state.instance);
+  const [username, setUsername] = useState(
+    showCreateAccount ? "" : state.instance,
+  );
 
   const { i18n } = useTranslationContext();
 
+  const [notification, safeFunctionHandler] = useLocalNotificationBetter();
   const mfa = useChallengeHandler();
-  const [doLogin, repeatLogin] = mfa.withMfaHandler(
-    ({ challengeIds, onChallengeRequired }) =>
-      async () => {
-        const api = getInstanceForUsername(username);
 
-        try {
-          const result = await api.createAccessToken(
-            username,
-            password,
-            FOREVER_REFRESHABLE_TOKEN(i18n.str`Logged in`),
-            {
-              challengeIds,
-            },
-          );
-          if (result.type === "ok") {
-            const { access_token: token } = result.body;
-            logIn(username, token);
-            return;
-          } else {
-            switch (result.case) {
-              case HttpStatusCode.Unauthorized: {
-                setNotif({
-                  message: i18n.str`Your password is incorrect`,
-                  type: "ERROR",
-                });
-                return;
-              }
-              case HttpStatusCode.NotFound: {
-                setNotif({
-                  message: i18n.str`Your instance cannot be found`,
-                  type: "ERROR",
-                });
-                return;
-              }
-              case HttpStatusCode.Accepted: {
-                onChallengeRequired(result.body);
-                return;
-              }
-              default: {
-                assertUnreachable(result);
-              }
-            }
-          }
-        } catch (error) {
-          const details =
-            error instanceof TalerError
-              ? JSON.stringify(error.errorDetail)
-              : undefined;
-          setNotif({
-            message: i18n.str`Failed to login.`,
-            type: "ERROR",
-            description: error instanceof Error ? error.message : undefined,
-            details,
-          });
-        }
-      },
+  const login = safeFunctionHandler(
+    (usr: string, pwd: string, challengeIds: string[]) => {
+      const api = getInstanceForUsername(username);
+      return api.createAccessToken(
+        usr,
+        pwd,
+        FOREVER_REFRESHABLE_TOKEN(i18n.str`Logged in`),
+        { challengeIds },
+      );
+    },
+    !username || !password ? undefined : [username, password, []],
   );
+  login.onSuccess = (success, usr) => {
+    logIn(usr, success.access_token);
+  };
+  login.onFail = (fail) => {
+    switch (fail.case) {
+      case HttpStatusCode.Accepted:
+        mfa.onChallengeRequired(fail.body);
+        return i18n.str`Second factor authorization required.`;
+      case HttpStatusCode.Unauthorized:
+        return i18n.str`Unauthorized`;
+      case HttpStatusCode.NotFound:
+        return i18n.str`Not found`;
+    }
+  };
+  const retry = login.lambda((ids: string[]) => [
+    login.args![0],
+    login.args![1],
+    ids,
+  ]);
 
   if (mfa.pendingChallenge) {
     return (
       <SolveMFAChallenges
         currentChallenge={mfa.pendingChallenge}
-        onCompleted={repeatLogin}
+        onCompleted={retry}
         onCancel={mfa.doCancelChallenge}
       />
     );
@@ -138,7 +116,7 @@ export function LoginPage({ showCreateAccount }: Props): VNode {
 
   return (
     <Fragment>
-      <NotificationCard notification={notif} />
+      <LocalNotificationBannerBulma notification={notification} />
       <div class="columns is-centered" style={{ margin: "auto" }}>
         <div class="column is-two-thirds ">
           <div class="modal-card" style={{ width: "100%", margin: 0 }}>
@@ -154,10 +132,6 @@ export function LoginPage({ showCreateAccount }: Props): VNode {
               class="modal-card-body"
               style={{ border: "1px solid", borderTop: 0, borderBottom: 0 }}
             >
-              <i18n.Translate>
-                Please enter your password for <b>"{state.instance}"</b>.
-              </i18n.Translate>
-
               <div class="field is-horizontal">
                 <div class="field-label is-normal">
                   <label class="label">
@@ -173,7 +147,7 @@ export function LoginPage({ showCreateAccount }: Props): VNode {
                         placeholder={"instance name"}
                         name="username"
                         onKeyPress={(e) =>
-                          e.keyCode === 13 ? doLogin() : null
+                          e.keyCode === 13 ? login.call() : null
                         }
                         value={username}
                         onInput={(e): void =>
@@ -199,7 +173,7 @@ export function LoginPage({ showCreateAccount }: Props): VNode {
                         placeholder={"current password"}
                         name="token"
                         onKeyPress={(e) =>
-                          e.keyCode === 13 ? doLogin() : null
+                          e.keyCode === 13 ? login.call() : null
                         }
                         value={password}
                         onInput={(e): void =>
@@ -234,9 +208,9 @@ export function LoginPage({ showCreateAccount }: Props): VNode {
                   <i18n.Translate>Forgot password</i18n.Translate>
                 </a>
               )}
-              <AsyncButton disabled={!username || !password} onClick={doLogin}>
+              <ButtonBetterBulma onClick={login} type="submit">
                 <i18n.Translate>Confirm</i18n.Translate>
-              </AsyncButton>
+              </ButtonBetterBulma>
             </footer>
           </div>
           {!showCreateAccount ? undefined : (
