@@ -185,21 +185,34 @@ function processTaggedTemplateExpression(
   return res;
 }
 
-const SCREEN_ID_MAP: Record<string, Set<string>> = {};
-const MISSING_SCREEN_ID: Set<string> = new Set();
+type ScreenInfo = {
+  fileMap: Record<string, Set<string>>;
+  missing: Set<String>;
+  maxId: number;
+};
+const SCREEN_INFO: ScreenInfo = {
+  fileMap: {},
+  missing: new Set(),
+  maxId: 0,
+};
 
 function formatScreenId(
   sourceFile: ts.SourceFile,
   outChunks: string[],
   screenId: string,
 ) {
-  if (!screenId) {
-    MISSING_SCREEN_ID.add(sourceFile.fileName);
+  const screen = Number.parseInt(screenId, 10);
+  if (!screen || Number.isNaN(screen)) {
+    SCREEN_INFO.missing.add(sourceFile.fileName);
   } else {
-    if (!SCREEN_ID_MAP[screenId]) {
-      SCREEN_ID_MAP[screenId] = new Set();
+    if (!SCREEN_INFO.fileMap[screenId]) {
+      SCREEN_INFO.fileMap[screenId] = new Set();
+    }
+    SCREEN_INFO.fileMap[screenId].add(sourceFile.fileName);
+
+    if (SCREEN_INFO.maxId < screen) {
+      SCREEN_INFO.maxId = screen;
     }
-    SCREEN_ID_MAP[screenId].add(sourceFile.fileName);
 
     outChunks.push(`#. screenid: ${screenId}\n`);
   }
@@ -430,9 +443,15 @@ function processNode(
   switch (node.kind) {
     case ts.SyntaxKind.JsxElement:
       let path = getJsxElementPath(node);
+      // <i18n.Translate>text</i18n.Translate>
       if (arrayEq(path, ["i18n", "Translate"])) {
         const content = getJsxContent(node);
         const { line } = ts.getLineAndCharacterOfPosition(sourceFile, node.pos);
+        if (!content) {
+          throw Error(
+            `string to be translated can't be empty: ${sourceFile.fileName}:${line}`,
+          );
+        }
         const comment = getComment(
           sourceFile,
           preLastTokLine,
@@ -454,27 +473,34 @@ function processNode(
         return;
       }
       if (arrayEq(path, ["i18n", "TranslateSwitch"])) {
-        let { line } = ts.getLineAndCharacterOfPosition(sourceFile, node.pos);
-        let comment = getComment(sourceFile, preLastTokLine, lastTokLine, node);
+        const { line } = ts.getLineAndCharacterOfPosition(sourceFile, node.pos);
+        const comment = getComment(
+          sourceFile,
+          preLastTokLine,
+          lastTokLine,
+          node,
+        );
         formatMsgComment(projectPrefix, sourceFile, outChunks, line, comment);
-        let singularForm = getJsxSingular(node);
-        if (!singularForm) {
-          console.error("singular form missing");
-          process.exit(1);
+        const content = getJsxSingular(node);
+        if (!content) {
+          throw Error(
+            `string to be translated can't be empty, singular is missing: ${sourceFile.fileName}:${line}`,
+          );
         }
-        let pluralForm = getJsxPlural(node);
+        const pluralForm = getJsxPlural(node);
         if (!pluralForm) {
-          console.error("plural form missing");
-          process.exit(1);
+          throw Error(
+            `string to be translated can't be empty, plural is missing: ${sourceFile.fileName}:${line}`,
+          );
         }
         const context = getJsxAttribute(sourceFile, node)["context"] ?? "";
-        const msgid = context + singularForm;
+        const msgid = context + content;
         if (!knownMessageIds.has(msgid)) {
           knownMessageIds.add(msgid);
           const screenId = searchScreenId(parents, sourceFile);
           formatScreenId(sourceFile, outChunks, screenId);
           formatMsgLine(outChunks, "msgctxt", context);
-          formatMsgLine(outChunks, "msgid", singularForm);
+          formatMsgLine(outChunks, "msgid", content);
           formatMsgLine(outChunks, "msgid_plural", pluralForm);
           outChunks.push(`msgstr[0] ""\n`);
           outChunks.push(`msgstr[1] ""\n`);
@@ -503,12 +529,18 @@ function processNode(
         preLastTokLine = lastTokLine; // HERE
         lastTokLine = lc1.line;
       }
-      let t1 = processTaggedTemplateExpression(
+      const t1 = processTaggedTemplateExpression(
         sourceFile,
         preLastTokLine,
         lastTokLine,
         tte1,
       );
+      const content = t1.template;
+      if (!content) {
+        throw Error(
+          `string to be translated can't be empty: ${sourceFile.fileName}:${line}`,
+        );
+      }
 
       const tte2 = <ts.TaggedTemplateExpression>ce.arguments[1];
       let lc2 = ts.getLineAndCharacterOfPosition(sourceFile, tte2.pos);
@@ -523,14 +555,14 @@ function processNode(
         tte2,
       );
       let comment = getComment(sourceFile, preLastTokLine, lastTokLine, ce);
-      const msgid = path.ctx + t1.template;
+      const msgid = path.ctx + content;
       if (!knownMessageIds.has(msgid)) {
         knownMessageIds.add(msgid);
         const screenId = searchScreenId(parents, sourceFile);
         formatScreenId(sourceFile, outChunks, screenId);
         formatMsgComment(projectPrefix, sourceFile, outChunks, line, comment);
         formatMsgLine(outChunks, "msgctxt", path.ctx);
-        formatMsgLine(outChunks, "msgid", t1.template);
+        formatMsgLine(outChunks, "msgid", content);
         formatMsgLine(outChunks, "msgid_plural", t2.template);
         outChunks.push(`msgstr[0] ""\n`);
         outChunks.push(`msgstr[1] ""\n`);
@@ -547,7 +579,7 @@ function processNode(
         preLastTokLine = lastTokLine;
         lastTokLine = lc2.line;
       }
-      let { comment, template, line, path, context } =
+      const { comment, template, line, path, context } =
         processTaggedTemplateExpression(
           sourceFile,
           preLastTokLine,
@@ -557,6 +589,11 @@ function processNode(
       if (path[0] != "i18n") {
         break;
       }
+      if (!template) {
+        throw Error(
+          `string to be translated can't be empty: ${sourceFile.fileName}:${line}`,
+        );
+      }
       const msgid = context + template;
       if (!knownMessageIds.has(msgid)) {
         knownMessageIds.add(msgid);
@@ -679,19 +716,31 @@ export function potextract() {
     processFile(f, chunks, knownMessageIds, gitRoot);
   }
 
-  MISSING_SCREEN_ID.forEach((fileName) => {
-    console.error(`missing SCREEN_ID for file: ${fileName}`);
-  });
-
-  Object.entries(SCREEN_ID_MAP).forEach(([screenId, files]) => {
-    if (files.size > 1) {
-      console.error(
-        `SCREEN_ID ${screenId} is in multiple files: \n * ${[...files].join(
-          "\n * ",
-        )}`,
-      );
-    }
-  });
+  if (SCREEN_INFO.missing.size) {
+    console.error(
+      `There are some files with translation strings that doesnt have a constant TALER_SCREEN_ID on the root. This constant should be a unquie integer number to facilitate the identification from other locations. Add a "const TALER_SCREEN_ID = <number>;" after the last import. It doesn't need to be exported and it can be removed from runtime. Files:`,
+    );
+    SCREEN_INFO.missing.forEach((fileName) => {
+      console.error(` * ${fileName}`);
+    });
+  }
+  const haveRepetition =
+    Object.values(SCREEN_INFO.fileMap).find((files) => files.size > 1) !==
+    undefined;
+  if (haveRepetition) {
+    console.error(
+      `There are more than one file with the same TALER_SCREEN_ID, the current max number is "${SCREEN_INFO.maxId}". Repeated ids:`,
+    );
+    Object.entries(SCREEN_INFO.fileMap).forEach(([screenId, files]) => {
+      if (files.size > 1) {
+        console.error(
+          `const TALER_SCREEN_ID = ${screenId};\n * ${[...files].join(
+            "\n * ",
+          )}`,
+        );
+      }
+    });
+  }
 
   const pot = chunks.join("");
 
