@@ -275,18 +275,16 @@ export async function useSharedTestkudosEnvironment(t: GlobalTestState) {
     await merchant.dbinit();
   }
 
-  const bankStart = async () => {
-    await bank.start();
-    await bank.pingUntilAvailable();
-  };
+  await bank.start();
 
-  const exchangeStart = async () => {
-    await exchange.start({
-      skipDbinit: true,
-      skipKeyup: prevSetupDone,
-    });
-    await exchange.pingUntilAvailable();
-  };
+  // We *must* wait for the exchange to be started.
+  // Otherwise the merchant might be able to see
+  // partial /keys (e.g. with missing wire fees),
+  // leading to flaky tests.
+  await exchange.start({
+    skipDbinit: true,
+    skipKeyup: prevSetupDone,
+  });
 
   const merchStart = async () => {
     await merchant.start({
@@ -331,18 +329,11 @@ export async function useSharedTestkudosEnvironment(t: GlobalTestState) {
     return { merchantAdminAccessToken };
   };
 
-  await bankStart();
+  const res = await Promise.all([merchStart(), walletStartProm]);
 
-  const res = await Promise.all([
-    exchangeStart(),
-    merchStart(),
-    undefined,
-    walletStartProm,
-  ]);
-
-  const merchantAdminAccessToken = res[1].merchantAdminAccessToken;
-  const walletClient = res[3].walletClient;
-  const walletService = res[3].walletService;
+  const merchantAdminAccessToken = res[0].merchantAdminAccessToken;
+  const walletClient = res[1].walletClient;
+  const walletService = res[1].walletService;
 
   fs.writeFileSync(sharedDir + "/setup-done", "OK");
 
