@@ -1769,7 +1769,6 @@ export class ExchangeService implements ExchangeServiceInterface {
     await sh(this.globalState, "rm-secmod-keys", `rm ${eddsaKeydir}/*`);
   }
 
-
   /**
    * Purge denom signing keys.
    * Keeps message signing keys (eddsa) in place.
@@ -2316,7 +2315,7 @@ export class MerchantService implements MerchantServiceInterface {
         ),
       default_pay_delay:
         instanceConfig.defaultPayDelay ??
-        Duration.toTalerProtocolDuration(Duration.getForever()),
+        Duration.toTalerProtocolDuration(Duration.fromSpec({ days: 1 })),
     };
     const headers: Record<string, string> = {};
     if (adminAccessToken) {
