@@ -20,13 +20,13 @@
 import { AmountString, TalerMerchantApi } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { defaultCoinConfig } from "../harness/denomStructures.js";
-import { GlobalTestState } from "../harness/harness.js";
 import {
   createSimpleTestkudosEnvironmentV3,
   createWalletDaemonWithClient,
   makeTestPaymentV2,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
+import { GlobalTestState } from "../harness/harness.js";
 
 /**
  * Run test for basic, bank-integrated withdrawal and payment.
@@ -38,7 +38,7 @@ export async function runAgeRestrictionsMixedMerchantTest(t: GlobalTestState) {
     walletClient: walletOne,
     bankClient,
     exchange,
-    merchant,
+    merchant, merchantAdminAccessToken,
   } = await createSimpleTestkudosEnvironmentV3(
     t,
     defaultCoinConfig.map((x) => x("TESTKUDOS")),
@@ -76,7 +76,7 @@ export async function runAgeRestrictionsMixedMerchantTest(t: GlobalTestState) {
       minimum_age: 9,
     };
 
-    await makeTestPaymentV2(t, { walletClient, merchant, order });
+    await makeTestPaymentV2(t, { walletClient, merchant, order, merchantAdminAccessToken });
     await walletClient.call(
       WalletApiOperation.TestingWaitTransactionsFinal,
       {},
@@ -100,7 +100,7 @@ export async function runAgeRestrictionsMixedMerchantTest(t: GlobalTestState) {
       fulfillment_url: "taler://fulfillment-success/thx",
     };
 
-    await makeTestPaymentV2(t, { walletClient: walletTwo, merchant, order });
+    await makeTestPaymentV2(t, { walletClient: walletTwo, merchant, order, merchantAdminAccessToken });
     await walletTwo.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
   }
 
@@ -121,9 +121,10 @@ export async function runAgeRestrictionsMixedMerchantTest(t: GlobalTestState) {
       minimum_age: 9,
     };
 
-    await makeTestPaymentV2(t, { walletClient: walletThree, merchant, order });
+    await makeTestPaymentV2(t, { walletClient: walletThree, merchant, order, merchantAdminAccessToken });
     await walletThree.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
   }
 }
 
 runAgeRestrictionsMixedMerchantTest.suites = ["wallet"];
+runAgeRestrictionsMixedMerchantTest.experimental = true;
