@@ -18,23 +18,25 @@
  * Imports.
  */
 import {
-  MerchantApiClient,
+  Logger,
   PreparePayResultType,
+  succeedOrThrow,
   TalerErrorCode,
-  TalerMerchantApi,
-  TransactionType,
+  TalerMerchantInstanceHttpClient,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { GlobalTestState } from "../harness/harness.js";
 import {
   createSimpleTestkudosEnvironmentV3,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
+import { GlobalTestState } from "../harness/harness.js";
+
+const logger = new Logger("test-denom-unoffered");
 
 export async function runDenomUnofferedTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bankClient, exchange, merchant } =
+  const { walletClient, bankClient, exchange, merchant, merchantAdminAccessToken } =
     await createSimpleTestkudosEnvironmentV3(t);
 
   // Withdraw digital cash into the wallet.
@@ -48,6 +50,10 @@ export async function runDenomUnofferedTest(t: GlobalTestState) {
 
   await wres.withdrawalFinishedCond;
 
+  const merchantClient = new TalerMerchantInstanceHttpClient(
+    merchant.makeInstanceBaseUrl(),
+  );
+
   // Make the exchange forget the denomination.
   // Effectively we completely reset the exchange,
   // but keep the exchange master public key.
@@ -63,21 +69,21 @@ export async function runDenomUnofferedTest(t: GlobalTestState) {
   await merchant.start();
   await merchant.pingUntilAvailable();
 
-  const order: TalerMerchantApi.Order = {
-    summary: "Buy me!",
-    amount: "TESTKUDOS:5",
-    fulfillment_url: "taler://fulfillment-success/thx",
-  };
+  t.logStep("services restarted");
 
-  const merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl());
+  const orderResp = succeedOrThrow(
+    await merchantClient.createOrder(merchantAdminAccessToken, {
+      order: {
+        summary: "Buy me!",
+        amount: "TESTKUDOS:5",
+        fulfillment_url: "taler://fulfillment-success/thx",
+      },
+    }),
+  );
 
-  const orderResp = await merchantClient.createOrder({
-    order: order,
-  });
-
-  let orderStatus = await merchantClient.queryPrivateOrderStatus({
-    orderId: orderResp.order_id,
-  });
+  let orderStatus = succeedOrThrow(
+    await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id),
+  );
 
   t.assertTrue(orderStatus.order_status === "unpaid");
 
@@ -94,71 +100,60 @@ export async function runDenomUnofferedTest(t: GlobalTestState) {
     preparePayResult.status === PreparePayResultType.PaymentPossible,
   );
 
+  t.logStep("order prepared");
+
   const confirmResp = await walletClient.call(WalletApiOperation.ConfirmPay, {
     transactionId: preparePayResult.transactionId,
   });
 
+  t.logStep("order confirmed");
+
+  // Currently, the payment transaction will be aborted.
+  // In the future, the wallet should also do coin re-selection
+  // when a denomination is lost.  But we might want to ask the user
+  // first, since they lost money.
+  await walletClient.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
+
   const tx = await walletClient.call(WalletApiOperation.GetTransactionById, {
     transactionId: confirmResp.transactionId,
   });
 
-  t.assertTrue(tx.error != null);
-  t.assertTrue(
-    tx.error.code === TalerErrorCode.WALLET_PAY_MERCHANT_SERVER_ERROR,
-  );
-
-  const merchantErrorCode = (tx.error as any).requestError.errorResponse.code;
-
   t.assertDeepEqual(
-    merchantErrorCode,
+    (tx.abortReason?.errorDetails as any)?.code,
     TalerErrorCode.MERCHANT_GENERIC_EXCHANGE_UNEXPECTED_STATUS,
   );
 
-  const exchangeErrorCode = (tx.error as any).requestError.errorResponse
-    .exchange_ec;
-
-  t.assertDeepEqual(
-    exchangeErrorCode,
-    TalerErrorCode.EXCHANGE_GENERIC_DENOMINATION_KEY_UNKNOWN,
-  );
-
-  // Depending on whether the merchant has seen the new denominations or not,
-  // the error code might be different here.
-  // t.assertDeepEqual(
-  //   merchantErrorCode,
-  //   TalerErrorCode.MERCHANT_POST_ORDERS_ID_PAY_DENOMINATION_KEY_NOT_FOUND,
-  // );
-
   // Force updating the exchange entry so that the wallet knows about the new denominations.
   await walletClient.call(WalletApiOperation.UpdateExchangeEntry, {
     exchangeBaseUrl: exchange.baseUrl,
     force: true,
   });
 
-  await walletClient.call(WalletApiOperation.DeleteTransaction, {
-    transactionId: confirmResp.transactionId,
-  });
+  logger.info(`withdrawing (with new denoms)`);
 
   // Now withdrawal should work again.
-  await withdrawViaBankV3(t, {
+  const wres2 = await withdrawViaBankV3(t, {
     walletClient,
     bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
 
-  await walletClient.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
+  await wres2.withdrawalFinishedCond;
 
   const txs = await walletClient.call(WalletApiOperation.GetTransactions, {
     sort: "stable-ascending",
-    includeRefreshes: true,
   });
   console.log(JSON.stringify(txs, undefined, 2));
 
-  t.assertDeepEqual(txs.transactions[0].type, TransactionType.Withdrawal);
-  t.assertDeepEqual(txs.transactions[1].type, TransactionType.Refresh);
-  t.assertDeepEqual(txs.transactions[2].type, TransactionType.DenomLoss);
-  t.assertDeepEqual(txs.transactions[3].type, TransactionType.Withdrawal);
+  const txTypes = txs.transactions.map((x) => x.type);
+
+  t.assertDeepEqual(txTypes, [
+    "withdrawal",
+    "payment",
+    "denom-loss",
+    "withdrawal",
+  ]);
 }
 
 runDenomUnofferedTest.suites = ["wallet"];
