@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2020 Taler Systems S.A.
+ (C) 2020-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -31,12 +31,12 @@ import {
   FaultInjectionResponseContext,
 } from "../harness/faultInjection.js";
 import {
-    BankService,
+  BankService,
   ExchangeService,
   GlobalTestState,
   MerchantService,
   WalletCli,
-  generateRandomPayto,
+  getTestHarnessPaytoForLabel,
   setupDb,
 } from "../harness/harness.js";
 
@@ -66,20 +66,24 @@ export async function runExchangeManagementFaultTest(
 
   const merchant = await MerchantService.create(t, {
     name: "testmerchant-1",
-    currency: "TESTKUDOS",
     httpPort: 8083,
     database: db.connStr,
   });
 
   let receiverName = "Exchange";
   let exchangeBankUsername = "exchange";
-  let exchangeBankPassword = "mypw";
-  let exchangePaytoUri = generateRandomPayto(exchangeBankUsername);
+  let exchangeBankPassword = "mypw-password";
+  let exchangePaytoUri = getTestHarnessPaytoForLabel(exchangeBankUsername);
 
   await exchange.addBankAccount("1", {
-    accountName: exchangeBankUsername,
-    accountPassword: exchangeBankPassword,
-    wireGatewayApiBaseUrl: new URL("accounts/exchange/taler-wire-gateway/", bank.baseUrl).href,
+    wireGatewayAuth: {
+      username: exchangeBankUsername,
+      password: exchangeBankPassword,
+    },
+    wireGatewayApiBaseUrl: new URL(
+      "accounts/exchange/taler-wire-gateway/",
+      bank.baseUrl,
+    ).href,
     accountPaytoUri: exchangePaytoUri,
   });
 
@@ -89,10 +93,7 @@ export async function runExchangeManagementFaultTest(
     config.setString("exchange", "base_url", "http://localhost:8091/");
   });
 
-  bank.setSuggestedExchange(
-    faultyExchange,
-    exchangePaytoUri,
-  );
+  bank.setSuggestedExchange(faultyExchange, exchangePaytoUri);
 
   await bank.start();
 
@@ -108,17 +109,17 @@ export async function runExchangeManagementFaultTest(
   await merchant.start();
   await merchant.pingUntilAvailable();
 
-  await merchant.addInstanceWithWireAccount({
-    id: "default",
+  const { accessToken: adminAccessToken } = await merchant.addInstanceWithWireAccount({
+    id: "admin",
     name: "Default Instance",
-    paytoUris: [generateRandomPayto("merchant-default")],
+    paytoUris: [getTestHarnessPaytoForLabel("merchant-default")],
   });
 
   await merchant.addInstanceWithWireAccount({
     id: "minst1",
     name: "minst1",
-    paytoUris: [generateRandomPayto("minst1")],
-  });
+    paytoUris: [getTestHarnessPaytoForLabel("minst1")],
+  }, {adminAccessToken});
 
   console.log("setup done!");
 
@@ -190,11 +191,11 @@ export async function runExchangeManagementFaultTest(
     },
   });
 
-  const err1 = await t.assertThrowsTalerErrorAsync(async () => {
-    await wallet.client.call(WalletApiOperation.AddExchange, {
+  const err1 = await t.assertThrowsTalerErrorAsyncLegacy(
+    wallet.client.call(WalletApiOperation.AddExchange, {
       exchangeBaseUrl: faultyExchange.baseUrl,
-    });
-  });
+    }),
+  );
 
   console.log("got error", err1);
 
@@ -238,11 +239,11 @@ export async function runExchangeManagementFaultTest(
     },
   });
 
-  const err2 = await t.assertThrowsTalerErrorAsync(async () => {
-    await wallet.client.call(WalletApiOperation.AddExchange, {
+  const err2 = await t.assertThrowsTalerErrorAsyncLegacy(
+    wallet.client.call(WalletApiOperation.AddExchange, {
       exchangeBaseUrl: faultyExchange.baseUrl,
-    });
-  });
+    }),
+  );
 
   t.assertTrue(err2.hasErrorCode(TalerErrorCode.WALLET_EXCHANGE_UNAVAILABLE));
 
@@ -271,7 +272,7 @@ export async function runExchangeManagementFaultTest(
   const bankClient = new TalerCorebankApiClient(bank.corebankApiBaseUrl, {
     auth: {
       username: "admin",
-      password: "adminpw",
+      password: "admin-password",
     },
   });
 
@@ -300,3 +301,4 @@ export async function runExchangeManagementFaultTest(
 }
 
 runExchangeManagementFaultTest.suites = ["wallet", "exchange"];
+runExchangeManagementFaultTest.experimental = true;
