@@ -22,20 +22,21 @@ import {
   TalerMerchantApi,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import {
-  BankService,
-  ExchangeService,
-  GlobalTestState,
-  MerchantService,
-  generateRandomPayto,
-  setupDb,
-} from "../harness/harness.js";
 import {
   SimpleTestEnvironmentNg3,
   createWalletDaemonWithClient,
   makeTestPaymentV2,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
+import {
+  BankService,
+  ExchangeService,
+  GlobalTestState,
+  HarnessExchangeBankAccount,
+  MerchantService,
+  getTestHarnessPaytoForLabel,
+  setupDb,
+} from "../harness/harness.js";
 
 /**
  * Run a test case with a simple TESTKUDOS Taler environment, consisting
@@ -62,25 +63,28 @@ export async function createMyTestkudosEnvironment(
 
   const merchant = await MerchantService.create(t, {
     name: "testmerchant-1",
-    currency: "TESTKUDOS",
     httpPort: 8083,
     database: db.connStr,
   });
 
   let receiverName = "Exchange";
   let exchangeBankUsername = "exchange";
-  let exchangeBankPassword = "mypw";
-  let exchangePaytoUri = generateRandomPayto(exchangeBankUsername);
+  let exchangeBankPassword = "mypw-password";
+  let exchangePaytoUri = getTestHarnessPaytoForLabel(exchangeBankUsername);
 
-  await exchange.addBankAccount("1", {
-    accountName: exchangeBankUsername,
-    accountPassword: exchangeBankPassword,
+  const exchangeBankAccount: HarnessExchangeBankAccount = {
+    wireGatewayAuth: {
+      password: exchangeBankPassword,
+      username: exchangeBankUsername,
+    },
+    accountPaytoUri: exchangePaytoUri,
     wireGatewayApiBaseUrl: new URL(
       "accounts/exchange/taler-wire-gateway/",
       bank.baseUrl,
     ).href,
-    accountPaytoUri: exchangePaytoUri,
-  });
+  };
+
+  await exchange.addBankAccount("1", exchangeBankAccount);
 
   bank.setSuggestedExchange(exchange, exchangePaytoUri);
 
@@ -91,7 +95,7 @@ export async function createMyTestkudosEnvironment(
   const bankClient = new TalerCorebankApiClient(bank.corebankApiBaseUrl, {
     auth: {
       username: "admin",
-      password: "adminpw",
+      password: "admin-password",
     },
   });
 
@@ -166,11 +170,13 @@ export async function createMyTestkudosEnvironment(
   await merchant.start();
   await merchant.pingUntilAvailable();
 
-  await merchant.addDefaultInstance();
+  const {accessToken: adminAccessToken} = await merchant.addDefaultInstance();
   await merchant.addInstanceWithWireAccount({
     id: "minst1",
     name: "minst1",
-    paytoUris: [generateRandomPayto("minst1")],
+    paytoUris: [getTestHarnessPaytoForLabel("minst1")],
+  }, {
+    adminAccessToken
   });
 
   console.log("setup done!");
@@ -186,15 +192,12 @@ export async function createMyTestkudosEnvironment(
     commonDb: db,
     exchange,
     merchant,
+    merchantAdminAccessToken: adminAccessToken,
     walletClient,
     walletService,
     bankClient,
-    exchangeBankAccount: {
-      accountName: "",
-      accountPassword: "",
-      accountPaytoUri: "",
-      wireGatewayApiBaseUrl: "",
-    },
+    bank,
+    exchangeBankAccount,
   };
 }
 
@@ -204,7 +207,7 @@ export async function createMyTestkudosEnvironment(
 export async function runFeeRegressionTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bankClient, exchange, merchant } =
+  const { walletClient, bankClient, exchange, merchantAdminAccessToken, merchant } =
     await createMyTestkudosEnvironment(t);
 
   // Withdraw digital cash into the wallet.
@@ -229,7 +232,7 @@ export async function runFeeRegressionTest(t: GlobalTestState) {
     fulfillment_url: "taler://fulfillment-success/thx",
   };
 
-  await makeTestPaymentV2(t, { walletClient, merchant, order });
+  await makeTestPaymentV2(t, { walletClient, merchant, order, merchantAdminAccessToken });
 
   await walletClient.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
 
