@@ -17,14 +17,20 @@
 /**
  * Imports.
  */
-import { MerchantApiClient, TalerError, URL } from "@gnu-taler/taler-util";
+import {
+  HttpStatusCode,
+  MerchantAuthMethod,
+  TalerMerchantManagementHttpClient,
+  URL
+} from "@gnu-taler/taler-util";
 import {
   ExchangeService,
   GlobalTestState,
+  MERCHANT_DEFAULT_LOGIN_SCOPE,
   MerchantService,
-  generateRandomPayto,
+  getTestHarnessPaytoForLabel,
   harnessHttpLib,
-  setupDb,
+  setupDb
 } from "../harness/harness.js";
 
 /**
@@ -44,7 +50,6 @@ export async function runMerchantInstancesDeleteTest(t: GlobalTestState) {
 
   const merchant = await MerchantService.create(t, {
     name: "testmerchant-1",
-    currency: "TESTKUDOS",
     httpPort: 8083,
     database: db.connStr,
   });
@@ -75,12 +80,13 @@ export async function runMerchantInstancesDeleteTest(t: GlobalTestState) {
   }
 
   // Add an instance, no auth!
-  await merchant.addInstanceWithWireAccount({
-    id: "default",
+  const { accessToken: adminAccessToken } = await merchant.addInstanceWithWireAccount({
+    id: "admin",
     name: "Default Instance",
-    paytoUris: [generateRandomPayto("merchant-default")],
+    paytoUris: [getTestHarnessPaytoForLabel("merchant-default")],
     auth: {
-      method: "external",
+      method: MerchantAuthMethod.TOKEN,
+      password: "admin-pwd"
     },
   });
 
@@ -88,49 +94,30 @@ export async function runMerchantInstancesDeleteTest(t: GlobalTestState) {
   await merchant.addInstanceWithWireAccount({
     id: "myinst",
     name: "Second Instance",
-    paytoUris: [generateRandomPayto("merchant-default")],
+    paytoUris: [getTestHarnessPaytoForLabel("merchant-default")],
     auth: {
-      method: "external",
+      method: MerchantAuthMethod.TOKEN,
+      password: "myinst-pwd"
     },
-  });
+  }, {adminAccessToken});
 
-  let merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl(), {
-    auth: {
-      method: "external",
-    },
-  });
+  const merchantApi = new TalerMerchantManagementHttpClient(
+    merchant.makeInstanceBaseUrl(),
+  );
 
-  await merchantClient.changeAuth({
-    method: "token",
-    token: "secret-token:foobar",
-  });
+  const adminPwd = await merchantApi.createAccessToken("admin", "admin-pwd", MERCHANT_DEFAULT_LOGIN_SCOPE)
+  t.assertTrue(adminPwd.type === "ok")
 
-  merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl(), {
-    auth: {
-      method: "token",
-      token: "secret-token:foobar",
-    },
-  });
 
   // Check that deleting an instance checks the auth
   // of the default instance.
   {
-    const unauthMerchantClient = new MerchantApiClient(
-      merchant.makeInstanceBaseUrl(),
-      {
-        auth: {
-          method: "token",
-          token: "secret-token:invalid",
-        },
-      },
+    const res = await merchantApi.deleteInstance(
+      undefined,
+      "myinst",
     );
-
-    const exc = await t.assertThrowsAsync(async () => {
-      await unauthMerchantClient.deleteInstance("myinst");
-    });
-    console.log("Got expected exception", exc);
-    t.assertTrue(exc instanceof TalerError);
-    t.assertDeepEqual(exc.errorDetail.httpStatusCode, 401);
+    t.assertTrue(res.type === "fail");
+    t.assertTrue(res.case === HttpStatusCode.Unauthorized);
   }
 }
 
