@@ -19,24 +19,25 @@
  */
 import {
   ConfirmPayResultType,
-  MerchantApiClient,
   PreparePayResultType,
+  TalerMerchantInstanceHttpClient,
   URL,
   codecForMerchantOrderStatusUnpaid,
+  succeedOrThrow,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { GlobalTestState, harnessHttpLib } from "../harness/harness.js";
 import {
   createSimpleTestkudosEnvironmentV3,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
+import { GlobalTestState, harnessHttpLib } from "../harness/harness.js";
 
 /**
  * Run test for basic, bank-integrated withdrawal.
  */
 export async function runMerchantLongpollingTest(t: GlobalTestState) {
   // Set up test environment
-  const { walletClient, bankClient, exchange, merchant } =
+  const { walletClient, bankClient, exchange, merchant, merchantAdminAccessToken } =
     await createSimpleTestkudosEnvironmentV3(t);
 
   // Withdraw digital cash into the wallet.
@@ -50,9 +51,11 @@ export async function runMerchantLongpollingTest(t: GlobalTestState) {
 
   await wres.withdrawalFinishedCond;
 
-  const merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl());
+  const merchantClient = new TalerMerchantInstanceHttpClient(
+    merchant.makeInstanceBaseUrl(),
+  );
 
-  /**
+  /*
    * =========================================================================
    * Create an order and let the wallet pay under a session ID
    *
@@ -61,19 +64,22 @@ export async function runMerchantLongpollingTest(t: GlobalTestState) {
    * =========================================================================
    */
 
-  let orderResp = await merchantClient.createOrder({
-    order: {
-      summary: "Buy me!",
-      amount: "TESTKUDOS:5",
-      fulfillment_url: "https://example.com/article42",
-    },
-    create_token: false,
-  });
+  let orderResp = succeedOrThrow(
+    await merchantClient.createOrder(merchantAdminAccessToken, {
+      order: {
+        summary: "Buy me!",
+        amount: "TESTKUDOS:5",
+        fulfillment_url: "https://example.com/article42",
+      },
+      create_token: false,
+    }),
+  );
 
-  let orderStatus = await merchantClient.queryPrivateOrderStatus({
-    orderId: orderResp.order_id,
-    sessionId: "mysession-one",
-  });
+  let orderStatus = succeedOrThrow(
+    await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id, {
+      sessionId: "mysession-one",
+    }),
+  );
 
   t.assertTrue(orderStatus.order_status === "unpaid");
 
@@ -141,7 +147,7 @@ export async function runMerchantLongpollingTest(t: GlobalTestState) {
 
   t.assertTrue(preparePayResp.status === PreparePayResultType.PaymentPossible);
 
-  const proposalId = preparePayResp.proposalId;
+  const proposalTransactionId = preparePayResp.transactionId;
 
   publicOrderStatusResp = await publicOrderStatusPromise;
 
@@ -156,7 +162,7 @@ export async function runMerchantLongpollingTest(t: GlobalTestState) {
   );
 
   const confirmPayRes = await walletClient.call(WalletApiOperation.ConfirmPay, {
-    proposalId: proposalId,
+    transactionId: proposalTransactionId,
   });
 
   t.assertTrue(confirmPayRes.type === ConfirmPayResultType.Done);
