@@ -24,7 +24,7 @@ import {
   createSimpleTestkudosEnvironmentV3,
   makeTestPaymentV2,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
 
 /**
  * Run test for basic, bank-integrated withdrawal and payment.
@@ -32,7 +32,7 @@ import {
 export async function runPaymentTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { bankClient, walletClient, exchange, merchant } =
+  const { bankClient, walletClient, exchange, merchant, merchantAdminAccessToken } =
     await createSimpleTestkudosEnvironmentV3(t);
 
   // Withdraw digital cash into the wallet.
@@ -53,7 +53,7 @@ export async function runPaymentTest(t: GlobalTestState) {
     fulfillment_url: "taler://fulfillment-success/thx",
   } satisfies TalerMerchantApi.Order;
 
-  await makeTestPaymentV2(t, { walletClient, merchant, order });
+  await makeTestPaymentV2(t, { walletClient, merchant, order, merchantAdminAccessToken });
   await walletClient.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
 
   // Test JSON normalization of contract terms: Does the wallet
@@ -64,7 +64,7 @@ export async function runPaymentTest(t: GlobalTestState) {
     fulfillment_url: "taler://fulfillment-success/thx",
   } satisfies TalerMerchantApi.Order;
 
-  await makeTestPaymentV2(t, { walletClient, merchant, order: order2 });
+  await makeTestPaymentV2(t, { walletClient, merchant, order: order2, merchantAdminAccessToken });
   await walletClient.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
 
   // Test JSON normalization of contract terms: Does the wallet
@@ -75,7 +75,7 @@ export async function runPaymentTest(t: GlobalTestState) {
     fulfillment_url: "taler://fulfillment-success/thx",
   } satisfies TalerMerchantApi.Order;
 
-  await makeTestPaymentV2(t, { walletClient, merchant, order: order3 });
+  await makeTestPaymentV2(t, { walletClient, merchant, order: order3, merchantAdminAccessToken });
   await walletClient.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
 
   const bal = await walletClient.call(WalletApiOperation.GetBalances, {});
