@@ -19,17 +19,20 @@
  */
 import {
   ConfirmPayResultType,
-  MerchantApiClient,
   PreparePayResultType,
+  TalerMerchantInstanceHttpClient,
+  TransactionMajorState,
   URL,
   codecForMerchantOrderStatusUnpaid,
+  j2s,
+  succeedOrThrow,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { GlobalTestState, harnessHttpLib } from "../harness/harness.js";
 import {
   createSimpleTestkudosEnvironmentV3,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
+import { GlobalTestState, harnessHttpLib } from "../harness/harness.js";
 
 /**
  * Run test for basic, bank-integrated withdrawal.
@@ -37,10 +40,12 @@ import {
 export async function runPaywallFlowTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bankClient, exchange, merchant } =
+  const { walletClient, bankClient, exchange, merchant, merchantAdminAccessToken } =
     await createSimpleTestkudosEnvironmentV3(t);
 
-  const merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl());
+  const merchantClient = new TalerMerchantInstanceHttpClient(
+    merchant.makeInstanceBaseUrl(),
+  );
 
   // Withdraw digital cash into the wallet.
 
@@ -53,7 +58,7 @@ export async function runPaywallFlowTest(t: GlobalTestState) {
 
   await wres.withdrawalFinishedCond;
 
-  /**
+  /*
    * =========================================================================
    * Create an order and let the wallet pay under a session ID
    *
@@ -62,21 +67,24 @@ export async function runPaywallFlowTest(t: GlobalTestState) {
    * =========================================================================
    */
 
-  let orderResp = await merchantClient.createOrder({
-    order: {
-      summary: "Buy me!",
-      amount: "TESTKUDOS:5",
-      fulfillment_url: "https://example.com/article42",
-      public_reorder_url: "https://example.com/article42-share",
-    },
-  });
+  let orderResp = succeedOrThrow(
+    await merchantClient.createOrder(merchantAdminAccessToken, {
+      order: {
+        summary: "Buy me!",
+        amount: "TESTKUDOS:5",
+        fulfillment_url: "https://example.com/article42",
+        public_reorder_url: "https://example.com/article42-share",
+      },
+    }),
+  );
 
   const firstOrderId = orderResp.order_id;
 
-  let orderStatus = await merchantClient.queryPrivateOrderStatus({
-    orderId: orderResp.order_id,
-    sessionId: "mysession-one",
-  });
+  let orderStatus = succeedOrThrow(
+    await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id, {
+      sessionId: "mysession-one",
+    }),
+  );
 
   t.assertTrue(orderStatus.order_status === "unpaid");
 
@@ -110,7 +118,7 @@ export async function runPaywallFlowTest(t: GlobalTestState) {
 
   t.assertTrue(preparePayResp.status === PreparePayResultType.PaymentPossible);
 
-  const proposalId = preparePayResp.proposalId;
+  const proposalTransactionId = preparePayResp.transactionId;
 
   console.log("requesting", publicOrderStatusUrl.href);
   publicOrderStatusResp = await harnessHttpLib.fetch(publicOrderStatusUrl.href);
@@ -126,7 +134,7 @@ export async function runPaywallFlowTest(t: GlobalTestState) {
   );
 
   const confirmPayRes = await walletClient.call(WalletApiOperation.ConfirmPay, {
-    proposalId: proposalId,
+    transactionId: proposalTransactionId,
   });
 
   t.assertTrue(confirmPayRes.type === ConfirmPayResultType.Done);
@@ -147,10 +155,11 @@ export async function runPaywallFlowTest(t: GlobalTestState) {
    * =========================================================================
    */
 
-  orderStatus = await merchantClient.queryPrivateOrderStatus({
-    orderId: orderResp.order_id,
-    sessionId: "mysession-two",
-  });
+  orderStatus = succeedOrThrow(
+    await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id, {
+      sessionId: "mysession-two",
+    }),
+  );
 
   // Should be claimed (not paid!) because of a new session ID
   t.assertTrue(orderStatus.order_status === "claimed");
@@ -168,28 +177,38 @@ export async function runPaywallFlowTest(t: GlobalTestState) {
   t.assertTrue(preparePayResp.status === PreparePayResultType.AlreadyConfirmed);
   t.assertTrue(preparePayResp.paid);
 
+  await walletClient.call(WalletApiOperation.TestingWaitTransactionState, {
+    transactionId: preparePayResp.transactionId,
+    txState: {
+      major: TransactionMajorState.Done,
+    },
+  });
+
   /**
    * =========================================================================
    * Now we test re-purchase detection.
    * =========================================================================
    */
 
-  orderResp = await merchantClient.createOrder({
-    order: {
-      summary: "Buy me!",
-      amount: "TESTKUDOS:5",
-      // Same fulfillment URL as previously!
-      fulfillment_url: "https://example.com/article42",
-      public_reorder_url: "https://example.com/article42-share",
-    },
-  });
+  orderResp = succeedOrThrow(
+    await merchantClient.createOrder(merchantAdminAccessToken, {
+      order: {
+        summary: "Buy me!",
+        amount: "TESTKUDOS:5",
+        // Same fulfillment URL as previously!
+        fulfillment_url: "https://example.com/article42",
+        public_reorder_url: "https://example.com/article42-share",
+      },
+    }),
+  );
 
   const secondOrderId = orderResp.order_id;
 
-  orderStatus = await merchantClient.queryPrivateOrderStatus({
-    orderId: secondOrderId,
-    sessionId: "mysession-three",
-  });
+  orderStatus = succeedOrThrow(
+    await merchantClient.getOrderDetails(merchantAdminAccessToken, secondOrderId, {
+      sessionId: "mysession-three",
+    }),
+  );
 
   t.assertTrue(orderStatus.order_status === "unpaid");
 
@@ -209,21 +228,34 @@ export async function runPaywallFlowTest(t: GlobalTestState) {
   t.assertTrue(preparePayResp.status === PreparePayResultType.AlreadyConfirmed);
   t.assertTrue(preparePayResp.paid);
 
+  await walletClient.call(WalletApiOperation.TestingWaitTransactionState, {
+    transactionId: preparePayResp.transactionId,
+    txState: {
+      major: TransactionMajorState.Done,
+    },
+  });
+
   // The first order should now be paid under "mysession-three",
   // as the wallet did re-purchase detection
-  orderStatus = await merchantClient.queryPrivateOrderStatus({
-    orderId: firstOrderId,
-    sessionId: "mysession-three",
-  });
+  orderStatus = succeedOrThrow(
+    await merchantClient.getOrderDetails(merchantAdminAccessToken, firstOrderId, {
+      sessionId: "mysession-three",
+    }),
+  );
+
+  console.log(`order status (mysession-three)`, j2s(orderStatus));
 
   t.assertTrue(orderStatus.order_status === "paid");
 
   // Check that with a completely new session ID, the status would NOT
   // be paid.
-  orderStatus = await merchantClient.queryPrivateOrderStatus({
-    orderId: firstOrderId,
-    sessionId: "mysession-four",
-  });
+  orderStatus = succeedOrThrow(
+    await merchantClient.getOrderDetails(merchantAdminAccessToken, firstOrderId, {
+      sessionId: "mysession-four",
+    }),
+  );
+
+  console.log(`order status (mysession-four)`, j2s(orderStatus));
 
   t.assertTrue(orderStatus.order_status === "claimed");
 
