@@ -19,16 +19,17 @@
  */
 import {
   Duration,
-  MerchantApiClient,
+  succeedOrThrow,
+  TalerMerchantInstanceHttpClient,
   TransactionMajorState,
   TransactionMinorState,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { GlobalTestState } from "../harness/harness.js";
 import {
   createSimpleTestkudosEnvironmentV3,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
+import { GlobalTestState } from "../harness/harness.js";
 
 /**
  * Run test for basic, bank-integrated withdrawal.
@@ -36,10 +37,12 @@ import {
 export async function runRefundAutoTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bankClient, exchange, merchant } =
+  const { walletClient, bankClient, exchange, merchant, merchantAdminAccessToken } =
     await createSimpleTestkudosEnvironmentV3(t);
 
-  const merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl());
+  const merchantClient = new TalerMerchantInstanceHttpClient(
+    merchant.makeInstanceBaseUrl(),
+  );
 
   // Withdraw digital cash into the wallet.
 
@@ -57,23 +60,25 @@ export async function runRefundAutoTest(t: GlobalTestState) {
     t.logStep("start-test-autorefund");
 
     // Set up order.
-    const orderResp = await merchantClient.createOrder({
-      order: {
-        summary: "Buy me!",
-        amount: "TESTKUDOS:5",
-        fulfillment_url: "taler://fulfillment-success/thx",
-        auto_refund: Duration.toTalerProtocolDuration(
-          Duration.fromSpec({ minutes: 5 }),
+    const orderResp = succeedOrThrow(
+      await merchantClient.createOrder(merchantAdminAccessToken, {
+        order: {
+          summary: "Buy me!",
+          amount: "TESTKUDOS:5",
+          fulfillment_url: "taler://fulfillment-success/thx",
+          auto_refund: Duration.toTalerProtocolDuration(
+            Duration.fromSpec({ minutes: 5 }),
+          ),
+        },
+        refund_delay: Duration.toTalerProtocolDuration(
+          Duration.fromSpec({ minutes: 10 }),
         ),
-      },
-      refund_delay: Duration.toTalerProtocolDuration(
-        Duration.fromSpec({ minutes: 10 }),
-      ),
-    });
+      }),
+    );
 
-    let orderStatus = await merchantClient.queryPrivateOrderStatus({
-      orderId: orderResp.order_id,
-    });
+    let orderStatus = succeedOrThrow(
+      await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id),
+    );
 
     t.assertTrue(orderStatus.order_status === "unpaid");
 
@@ -89,18 +94,18 @@ export async function runRefundAutoTest(t: GlobalTestState) {
 
     // Check if payment was successful.
 
-    orderStatus = await merchantClient.queryPrivateOrderStatus({
-      orderId: orderResp.order_id,
-    });
+    orderStatus = succeedOrThrow(
+      await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id),
+    );
 
     t.assertTrue(orderStatus.order_status === "paid");
 
-    const ref = await merchantClient.giveRefund({
-      amount: "TESTKUDOS:5",
-      instance: "default",
-      justification: "foo",
-      orderId: orderResp.order_id,
-    });
+    const ref = succeedOrThrow(
+      await merchantClient.addRefund(merchantAdminAccessToken, orderResp.order_id, {
+        refund: "TESTKUDOS:5",
+        reason: "foo",
+      }),
+    );
 
     console.log(ref);
 
@@ -130,23 +135,25 @@ export async function runRefundAutoTest(t: GlobalTestState) {
 
   {
     // Set up order.
-    const orderResp = await merchantClient.createOrder({
-      order: {
-        summary: "Buy me!",
-        amount: "TESTKUDOS:5",
-        fulfillment_url: "taler://fulfillment-success/thx",
-        auto_refund: Duration.toTalerProtocolDuration(
-          Duration.fromSpec({ minutes: 5 }),
+    const orderResp = succeedOrThrow(
+      await merchantClient.createOrder(merchantAdminAccessToken, {
+        order: {
+          summary: "Buy me!",
+          amount: "TESTKUDOS:5",
+          fulfillment_url: "taler://fulfillment-success/thx",
+          auto_refund: Duration.toTalerProtocolDuration(
+            Duration.fromSpec({ minutes: 5 }),
+          ),
+        },
+        refund_delay: Duration.toTalerProtocolDuration(
+          Duration.fromSpec({ minutes: 10 }),
         ),
-      },
-      refund_delay: Duration.toTalerProtocolDuration(
-        Duration.fromSpec({ minutes: 10 }),
-      ),
-    });
+      }),
+    );
 
-    let orderStatus = await merchantClient.queryPrivateOrderStatus({
-      orderId: orderResp.order_id,
-    });
+    let orderStatus = succeedOrThrow(
+      await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id),
+    );
 
     t.assertTrue(orderStatus.order_status === "unpaid");
 
@@ -162,9 +169,9 @@ export async function runRefundAutoTest(t: GlobalTestState) {
 
     // Check if payment was successful.
 
-    orderStatus = await merchantClient.queryPrivateOrderStatus({
-      orderId: orderResp.order_id,
-    });
+    orderStatus = succeedOrThrow(
+      await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id),
+    );
 
     t.assertTrue(orderStatus.order_status === "paid");
 
