@@ -17,24 +17,30 @@
 /**
  * Imports.
  */
-import { MerchantApiClient, PreparePayResultType } from "@gnu-taler/taler-util";
+import {
+  PreparePayResultType,
+  succeedOrThrow,
+  TalerMerchantInstanceHttpClient,
+} from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { GlobalTestState } from "../harness/harness.js";
 import {
   createSimpleTestkudosEnvironmentV3,
   createWalletDaemonWithClient,
   makeTestPaymentV2,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
+import { GlobalTestState } from "../harness/harness.js";
 import { SyncService } from "../harness/sync.js";
 
 export async function runWalletBackupDoublespendTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { commonDb, merchant, walletClient, bankClient, exchange } =
+  const { commonDb, merchant, walletClient, bankClient, exchange, merchantAdminAccessToken } =
     await createSimpleTestkudosEnvironmentV3(t);
 
-  const merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl());
+  const merchantClient = new TalerMerchantInstanceHttpClient(
+    merchant.makeInstanceBaseUrl(),
+  );
 
   const sync = await SyncService.create(t, {
     currency: "TESTKUDOS",
@@ -92,6 +98,7 @@ export async function runWalletBackupDoublespendTest(t: GlobalTestState) {
 
   await makeTestPaymentV2(t, {
     merchant,
+    merchantAdminAccessToken,
     walletClient,
     order: {
       summary: "foo",
@@ -116,19 +123,19 @@ export async function runWalletBackupDoublespendTest(t: GlobalTestState) {
   // Now we double-spend with the second wallet
 
   {
-    const instance = "default";
+    const orderResp = succeedOrThrow(
+      await merchantClient.createOrder(merchantAdminAccessToken, {
+        order: {
+          amount: "TESTKUDOS:8",
+          summary: "bla",
+          fulfillment_url: "taler://fulfillment-success",
+        },
+      }),
+    );
 
-    const orderResp = await merchantClient.createOrder({
-      order: {
-        amount: "TESTKUDOS:8",
-        summary: "bla",
-        fulfillment_url: "taler://fulfillment-success",
-      },
-    });
-
-    let orderStatus = await merchantClient.queryPrivateOrderStatus({
-      orderId: orderResp.order_id,
-    });
+    let orderStatus = succeedOrThrow(
+      await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id),
+    );
 
     t.assertTrue(orderStatus.order_status === "unpaid");
 
