@@ -19,12 +19,12 @@
  */
 import {
   NotificationType,
-  TalerCorebankApiClient,
   TransactionMajorState,
+  TransactionMinorState,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
+import { createSimpleTestkudosEnvironmentV3 } from "../harness/environments.js";
 import { GlobalTestState } from "../harness/harness.js";
-import { createSimpleTestkudosEnvironmentV3 } from "../harness/helpers.js";
 
 /**
  * Test behavior when an order is deleted while the wallet is paying for it.
@@ -78,6 +78,14 @@ export async function runWalletBalanceNotificationsTest(t: GlobalTestState) {
 
   // Confirm it
 
+  await walletClient.call(WalletApiOperation.TestingWaitTransactionState, {
+    transactionId: acceptRes.transactionId,
+    txState: {
+      major: TransactionMajorState.Pending,
+      minor: TransactionMinorState.BankConfirmTransfer,
+    },
+  });
+
   await bankClient.confirmWithdrawalOperation(user.username, {
     withdrawalOperationId: wop.withdrawal_id,
   });
