@@ -18,19 +18,21 @@
  * Imports.
  */
 import {
-  MerchantApiClient,
   PreparePayResultType,
+  TalerCorebankApiClient,
+  TalerMerchantInstanceHttpClient,
   j2s,
+  succeedOrThrow,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { CoinConfig } from "../harness/denomStructures.js";
-import { GlobalTestState } from "../harness/harness.js";
 import {
   createSimpleTestkudosEnvironmentV2,
   createWalletDaemonWithClient,
   makeTestPaymentV2,
-  withdrawViaBankV2,
-} from "../harness/helpers.js";
+  withdrawViaBankV3,
+} from "../harness/environments.js";
+import { GlobalTestState } from "../harness/harness.js";
 
 const coinCommon = {
   cipher: "RSA" as const,
@@ -63,11 +65,13 @@ const coinConfigList: CoinConfig[] = [
 export async function runWalletBlockedPayMerchantTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { bank, exchange, merchant } = await createSimpleTestkudosEnvironmentV2(
+  const { bank, exchange, merchant, merchantAdminAccessToken } = await createSimpleTestkudosEnvironmentV2(
     t,
     coinConfigList,
   );
 
+  const bankClient = new TalerCorebankApiClient(bank.baseUrl);
+
   // Withdraw digital cash into the wallet.
 
   const { walletClient: w1 } = await createWalletDaemonWithClient(t, {
@@ -80,9 +84,9 @@ export async function runWalletBlockedPayMerchantTest(t: GlobalTestState) {
     },
   });
 
-  await withdrawViaBankV2(t, {
+  await withdrawViaBankV3(t, {
     walletClient: w1,
-    bank,
+    bankClient,
     exchange,
     amount: "TESTKUDOS:20",
   });
@@ -98,6 +102,7 @@ export async function runWalletBlockedPayMerchantTest(t: GlobalTestState) {
   // Do a payment that causes a refresh.
   await makeTestPaymentV2(t, {
     merchant,
+    merchantAdminAccessToken,
     walletClient: w1,
     order: {
       summary: "test",
@@ -105,18 +110,22 @@ export async function runWalletBlockedPayMerchantTest(t: GlobalTestState) {
     },
   });
 
-  const merchantClient = new MerchantApiClient(merchant.makeInstanceBaseUrl());
+  const merchantClient = new TalerMerchantInstanceHttpClient(
+    merchant.makeInstanceBaseUrl(),
+  );
 
-  const orderResp = await merchantClient.createOrder({
-    order: {
-      summary: "My Payment",
-      amount: "TESTKUDOS:18",
-    },
-  });
+  const orderResp = succeedOrThrow(
+    await merchantClient.createOrder(merchantAdminAccessToken, {
+      order: {
+        summary: "My Payment",
+        amount: "TESTKUDOS:18",
+      },
+    }),
+  );
 
-  let orderStatus = await merchantClient.queryPrivateOrderStatus({
-    orderId: orderResp.order_id,
-  });
+  let orderStatus = succeedOrThrow(
+    await merchantClient.getOrderDetails(merchantAdminAccessToken, orderResp.order_id),
+  );
 
   t.assertTrue(orderStatus.order_status === "unpaid");
 
