@@ -23,21 +23,22 @@ import {
   TalerCorebankApiClient,
   TalerError,
   TalerErrorCode,
+  TalerWireGatewayAuth,
   WalletNotification,
   j2s,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { CoinConfig, defaultCoinConfig } from "../harness/denomStructures.js";
+import { withdrawViaBankV3 } from "../harness/environments.js";
 import {
-    BankService,
+  BankService,
   ExchangeService,
   GlobalTestState,
   WalletClient,
   WalletService,
-  generateRandomPayto,
+  getTestHarnessPaytoForLabel,
   setupDb,
 } from "../harness/harness.js";
-import { withdrawViaBankV3 } from "../harness/helpers.js";
 
 /**
  * Test for DD48 notifications.
@@ -63,13 +64,19 @@ export async function runWalletDd48Test(t: GlobalTestState) {
 
   let receiverName = "Exchange";
   let exchangeBankUsername = "exchange";
-  let exchangeBankPassword = "mypw";
-  let exchangePaytoUri = generateRandomPayto(exchangeBankUsername);
+  let exchangeBankPassword = "mypw-password";
+  let exchangePaytoUri = getTestHarnessPaytoForLabel(exchangeBankUsername);
+  const wireGatewayAuth: TalerWireGatewayAuth = {
+    username: exchangeBankUsername,
+    password: exchangeBankPassword,
+  };
 
   await exchange.addBankAccount("1", {
-    accountName: exchangeBankUsername,
-    accountPassword: exchangeBankPassword,
-    wireGatewayApiBaseUrl: new URL("accounts/exchange/taler-wire-gateway/", bank.baseUrl).href,
+    wireGatewayAuth,
+    wireGatewayApiBaseUrl: new URL(
+      "accounts/exchange/taler-wire-gateway/",
+      bank.baseUrl,
+    ).href,
     accountPaytoUri: exchangePaytoUri,
   });
 
@@ -82,7 +89,7 @@ export async function runWalletDd48Test(t: GlobalTestState) {
   const bankClient = new TalerCorebankApiClient(bank.corebankApiBaseUrl, {
     auth: {
       username: "admin",
-      password: "adminpw",
+      password: "admin-password",
     },
   });
 
@@ -128,6 +135,7 @@ export async function runWalletDd48Test(t: GlobalTestState) {
 
   await walletClient.call(WalletApiOperation.AddExchange, {
     exchangeBaseUrl: exchange.baseUrl,
+    ephemeral: true,
   });
 
   {
@@ -178,7 +186,7 @@ export async function runWalletDd48Test(t: GlobalTestState) {
       (x) =>
         x.type === NotificationType.ExchangeStateTransition &&
         x.oldExchangeState == null &&
-        x.newExchangeState.exchangeEntryStatus ===
+        x.newExchangeState?.exchangeEntryStatus ===
           ExchangeEntryStatus.Ephemeral,
     ),
   );
