@@ -30,20 +30,23 @@ import {
   WalletApiOperation,
   parseTransactionIdentifier,
 } from "@gnu-taler/taler-wallet-core";
-import { GlobalTestState, generateRandomPayto } from "../harness/harness.js";
 import {
   createSimpleTestkudosEnvironmentV3,
   makeTestPaymentV2,
   withdrawViaBankV3,
-} from "../harness/helpers.js";
+} from "../harness/environments.js";
+import {
+  GlobalTestState,
+  getTestHarnessPaytoForLabel,
+} from "../harness/harness.js";
 
 /**
- * Run test for refreshe after a payment.
+ * Run test for refresh after a payment.
  */
 export async function runWalletRefreshTest(t: GlobalTestState) {
   // Set up test environment
 
-  const { walletClient, bankClient, exchange, merchant } =
+  const { walletClient, bankClient, exchange, merchant, merchantAdminAccessToken } =
     await createSimpleTestkudosEnvironmentV3(t);
 
   // Withdraw digital cash into the wallet.
@@ -63,7 +66,7 @@ export async function runWalletRefreshTest(t: GlobalTestState) {
     fulfillment_url: "taler://fulfillment-success/thx",
   };
 
-  await makeTestPaymentV2(t, { walletClient, merchant, order });
+  await makeTestPaymentV2(t, { walletClient, merchant, order, merchantAdminAccessToken });
   await walletClient.call(WalletApiOperation.TestingWaitTransactionsFinal, {});
 
   const txns = await walletClient.call(WalletApiOperation.GetTransactions, {
@@ -120,7 +123,7 @@ export async function runWalletRefreshTest(t: GlobalTestState) {
       WalletApiOperation.CreateDepositGroup,
       {
         amount: "TESTKUDOS:10.5" as AmountString,
-        depositPaytoUri: generateRandomPayto("foo"),
+        depositPaytoUri: getTestHarnessPaytoForLabel("foo"),
       },
     );
 
@@ -175,7 +178,7 @@ export async function runWalletRefreshTest(t: GlobalTestState) {
       WalletApiOperation.CreateDepositGroup,
       {
         amount: "TESTKUDOS:10.5" as AmountString,
-        depositPaytoUri: generateRandomPayto("foo"),
+        depositPaytoUri: getTestHarnessPaytoForLabel("foo"),
       },
     );
 
