@@ -17,7 +17,7 @@
 /**
  * Imports.
  */
-import { AmountString, URL } from "@gnu-taler/taler-util";
+import { AmountString, PaytoString, URL } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { CoinConfig, defaultCoinConfig } from "../harness/denomStructures.js";
 import {
@@ -52,17 +52,19 @@ export async function runWithdrawalFakebankTest(t: GlobalTestState) {
   });
 
   exchange.addBankAccount("1", {
-    accountName: "exchange",
-    accountPassword: "x",
+    wireGatewayAuth: {
+      username: "exchange",
+      password: "password",
+    },
     wireGatewayApiBaseUrl: new URL(
       "/accounts/exchange/taler-wire-gateway/",
       bank.baseUrl,
     ).href,
     accountPaytoUri:
-      "payto://x-taler-bank/localhost/exchange?receiver-name=Exchange",
+      "payto://x-taler-bank/localhost/exchange?receiver-name=Exchange" as PaytoString,
   });
 
-  await bank.createExchangeAccount("exchange", "x");
+  await bank.createExchangeAccount("exchange", "password");
 
   await bank.start();
 
