@@ -17,7 +17,12 @@
 /**
  * Imports.
  */
-import { TalerCorebankApiClient, j2s } from "@gnu-taler/taler-util";
+import {
+  TalerCorebankApiClient,
+  TransactionMajorState,
+  TransactionMinorState,
+  j2s,
+} from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { CoinConfig } from "../harness/denomStructures.js";
 import {
@@ -25,7 +30,7 @@ import {
   ExchangeService,
   GlobalTestState,
   WalletCli,
-  generateRandomPayto,
+  getTestHarnessPaytoForLabel,
   setupDb,
 } from "../harness/harness.js";
 
@@ -84,12 +89,14 @@ export async function runWithdrawalFeesTest(t: GlobalTestState) {
 
   let receiverName = "Exchange";
   let exchangeBankUsername = "exchange";
-  let exchangeBankPassword = "mypw";
-  let exchangePaytoUri = generateRandomPayto(exchangeBankUsername);
+  let exchangeBankPassword = "mypw-password";
+  let exchangePaytoUri = getTestHarnessPaytoForLabel(exchangeBankUsername);
 
   await exchange.addBankAccount("1", {
-    accountName: exchangeBankUsername,
-    accountPassword: exchangeBankPassword,
+    wireGatewayAuth: {
+      username: exchangeBankUsername,
+      password: exchangeBankPassword,
+    },
     wireGatewayApiBaseUrl: new URL(
       "accounts/exchange/taler-wire-gateway/",
       bank.baseUrl,
@@ -106,7 +113,7 @@ export async function runWithdrawalFeesTest(t: GlobalTestState) {
   const bankClient = new TalerCorebankApiClient(bank.corebankApiBaseUrl, {
     auth: {
       username: "admin",
-      password: "adminpw",
+      password: "admin-password",
     },
   });
 
@@ -138,7 +145,7 @@ export async function runWithdrawalFeesTest(t: GlobalTestState) {
   bankClient.setAuth(user);
   const wop = await bankClient.createWithdrawalOperation(user.username, amount);
 
-  t.logStep("Hand it to the wallet")
+  t.logStep("Hand it to the wallet");
 
   const details = await wallet.client.call(
     WalletApiOperation.GetWithdrawalDetailsForUri,
@@ -165,25 +172,36 @@ export async function runWithdrawalFeesTest(t: GlobalTestState) {
   t.assertAmountEquals(amountDetails.amountEffective, "TESTKUDOS:5");
   t.assertAmountEquals(amountDetails.amountRaw, "TESTKUDOS:7.5");
 
-  t.logStep("Complete all pending operations")
+  t.logStep("Complete all pending operations");
 
   await wallet.runPending();
 
-  t.logStep("Withdraw (AKA select)")
+  t.logStep("Withdraw (AKA select)");
 
-  await wallet.client.call(WalletApiOperation.AcceptBankIntegratedWithdrawal, {
-    exchangeBaseUrl: exchange.baseUrl,
-    talerWithdrawUri: wop.taler_withdraw_uri,
+  const acceptResp = await wallet.client.call(
+    WalletApiOperation.AcceptBankIntegratedWithdrawal,
+    {
+      exchangeBaseUrl: exchange.baseUrl,
+      talerWithdrawUri: wop.taler_withdraw_uri,
+    },
+  );
+
+  t.logStep("Confirm it");
+
+  await wallet.client.call(WalletApiOperation.TestingWaitTransactionState, {
+    transactionId: acceptResp.transactionId,
+    txState: {
+      major: TransactionMajorState.Pending,
+      minor: TransactionMinorState.BankConfirmTransfer,
+    },
   });
 
-  t.logStep("Confirm it")
-
   await bankClient.confirmWithdrawalOperation(user.username, {
     withdrawalOperationId: wop.withdrawal_id,
   });
   await wallet.runUntilDone();
 
-  t.logStep("Check balance")
+  t.logStep("Check balance");
 
   const balResp = await wallet.client.call(WalletApiOperation.GetBalances, {});
   console.log(j2s(balResp));
