@@ -54,6 +54,7 @@ import { runDepositTest } from "./test-deposit.js";
 import { runDonauCharityManagementTest } from "./test-donau-charity-management.js";
 import { runDonauCompatTest } from "./test-donau-compat.js";
 import { runDonauIdempotencyTest } from "./test-donau-idempotency.js";
+import { runDonauKeychangeTest } from "./test-donau-keychange.js";
 import { runDonauMinusTTest } from "./test-donau-minus-t.js";
 import { runDonauMultiTest } from "./test-donau-multi.js";
 import { runDonauTest } from "./test-donau.js";
@@ -168,6 +169,7 @@ import { runWalletBlockedPayPeerPullTest } from "./test-wallet-blocked-pay-peer-
 import { runWalletBlockedPayPeerPushTest } from "./test-wallet-blocked-pay-peer-push.js";
 import { runWalletCliTerminationTest } from "./test-wallet-cli-termination.js";
 import { runWalletConfigTest } from "./test-wallet-config.js";
+import { runWalletContactsBasicTest } from "./test-wallet-contacts-basic.js";
 import { runWalletCryptoWorkerTest } from "./test-wallet-cryptoworker.js";
 import { runWalletDblessTest } from "./test-wallet-dbless.js";
 import { runWalletDd48Test } from "./test-wallet-dd48.js";
@@ -179,6 +181,7 @@ import { runWalletExchangeMigrationTest } from "./test-wallet-exchange-migration
 import { runWalletExchangeUpdateTest } from "./test-wallet-exchange-update.js";
 import { runWalletGenDbTest } from "./test-wallet-gendb.js";
 import { runWalletInsufficientBalanceTest } from "./test-wallet-insufficient-balance.js";
+import { runWalletMailboxBasicTest } from "./test-wallet-mailbox-basic.js";
 import { runWalletNetworkAvailabilityTest } from "./test-wallet-network-availability.js";
 import { runWalletNotificationsTest } from "./test-wallet-notifications.js";
 import { runWalletObservabilityTest } from "./test-wallet-observability.js";
@@ -273,6 +276,8 @@ const allTests: TestMainFunction[] = [
   runWalletBackupDoublespendTest,
   runWalletNotificationsTest,
   runWalletCryptoWorkerTest,
+  runWalletContactsBasicTest,
+  runWalletMailboxBasicTest,
   runWalletDblessTest,
   runWallettestingTest,
   runWithdrawalAbortBankTest,
@@ -393,6 +398,7 @@ const allTests: TestMainFunction[] = [
   runDonauCharityManagementTest,
   runDonauMultiTest,
   runDonauIdempotencyTest,
+  runDonauKeychangeTest,
 ];
 
 export interface TestRunSpec {
