@@ -30,8 +30,8 @@ import {
   codecForString,
   renderContext,
 } from "./codec.js";
-import { CurrencySpecification } from "./index.js";
-import { AmountString } from "./types-taler-common.js";
+import { opFixedSuccess, opKnownFailure } from "./operation.js";
+import { AmountString, CurrencySpecification } from "./types-taler-common.js";
 
 /**
  * Number of fractional units that one value unit represents.
@@ -55,6 +55,11 @@ export const amountMaxValue = 2 ** 52;
  */
 export const FRAC_SEPARATOR = ".";
 
+/**
+ * Separator character between integer and fractional
+ */
+export const CURRENCY_SEPARATOR = ":";
+
 /**
  * Non-negative financial amount.  Fractional values are expressed as multiples
  * of 1e-8.
@@ -96,6 +101,18 @@ export class Amount {
     return new Amount(r.amount, r.saturated ? 1 : 0);
   }
 
+  isZero(): boolean {
+    return this.val.fraction === 0 && this.val.value === 0;
+  }
+
+  sub(...a: AmountLike[]): Amount {
+    if (this.saturated) {
+      return this;
+    }
+    const r = Amounts.sub(this.val, ...a);
+    return new Amount(r.amount, r.saturated ? 1 : 0);
+  }
+
   mult(n: number): Amount {
     if (this.saturated) {
       return this;
@@ -167,6 +184,33 @@ export interface DivmodResult {
   remainder: AmountJson;
 }
 
+export enum AmountParseError {
+  /**
+   * Should have a string separated with a colon
+   */
+  MISSING_CURRENCY,
+  /**
+   * Currency should be less than 11 characters
+   */
+  CURRENCY_TOO_LONG,
+  /**
+   * Currency shoul be only letter from a to z
+   */
+  BAD_CURRENCY,
+  /**
+   * Number can only be digits from 0 to 9
+   */
+  BAD_NUMBER,
+  /**
+   * Integer part should be less than 2 ** 52
+   */
+  TOO_HIGH,
+  /**
+   * Fractional part should be shorter than 8 digits
+   */
+  TOO_PRECISE,
+}
+
 /**
  * Helper class for dealing with amounts.
  */
@@ -420,6 +464,57 @@ export class Amounts {
     return /^[a-zA-Z]{1,11}$/.test(s);
   }
 
+  /**
+   * Parse an amount like 'EUR:20.5' for 20 Euros and 50 ct.
+   *
+   * Currency name size limit is 11 of ASCII letters
+   * Fraction size limit is 8
+   */
+  static parseWithError(s: string) {
+    const c_idx = s.indexOf(CURRENCY_SEPARATOR);
+
+    if (c_idx === -1 || c_idx === 0) {
+      return opKnownFailure(AmountParseError.MISSING_CURRENCY);
+    }
+    if (c_idx > 11) {
+      return opKnownFailure(AmountParseError.MISSING_CURRENCY);
+    }
+    const currency = s.substring(0, c_idx).toUpperCase();
+    if (!/^[a-zA-Z]+$/.test(currency)) {
+      return opKnownFailure(AmountParseError.BAD_CURRENCY);
+    }
+    const number = s.substring(c_idx + 1);
+    const d_idx = number.indexOf(FRAC_SEPARATOR);
+    const integerStr = d_idx === -1 ? number : number.substring(0, d_idx);
+    const fractStr =
+      d_idx === -1 || d_idx === number.length
+        ? "0"
+        : number.substring(d_idx + 1);
+
+    if (!/^[0-9]+$/.test(integerStr) || !/^[0-9]+$/.test(fractStr)) {
+      return opKnownFailure(AmountParseError.BAD_NUMBER);
+    }
+
+    const value = Number.parseInt(integerStr, 10);
+    const fraction = Math.round(
+      amountFractionalBase * Number.parseFloat(FRAC_SEPARATOR + fractStr),
+    );
+    if (!Number.isInteger(value) || !Number.isInteger(fraction)) {
+      return opKnownFailure(AmountParseError.BAD_NUMBER);
+    }
+    if (value > amountMaxValue) {
+      return opKnownFailure(AmountParseError.TOO_HIGH);
+    }
+    if (fractStr.length > amountFractionalLength) {
+      return opKnownFailure(AmountParseError.TOO_PRECISE);
+    }
+    return opFixedSuccess({
+      currency,
+      fraction,
+      value,
+    });
+  }
+
   /**
    * Parse an amount like 'EUR:20.5' for 20 Euros and 50 ct.
    *
@@ -556,6 +651,16 @@ export class Amounts {
     return `${a.currency}:${s}` as AmountString;
   }
 
+  /**
+   * Show an amount in a form suitable for the user.
+   * FIXME:  In the future, this should consider currency-specific
+   * settings such as significant digits or currency symbols.
+   */
+  static toPretty(amount: AmountJson): string {
+    const x = amount.value + amount.fraction / amountFractionalBase;
+    return `${x} ${amount.currency}`;
+  }
+
   static amountHasSameCurrency(a1: AmountLike, a2: AmountLike): boolean {
     const x1 = this.jsonifyAmount(a1);
     const x2 = this.jsonifyAmount(a2);
