@@ -22,18 +22,19 @@ import { Logger } from "../logging.js";
 import {
   FailCasesByMethod,
   ResultByMethod,
+  carefullyParseConfig,
   opEmptySuccess,
   opKnownHttpFailure,
   opKnownTalerFailure,
   opSuccessFromHttp,
-  opUnknownFailure,
+  opUnknownHttpFailure
 } from "../operation.js";
 import { TalerErrorCode } from "../taler-error-codes.js";
 import {
   BankWithdrawalOperationPostRequest,
-  WithdrawalOperationStatus,
+  WithdrawalOperationStatusFlag,
   codecForBankWithdrawalOperationPostResponse,
-  codecForBankWithdrawalOperationStatus,
+  codecForBankWithdrawalOperationStatus
 } from "../types-taler-bank-integration.js";
 import { LongPollParams } from "../types-taler-common.js";
 import { codecForIntegrationBankConfig } from "../types-taler-corebank.js";
@@ -53,9 +54,7 @@ const logger = new Logger("bank-integration.ts");
  * The API is used by the wallets.
  */
 export class TalerBankIntegrationHttpClient {
-  public static readonly PROTOCOL_VERSION = "2:0:1";
-  public readonly PROTOCOL_VERSION =
-    TalerBankIntegrationHttpClient.PROTOCOL_VERSION;
+  public static readonly PROTOCOL_VERSION = "5:0:0";
 
   httpLib: HttpRequestLibrary;
 
@@ -66,7 +65,7 @@ export class TalerBankIntegrationHttpClient {
     this.httpLib = httpClient ?? createPlatformHttpLib();
   }
 
-  isCompatible(version: string): boolean {
+  static isCompatible(version: string): boolean {
     const compare = LibtoolVersion.compare(this.PROTOCOL_VERSION, version);
     return compare?.compatible ?? false;
   }
@@ -82,10 +81,14 @@ export class TalerBankIntegrationHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccessFromHttp(resp, codecForIntegrationBankConfig());
+        return carefullyParseConfig(
+          "taler-bank-integration",
+          TalerBankIntegrationHttpClient.PROTOCOL_VERSION,
+          resp,
+          codecForIntegrationBankConfig(),
+        );
       default:
-        logger.warn(`config request failed, status ${resp.status}`);
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 
@@ -96,7 +99,7 @@ export class TalerBankIntegrationHttpClient {
   async getWithdrawalOperationById(
     woid: string,
     params?: {
-      old_state?: WithdrawalOperationStatus;
+      old_state?: WithdrawalOperationStatusFlag;
     } & LongPollParams,
   ) {
     const url = new URL(`withdrawal-operation/${woid}`, this.baseUrl);
@@ -116,13 +119,14 @@ export class TalerBankIntegrationHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 
   /**
-   * https://docs.taler.net/core/api-bank-integration.html#post-$BANK_API_BASE_URL-withdrawal-operation-$wopid
+   * FIXME: This is a misnomer!
    *
+   * https://docs.taler.net/core/api-bank-integration.html#post-$BANK_API_BASE_URL-withdrawal-operation-$wopid
    */
   async completeWithdrawalOperationById(
     woid: string,
@@ -145,6 +149,8 @@ export class TalerBankIntegrationHttpClient {
         const body = await readTalerErrorResponse(resp);
         const details = codecForTalerErrorDetail().decode(body);
         switch (details.code) {
+          case TalerErrorCode.BANK_UPDATE_ABORT_CONFLICT:
+            return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_WITHDRAWAL_OPERATION_RESERVE_SELECTION_CONFLICT:
             return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_DUPLICATE_RESERVE_PUB_SUBJECT:
@@ -155,14 +161,14 @@ export class TalerBankIntegrationHttpClient {
             return opKnownTalerFailure(details.code, details);
           case TalerErrorCode.BANK_AMOUNT_DIFFERS:
             return opKnownTalerFailure(details.code, details);
-          case TalerErrorCode.BANK_AMOUNT_REQUIRED:
+          case TalerErrorCode.BANK_UNALLOWED_DEBIT:
             return opKnownTalerFailure(details.code, details);
           default:
-            return opUnknownFailure(resp, details);
+            return opUnknownHttpFailure(resp, details);
         }
       }
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 
@@ -177,13 +183,13 @@ export class TalerBankIntegrationHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.NoContent:
-        return opEmptySuccess(resp);
+        return opEmptySuccess();
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Conflict:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 }
