@@ -26,10 +26,12 @@ import { LibtoolVersion } from "../libtool-version.js";
 import {
   FailCasesByMethod,
   ResultByMethod,
-  opKnownAlternativeFailure,
+  carefullyParseConfig,
+  opKnownAlternativeHttpFailure,
   opKnownHttpFailure,
   opSuccessFromHttp,
   opUnknownFailure,
+  opUnknownHttpFailure,
 } from "../operation.js";
 import {
   codecForChallengeInvalidPinResponse,
@@ -63,7 +65,7 @@ export enum ChallengerCacheEviction {
 export class ChallengerHttpClient {
   httpLib: HttpRequestLibrary;
   cacheEvictor: CacheEvictor<ChallengerCacheEviction>;
-  public readonly PROTOCOL_VERSION = "1:0:0";
+  public static readonly PROTOCOL_VERSION = "2:0:0";
 
   constructor(
     readonly baseUrl: string,
@@ -74,7 +76,7 @@ export class ChallengerHttpClient {
     this.cacheEvictor = cacheEvictor ?? nullEvictor;
   }
 
-  isCompatible(version: string): boolean {
+  static isCompatible(version: string): boolean {
     const compare = LibtoolVersion.compare(this.PROTOCOL_VERSION, version);
     return compare?.compatible ?? false;
   }
@@ -89,24 +91,27 @@ export class ChallengerHttpClient {
     });
     switch (resp.status) {
       case HttpStatusCode.Ok:
-        return opSuccessFromHttp(
+        return carefullyParseConfig(
+          "challenger",
+          ChallengerHttpClient.PROTOCOL_VERSION,
           resp,
           codecForChallengerTermsOfServiceResponse(),
         );
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
   /**
    * https://docs.taler.net/core/api-challenger.html#post--setup-$CLIENT_ID
    *
    */
-  async setup(clientId: string, token: AccessToken) {
+  async setup(clientId: string, token: AccessToken, body?: object) {
     const url = new URL(`setup/${clientId}`, this.baseUrl);
     const resp = await this.httpLib.fetch(url.href, {
       method: "POST",
+      body,
       headers: {
         Authorization: makeBearerTokenAuthHeader(token),
       },
@@ -117,7 +122,7 @@ export class ChallengerHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 
@@ -157,7 +162,7 @@ export class ChallengerHttpClient {
       case HttpStatusCode.InternalServerError:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 
@@ -172,11 +177,7 @@ export class ChallengerHttpClient {
 
     const resp = await this.httpLib.fetch(url.href, {
       method: "POST",
-      body: new URLSearchParams(Object.entries(body)).toString(),
-      headers: {
-        "Content-Type": "application/x-www-form-urlencoded",
-      },
-      redirect: "manual",
+      body,
     });
     switch (resp.status) {
       case HttpStatusCode.Ok: {
@@ -196,7 +197,7 @@ export class ChallengerHttpClient {
       case HttpStatusCode.InternalServerError:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 
@@ -226,7 +227,7 @@ export class ChallengerHttpClient {
       case HttpStatusCode.BadRequest:
         return opKnownHttpFailure(resp.status, resp);
       case HttpStatusCode.Forbidden:
-        return opKnownAlternativeFailure(
+        return opKnownAlternativeHttpFailure(
           resp,
           HttpStatusCode.Forbidden,
           codecForChallengeInvalidPinResponse(),
@@ -240,7 +241,7 @@ export class ChallengerHttpClient {
       case HttpStatusCode.InternalServerError:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 
@@ -280,7 +281,7 @@ export class ChallengerHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 
@@ -306,7 +307,7 @@ export class ChallengerHttpClient {
       case HttpStatusCode.NotFound:
         return opKnownHttpFailure(resp.status, resp);
       default:
-        return opUnknownFailure(resp, await readTalerErrorResponse(resp));
+        return opUnknownHttpFailure(resp);
     }
   }
 }
