@@ -47,13 +47,13 @@ import {
   LongPollParams,
   OfficerAccount,
   PaginationParams,
-  PaytoHash
 } from "../types-taler-common.js";
 import {
   AccountKycStatus,
   AmlDecisionRequest,
   AmlDecisionsResponse,
   AvailableMeasureSummary,
+  EventCounter,
   ExchangeGetContractResponse,
   ExchangeKeysResponse,
   ExchangeKycUploadFormRequest,
@@ -81,6 +81,7 @@ import {
   PurseConflictPartial,
   WalletKycRequest,
   codecForAccountKycStatus,
+  codecForAmlDecisionsAccounts,
   codecForAmlDecisionsResponse,
   codecForAmlKycAttributes,
   codecForAmlStatisticsResponse,
@@ -100,7 +101,7 @@ import {
   codecForLegitimizationMeasuresList,
   codecForLegitimizationNeededResponse,
   codecForPurseConflict,
-  codecForPurseConflictPartial
+  codecForPurseConflictPartial,
 } from "../types-taler-exchange.js";
 import {
   CacheEvictor,
@@ -114,6 +115,7 @@ import {
   Amounts,
   CancellationToken,
   LongpollQueue,
+  PaytoHash,
   signAmlDecision,
   signAmlQuery,
 } from "../index.js";
@@ -136,7 +138,7 @@ export enum TalerExchangeCacheEviction {
  * Client library for the GNU Taler exchange service.
  */
 export class TalerExchangeHttpClient {
-  public static readonly SUPPORTED_EXCHANGE_PROTOCOL_VERSION = "27:0:2";
+  public static readonly SUPPORTED_EXCHANGE_PROTOCOL_VERSION = "31:0:6";
   private httpLib: HttpRequestLibrary;
   private cacheEvictor: CacheEvictor<TalerExchangeCacheEviction>;
   private preventCompression: boolean;
@@ -583,14 +585,7 @@ export class TalerExchangeHttpClient {
     accountSig: EddsaSignatureString;
     longpoll?: boolean;
     awaitAuth?: boolean;
-  }): Promise<
-    | OperationOk<void>
-    | OperationAlternative<HttpStatusCode.Ok, AccountKycStatus>
-    | OperationAlternative<HttpStatusCode.Accepted, AccountKycStatus>
-    | OperationFail<HttpStatusCode.Forbidden>
-    | OperationFail<HttpStatusCode.NotFound>
-    | OperationFail<HttpStatusCode.Conflict>
-  > {
+  }) {
     const { paytoHash, accountPub, accountSig, longpoll, awaitAuth } = args;
     const url = new URL(`kyc-check/${paytoHash}`, this.baseUrl);
     if (awaitAuth !== undefined) {
@@ -609,14 +604,14 @@ export class TalerExchangeHttpClient {
     );
 
     switch (resp.status) {
-      case HttpStatusCode.Ok:
-      case HttpStatusCode.Accepted:
+      case HttpStatusCode.Ok: // means there are voluntary checks
+      case HttpStatusCode.Accepted: // means there are requried checks
         return opKnownAlternativeHttpFailure(
           resp,
           resp.status,
           codecForAccountKycStatus(),
         );
-      case HttpStatusCode.NoContent:
+      case HttpStatusCode.NoContent: // no checks can be done
         return opEmptySuccess();
       case HttpStatusCode.Forbidden:
       case HttpStatusCode.NotFound:
@@ -933,8 +928,13 @@ export class TalerExchangeHttpClient {
     switch (resp.status) {
       case HttpStatusCode.Ok:
         return opSuccessFromHttp(resp, codecForAmlStatisticsResponse());
-      case HttpStatusCode.NoContent:
-        return opFixedSuccess({ statistics: [] });
+      case HttpStatusCode.NoContent: {
+        return opFixedSuccess({
+          statistics: names.map(
+            (name) => ({ counter: 0, name }) as EventCounter,
+          ),
+        });
+      }
       case HttpStatusCode.Conflict:
       case HttpStatusCode.NotFound:
       case HttpStatusCode.Forbidden:
@@ -944,6 +944,56 @@ export class TalerExchangeHttpClient {
     }
   }
 
+  /**
+   * https://docs.taler.net/core/api-exchange.html#get--aml-$OFFICER_PUB-decisions
+   *
+   */
+  async getAmlAccounts(
+    auth: OfficerAccount,
+    params: PaginationParams & {
+      highRisk?: boolean;
+      open?: boolean;
+      investigation?: boolean;
+    } = {},
+  ) {
+    const url = new URL(`aml/${auth.id}/accounts`, this.baseUrl);
+
+    addPaginationParams(url, params);
+    if (params.investigation !== undefined) {
+      url.searchParams.set(
+        "investigation",
+        params.investigation ? "YES" : "NO",
+      );
+    }
+    if (params.open !== undefined && params.open) {
+      url.searchParams.set("open", "YES");
+    }
+    if (params.highRisk !== undefined && params.highRisk) {
+      url.searchParams.set("high_risk", "YES");
+    }
+
+    const resp = await this.fetch(url, {
+      headers: {
+        "Taler-AML-Officer-Signature": encodeCrock(
+          signAmlQuery(auth.signingKey),
+        ),
+      },
+    });
+
+    switch (resp.status) {
+      case HttpStatusCode.Ok:
+        return opSuccessFromHttp(resp, codecForAmlDecisionsAccounts());
+      case HttpStatusCode.NoContent:
+        return opFixedSuccess({ accounts: [] });
+      case HttpStatusCode.Forbidden:
+      case HttpStatusCode.NotFound:
+      case HttpStatusCode.Conflict:
+        return opKnownHttpFailure(resp.status, resp);
+      default:
+        return opUnknownHttpFailure(resp);
+    }
+  }
+
   /**
    * https://docs.taler.net/core/api-exchange.html#get--aml-$OFFICER_PUB-decisions
    *
