@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2023 Taler Systems S.A.
+ (C) 2023-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -16,7 +16,7 @@
  SPDX-License-Identifier: AGPL3.0-or-later
 */
 
-import type { CancellationToken } from "./CancellationToken.js";
+import { CancellationToken } from "@gnu-taler/taler-util";
 import { Codec } from "./codec.js";
 import { j2s } from "./helpers.js";
 import {
@@ -71,12 +71,23 @@ export interface HttpRequestOptions {
    * Same semantics as WHATWG fetch.
    */
   redirect?: "follow" | "error" | "manual";
+
+  /**
+   * How to compress the payload
+   */
+  compress?: "gzip" | "deflate";
 }
 
 /**
  * Headers, roughly modeled after the fetch API's headers object.
  */
-export class Headers {
+export interface Headers {
+  get(name: string): string | null;
+  set(name: string, value: string): void;
+  toJSON(): any;
+}
+
+export class HeadersImpl {
   private headerMap = new Map<string, string>();
 
   get(name: string): string | null {
@@ -139,6 +150,7 @@ export async function readTalerErrorResponse(
         requestUrl: httpResponse.requestUrl,
         requestMethod: httpResponse.requestMethod,
         httpStatusCode: httpResponse.status,
+        response: await httpResponse.text(),
         contentType: contentType || "<null>",
       },
       "Error response did not even contain JSON. The request URL might be wrong or the service might be unavailable.",
@@ -333,14 +345,16 @@ export function throwUnexpectedRequestError(
   httpResponse: HttpResponse,
   talerErrorResponse: TalerErrorResponse,
 ): never {
+  const errorDetails = {
+    requestUrl: httpResponse.requestUrl,
+    requestMethod: httpResponse.requestMethod,
+    httpStatusCode: httpResponse.status,
+    errorResponse: talerErrorResponse,
+  };
+  logger.trace(`unexpected request error: ${j2s(errorDetails)}`);
   throw TalerError.fromDetail(
     TalerErrorCode.WALLET_UNEXPECTED_REQUEST_ERROR,
-    {
-      requestUrl: httpResponse.requestUrl,
-      requestMethod: httpResponse.requestMethod,
-      httpStatusCode: httpResponse.status,
-      errorResponse: talerErrorResponse,
-    },
+    errorDetails,
     `Unexpected HTTP status ${httpResponse.status} in response`,
   );
 }
@@ -503,7 +517,7 @@ export function encodeBody(body: unknown): ArrayBuffer {
   } else if (body instanceof ArrayBuffer) {
     return body;
   } else if (body instanceof URLSearchParams) {
-    return body as unknown as ArrayBuffer;
+    return textEncoder.encode(body.toString()).buffer;
   } else if (typeof body === "object" && body.constructor.name === "FormData") {
     return body as ArrayBuffer;
   } else if (typeof body === "object") {
