@@ -29,19 +29,6 @@ import {
  * Implementation of the HTTP request library interface for node.
  */
 export class HttpLibImpl implements HttpRequestLibrary {
-  get(
-    url: string,
-    opt?: HttpRequestOptions | undefined,
-  ): Promise<HttpResponse> {
-    throw new Error("Method not implemented.");
-  }
-  postJson(
-    url: string,
-    body: any,
-    opt?: HttpRequestOptions | undefined,
-  ): Promise<HttpResponse> {
-    throw new Error("Method not implemented.");
-  }
   fetch(
     url: string,
     opt?: HttpRequestOptions | undefined,
