@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2019 Taler Systems S.A.
+ (C) 2019-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -16,19 +16,17 @@
  SPDX-License-Identifier: AGPL3.0-or-later
 */
 
-/**
- * Imports.
- */
 import type { FollowOptions, RedirectableRequest } from "follow-redirects";
 import followRedirects from "follow-redirects";
 import type { ClientRequest, IncomingMessage } from "node:http";
 import { RequestOptions } from "node:http";
 import * as net from "node:net";
+import { deflateSync } from "node:zlib";
 import { TalerError } from "./errors.js";
 import { HttpLibArgs, encodeBody, getDefaultHeaders } from "./http-common.js";
 import {
   DEFAULT_REQUEST_TIMEOUT_MS,
-  Headers,
+  HeadersImpl,
   HttpRequestLibrary,
   HttpRequestOptions,
   HttpResponse,
@@ -38,6 +36,7 @@ import {
   RequestThrottler,
   TalerErrorCode,
   URL,
+  j2s,
   typedArrayConcat,
 } from "./index.js";
 
@@ -47,11 +46,12 @@ const https = followRedirects.https;
 // Work around a node v20.0.0, v20.1.0, and v20.2.0 bug. The issue was fixed
 // in v20.3.0.
 // https://github.com/nodejs/node/issues/47822#issuecomment-1564708870
-// Safe to remove once support for Node v20 is dropped.
+// The issues is not fixed on 22 and 24.
+// More info https://bugs.gnunet.org/view.php?id=9942
 if (
   // check for `node` in case we want to use this in "exotic" JS envs
   process.versions.node &&
-  process.versions.node.match(/20\.[0-2]\.0/)
+  process.versions.node.match(/(20|22|24)\./)
 ) {
   //@ts-ignore
   net.setDefaultAutoSelectFamily(false);
@@ -72,6 +72,7 @@ export class HttpLibImpl implements HttpRequestLibrary {
   private throttle = new RequestThrottler();
   private throttlingEnabled = true;
   private requireTls = false;
+  private idCounter: number = 1;
 
   constructor(args?: HttpLibArgs) {
     this.throttlingEnabled = args?.enableThrottling ?? true;
@@ -87,8 +88,15 @@ export class HttpLibImpl implements HttpRequestLibrary {
 
   async fetch(url: string, opt?: HttpRequestOptions): Promise<HttpResponse> {
     const method = opt?.method?.toUpperCase() ?? "GET";
+    const rid = this.idCounter++;
 
-    logger.trace(`Requesting ${method} ${url}`);
+    if (logger.shouldLogTrace()) {
+      if (opt?.body != null) {
+        logger.trace(`request ${rid} ${method} ${url}: ${opt.body}`);
+      } else {
+        logger.trace(`request ${rid} ${method} ${url}`);
+      }
+    }
 
     const parsedUrl = new URL(url);
     if (this.throttlingEnabled && this.throttle.applyThrottle(url)) {
@@ -99,7 +107,7 @@ export class HttpLibImpl implements HttpRequestLibrary {
           requestUrl: url,
           throttleStats: this.throttle.getThrottleStats(url),
         },
-        `request to origin ${parsedUrl.origin} was throttled`,
+        `request ${rid} to ${parsedUrl.origin} was throttled`,
       );
     }
     if (this.requireTls && parsedUrl.protocol !== "https:") {
@@ -109,7 +117,7 @@ export class HttpLibImpl implements HttpRequestLibrary {
           requestMethod: method,
           requestUrl: url,
         },
-        `request to ${parsedUrl.origin} is not possible with protocol ${parsedUrl.protocol}`,
+        `request ${rid} to ${parsedUrl.origin} is not possible with protocol ${parsedUrl.protocol}`,
       );
     }
     let timeoutMs: number | undefined;
@@ -126,7 +134,7 @@ export class HttpLibImpl implements HttpRequestLibrary {
         requestHeadersMap[key] = value;
       });
     }
-    logger.trace(`request timeout ${timeoutMs} ms`);
+    logger.trace(`request ${rid} timeout ${timeoutMs} ms`);
 
     let reqBody: ArrayBuffer | undefined;
 
@@ -138,6 +146,15 @@ export class HttpLibImpl implements HttpRequestLibrary {
       reqBody = encodeBody(opt.body);
     }
 
+    if (reqBody && opt?.compress === "deflate") {
+      reqBody = deflateSync(reqBody);
+      requestHeadersMap["Content-Encoding"] = "deflate";
+    }
+
+    if (opt?.body instanceof URLSearchParams) {
+      requestHeadersMap["Content-Type"] = "application/x-www-form-urlencoded";
+    }
+
     let path = parsedUrl.pathname;
     if (parsedUrl.search != null) {
       path += parsedUrl.search;
@@ -181,10 +198,9 @@ export class HttpLibImpl implements HttpRequestLibrary {
         return arg + " '" + String(v) + "'";
       }
       console.log(
-        `TALER_API_DEBUG: curl -X ${options.method} "${parsedUrl.href}" ${headers} ${ifUndefined(
-          "-d",
-          payload,
-        )}`,
+        `TALER_API_DEBUG: curl -X ${options.method} "${
+          parsedUrl.href
+        }" ${headers} ${ifUndefined("-d", payload)}`,
       );
     }
 
@@ -206,7 +222,7 @@ export class HttpLibImpl implements HttpRequestLibrary {
           chunks.push(d);
         });
         res.on("end", () => {
-          const headers: Headers = new Headers();
+          const headers: HeadersImpl = new HeadersImpl();
           for (const [k, v] of Object.entries(res.headers)) {
             if (!v) {
               continue;
@@ -228,16 +244,26 @@ export class HttpLibImpl implements HttpRequestLibrary {
             },
             json() {
               const text = textDecoder.decode(data);
-              return JSON.parse(text);
+              const json = JSON.parse(text);
+              if (logger.shouldLogTrace()) {
+                logger.trace(`request ${rid} JSON: ${j2s(json)}`);
+              }
+              return json;
             },
             async text() {
               const text = textDecoder.decode(data);
+              if (logger.shouldLogTrace()) {
+                logger.trace(`request ${rid} TEXT: ${text}`);
+              }
               return text;
             },
           };
+          logger.trace(`request ${rid} status code ${resp.status}`);
           doCleanup();
           if (SHOW_CURL_HTTP_REQUEST) {
-            console.log(`TALER_API_DEBUG: ${res.statusCode} ${textDecoder.decode(data)}`)
+            console.log(
+              `TALER_API_DEBUG: ${res.statusCode} ${textDecoder.decode(data)}`,
+            );
           }
           resolve(resp);
         });
