@@ -57,3 +57,19 @@ export function checkLogicInvariant(b: boolean, m?: string): asserts b {
     }
   }
 }
+
+/**
+ * Check a logic invariant.
+ *
+ * A violation of this invariant means that another peer violated a
+ * protocol invariant.
+ */
+export function checkProtocolInvariant(b: boolean, m?: string): asserts b {
+  if (!b) {
+    if (m) {
+      throw Error(`BUG: protocol invariant failed (${m})`);
+    } else {
+      throw Error("BUG: protocol invariant failed");
+    }
+  }
+}
