@@ -68,7 +68,7 @@ function vn(
   return (1 & ((d - 1) >>> 8)) - 1;
 }
 
-function crypto_verify_16(
+export function crypto_verify_16(
   x: Uint8Array,
   xi: number,
   y: Uint8Array,
@@ -687,7 +687,7 @@ function crypto_stream_xor(
  * https://github.com/floodyberry/poly1305-donna
  */
 
-class poly1305 {
+export class poly1305 {
   buffer = new Uint8Array(16);
   r = new Uint16Array(10);
   h = new Uint16Array(10);
