@@ -25,7 +25,9 @@
 import { AbsoluteTime } from "./time.js";
 import { TransactionState } from "./types-taler-wallet-transactions.js";
 import {
+  ContactEntry,
   ExchangeEntryState,
+  MailboxMessageRecord,
   TalerErrorDetail,
   TransactionIdStr,
 } from "./types-taler-wallet.js";
@@ -34,6 +36,10 @@ export enum NotificationType {
   BalanceChange = "balance-change",
   BankAccountChange = "bank-account-change",
   BackupOperationError = "backup-error",
+  ContactAdded = "contact-added",
+  ContactDeleted = "contact-deleted",
+  MailboxMessageAdded = "mailbox-message-added",
+  MailboxMessageDeleted = "mailbox-message-deleted",
   TransactionStateTransition = "transaction-state-transition",
   ExchangeStateTransition = "exchange-state-transition",
   Idle = "idle",
@@ -116,6 +122,54 @@ export interface ExchangeStateTransitionNotification {
   errorInfo?: ErrorInfoSummary;
 }
 
+/**
+ * Notification emitted when a contact is added
+ */
+export interface ContactAddedNotification {
+  type: NotificationType.ContactAdded;
+
+  /**
+   * The contact that was added
+   */
+  contact: ContactEntry;
+}
+
+/**
+ * Notification emitted when a contact is deleted
+ */
+export interface ContactDeletedNotification {
+  type: NotificationType.ContactDeleted;
+
+  /**
+   * The contact that was deleted
+   */
+  contact: ContactEntry;
+}
+
+/**
+ * Notification emitted when a mailbox message is added
+ */
+export interface MailboxMessageAddedNotification {
+  type: NotificationType.MailboxMessageAdded;
+
+  /**
+   * The message that was added
+   */
+  message: MailboxMessageRecord;
+}
+
+/**
+ * Notification emitted when a mailbox message is deleted
+ */
+export interface MailboxMessageDeletedNotification {
+  type: NotificationType.MailboxMessageDeleted;
+
+  /**
+   * The message that was deleted
+   */
+  message: MailboxMessageRecord;
+}
+
 /**
  * Transaction emitted when a bank account changes.
  */
@@ -289,6 +343,10 @@ export type WalletNotification =
   | BalanceChangeNotification
   | BankAccountChangeNotification
   | BackupOperationErrorNotification
+  | ContactAddedNotification
+  | ContactDeletedNotification
+  | MailboxMessageAddedNotification
+  | MailboxMessageDeletedNotification
   | ExchangeStateTransitionNotification
   | TransactionStateTransitionNotification
   | TaskProgressNotification
