@@ -17,13 +17,14 @@
 import { BitcoinBech32 } from "./bech32.js";
 import { generateFakeSegwitAddress } from "./bitcoin.js";
 import { Codec, Context, DecodingError, renderContext } from "./codec.js";
+import { assertUnreachable } from "./errors.js";
+import { IbanString, parseIban } from "./iban.js";
 import {
   opFixedSuccess,
   opKnownFailure,
   opKnownFailureWithBody,
+  succeedOrThrow,
 } from "./operation.js";
-import { IbanString, parseIban } from "./iban.js";
-import { assertUnreachable } from "./errors.js";
 import {
   decodeCrock,
   encodeCrock,
@@ -86,6 +87,10 @@ export enum PaytoParseError {
   INVALID_TARGET_PATH,
 }
 
+// TODO: use unique symbol to remove compat with string
+// TODO: verify in account aml types if we can merge it since this is also an aml account
+export type PaytoHash = string;
+
 export namespace Paytos {
   export type URI =
     | PaytoUnsupported
@@ -339,12 +344,13 @@ export namespace Paytos {
     bic: string | undefined,
     params: Record<string, string> = {},
   ): PaytoIBAN {
+    iban = iban.toUpperCase() as IbanString;
     return {
       targetType: PaytoType.IBAN,
       iban,
       bic,
       params,
-      normalizedPath: iban.toLocaleLowerCase(),
+      normalizedPath: iban,
       fullPath: !bic ? iban : `${bic}/${iban}`,
       displayName: iban,
     };
@@ -439,6 +445,9 @@ export namespace Paytos {
   //////////////////////
   // parsing function
   ///////////////////////
+  export function asString(p: FullPaytoString): Paytos.URI {
+    return succeedOrThrow(fromString(p))
+  }
 
   export function fromString(
     s: string,
@@ -674,42 +683,58 @@ export namespace Paytos {
       }
     }
   }
+}
 
-  export function codecFullForPaytoString(): Codec<FullPaytoString> {
-    return {
-      decode(x: any, c?: Context): FullPaytoString {
-        if (typeof x !== "string") {
-          throw new DecodingError(
-            `expected string at ${renderContext(c)} but got ${typeof x}`,
-          );
-        }
-        if (!x.startsWith(PAYTO_PREFIX)) {
-          throw new DecodingError(
-            `expected start with payto at ${renderContext(c)} but got "${x}"`,
-          );
-        }
-        return x as FullPaytoString;
-      },
-    };
-  }
+export function codecForPaytoHash(): Codec<PaytoHash> {
+  return {
+    decode(x: any, c?: Context): PaytoHash {
+      // TODO: implement a stronger validation and reject invalid hash
+      // maybe check charset and also length
+      if (typeof x !== "string") {
+        throw new DecodingError(
+          `expected string at ${renderContext(c)} but got ${typeof x}`,
+        );
+      }
+      return x as PaytoHash;
+    },
+  };
+}
+export function codecFullForPaytoString(): Codec<Paytos.FullPaytoString> {
+  return {
+    decode(x: any, c?: Context): Paytos.FullPaytoString {
+      // TODO: implement a stronger validation and reject invalid paytos
+      if (typeof x !== "string") {
+        throw new DecodingError(
+          `expected string at ${renderContext(c)} but got ${typeof x}`,
+        );
+      }
+      if (!x.startsWith(PAYTO_PREFIX)) {
+        throw new DecodingError(
+          `expected start with payto at ${renderContext(c)} but got "${x}"`,
+        );
+      }
+      return x as Paytos.FullPaytoString;
+    },
+  };
+}
 
-  export function codecNormalizedForPaytoString(): Codec<NormalizedPaytoString> {
-    return {
-      decode(x: any, c?: Context): NormalizedPaytoString {
-        if (typeof x !== "string") {
-          throw new DecodingError(
-            `expected string at ${renderContext(c)} but got ${typeof x}`,
-          );
-        }
-        if (!x.startsWith(PAYTO_PREFIX)) {
-          throw new DecodingError(
-            `expected start with payto at ${renderContext(c)} but got "${x}"`,
-          );
-        }
-        return x as NormalizedPaytoString;
-      },
-    };
-  }
+export function codecNormalizedForPaytoString(): Codec<Paytos.NormalizedPaytoString> {
+  return {
+    decode(x: any, c?: Context): Paytos.NormalizedPaytoString {
+      // TODO: implement a stronger validation and reject invalid paytos
+      if (typeof x !== "string") {
+        throw new DecodingError(
+          `expected string at ${renderContext(c)} but got ${typeof x}`,
+        );
+      }
+      if (!x.startsWith(PAYTO_PREFIX)) {
+        throw new DecodingError(
+          `expected start with payto at ${renderContext(c)} but got "${x}"`,
+        );
+      }
+      return x as Paytos.NormalizedPaytoString;
+    },
+  };
 }
 
 /**
