@@ -31,7 +31,7 @@ import { Logger } from "./logging.js";
 import * as nacl from "./nacl-fast.js";
 import { secretbox } from "./nacl-fast.js";
 import { TalerProtocolDuration, TalerProtocolTimestamp } from "./time.js";
-import { CoinPublicKeyString, HashCodeString } from "./types-taler-common.js";
+import { CoinPublicKeyString, EcdhePublicKey, HashCodeString } from "./types-taler-common.js";
 import {
   CoinEnvelope,
   DenomKeyType,
@@ -39,6 +39,8 @@ import {
 } from "./types-taler-exchange.js";
 import { TokenEnvelope, TokenIssuePublicKey } from "./types-taler-merchant.js";
 import { PayWalletData } from "./types-taler-wallet.js";
+import { arrayBuffer } from "node:stream/consumers";
+import { TalerSignaturePurpose } from "./taler_signatures.js";
 
 const isEddsaPubP: unique symbol = Symbol("isEddsaPubP");
 type FlavorEddsaPubP = {
@@ -1095,44 +1097,6 @@ export function hashWire(paytoUri: string, salt: string): string {
   return encodeCrock(r);
 }
 
-export enum TalerSignaturePurpose {
-  MERCHANT_TRACK_TRANSACTION = 1103,
-  WALLET_RESERVE_WITHDRAW = 1200,
-  WALLET_RESERVE_HISTORY = 1208,
-  WALLET_COIN_DEPOSIT = 1201,
-  GLOBAL_FEES = 1022,
-  MASTER_DENOMINATION_KEY_VALIDITY = 1025,
-  MASTER_WIRE_FEES = 1028,
-  MASTER_WIRE_DETAILS = 1030,
-  WALLET_COIN_MELT = 1202,
-  TEST = 4242,
-  MERCHANT_PAYMENT_OK = 1104,
-  MERCHANT_CONTRACT = 1101,
-  MERCHANT_REFUND = 1102,
-  WALLET_COIN_RECOUP = 1203,
-  WALLET_COIN_LINK = 1204,
-  WALLET_ACCOUNT_SETUP = 1205,
-  WALLET_COIN_RECOUP_REFRESH = 1206,
-  WALLET_AGE_ATTESTATION = 1207,
-  WALLET_PURSE_CREATE = 1210,
-  WALLET_PURSE_DEPOSIT = 1211,
-  WALLET_PURSE_MERGE = 1213,
-  WALLET_ACCOUNT_MERGE = 1214,
-  WALLET_PURSE_ECONTRACT = 1216,
-  WALLET_PURSE_DELETE = 1220,
-  WALLET_TOKEN_USE = 1222,
-  WALLET_COIN_HISTORY = 1209,
-  EXCHANGE_CONFIRM_RECOUP = 1039,
-  EXCHANGE_CONFIRM_RECOUP_REFRESH = 1041,
-  AML_DECISION = 1350,
-  AML_QUERY = 1351,
-  MASTER_AML_KEY = 1017,
-  KYC_AUTH = 1360,
-  ANASTASIS_POLICY_UPLOAD = 1400,
-  ANASTASIS_POLICY_DOWNLOAD = 1401,
-  SYNC_BACKUP_UPLOAD = 1450,
-}
-
 export enum WalletAccountMergeFlags {
   /**
    * Not a legal mode!
@@ -1756,3 +1720,565 @@ export function toHexString(byteArray: Uint8Array) {
     "",
   );
 }
+
+// RFC 9180 Hybrid Public-Key Encryption
+// Currently, no agility implemented, we only support
+// DHKEM(X25519, HKDF-SHA256), HKDF-SHA256, ChaCha20Poly1305
+
+// A X25519 public key
+export type HpkePublicKey = Uint8Array
+
+// A X25519 secret key
+export type HpkeSecretKey = Uint8Array;
+
+// An X25519 public key
+export type HpkeEncapsulation = Uint8Array
+
+// This makes sure that sender confusion is
+// avoided.
+// In a non-oneshot API (currently not implemented)
+// this is a requried input.
+export enum HpkeRole {
+  Sender,
+  Receiver,
+}
+
+// We do support pre-shared keys in this API.
+export enum HpkeMode {
+  Base = 0x00,
+  PSK = 0x01,
+}
+
+// Currently only used internally.
+// Necessary when we support anything
+// else than the oneshot APIs
+export interface HpkeContext {
+  key: Uint8Array,
+  nonce: Uint8Array,
+  seq: number,
+  role: HpkeRole,
+}
+
+
+export function hkdf_extract_sha256(
+  ikm: Uint8Array,
+  salt?: Uint8Array,
+): Uint8Array {
+  salt = salt ?? new Uint8Array(64);
+  // extract
+  return hmacSha256(salt, ikm);
+
+}
+
+export function hkdf_expand_sha256(
+  outputLength: number,
+  prk: Uint8Array,
+  info?: Uint8Array,
+): Uint8Array {
+
+  info = info ?? new Uint8Array(0);
+
+  // expand
+  const N = Math.ceil(outputLength / 32);
+  const output = new Uint8Array(N * 32);
+  for (let i = 0; i < N; i++) {
+    let buf;
+    if (i == 0) {
+      buf = new Uint8Array(info.byteLength + 1);
+      buf.set(info, 0);
+    } else {
+      buf = new Uint8Array(info.byteLength + 1 + 32);
+      for (let j = 0; j < 32; j++) {
+        buf[j] = output[(i - 1) * 32 + j];
+      }
+      buf.set(info, 32);
+    }
+    buf[buf.length - 1] = i + 1;
+    const chunk = hmacSha256(prk, buf);
+    output.set(chunk, i * 32);
+  }
+
+  return output.slice(0, outputLength);
+}
+
+
+export function hpkeLabeledExpand(
+  ctx: string,
+  prk: Uint8Array,
+  label: string,
+  info: Uint8Array,
+  suiteId: Uint8Array,
+  outLength: number) : Uint8Array {
+  const outLenBytes = new ArrayBuffer(2);
+  const out = new DataView(outLenBytes);
+  out.setUint16(0, outLength);
+  const labelBytes = stringToBytes(label);
+  const ctxBytes = stringToBytes(ctx);
+  const labeledInfo = new Uint8Array([...(new Uint8Array(outLenBytes)), ...ctxBytes, ...suiteId, ...labelBytes, ...info]);
+  return hkdf_expand_sha256(outLength, prk, labeledInfo);
+}
+
+export function hpkeLabeledExtract(
+  ctx: string,
+  label: Uint8Array,
+  ikm: Uint8Array,
+  suiteId: Uint8Array,
+  salt?: Uint8Array): Uint8Array {
+  const ctxBytes = stringToBytes(ctx);
+  const labeledIkm = new Uint8Array([...ctxBytes, ...suiteId, ...label, ...ikm]);
+  return hkdf_extract_sha256(labeledIkm, salt);
+
+}
+
+export function ecdh_x25519(
+  ecdhPriv: Uint8Array,
+  ecdhPub: Uint8Array,
+): Uint8Array {
+  var checkbyte = 0;
+  const res = nacl.scalarMult(ecdhPriv, ecdhPub);
+  for (let i = 0; i < res.length; i++) {
+    checkbyte = res[i] | checkbyte;
+  }
+  if (checkbyte == 0) {
+    throw Error("x25519 failed");
+  }
+  return res;
+}
+
+export function hpkeKemEncapsNorand(
+  pkR: Uint8Array,
+  skE: Uint8Array): [HpkeEncapsulation, Uint8Array] {
+  const enc = ecdhGetPublic(skE);
+  const kem_context = new Uint8Array([...enc, ...pkR]);
+  const dh = ecdh_x25519 (skE, pkR);
+  const suiteId = new Uint8Array([0x4B,0x45,0x4D,0x00,0x20]);
+  const prk = hpkeLabeledExtract("HPKE-v1",
+                             stringToBytes("eae_prk"),
+                             dh,
+                             suiteId);
+  const ss = hpkeLabeledExpand("HPKE-v1",
+                           prk,
+                           "shared_secret",
+                           kem_context,
+                           suiteId,
+                           32);
+  return [enc, ss];
+}
+
+export function hpkeKemDecaps(
+  skR: Uint8Array,
+  enc: Uint8Array): Uint8Array {
+  const pkR = ecdhGetPublic(skR);
+  const dh = ecdh_x25519(skR, enc);
+  const kem_context = new Uint8Array([...enc, ...pkR]);
+  const suiteId = new Uint8Array([0x4B,0x45,0x4D,0x00,0x20]);
+  const prk = hpkeLabeledExtract("HPKE-v1",
+                             stringToBytes("eae_prk"),
+                             dh,
+                             suiteId);
+  const ss = hpkeLabeledExpand("HPKE-v1",
+                           prk,
+                           "shared_secret",
+                           kem_context,
+                           suiteId,
+                           32);
+  return ss;
+
+}
+
+export function hpkeKeySchedule(
+  role: HpkeRole,
+  mode: HpkeMode,
+  sharedSecret: Uint8Array,
+  info: Uint8Array,
+  psk?: Uint8Array,
+  pskId?: Uint8Array) : HpkeContext {
+  const suiteId = new ArrayBuffer(4 + 3 * 2);
+  const v = new DataView(suiteId);
+  v.setUint8(0, 0x48); // "H"
+  v.setUint8(1, 0x50); // "P"
+  v.setUint8(2, 0x4B); // "K"
+  v.setUint8(3, 0x45); // "E"
+  v.setUint16(4, 32); // kemID (DHKEM(X25519, HKDF-256))
+  v.setUint16(6, 1); // KDF ID (HKDF-SHA256)
+  v.setUint16(8, 3); // AEAD ID (ChaChaPoly1305)
+
+  if (mode == HpkeMode.PSK) {
+    if (!psk) {
+      throw Error("Mode is PSK, but PSK not provided");
+    }
+    if (!psk) {
+      throw Error("Mode is PSK, but PSKID not provided");
+    }
+  } else if (mode == HpkeMode.Base) {
+    if (psk) {
+      throw Error("PSK provided, but mode is Base");
+    }
+    if (pskId) {
+      throw Error("PSKID provided, but mode is Base");
+    }
+  }
+  const suiteIdBytes = new Uint8Array(suiteId);
+  const pskIdHash = hpkeLabeledExtract("HPKE-v1",
+                                   stringToBytes("psk_id_hash"),
+                                   new Uint8Array([]),
+                                   suiteIdBytes);
+  const infoHash = hpkeLabeledExtract("HPKE-v1",
+                                  stringToBytes("info_hash"),
+                                  info,
+                                  suiteIdBytes);
+  const keyScheduleCtx = new Uint8Array([mode, ...pskIdHash, ...infoHash]);
+  const secret = hpkeLabeledExtract("HPKE-v1",
+                                stringToBytes("secret"),
+                                psk || new Uint8Array([]),
+                                suiteIdBytes,
+                                sharedSecret);
+  const ctxKey = hpkeLabeledExpand("HPKE-v1",
+                               secret,
+                               "key",
+                               keyScheduleCtx,
+                               suiteIdBytes,
+                               32); // key 32 bytes / 256 bit
+  const ctxNonce = hpkeLabeledExpand("HPKE-v1",
+                                 secret,
+                                 "base_nonce",
+                                 keyScheduleCtx,
+                                 suiteIdBytes,
+                                 12); // nonce 12 bytes
+  // We do not support secret export hence no secret export labeledExtract
+
+  return {key: ctxKey, nonce: ctxNonce, role: role, seq: 0};
+}
+
+
+export function hpkeSenderSetupNorand(
+  pkR: HpkePublicKey,
+  skE: HpkeSecretKey,
+  info: Uint8Array) : [HpkeEncapsulation, HpkeContext] {
+  const [enc, sharedSecret] = hpkeKemEncapsNorand(pkR, skE);
+  const ctx = hpkeKeySchedule(HpkeRole.Sender,
+                              HpkeMode.Base,
+                              sharedSecret,
+                              info);
+  return [enc, ctx];
+}
+
+
+export function hpkeSenderSetup(
+  pkR: HpkePublicKey,
+  info: Uint8Array) : [HpkeEncapsulation, HpkeContext] {
+  const keypair = createEcdheKeyPair();
+  return hpkeSenderSetupNorand(pkR, keypair.ecdhePriv, info);
+}
+
+export function hpkeComputeNonce(
+  ctx: HpkeContext) : Uint8Array {
+  const nonce = new ArrayBuffer(12);
+  const seqNboBuf = new ArrayBuffer(8);
+  const v = new DataView(nonce);
+  const vSeq = new DataView(seqNboBuf);
+  const seqNbo = new Uint8Array(seqNboBuf);
+  vSeq.setBigUint64(0, BigInt(ctx.seq));
+  const offset = 12 - 8; // nonce length - sequence counter of 64 bits
+  let i = 0;
+  let j = 0;
+  for (i = 0; i < 12; i++) {
+    if (i < offset) {
+      v.setUint8(i, ctx.nonce[i]);
+    } else {
+      v.setUint8(i, ctx.nonce[i] ^ seqNbo[j++]);
+    }
+  }
+  return new Uint8Array(nonce);
+}
+
+export function hpkeSealOneshotNorand(
+  pkR: HpkePublicKey,
+  skE: HpkeSecretKey,
+  info: Uint8Array,
+  aad: Uint8Array,
+  plaintext: Uint8Array): Uint8Array {
+  const [enc, ctx] = hpkeSenderSetupNorand(pkR, skE, info);
+  const nonce = hpkeComputeNonce(ctx);
+  const ct = chacha20poly1305_ietf_encrypt(plaintext,
+                                           aad,
+                                           nonce,
+                                           ctx.key);
+  ctx.seq++;
+  return new Uint8Array([...enc, ...ct]);
+}
+
+export function hpkeSealOneshot(
+  pkR: HpkePublicKey,
+  info: Uint8Array,
+  aad: Uint8Array,
+  plaintext: Uint8Array): Uint8Array {
+  const [enc, ctx] = hpkeSenderSetup(pkR, info);
+  const nonce = hpkeComputeNonce(ctx);
+  const ct = chacha20poly1305_ietf_encrypt(plaintext,
+                                           aad,
+                                           nonce,
+                                           ctx.key);
+  ctx.seq++;
+  return new Uint8Array([...enc, ...ct]);
+}
+
+
+export function hpkeReceiverSetup(
+  enc: Uint8Array,
+  skR: HpkeSecretKey,
+  info: Uint8Array) : HpkeContext {
+  const sharedSecret = hpkeKemDecaps(skR, enc);
+  return hpkeKeySchedule(HpkeRole.Receiver,
+                         HpkeMode.Base,
+                         sharedSecret,
+                         info);
+}
+
+export function hpkeOpenOneshot(
+  skR: HpkeSecretKey,
+  info: Uint8Array,
+  aad: Uint8Array,
+  ciphertext: Uint8Array): Uint8Array | undefined {
+  try {
+    const enc = ciphertext.slice(0, 32);
+    const ctx = hpkeReceiverSetup(enc,
+                                  skR,
+                                  info);
+    const nonce = hpkeComputeNonce(ctx);
+    return chacha20poly1305_ietf_decrypt(ciphertext.slice(32),
+                                         aad,
+                                         nonce,
+                                         ctx.key);
+  } catch (e) {
+    logger.error("hpkeOpenOneshot failed:" + e)
+    return undefined;
+  }
+}
+
+export function hpkeSecretKeyGetPublic(
+  sk: HpkeSecretKey) : HpkePublicKey {
+  return ecdhGetPublic(sk as Uint8Array) as HpkePublicKey;
+}
+
+export function hpkeCreateSecretKey() : HpkeSecretKey {
+  const keypair = createEcdheKeyPair();
+  return keypair.ecdhePriv as HpkeSecretKey;
+}
+
+// RFC 8439 ChaCha20-Poly1305 (IETF variants)
+
+function chacha20_toUint32(data: Uint8Array | number[], index: number): number {
+  return data[index++] ^ (data[index++] << 8) ^ (data[index++] << 16) ^ (data[index] << 24)
+}
+
+function chacha20_rotl(data: number, shift: number): number {
+  return ((data << shift) | (data >>> (32 - shift)))
+}
+
+export function chacha20_quarterround(
+  out: number[],
+  a: number,
+  b: number,
+  c: number,
+  d: number) {
+  out[d] = chacha20_rotl(out[d] ^ (out[a] += out[b]), 16);
+  out[b] = chacha20_rotl(out[b] ^ (out[c] += out[d]), 12);
+  out[d] = chacha20_rotl(out[d] ^ (out[a] += out[b]), 8);
+  out[b] = chacha20_rotl(out[b] ^ (out[c] += out[d]), 7);
+
+  out[a] >>>= 0;
+  out[b] >>>= 0;
+  out[c] >>>= 0;
+  out[d] >>>= 0;
+}
+
+export function chacha20_block(
+  input: number[]) : Uint8Array {
+  const out = Array<number>(64).fill(0);
+  // copy param array to x
+  const x = Array.from(input);
+  var i = 0;
+  var bytesWritten = 0;
+
+  // 10 loops × 2 rounds/loop = 20 rounds
+  for (i = 0; i < 20; i += 2) {
+    // Odd round
+    chacha20_quarterround(x, 0, 4, 8, 12)
+    chacha20_quarterround(x, 1, 5, 9, 13)
+    chacha20_quarterround(x, 2, 6, 10, 14)
+    chacha20_quarterround(x, 3, 7, 11, 15)
+
+    // Even round
+    chacha20_quarterround(x, 0, 5, 10, 15)
+    chacha20_quarterround(x, 1, 6, 11, 12)
+    chacha20_quarterround(x, 2, 7, 8, 13)
+    chacha20_quarterround(x, 3, 4, 9, 14)
+  }
+
+  for (i = 0; i < 16; i++) {
+    // out[i] = x[i] + in[i]
+    let tmp = x[i] + input[i]
+
+    // update pad
+    out[bytesWritten++] = tmp & 0xFF
+    out[bytesWritten++] = (tmp >>> 8) & 0xFF
+    out[bytesWritten++] = (tmp >>> 16) & 0xFF
+    out[bytesWritten++] = (tmp >>> 24) & 0xFF
+  }
+  return new Uint8Array([...out]);
+}
+
+export function chacha20_ietf_xor(
+  key: Uint8Array,
+  nonce: Uint8Array,
+  m: Uint8Array,
+  c?: number) : Uint8Array {
+  invariant (0 != m.length);
+  var bytesWritten = 0;
+  const out = new Uint8Array(m.length);
+  const sigma: number[] = [0x61707865, 0x3320646e, 0x79622d32, 0x6b206574];
+  const keybytes = [
+    chacha20_toUint32(key, 0),
+    chacha20_toUint32(key, 4),
+    chacha20_toUint32(key, 8),
+    chacha20_toUint32(key, 12),
+    chacha20_toUint32(key, 16),
+    chacha20_toUint32(key, 20),
+    chacha20_toUint32(key, 24),
+    chacha20_toUint32(key, 28),
+  ];
+  const noncebytes = [
+    chacha20_toUint32(nonce, 0),
+    chacha20_toUint32(nonce, 4),
+    chacha20_toUint32(nonce, 8),
+  ];
+  const param: number[] = [
+    ...sigma,
+    ...keybytes,
+    c? c: 0, // Counter, index is 12
+    ...noncebytes,
+  ];
+  for (let i = 0; i < m.length; i++) {
+    var pad;
+    if (bytesWritten === 0 || bytesWritten === 64) {
+      // generate new block //
+
+      pad = chacha20_block(param);
+      // counter increment
+      param[12]++;
+
+      // bytes counter for wrap around
+      bytesWritten = 0;
+    }
+    invariant(pad != undefined);
+    out[i] = m[i] ^ pad[bytesWritten++]
+  }
+
+  return out;
+}
+
+
+export function chacha20_ietf(
+  clen: number,
+  key: Uint8Array,
+  nonce: Uint8Array) : Uint8Array {
+  var bytesWritten = 0;
+  const m = Array<number>(clen).fill(0);
+  const out = new Uint8Array(m.length);
+  const sigma: number[] = [0x61707865, 0x3320646e, 0x79622d32, 0x6b206574];
+  const keybytes = [
+    chacha20_toUint32(key, 0),
+    chacha20_toUint32(key, 4),
+    chacha20_toUint32(key, 8),
+    chacha20_toUint32(key, 12),
+    chacha20_toUint32(key, 16),
+    chacha20_toUint32(key, 20),
+    chacha20_toUint32(key, 24),
+    chacha20_toUint32(key, 28),
+  ];
+  const noncebytes = [
+    chacha20_toUint32(nonce, 0),
+    chacha20_toUint32(nonce, 4),
+    chacha20_toUint32(nonce, 8),
+  ];
+  const param: number[] = [
+    ...sigma,
+    ...keybytes,
+    0, // Counter, index is 12
+    ...noncebytes,
+  ];
+  for (let i = 0; i < m.length; i++) {
+    var pad;
+    if (bytesWritten === 0 || bytesWritten === 64) {
+      // generate new block //
+
+      pad = chacha20_block(param);
+      // counter increment
+      param[12]++;
+
+      // bytes counter for wrap around
+      bytesWritten = 0;
+    }
+    invariant(pad != undefined);
+    out[i] = m[i] ^ pad[bytesWritten++]
+  }
+
+  return out;
+}
+
+export function chacha20poly1305_ietf_encrypt(
+  m: Uint8Array,
+  ad: Uint8Array,
+  npub: Uint8Array,
+  k: Uint8Array) : Uint8Array {
+  invariant(k.length == 32);
+  invariant(npub.length == 12);
+  const slenBuf = new ArrayBuffer(8);
+  const slenDv = new DataView(slenBuf);
+  const pad0 = new Uint8Array(16).fill(0);
+  const block0 = chacha20_ietf(64, k, npub);
+  const tag = new Uint8Array(16);
+  const p = new nacl.poly1305(block0);
+  p.update(ad, 0, ad.length);
+  p.update(pad0, 0, (0x10 - ad.length) & 0xf);
+  const ct = chacha20_ietf_xor(k, npub, m, 1);
+  p.update(ct, 0, ct.length);
+  p.update(pad0, 0, (0x10 - m.length) & 0xf);
+  slenDv.setBigUint64(0, BigInt(ad.length), true);
+  p.update(new Uint8Array(slenBuf), 0, 8);
+  slenDv.setBigUint64(0, BigInt(ct.length), true);
+  p.update(new Uint8Array(slenBuf), 0, 8);
+  p.finish(tag, 0);
+  return new Uint8Array([...ct, ...tag]);
+}
+
+export function chacha20poly1305_ietf_decrypt(
+  ct: Uint8Array,
+  ad: Uint8Array,
+  npub: Uint8Array,
+  k: Uint8Array) : Uint8Array | undefined {
+  invariant(k.length == 32);
+  invariant(npub.length == 12);
+  const slenBuf = new ArrayBuffer(8);
+  const slenDv = new DataView(slenBuf);
+  const pad0 = new Uint8Array(16).fill(0);
+  const block0 = chacha20_ietf(64, k, npub);
+  const tag = new Uint8Array(16);
+  const p = new nacl.poly1305(block0);
+  const mlen = ct.length - tag.length;
+  p.update(ad, 0, ad.length);
+  p.update(pad0, 0, (0x10 - ad.length) & 0xf);
+  p.update(ct, 0, mlen);
+  p.update(pad0, 0, (0x10 - (mlen)) & 0xf);
+  slenDv.setBigUint64(0, BigInt(ad.length), true);
+  p.update(new Uint8Array(slenBuf), 0, 8);
+  slenDv.setBigUint64(0, BigInt(mlen), true);
+  p.update(new Uint8Array(slenBuf), 0, 8);
+  p.finish(tag, 0);
+  if (nacl.crypto_verify_16(tag, 0, ct, mlen) !== 0) {
+    return undefined;
+  };
+  const m = chacha20_ietf_xor(k, npub, ct.slice(0, mlen), 1);
+  return m;
+}
