@@ -26,9 +26,9 @@
 import { AmountJson, Amounts } from "./amounts.js";
 import { Logger } from "./logging.js";
 
-import nodejs_fs from "fs";
-import nodejs_os from "os";
-import nodejs_path from "path";
+import nodejs_fs from "node:fs";
+import nodejs_os from "node:os";
+import nodejs_path from "node:path";
 
 const logger = new Logger("talerconfig.ts");
 
@@ -91,12 +91,19 @@ export interface ConfigSource {
 export type ConfigSourceDef = { [x: string]: ConfigSource | undefined };
 
 export const ConfigSources = {
-  ["taler"]: {
-    projectName: "taler",
-    componentName: "taler",
-    installPathBinary: "taler-config",
-    baseConfigVarname: "TALER_BASE_CONFIG",
-    prefixVarname: "TALER_PREFIX",
+  ["taler-exchange"]: {
+    projectName: "taler-exchange",
+    componentName: "taler-exchange",
+    installPathBinary: "taler-exchange-config",
+    baseConfigVarname: "TALER_EXCHANGE_BASE_CONFIG",
+    prefixVarname: "TALER_EXCHANGE_PREFIX",
+  } satisfies ConfigSource,
+  ["taler-merchant"]: {
+    projectName: "taler-merchant",
+    componentName: "taler-merchant",
+    installPathBinary: "taler-merchant-config",
+    baseConfigVarname: "TALER_MERCHANT_BASE_CONFIG",
+    prefixVarname: "TALER_MERCHANT_PREFIX",
   } satisfies ConfigSource,
   ["libeufin-bank"]: {
     projectName: "libeufin",
@@ -119,9 +126,16 @@ export const ConfigSources = {
     baseConfigVarname: "GNUNET_BASE_CONFIG",
     prefixVarname: "GNUNET_PREFIX",
   } satisfies ConfigSource,
+  ["donau"]: {
+    projectName: "donau",
+    componentName: "donau",
+    installPathBinary: "donau-config",
+    baseConfigVarname: "DONAU_BASE_CONFIG",
+    prefixVarname: "DONAU_PREFIX",
+  } satisfies ConfigSource,
 } satisfies ConfigSourceDef;
 
-const defaultConfigSource: ConfigSource = ConfigSources.taler;
+const defaultConfigSource: ConfigSource = ConfigSources["taler-exchange"];
 
 export class ConfigValue<T> {
   constructor(
@@ -598,6 +612,19 @@ export class Configuration {
     return this.sectionMap[secNorm]?.entries[optNorm];
   }
 
+  /**
+   * Load configuration values from a string in the format
+   * of a configuration file.  Directives are not allowed.
+   *
+   *
+   * Has the same effect as setting each option manually.
+   */
+  loadFromString(s: string): void {
+    this.internalLoadFromString(s, false, {
+      banDirectives: true,
+    });
+  }
+
   setString(section: string, option: string, value: string): void {
     const sec = this.provideSection(section);
     sec.entries[option.toUpperCase()] = {
@@ -708,6 +735,8 @@ export class Configuration {
   }
 
   private loadDefaultsFromDir(dirname: string): void {
+    const exists = nodejs_fs.existsSync(dirname);
+    if (!exists) return;
     const files = nodejs_fs.readdirSync(dirname);
     for (const f of files) {
       const fn = nodejs_path.join(dirname, f);
