@@ -27,7 +27,7 @@ import { Codec, Context, DecodingError, renderContext } from "./codec.js";
 import { HostPortPath, Paytos } from "./payto.js";
 import { Result } from "./result.js";
 import { TalerErrorCode } from "./taler-error-codes.js";
-import { AmountString } from "./types-taler-common.js";
+import { AmountString, EddsaPublicKeyString, HashCodeString } from "./types-taler-common.js";
 import { URL, URLSearchParams } from "./url.js";
 import {
   opFixedSuccess,
@@ -50,7 +50,8 @@ export type TalerUri =
   | WithdrawUriResult
   | WithdrawExchangeUri
   | AddExchangeUri
-  | WithdrawalTransferResultUri;
+  | WithdrawalTransferResultUri
+  | AddContactUri;
 
 declare const __action_str: unique symbol;
 export type TalerUriString = string & { [__action_str]: true };
@@ -107,6 +108,7 @@ export namespace TalerUris {
   export type URI = TalerUri;
 
   const supported_targets: Record<TalerUriAction, true> = {
+    "add-contact": true,
     "add-exchange": true,
     "dev-experiment": true,
     pay: true,
@@ -232,6 +234,22 @@ export namespace TalerUris {
       exchangeBaseUrl,
     };
   }
+  export function createTalerAddContact(
+    alias: string,
+    aliasType: string,
+    mailboxUri: string,
+    mailboxIdentity: string,
+    sourceBaseUrl: string,
+  ): AddContactUri {
+    return {
+      type: TalerUriAction.AddContact,
+      alias: alias,
+      aliasType: aliasType,
+      mailboxBaseUri: mailboxUri,
+      mailboxIdentity: mailboxIdentity,
+      sourceBaseUrl: sourceBaseUrl,
+    };
+  }
   export function createTalerWithdrawalTransferResult(
     ref: string,
     opts: {
@@ -273,6 +291,10 @@ export namespace TalerUris {
         if (p.status) result.push(["status", p.status]);
         return result;
       }
+      case TalerUriAction.AddContact: {
+        if (p.sourceBaseUrl) result.push(["sourceBaseUrl", p.sourceBaseUrl]);
+        return result;
+      }
       case TalerUriAction.Refund:
       case TalerUriAction.PayPush:
       case TalerUriAction.PayPull:
@@ -310,6 +332,7 @@ export namespace TalerUris {
       case TalerUriAction.Restore:
       case TalerUriAction.DevExperiment:
       case TalerUriAction.WithdrawalTransferResult:
+      case TalerUriAction.AddContact:
         return TALER_PREFIX;
       default:
         assertUnreachable(p);
@@ -349,6 +372,8 @@ export namespace TalerUris {
         return `/${p.devExperimentId}`;
       case TalerUriAction.WithdrawalTransferResult:
         return `/`;
+      case TalerUriAction.AddContact:
+        return `/${p.aliasType}/${p.alias}/${asHost(p.mailboxBaseUri as HostPortPath)}/${p.mailboxIdentity}`
       default:
         assertUnreachable(p);
     }
@@ -802,6 +827,35 @@ export namespace TalerUris {
           }),
         );
       }
+      case TalerUriAction.AddContact: {
+        // check number of segments
+        if (cs.length < 4) {
+          return opKnownFailureWithBody(TalerUriParseError.COMPONENTS_LENGTH, {
+            uriType,
+          });
+        }
+
+        const mailboxBaseUri = Paytos.parseHostPortPath2(
+          cs[2],
+          cs.slice(2, cs.length - 2).join("/"),
+          scheme,
+        );
+        if (!mailboxBaseUri) {
+          return opKnownFailureWithBody(
+            TalerUriParseError.INVALID_TARGET_PATH,
+            {
+              pos: 0 as const,
+              uriType,
+              error: mailboxBaseUri,
+            },
+          );
+        }
+        const mailboxIdentity = cs[cs.length - 1];
+
+        return opFixedSuccess<URI>(
+          createTalerAddContact(cs[0], cs[1], mailboxBaseUri, mailboxIdentity, params["sourceBaseUrl"]),
+        );
+      }
       default: {
         assertUnreachable(uriType);
       }
@@ -889,6 +943,15 @@ export interface WithdrawalTransferResultUri {
   status?: "success" | "aborted";
 }
 
+export interface AddContactUri {
+  type: TalerUriAction.AddContact;
+  alias: string;
+  aliasType: string;
+  mailboxBaseUri: string;
+  mailboxIdentity: HashCodeString;
+  sourceBaseUrl: string;
+}
+
 /**
  * Parse a taler[+http]://withdraw URI.
  * Return undefined if not passed a valid URI.
@@ -981,6 +1044,42 @@ export function parseAddExchangeUriWithError(s: string) {
   return Result.of(result);
 }
 
+/**
+ * Parse a taler[+http]://add-contact URI.
+ * Return undefined if not passed a valid URI.
+ */
+export function parseAddContactUriWithError(s: string) {
+  const pi = parseProtoInfoWithError(s, "add-contact");
+  if (pi.tag === "error") {
+    return pi;
+  }
+  const parts = pi.value.rest.split("/");
+
+  if (parts.length < 4) {
+    return Result.error(TalerErrorCode.WALLET_TALER_URI_MALFORMED);
+  }
+  const mailboxBaseUri = parts[2];
+  const pathSegments = parts.slice(3, parts.length - 2);
+  const lastPart = parts[parts.length-1];
+  const q = new URLSearchParams(lastPart ?? "");
+  const mailboxIdentity = lastPart.split("?")[0];
+  const sourceBaseUrl = q.get("sourceBaseUrl") ?? "";
+  const mailboxHostPort = Paytos.parseHostPortPath2(
+    mailboxBaseUri,
+    pathSegments.join("/"),
+    pi.value.innerProto,
+  );
+  const result: AddContactUri = {
+    type: TalerUriAction.AddContact,
+    aliasType: parts[0],
+    alias: parts[1],
+    mailboxBaseUri: mailboxHostPort!,
+    mailboxIdentity: mailboxIdentity,
+    sourceBaseUrl: sourceBaseUrl,
+  };
+  return Result.of(result);
+}
+
 /**
  *
  * @deprecated use parseWithdrawUriWithError
@@ -991,6 +1090,16 @@ export function parseAddExchangeUri(s: string): AddExchangeUri | undefined {
   return r.value;
 }
 
+/**
+ *
+ * @deprecated use parseWithdrawUriWithError
+ */
+export function parseAddContactUri(s: string): AddContactUri | undefined {
+  const r = parseAddContactUriWithError(s);
+  if (r.tag === "error") return undefined;
+  return r.value;
+}
+
 export enum TalerUriAction {
   /**
    * https://lsd.gnunet.org/lsd0006/#section-5.1
@@ -1036,6 +1145,11 @@ export enum TalerUriAction {
    * https://lsd.gnunet.org/lsd0006/#section-5.11
    */
   WithdrawalTransferResult = "withdrawal-transfer-result",
+  /**
+   * FIXME: LSD
+   * Add a contact to the wallet
+   */
+  AddContact = "add-contact"
 }
 
 interface TalerUriProtoInfo {
@@ -1115,6 +1229,7 @@ const parsers: { [A in TalerUriAction]: Parser } = {
   [TalerUriAction.DevExperiment]: parseDevExperimentUri,
   [TalerUriAction.WithdrawExchange]: parseWithdrawExchangeUri,
   [TalerUriAction.AddExchange]: parseAddExchangeUri,
+  [TalerUriAction.AddContact]: parseAddContactUri,
   [TalerUriAction.WithdrawalTransferResult]: () => {
     throw new Error("not supported");
   },
@@ -1176,6 +1291,9 @@ export function stringifyTalerUri(uri: TalerUri): string {
     case TalerUriAction.AddExchange: {
       return stringifyAddExchange(uri);
     }
+    case TalerUriAction.AddContact: {
+      return stringifyAddContact(uri);
+    }
     case TalerUriAction.WithdrawalTransferResult: {
       throw Error("not supported");
     }
@@ -1577,6 +1695,27 @@ export function stringifyAddExchange({
   return `${proto}://add-exchange/${path}`;
 }
 
+/**
+ * @deprecated
+ * @param param0
+ * @returns
+ */
+export function stringifyAddContact({
+  alias,
+  aliasType,
+  mailboxBaseUri: mailboxBaseUri,
+  mailboxIdentity: mailboxIdentity,
+  sourceBaseUrl,
+}: Omit<AddContactUri, "type">): string {
+  const { proto, path } = getUrlInfo(mailboxBaseUri);
+  const baseUri = `${proto}://add-contact/${aliasType}/${alias}/${path}${mailboxIdentity}`;
+  if (sourceBaseUrl) {
+    return baseUri + `?sourceBaseUrl=${encodeURIComponent(sourceBaseUrl)}`;
+  } else {
+    return baseUri;
+  }
+}
+
 /**
  * @deprecated
  * @param param0
