@@ -586,6 +586,18 @@ test("taler-new add exchange URI (stringify)", (t) => {
   t.deepEqual(url, "taler://add-exchange/exchange.demo.taler.net/");
 });
 
+test("taler-new add contact URI (stringify)", (t) => {
+  const url = TalerUris.toString({
+    type: TalerUriAction.AddContact,
+    alias: "bob@example.com",
+    aliasType: "email",
+    mailboxBaseUri: "https://mailbox.example.com/mb",
+    mailboxIdentity: "SOMEHADDR",
+    sourceBaseUrl: "https://taldir.example.com",
+  });
+  t.deepEqual(url, "taler://add-contact/email/bob@example.com/mailbox.example.com/mb/SOMEHADDR?sourceBaseUrl=https%3A%2F%2Ftaldir.example.com");
+});
+
 /**
  * wrong uris
  */
