@@ -15,7 +15,7 @@
  */
 
 import test from "ava";
-import { AbsoluteTime, Duration } from "./time.js";
+import { Duration } from "./time.js";
 
 test("duration parsing", (t) => {
   const d1 = Duration.fromPrettyString("1h");
