@@ -110,6 +110,14 @@ export namespace TalerProtocolDuration {
 }
 
 export namespace TalerProtocolTimestamp {
+  export function isTimestamp(x: unknown): x is TalerProtocolTimestamp {
+    return (
+      typeof x === "object" &&
+      x !== null &&
+      "t_s" in x &&
+      (typeof x.t_s === "number" || x.t_s === "never")
+    );
+  }
   export function now(): TalerProtocolTimestamp {
     return AbsoluteTime.toProtocolTimestamp(AbsoluteTime.now());
   }
