@@ -34,6 +34,7 @@ import {
   codecForConstString,
   codecForEither,
   codecForList,
+  codecForNumber,
   codecForString,
   codecOptional,
 } from "./codec.js";
@@ -41,13 +42,18 @@ import {
   TalerPreciseTimestamp,
   TalerProtocolDuration,
   TalerProtocolTimestamp,
+  codecForPreciseTimestamp,
 } from "./time.js";
 import {
   AmountString,
   InternationalizedString,
   codecForInternationalizedString,
 } from "./types-taler-common.js";
-import { MerchantInfo, codecForMerchantInfo } from "./types-taler-merchant.js";
+import {
+  MerchantContractTerms,
+  MerchantInfo,
+  codecForMerchantInfo,
+} from "./types-taler-merchant.js";
 import {
   RefreshReason,
   ScopeInfo,
@@ -98,11 +104,71 @@ export interface TransactionsRequest {
   filterByState?: TransactionStateFilter;
 }
 
+export interface GetTransactionsV2Request {
+  /**
+   * Return only transactions in the given currency.
+   */
+  currency?: string;
+
+  /**
+   * Return only transactions in the given scopeInfo
+   */
+  scopeInfo?: ScopeInfo;
+
+  /**
+   * If true, include all refreshes in the transactions list.
+   */
+  includeRefreshes?: boolean;
+
+  /**
+   * If true, include transactions that would usually be filtered out.
+   * Implies includeRefreshes.
+   */
+  includeAll?: boolean;
+
+  /**
+   * Only return transactions before/after this offset.
+   */
+  offsetTransactionId?: TransactionIdStr;
+
+  /**
+   * Only return transactions before/after the transaction with this
+   * timestamp.
+   *
+   * Used as a fallback if the offsetTransactionId was deleted.
+   */
+  offsetTimestamp?: TalerPreciseTimestamp;
+
+  /**
+   * Number of transactions to return.
+   *
+   * When the limit is positive, results are returned
+   * in ascending order of their timestamp.  If no offset is specified,
+   * the result list begins with the first transaction.
+   * If an offset is specified, transactions after the offset are returned.
+   *
+   * When the limit is negative, results are returned
+   * in descending order of their timestamp.  If no offset is specified,
+   * the result list begins with with the last transaction.
+   * If an offset is specified, transactions before the offset are returned.
+   */
+  limit?: number;
+
+  /**
+   * Filter transactions by their state / state category.
+   *
+   * If not specified, all transactions are returned.
+   */
+  filterByState?: "final" | "nonfinal" | "done";
+}
+
 export interface TransactionState {
   major: TransactionMajorState;
   minor?: TransactionMinorState;
 }
 
+export type TransactionStateWildcard = "*";
+
 export enum TransactionMajorState {
   // No state, only used when reporting transitions into the initial state
   None = "none",
@@ -127,9 +193,11 @@ export enum TransactionMinorState {
   Unknown = "unknown",
   Deposit = "deposit",
   KycRequired = "kyc",
+  KycInit = "kyc-init",
   MergeKycRequired = "merge-kyc",
   BalanceKycRequired = "balance-kyc",
   BalanceKycInit = "balance-kyc-init",
+  KycAuthRequired = "kyc-auth",
   Track = "track",
   SubmitPayment = "submit-payment",
   RebindSession = "rebind-session",
@@ -201,6 +269,11 @@ export interface TransactionCommon {
    */
   txState: TransactionState;
 
+  /**
+   * Wallet-internal state ID, only used for debugging and testing.
+   */
+  stId: number;
+
   /**
    * Possible transitions based on the current state.
    */
@@ -218,6 +291,10 @@ export interface TransactionCommon {
 
   error?: TalerErrorDetail;
 
+  abortReason?: TalerErrorDetail;
+
+  failReason?: TalerErrorDetail;
+
   /**
    * If the transaction minor state is in KycRequired this field is going to
    * have the location where the user need to go to complete KYC information.
@@ -233,6 +310,37 @@ export interface TransactionCommon {
    * KYC access token. Useful for testing, not so useful for UIs.
    */
   kycAccessToken?: string;
+
+  kycAuthTransferInfo?: KycAuthTransferInfo;
+}
+
+export interface KycAuthTransferInfo {
+  /**
+   * Payto URI of the account that must make the transfer.
+   *
+   * The KYC auth transfer will *not* work if it originates
+   * from a different account.
+   */
+  debitPaytoUri: string;
+
+  /**
+   * Account public key that must be included in the subject.
+   */
+  accountPub: string;
+
+  /**
+   * Amount that the exchange expects to be deposited.
+   *
+   * Usually corresponds to the TINY_AMOUNT configuration of the exchange,
+   * and thus is the smallest amount that can be transferred
+   * via a bank transfer.
+   */
+  amount: AmountString;
+
+  /**
+   * Possible target payto URIs.
+   */
+  creditPaytoUris: string[];
 }
 
 export type Transaction =
@@ -411,6 +519,7 @@ export interface TransactionInternalWithdrawal extends TransactionCommon {
 export interface PeerInfoShort {
   expiration: TalerProtocolTimestamp | undefined;
   summary: string | undefined;
+  iconId: string | undefined;
 }
 
 /**
@@ -553,9 +662,11 @@ export interface TransactionPayment extends TransactionCommon {
   info: OrderShortInfo;
 
   /**
-   * Wallet-internal end-to-end identifier for the payment.
+   * Full contract terms.
+   *
+   * Only included if explicitly included in the request.
    */
-  proposalId: string;
+  contractTerms?: MerchantContractTerms;
 
   /**
    * Amount that must be paid for the contract
@@ -593,9 +704,21 @@ export interface TransactionPayment extends TransactionCommon {
   refundQueryActive: boolean;
 
   /**
-   * Does this purchase has an pos validation
+   * PoS confirmation codes, separated by newlines.
+   * Only present for purchases that support PoS confirmation.
    */
   posConfirmation: string | undefined;
+
+  /**
+   * Until when will the posConfirmation be valid?
+   */
+  posConfirmationDeadline?: TalerProtocolTimestamp;
+
+  /**
+   * Did we receive the payment via a taler://pay-template/ URI
+   * and did the URI contain a nfc=1 flag?
+   */
+  posConfirmationViaNfc?: boolean;
 }
 
 export interface OrderShortInfo {
@@ -739,6 +862,11 @@ export interface TransactionDeposit extends TransactionCommon {
    */
   amountRaw: AmountString;
 
+  /**
+   * Deposit account public key.
+   */
+  accountPub: string;
+
   /**
    * Effective amount that is being deposited
    */
@@ -758,14 +886,46 @@ export interface TransactionDeposit extends TransactionCommon {
 
 export interface TransactionByIdRequest {
   transactionId: string;
+
+  /**
+   * If set to true, report the full contract terms in the response
+   * if the transaction has them.
+   */
+  includeContractTerms?: boolean;
 }
 
 export const codecForTransactionByIdRequest =
   (): Codec<TransactionByIdRequest> =>
     buildCodecForObject<TransactionByIdRequest>()
       .property("transactionId", codecForString())
+      .property("includeContractTerms", codecOptional(codecForBoolean()))
       .build("TransactionByIdRequest");
 
+export const codecForGetTransactionsV2Request =
+  (): Codec<GetTransactionsV2Request> =>
+    buildCodecForObject<GetTransactionsV2Request>()
+      .property("currency", codecOptional(codecForString()))
+      .property("scopeInfo", codecOptional(codecForScopeInfo()))
+      .property(
+        "offsetTransactionId",
+        codecOptional(codecForString() as Codec<TransactionIdStr>),
+      )
+      .property("offsetTimestamp", codecOptional(codecForPreciseTimestamp))
+      .property("limit", codecOptional(codecForNumber()))
+      .property(
+        "filterByState",
+        codecOptional(
+          codecForEither(
+            codecForConstString("final"),
+            codecForConstString("nonfinal"),
+            codecForConstString("done"),
+          ),
+        ),
+      )
+      .property("includeRefreshes", codecOptional(codecForBoolean()))
+      .property("includeAll", codecOptional(codecForBoolean()))
+      .build("GetTransactionsV2Request");
+
 export const codecForTransactionsRequest = (): Codec<TransactionsRequest> =>
   buildCodecForObject<TransactionsRequest>()
     .property("currency", codecOptional(codecForString()))
