@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2020 Taler Systems SA
+ (C) 2020-2025 Taler Systems SA
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -73,6 +73,7 @@ import {
 import { gunzipSync, gzipSync } from "fflate";
 import { addAttentionRequest, removeAttentionRequest } from "../attention.js";
 import {
+  cancelableFetch,
   TaskIdentifiers,
   TaskRunResult,
   TaskRunResultType,
@@ -225,7 +226,7 @@ async function runBackupCycleForProvider(
     accountBackupUrl.searchParams.set("fresh", "yes");
   }
 
-  const resp = await wex.http.fetch(accountBackupUrl.href, {
+  const resp = await cancelableFetch(wex, accountBackupUrl, {
     method: "POST",
     body: encBackup,
     headers: {
@@ -325,7 +326,7 @@ async function runBackupCycleForProvider(
         }
         // const opId = TaskIdentifiers.forBackup(prov);
         // await scheduleRetryInTx(ws, tx, opId);
-        prov.currentPaymentProposalId = result.proposalId;
+        prov.currentPaymentTransactionId = result.transactionId;
         prov.shouldRetryFreshProposal = false;
         prov.state = {
           tag: BackupProviderStateTag.Retrying,
@@ -595,7 +596,7 @@ export async function addBackupProvider(
     },
   );
   const termsUrl = new URL("config", canonUrl);
-  const resp = await wex.http.fetch(termsUrl.href);
+  const resp = await cancelableFetch(wex, termsUrl);
   const terms = await readSuccessResponseJsonOrThrow(
     resp,
     codecForSyncTermsOfServiceResponse(),
