@@ -51,6 +51,7 @@
  */
 import { GlobalIDB } from "@gnu-taler/idb-bridge";
 import {
+  AllowedExchangeInfo,
   AmountJson,
   AmountLike,
   Amounts,
@@ -58,6 +59,7 @@ import {
   BalanceFlag,
   BalancesResponse,
   checkDbInvariant,
+  DonauSummaryItem,
   GetBalanceDetailRequest,
   j2s,
   Logger,
@@ -68,16 +70,23 @@ import {
 import { ExchangeRestrictionSpec, findMatchingWire } from "./coinSelection.js";
 import {
   DepositOperationStatus,
+  DonationSummaryRecord,
   ExchangeEntryDbRecordStatus,
   OPERATION_STATUS_NONFINAL_FIRST,
   OPERATION_STATUS_NONFINAL_LAST,
+  PeerPullDebitRecordStatus,
+  PeerPullPaymentCreditStatus,
+  PeerPushCreditStatus,
   PeerPushDebitStatus,
+  PurchaseStatus,
   RefreshGroupRecord,
   RefreshOperationStatus,
   WalletDbReadOnlyTransaction,
   WithdrawalGroupStatus,
+  WithdrawalRecordType,
 } from "./db.js";
 import {
+  checkExchangeInScopeTx,
   getExchangeScopeInfo,
   getExchangeWireDetailsInTx,
 } from "./exchanges.js";
@@ -98,19 +107,27 @@ interface WalletBalance {
   flagOutgoingKyc: boolean;
 }
 
-/**
- * Compute the available amount that the wallet expects to get
- * out of a refresh group.
- */
-function computeRefreshGroupAvailableAmount(r: RefreshGroupRecord): AmountJson {
+function computeRefreshGroupAvailableAmountForExchanges(
+  r: RefreshGroupRecord,
+  restrictExchanges: AllowedExchangeInfo[] | undefined,
+): AmountJson {
   // Don't count finished refreshes, since the refresh already resulted
   // in coins being added to the wallet.
   let available = Amounts.zeroOfCurrency(r.currency);
   if (r.timestampFinished) {
     return available;
   }
-  for (let i = 0; i < r.oldCoinPubs.length; i++) {
-    available = Amounts.add(available, r.expectedOutputPerCoin[i]).amount;
+  if (!r.infoPerExchange) {
+    return available;
+  }
+  for (const exch of Object.keys(r.infoPerExchange)) {
+    const pe = r.infoPerExchange[exch];
+    if (
+      restrictExchanges == null ||
+      restrictExchanges.find((x) => x.exchangeBaseUrl === exch) != null
+    ) {
+      available = Amounts.add(available, pe.outputEffective).amount;
+    }
   }
   return available;
 }
@@ -126,9 +143,23 @@ function getBalanceKey(scopeInfo: ScopeInfo): string {
   }
 }
 
+function getScopeSortingOrder(scopeInfo: ScopeInfo): number {
+  switch (scopeInfo.type) {
+    case ScopeType.Global:
+      return 0;
+    case ScopeType.Auditor:
+      return 1;
+    case ScopeType.Exchange:
+      return 2;
+    default:
+      assertUnreachable(scopeInfo);
+  }
+}
+
 class BalancesStore {
   private exchangeScopeCache: Record<string, ScopeInfo> = {};
   private balanceStore: Record<string, WalletBalance> = {};
+  private donauSummaryItems: DonauSummaryItem[] | undefined = undefined;
 
   constructor(
     private wex: WalletExecutionContext,
@@ -232,15 +263,42 @@ class BalancesStore {
     b.flagOutgoingKyc = true;
   }
 
+  addDonauItem(rec: DonationSummaryRecord): void {
+    if (!this.donauSummaryItems) {
+      this.donauSummaryItems = [];
+    }
+
+    this.donauSummaryItems.push({
+      amountReceiptsAvailable: rec.amountReceiptsAvailable,
+      amountReceiptsSubmitted: rec.amountReceiptsSubmitted,
+      donauBaseUrl: rec.donauBaseUrl,
+      year: rec.year,
+      legalDomain: rec.legalDomain,
+      // Not yet implemented
+      amountStatement: undefined,
+    });
+  }
+
   toBalancesResponse(): BalancesResponse {
     const balancesResponse: BalancesResponse = {
       balances: [],
     };
 
+    if (this.donauSummaryItems) {
+      balancesResponse.donauSummary = this.donauSummaryItems;
+    }
+
     const balanceStore = this.balanceStore;
 
     Object.keys(balanceStore)
-      .sort()
+      .sort((a, b) => {
+        const balanceA = balanceStore[a];
+        const balanceB = balanceStore[b];
+        const scopeA = getScopeSortingOrder(balanceA.scopeInfo);
+        const scopeB = getScopeSortingOrder(balanceB.scopeInfo);
+        if (scopeA !== scopeB) return scopeA - scopeB;
+        return a.localeCompare(b);
+      })
       .forEach((c) => {
         const v = balanceStore[c];
         const flags: BalanceFlag[] = [];
@@ -285,6 +343,12 @@ export async function getBalancesInsideTransaction(
       "globalCurrencyAuditors",
       "globalCurrencyExchanges",
       "peerPushDebit",
+      "peerPushCredit",
+      "peerPullCredit",
+      "peerPullDebit",
+      "purchases",
+      "coins",
+      "donationSummaries",
     ]
   >,
 ): Promise<BalancesResponse> {
@@ -295,6 +359,10 @@ export async function getBalancesInsideTransaction(
     OPERATION_STATUS_NONFINAL_LAST,
   );
 
+  await tx.donationSummaries.iter().forEachAsync(async (rec) => {
+    balanceStore.addDonauItem(rec);
+  });
+
   await tx.exchanges.iter().forEachAsync(async (ex) => {
     if (
       ex.entryStatus === ExchangeEntryDbRecordStatus.Used ||
@@ -319,22 +387,24 @@ export async function getBalancesInsideTransaction(
     }
   });
 
-  await tx.refreshGroups.iter().forEachAsync(async (r) => {
-    switch (r.operationStatus) {
-      case RefreshOperationStatus.Pending:
-      case RefreshOperationStatus.Suspended:
-        break;
-      default:
+  await tx.refreshGroups.indexes.byStatus
+    .iter(keyRangeActive)
+    .forEachAsync(async (r) => {
+      switch (r.operationStatus) {
+        case RefreshOperationStatus.Pending:
+        case RefreshOperationStatus.Suspended:
+          break;
+        default:
+          return;
+      }
+      const perExchange = r.infoPerExchange;
+      if (!perExchange) {
         return;
-    }
-    const perExchange = r.infoPerExchange;
-    if (!perExchange) {
-      return;
-    }
-    for (const [e, x] of Object.entries(perExchange)) {
-      await balanceStore.addAvailable(r.currency, e, x.outputEffective);
-    }
-  });
+      }
+      for (const [e, x] of Object.entries(perExchange)) {
+        await balanceStore.addAvailable(r.currency, e, x.outputEffective);
+      }
+    });
 
   await tx.withdrawalGroups.indexes.byStatus
     .iter(keyRangeActive)
@@ -351,6 +421,8 @@ export async function getBalancesInsideTransaction(
           // Does not count as pendingIncoming
           return;
         case WithdrawalGroupStatus.PendingReady:
+        case WithdrawalGroupStatus.PendingRedenominate:
+        case WithdrawalGroupStatus.SuspendedRedenominate:
         case WithdrawalGroupStatus.AbortingBank:
         case WithdrawalGroupStatus.PendingQueryingStatus:
         case WithdrawalGroupStatus.SuspendedWaitConfirmBank:
@@ -389,15 +461,28 @@ export async function getBalancesInsideTransaction(
           break;
         }
         case WithdrawalGroupStatus.PendingWaitConfirmBank: {
-          checkDbInvariant(
-            wg.denomsSel !== undefined,
-            "wg in confirmed state should have been initialized",
-          );
           checkDbInvariant(
             wg.exchangeBaseUrl !== undefined,
-            "wg in kyc state should have been initialized",
+            "withdrawal group in PendingWaitConfirmBank state should have been initialized",
           );
-          const currency = Amounts.currencyOf(wg.denomsSel.totalCoinValue);
+
+          // FIXME: Consider just having the currency as a fixed field in the DB
+          // instead of having it in many locations.
+          let currency: string;
+
+          if (wg.denomsSel) {
+            currency = Amounts.currencyOf(wg.denomsSel.totalCoinValue);
+          } else if (
+            wg.wgInfo.withdrawalType === WithdrawalRecordType.BankIntegrated &&
+            wg.wgInfo.bankInfo.currency
+          ) {
+            currency = wg.wgInfo.bankInfo.currency;
+          } else {
+            logger.warn(
+              "could not determine currency for confirmed withdrawal group",
+            );
+            break;
+          }
           await balanceStore.setFlagIncomingConfirmation(
             currency,
             wg.exchangeBaseUrl,
@@ -436,6 +521,114 @@ export async function getBalancesInsideTransaction(
           );
           break;
         }
+        case PeerPushDebitStatus.Failed:
+        case PeerPushDebitStatus.Aborted:
+        case PeerPushDebitStatus.Done:
+        case PeerPushDebitStatus.Expired:
+          break;
+        default:
+          assertUnreachable(ppdRecord.status);
+      }
+    });
+
+  await tx.peerPushCredit.indexes.byStatus
+    .iter(keyRangeActive)
+    .forEachAsync(async (rec) => {
+      switch (rec.status) {
+        case PeerPushCreditStatus.PendingMerge:
+        case PeerPushCreditStatus.PendingBalanceKycInit:
+        case PeerPushCreditStatus.PendingBalanceKycRequired:
+        case PeerPushCreditStatus.SuspendedBalanceKycInit:
+        case PeerPushCreditStatus.PendingMergeKycRequired:
+        case PeerPushCreditStatus.SuspendedMerge:
+        case PeerPushCreditStatus.SuspendedBalanceKycRequired:
+        case PeerPushCreditStatus.SuspendedMergeKycRequired: {
+          const currency = Amounts.currencyOf(rec.estimatedAmountEffective);
+          const amount = rec.estimatedAmountEffective;
+          await balanceStore.addPendingIncoming(
+            currency,
+            rec.exchangeBaseUrl,
+            amount,
+          );
+          break;
+        }
+        case PeerPushCreditStatus.PendingWithdrawing:
+        case PeerPushCreditStatus.SuspendedWithdrawing: {
+          // Here, incoming balance should be covered by internal withdrawal tx
+          break;
+        }
+      }
+    });
+
+  await tx.peerPullCredit.indexes.byStatus
+    .iter(keyRangeActive)
+    .forEachAsync(async (rec) => {
+      switch (rec.status) {
+        case PeerPullPaymentCreditStatus.SuspendedBalanceKycInit:
+        case PeerPullPaymentCreditStatus.PendingBalanceKycInit:
+        case PeerPullPaymentCreditStatus.PendingBalanceKycRequired:
+        case PeerPullPaymentCreditStatus.SuspendedBalanceKycRequired:
+        case PeerPullPaymentCreditStatus.SuspendedMergeKycRequired:
+        case PeerPullPaymentCreditStatus.PendingMergeKycRequired:
+        case PeerPullPaymentCreditStatus.PendingReady:
+        case PeerPullPaymentCreditStatus.PendingCreatePurse:
+        case PeerPullPaymentCreditStatus.SuspendedReady:
+        case PeerPullPaymentCreditStatus.SuspendedCreatePurse: {
+          const currency = Amounts.currencyOf(rec.amount);
+          const amount = rec.estimatedAmountEffective;
+          await balanceStore.addPendingIncoming(
+            currency,
+            rec.exchangeBaseUrl,
+            amount,
+          );
+          break;
+        }
+      }
+    });
+
+  await tx.peerPullDebit.indexes.byStatus
+    .iter(keyRangeActive)
+    .forEachAsync(async (rec) => {
+      switch (rec.status) {
+        case PeerPullDebitRecordStatus.PendingDeposit:
+        case PeerPullDebitRecordStatus.AbortingRefresh:
+        case PeerPullDebitRecordStatus.SuspendedAbortingRefresh:
+        case PeerPullDebitRecordStatus.SuspendedDeposit:
+          const currency = Amounts.currencyOf(rec.amount);
+          const amount = rec.coinSel?.totalCost ?? rec.amount;
+          await balanceStore.addPendingOutgoing(
+            currency,
+            rec.exchangeBaseUrl,
+            amount,
+          );
+          break;
+      }
+    });
+
+  await tx.purchases.indexes.byStatus
+    .iter(keyRangeActive)
+    .forEachAsync(async (rec) => {
+      switch (rec.purchaseStatus) {
+        case PurchaseStatus.SuspendedPaying:
+        case PurchaseStatus.PendingPaying:
+          if (!rec.payInfo || !rec.payInfo.payCoinSelection?.coinPubs) {
+            break;
+          }
+          // FIXME: This is pretty slow, cache?
+          const currency = Amounts.currencyOf(rec.payInfo.totalPayCost);
+          const sel = rec.payInfo.payCoinSelection;
+          for (let i = 0; i < sel.coinPubs.length; i++) {
+            const coinPub = sel.coinPubs[i];
+            const coinRec = await tx.coins.get(coinPub);
+            if (!coinRec) {
+              continue;
+            }
+            await balanceStore.addPendingOutgoing(
+              currency,
+              coinRec.exchangeBaseUrl,
+              sel.coinContributions[i],
+            );
+          }
       }
     });
 
@@ -449,18 +642,25 @@ export async function getBalancesInsideTransaction(
       for (const [e, x] of Object.entries(perExchange)) {
         const currency = Amounts.currencyOf(dgRecord.amount);
         switch (dgRecord.operationStatus) {
-          case DepositOperationStatus.SuspendedKyc:
-          case DepositOperationStatus.PendingKyc:
+          case DepositOperationStatus.SuspendedAggregateKyc:
+          case DepositOperationStatus.PendingAggregateKyc:
+          case DepositOperationStatus.PendingDepositKyc:
+          case DepositOperationStatus.PendingDepositKycAuth:
+          case DepositOperationStatus.SuspendedDepositKyc:
+          case DepositOperationStatus.SuspendedDepositKycAuth:
             await balanceStore.setFlagOutgoingKyc(currency, e);
         }
-
         switch (dgRecord.operationStatus) {
-          case DepositOperationStatus.SuspendedKyc:
-          case DepositOperationStatus.PendingKyc:
-          case DepositOperationStatus.PendingTrack:
+          case DepositOperationStatus.SuspendedAggregateKyc:
+          case DepositOperationStatus.PendingAggregateKyc:
+          case DepositOperationStatus.LegacyPendingTrack:
           case DepositOperationStatus.SuspendedAborting:
           case DepositOperationStatus.SuspendedDeposit:
-          case DepositOperationStatus.SuspendedTrack:
+          case DepositOperationStatus.LegacySuspendedTrack:
+          case DepositOperationStatus.PendingDepositKyc:
+          case DepositOperationStatus.PendingDepositKycAuth:
+          case DepositOperationStatus.SuspendedDepositKyc:
+          case DepositOperationStatus.SuspendedDepositKycAuth:
           case DepositOperationStatus.PendingDeposit: {
             const perExchange = dgRecord.infoPerExchange;
             if (perExchange) {
@@ -488,26 +688,9 @@ export async function getBalances(
 ): Promise<BalancesResponse> {
   logger.trace("starting to compute balance");
 
-  const wbal = await wex.db.runReadWriteTx(
-    {
-      storeNames: [
-        "coinAvailability",
-        "coins",
-        "depositGroups",
-        "exchangeDetails",
-        "exchanges",
-        "globalCurrencyAuditors",
-        "globalCurrencyExchanges",
-        "purchases",
-        "refreshGroups",
-        "withdrawalGroups",
-        "peerPushDebit",
-      ],
-    },
-    async (tx) => {
-      return getBalancesInsideTransaction(wex, tx);
-    },
-  );
+  const wbal = await wex.db.runAllStoresReadOnlyTx({}, async (tx) => {
+    return getBalancesInsideTransaction(wex, tx);
+  });
 
   logger.trace("finished computing wallet balance");
 
@@ -516,9 +699,24 @@ export async function getBalances(
 
 export interface PaymentRestrictionsForBalance {
   currency: string;
+
   minAge: number;
-  restrictExchanges: ExchangeRestrictionSpec | undefined;
+
+  /**
+   * Restriction by the *sender* on the exchange.
+   */
+  restrictSenderScope: ScopeInfo | undefined;
+
+  /**
+   * Restrictions of the *receiver* on the exchanges.
+   */
+  restrictReceiverExchanges: ExchangeRestrictionSpec | undefined;
+
+  /**
+   * Restriction of the receiver on the supported wire methods.
+   */
   restrictWireMethods: string[] | undefined;
+
   depositPaytoUri: string | undefined;
 }
 
@@ -552,12 +750,12 @@ export interface PaymentBalanceDetails {
   balanceAgeAcceptable: AmountJson;
 
   /**
-   * Balance of type "merchant-acceptable" (see balance.ts for definition).
+   * Balance of type "receiver-acceptable" (see balance.ts for definition).
    */
   balanceReceiverAcceptable: AmountJson;
 
   /**
-   * Balance of type "merchant-depositable" (see balance.ts for definition).
+   * Balance of type "receiver-depositable" (see balance.ts for definition).
    */
   balanceReceiverDepositable: AmountJson;
 
@@ -568,7 +766,11 @@ export interface PaymentBalanceDetails {
    */
   balanceExchangeDepositable: AmountJson;
 
-  maxEffectiveSpendAmount: AmountJson;
+  /**
+   * Estimated maximum amount that the wallet could pay for, under the assumption
+   * that the merchant pays absolutely no fees.
+   */
+  maxMerchantEffectiveDepositAmount: AmountJson;
 }
 
 export async function getPaymentBalanceDetails(
@@ -582,6 +784,8 @@ export async function getPaymentBalanceDetails(
         "refreshGroups",
         "exchanges",
         "exchangeDetails",
+        "globalCurrencyAuditors",
+        "globalCurrencyExchanges",
         "denominations",
       ],
     },
@@ -599,6 +803,8 @@ export async function getPaymentBalanceDetailsInTx(
       "refreshGroups",
       "exchanges",
       "exchangeDetails",
+      "globalCurrencyAuditors",
+      "globalCurrencyExchanges",
       "denominations",
     ]
   >,
@@ -610,11 +816,11 @@ export async function getPaymentBalanceDetailsInTx(
     balanceAgeAcceptable: Amounts.zeroOfCurrency(req.currency),
     balanceReceiverAcceptable: Amounts.zeroOfCurrency(req.currency),
     balanceReceiverDepositable: Amounts.zeroOfCurrency(req.currency),
-    maxEffectiveSpendAmount: Amounts.zeroOfCurrency(req.currency),
+    maxMerchantEffectiveDepositAmount: Amounts.zeroOfCurrency(req.currency),
     balanceExchangeDepositable: Amounts.zeroOfCurrency(req.currency),
   };
 
-  logger.info(`computing balance details for ${j2s(req)}`);
+  logger.trace(`computing balance details for ${j2s(req)}`);
 
   const availableCoins = await tx.coinAvailability.getAll();
 
@@ -633,6 +839,18 @@ export async function getPaymentBalanceDetailsInTx(
       continue;
     }
 
+    // Skip exchanges if excluded by the receiver.
+    if (
+      req.restrictSenderScope &&
+      !(await checkExchangeInScopeTx(
+        tx,
+        ca.exchangeBaseUrl,
+        req.restrictSenderScope,
+      ))
+    ) {
+      continue;
+    }
+
     const wireDetails = await getExchangeWireDetailsInTx(
       tx,
       ca.exchangeBaseUrl,
@@ -671,16 +889,16 @@ export async function getPaymentBalanceDetailsInTx(
 
     let merchantExchangeAcceptable = false;
 
-    if (!req.restrictExchanges) {
+    if (!req.restrictReceiverExchanges) {
       merchantExchangeAcceptable = true;
     } else {
-      for (const ex of req.restrictExchanges.exchanges) {
+      for (const ex of req.restrictReceiverExchanges.exchanges) {
         if (ex.exchangeBaseUrl === ca.exchangeBaseUrl) {
           merchantExchangeAcceptable = true;
           break;
         }
       }
-      for (const acceptedAuditor of req.restrictExchanges.auditors) {
+      for (const acceptedAuditor of req.restrictReceiverExchanges.auditors) {
         for (const exchangeAuditor of wireDetails.auditors) {
           if (acceptedAuditor.auditorBaseUrl === exchangeAuditor.auditor_url) {
             merchantExchangeAcceptable = true;
@@ -719,13 +937,13 @@ export async function getPaymentBalanceDetailsInTx(
       merchantExchangeAcceptable &&
       merchantExchangeDepositable
     ) {
-      d.maxEffectiveSpendAmount = Amounts.add(
-        d.maxEffectiveSpendAmount,
+      d.maxMerchantEffectiveDepositAmount = Amounts.add(
+        d.maxMerchantEffectiveDepositAmount,
         Amounts.mult(ca.value, ca.freshCoinCount).amount,
       ).amount;
 
-      d.maxEffectiveSpendAmount = Amounts.sub(
-        d.maxEffectiveSpendAmount,
+      d.maxMerchantEffectiveDepositAmount = Amounts.sub(
+        d.maxMerchantEffectiveDepositAmount,
         Amounts.mult(denom.feeDeposit, ca.freshCoinCount).amount,
       ).amount;
     }
@@ -735,10 +953,11 @@ export async function getPaymentBalanceDetailsInTx(
     if (r.currency != req.currency) {
       return;
     }
-    d.balanceAvailable = Amounts.add(
-      d.balanceAvailable,
-      computeRefreshGroupAvailableAmount(r),
-    ).amount;
+    const balRefresh = computeRefreshGroupAvailableAmountForExchanges(
+      r,
+      req.restrictReceiverExchanges?.exchanges,
+    );
+    d.balanceAvailable = Amounts.add(d.balanceAvailable, balRefresh).amount;
   });
 
   return d;
@@ -775,7 +994,8 @@ export async function getBalanceDetail(
 
   return await getPaymentBalanceDetails(wex, {
     currency: req.currency,
-    restrictExchanges: {
+    restrictSenderScope: undefined,
+    restrictReceiverExchanges: {
       auditors: [],
       exchanges,
     },
