@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2021-2024 Taler Systems S.A.
+ (C) 2021-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -32,28 +32,39 @@ import {
   AllowedExchangeInfo,
   AmountJson,
   Amounts,
+  checkAccountRestriction,
   checkDbInvariant,
   checkLogicInvariant,
   CoinStatus,
   DenominationInfo,
   ExchangeGlobalFees,
   ForcedCoinSel,
-  InternationalizedString,
+  GetMaxDepositAmountRequest,
+  GetMaxDepositAmountResponse,
+  GetMaxPeerPushDebitAmountRequest,
+  GetMaxPeerPushDebitAmountResponse,
+  InsufficientBalanceHint,
   j2s,
   Logger,
   parsePaytoUri,
   PayCoinSelection,
   PaymentInsufficientBalanceDetails,
   ProspectivePayCoinSelection,
+  ScopeInfo,
+  ScopeType,
   SelectedCoin,
   SelectedProspectiveCoin,
   strcmp,
   TalerProtocolTimestamp,
 } from "@gnu-taler/taler-util";
-import { getPaymentBalanceDetailsInTx } from "./balance.js";
+import {
+  getPaymentBalanceDetailsInTx,
+  PaymentBalanceDetails,
+} from "./balance.js";
 import { getAutoRefreshExecuteThreshold } from "./common.js";
 import { DenominationRecord, WalletDbReadOnlyTransaction } from "./db.js";
 import {
+  checkExchangeInScopeTx,
   ExchangeWireDetails,
   getExchangeWireDetailsInTx,
 } from "./exchanges.js";
@@ -86,6 +97,8 @@ export interface CoinSelectionTally {
 
   customerDepositFees: AmountJson;
 
+  totalDepositFees: AmountJson;
+
   customerWireFees: AmountJson;
 
   wireFeeCoveredForExchange: Set<string>;
@@ -152,6 +165,10 @@ function tallyFees(
     dfRemaining,
   ).amount;
   tally.lastDepositFee = feeDeposit;
+  tally.totalDepositFees = Amounts.add(
+    tally.totalDepositFees,
+    feeDeposit,
+  ).amount;
 }
 
 export type SelectPayCoinsResult =
@@ -172,6 +189,8 @@ async function internalSelectPayCoins(
       "exchanges",
       "exchangeDetails",
       "coins",
+      "globalCurrencyAuditors",
+      "globalCurrencyExchanges",
     ]
   >,
   req: SelectPayCoinRequestNg,
@@ -180,19 +199,33 @@ async function internalSelectPayCoins(
   | { sel: SelResult; coinRes: SelectedCoin[]; tally: CoinSelectionTally }
   | undefined
 > {
+  let restrictWireMethod;
+  if (req.depositPaytoUri) {
+    const parsedPayto = parsePaytoUri(req.depositPaytoUri);
+    if (!parsedPayto) {
+      throw Error("invalid payto URI");
+    }
+    restrictWireMethod = parsedPayto.targetType;
+    if (restrictWireMethod !== req.restrictWireMethod) {
+      logger.warn(`conflicting payto URI and wire method restriction`);
+    }
+  } else {
+    restrictWireMethod = req.restrictWireMethod;
+  }
+
   const { contractTermsAmount, depositFeeLimit } = req;
-  const [candidateDenoms, wireFeesPerExchange] = await selectPayCandidates(
-    wex,
-    tx,
-    {
-      restrictExchanges: req.restrictExchanges,
-      instructedAmount: req.contractTermsAmount,
-      restrictWireMethod: req.restrictWireMethod,
-      depositPaytoUri: req.depositPaytoUri,
-      requiredMinimumAge: req.requiredMinimumAge,
-      includePendingCoins,
-    },
-  );
+  const candidateRes = await selectPayCandidates(wex, tx, {
+    currency: Amounts.currencyOf(req.contractTermsAmount),
+    restrictExchanges: req.restrictExchanges,
+    restrictWireMethod: req.restrictWireMethod,
+    restrictScope: req.restrictScope,
+    depositPaytoUri: req.depositPaytoUri,
+    requiredMinimumAge: req.requiredMinimumAge,
+    includePendingCoins,
+  });
+
+  const wireFeesPerExchange = candidateRes.currentWireFeePerExchange;
+  const candidateDenoms = candidateRes.coinAvailability;
 
   if (logger.shouldLogTrace()) {
     logger.trace(
@@ -210,6 +243,7 @@ async function internalSelectPayCoins(
     amountDepositFeeLimitRemaining: depositFeeLimit,
     customerDepositFees: Amounts.zeroOfCurrency(currency),
     customerWireFees: Amounts.zeroOfCurrency(currency),
+    totalDepositFees: Amounts.zeroOfCurrency(currency),
     wireFeeCoveredForExchange: new Set(),
     lastDepositFee: Amounts.zeroOfCurrency(currency),
   };
@@ -262,6 +296,8 @@ export async function selectPayCoinsInTx(
       "exchanges",
       "exchangeDetails",
       "coins",
+      "globalCurrencyAuditors",
+      "globalCurrencyExchanges",
     ]
   >,
   req: SelectPayCoinRequestNg,
@@ -270,6 +306,23 @@ export async function selectPayCoinsInTx(
     logger.trace(`selecting coins for ${j2s(req)}`);
   }
 
+  if (!!wex.ws.devExperimentState.merchantDepositInsufficient) {
+    return {
+      type: "failure",
+      insufficientBalanceDetails: await reportInsufficientBalanceDetails(
+        wex,
+        tx,
+        {
+          restrictExchanges: req.restrictExchanges,
+          instructedAmount: req.contractTermsAmount,
+          requiredMinimumAge: req.requiredMinimumAge,
+          wireMethod: req.restrictWireMethod,
+          depositPaytoUri: req.depositPaytoUri,
+        },
+      ),
+    } satisfies SelectPayCoinsResult;
+  }
+
   const materialAvSel = await internalSelectPayCoins(wex, tx, req, false);
 
   if (!materialAvSel) {
@@ -354,6 +407,8 @@ export async function selectPayCoins(
         "exchanges",
         "exchangeDetails",
         "coins",
+        "globalCurrencyAuditors",
+        "globalCurrencyExchanges",
       ],
     },
     async (tx) => {
@@ -452,6 +507,7 @@ async function assembleSelectPayCoinsSuccessResult(
     coins: coinRes,
     customerDepositFees: Amounts.stringify(tally.customerDepositFees),
     customerWireFees: Amounts.stringify(tally.customerWireFees),
+    totalDepositFees: Amounts.stringify(tally.totalDepositFees),
   };
 }
 
@@ -463,6 +519,48 @@ interface ReportInsufficientBalanceRequest {
   depositPaytoUri: string | undefined;
 }
 
+function getHint(
+  req: ReportInsufficientBalanceRequest,
+  exchDet: PaymentBalanceDetails,
+): InsufficientBalanceHint | undefined {
+  if (Amounts.cmp(exchDet.balanceAvailable, req.instructedAmount) >= 0) {
+    // It looks to the user like funds at this exchange should be sufficient.
+    // We now need to explain why that's not the case.
+
+    const isMerchant = !!req.wireMethod;
+
+    if (Amounts.cmp(exchDet.balanceMaterial, req.instructedAmount) <= 0) {
+      return InsufficientBalanceHint.WalletBalanceMaterialInsufficient;
+    } else if (
+      Amounts.cmp(exchDet.balanceAgeAcceptable, req.instructedAmount) < 0
+    ) {
+      return InsufficientBalanceHint.AgeRestricted;
+    } else if (
+      isMerchant &&
+      Amounts.cmp(exchDet.balanceReceiverAcceptable, req.instructedAmount)
+    ) {
+      return InsufficientBalanceHint.MerchantAcceptInsufficient;
+    } else if (
+      isMerchant &&
+      Amounts.cmp(exchDet.balanceExchangeDepositable, req.instructedAmount)
+    ) {
+      return InsufficientBalanceHint.MerchantDepositInsufficient;
+    } else if (
+      isMerchant &&
+      Amounts.cmp(
+        exchDet.maxMerchantEffectiveDepositAmount,
+        req.instructedAmount,
+      ) < 0
+    ) {
+      return InsufficientBalanceHint.FeesNotCovered;
+    }
+    // We don't exactly know why the balance is insufficient!
+    return undefined;
+  }
+  // Simply not enough funds.
+  return InsufficientBalanceHint.WalletBalanceAvailableInsufficient;
+}
+
 export async function reportInsufficientBalanceDetails(
   wex: WalletExecutionContext,
   tx: WalletDbReadOnlyTransaction<
@@ -472,14 +570,18 @@ export async function reportInsufficientBalanceDetails(
       "exchangeDetails",
       "refreshGroups",
       "denominations",
+      "globalCurrencyAuditors",
+      "globalCurrencyExchanges",
     ]
   >,
   req: ReportInsufficientBalanceRequest,
 ): Promise<PaymentInsufficientBalanceDetails> {
+  const currency = Amounts.currencyOf(req.instructedAmount);
   const details = await getPaymentBalanceDetailsInTx(wex, tx, {
-    restrictExchanges: req.restrictExchanges,
+    restrictSenderScope: undefined,
+    restrictReceiverExchanges: req.restrictExchanges,
     restrictWireMethods: req.wireMethod ? [req.wireMethod] : undefined,
-    currency: Amounts.currencyOf(req.instructedAmount),
+    currency,
     minAge: req.requiredMinimumAge ?? 0,
     depositPaytoUri: req.depositPaytoUri,
   });
@@ -493,28 +595,30 @@ export async function reportInsufficientBalanceDetails(
     let missingGlobalFees = false;
     const exchWire = await getExchangeWireDetailsInTx(tx, exch.baseUrl);
     if (!exchWire) {
+      // No wire details about the exchange known, skip!
+      continue;
+    }
+    const globalFees = getGlobalFees(exchWire);
+    if (!globalFees) {
       missingGlobalFees = true;
-    } else {
-      const globalFees = getGlobalFees(exchWire);
-      if (!globalFees) {
-        missingGlobalFees = true;
-      }
+    }
+    if (exchWire.currency !== Amounts.currencyOf(req.instructedAmount)) {
+      // Do not report anything for an exchange with a different currency.
+      continue;
     }
     const exchDet = await getPaymentBalanceDetailsInTx(wex, tx, {
-      restrictExchanges: {
-        exchanges: [
-          {
-            exchangeBaseUrl: exch.baseUrl,
-            exchangePub: exch.detailsPointer?.masterPublicKey,
-          },
-        ],
-        auditors: [],
+      restrictSenderScope: {
+        type: ScopeType.Exchange,
+        currency,
+        url: exch.baseUrl,
       },
-      restrictWireMethods: req.wireMethod ? [req.wireMethod] : [],
+      restrictReceiverExchanges: req.restrictExchanges,
+      restrictWireMethods: req.wireMethod ? [req.wireMethod] : undefined,
       currency: Amounts.currencyOf(req.instructedAmount),
       minAge: req.requiredMinimumAge ?? 0,
       depositPaytoUri: req.depositPaytoUri,
     });
+
     perExchange[exch.baseUrl] = {
       balanceAvailable: Amounts.stringify(exchDet.balanceAvailable),
       balanceMaterial: Amounts.stringify(exchDet.balanceMaterial),
@@ -529,14 +633,21 @@ export async function reportInsufficientBalanceDetails(
         exchDet.balanceReceiverDepositable,
       ),
       maxEffectiveSpendAmount: Amounts.stringify(
-        exchDet.maxEffectiveSpendAmount,
+        exchDet.maxMerchantEffectiveDepositAmount,
       ),
       missingGlobalFees,
+      causeHint: !!wex.ws.devExperimentState.merchantDepositInsufficient
+        ? InsufficientBalanceHint.MerchantDepositInsufficient
+        : getHint(req, exchDet),
     };
   }
 
   return {
     amountRequested: Amounts.stringify(req.instructedAmount),
+    wireMethod: req.wireMethod,
+    causeHint: !!wex.ws.devExperimentState.merchantDepositInsufficient
+      ? InsufficientBalanceHint.MerchantDepositInsufficient
+      : getHint(req, details),
     balanceAgeAcceptable: Amounts.stringify(details.balanceAgeAcceptable),
     balanceAvailable: Amounts.stringify(details.balanceAvailable),
     balanceMaterial: Amounts.stringify(details.balanceMaterial),
@@ -549,7 +660,9 @@ export async function reportInsufficientBalanceDetails(
     balanceReceiverDepositable: Amounts.stringify(
       details.balanceReceiverDepositable,
     ),
-    maxEffectiveSpendAmount: Amounts.stringify(details.maxEffectiveSpendAmount),
+    maxEffectiveSpendAmount: Amounts.stringify(
+      details.maxMerchantEffectiveDepositAmount,
+    ),
     perExchange,
   };
 }
@@ -590,7 +703,7 @@ export interface SelectGreedyRequest {
 
 function selectGreedy(
   req: SelectGreedyRequest,
-  candidateDenoms: AvailableDenom[],
+  candidateDenoms: AvailableCoinsOfDenom[],
   tally: CoinSelectionTally,
 ): SelResult | undefined {
   const selectedDenom: SelResult = {};
@@ -609,6 +722,9 @@ function selectGreedy(
       i < denom.numAvailable && Amounts.isNonZero(tally.amountPayRemaining);
       i++
     ) {
+      // Save the allowance *before* tallying.
+      const depositFeeAllowance = tally.amountDepositFeeLimitRemaining;
+
       tallyFees(
         tally,
         req.wireFeesPerExchange,
@@ -618,7 +734,8 @@ function selectGreedy(
 
       const coinSpend = Amounts.max(
         Amounts.min(tally.amountPayRemaining, denom.value),
-        denom.feeDeposit,
+        // Underflow saturates to zero
+        Amounts.sub(denom.feeDeposit, depositFeeAllowance).amount,
       );
 
       tally.amountPayRemaining = Amounts.sub(
@@ -653,7 +770,7 @@ function selectGreedy(
 
 function selectForced(
   req: SelectPayCoinRequestNg,
-  candidateDenoms: AvailableDenom[],
+  candidateDenoms: AvailableCoinsOfDenom[],
 ): SelResult | undefined {
   const selectedDenom: SelResult = {};
 
@@ -695,33 +812,9 @@ function selectForced(
   return selectedDenom;
 }
 
-export function checkAccountRestriction(
-  paytoUri: string,
-  restrictions: AccountRestriction[],
-): { ok: boolean; hint?: string; hintI18n?: InternationalizedString } {
-  for (const myRestriction of restrictions) {
-    switch (myRestriction.type) {
-      case "deny":
-        return { ok: false };
-      case "regex": {
-        const regex = new RegExp(myRestriction.payto_regex);
-        if (!regex.test(paytoUri)) {
-          return {
-            ok: false,
-            hint: myRestriction.human_hint,
-            hintI18n: myRestriction.human_hint_i18n,
-          };
-        }
-      }
-    }
-  }
-  return {
-    ok: true,
-  };
-}
-
 export interface SelectPayCoinRequestNg {
   restrictExchanges: ExchangeRestrictionSpec | undefined;
+  restrictScope?: ScopeInfo;
   restrictWireMethod: string;
   contractTermsAmount: AmountJson;
   depositFeeLimit: AmountJson;
@@ -739,7 +832,7 @@ export interface SelectPayCoinRequestNg {
   depositPaytoUri?: string;
 }
 
-export type AvailableDenom = DenominationInfo & {
+export type AvailableCoinsOfDenom = DenominationInfo & {
   maxAge: number;
   numAvailable: number;
 };
@@ -748,7 +841,11 @@ export function findMatchingWire(
   wireMethod: string,
   depositPaytoUri: string | undefined,
   exchangeWireDetails: ExchangeWireDetails,
-): { wireFee: AmountJson } | undefined {
+):
+  | { ok: true; wireFee: AmountJson }
+  | { ok: false; accountRestrictions: Record<string, AccountRestriction[]> }
+  | undefined {
+  const accountRestrictions: Record<string, AccountRestriction[]> = {};
   for (const acc of exchangeWireDetails.wireInfo.accounts) {
     const pp = parsePaytoUri(acc.payto_uri);
     checkLogicInvariant(!!pp);
@@ -788,10 +885,18 @@ export function findMatchingWire(
     }
 
     return {
+      ok: true,
       wireFee: Amounts.parseOrThrow(wireFeeStr),
     };
   }
-  return undefined;
+  if (Object.keys(accountRestrictions).length > 0) {
+    return {
+      ok: false,
+      accountRestrictions,
+    };
+  } else {
+    return undefined;
+  }
 }
 
 function checkExchangeAccepted(
@@ -820,9 +925,10 @@ function checkExchangeAccepted(
 }
 
 interface SelectPayCandidatesRequest {
-  instructedAmount: AmountJson;
+  currency: string;
   restrictWireMethod: string | undefined;
   depositPaytoUri?: string;
+  restrictScope?: ScopeInfo;
   restrictExchanges: ExchangeRestrictionSpec | undefined;
   requiredMinimumAge?: number;
 
@@ -834,40 +940,57 @@ interface SelectPayCandidatesRequest {
   includePendingCoins: boolean;
 }
 
+export interface PayCoinCandidates {
+  coinAvailability: AvailableCoinsOfDenom[];
+  currentWireFeePerExchange: Record<string, AmountJson>;
+  depositRestrictions?: Record<string, Record<string, AccountRestriction[]>>;
+}
+
 async function selectPayCandidates(
   wex: WalletExecutionContext,
   tx: WalletDbReadOnlyTransaction<
-    ["exchanges", "coinAvailability", "exchangeDetails", "denominations"]
+    [
+      "exchanges",
+      "coinAvailability",
+      "exchangeDetails",
+      "denominations",
+      "globalCurrencyAuditors",
+      "globalCurrencyExchanges",
+    ]
   >,
   req: SelectPayCandidatesRequest,
-): Promise<[AvailableDenom[], Record<string, AmountJson>]> {
+): Promise<PayCoinCandidates> {
   // FIXME: Use the existing helper (from balance.ts) to
   // get acceptable exchanges.
   logger.shouldLogTrace() &&
     logger.trace(`selecting available coin candidates for ${j2s(req)}`);
-  const denoms: AvailableDenom[] = [];
+  const denoms: AvailableCoinsOfDenom[] = [];
   const exchanges = await tx.exchanges.iter().toArray();
   const wfPerExchange: Record<string, AmountJson> = {};
+  const depositRestrictions: Record<
+    string,
+    Record<string, AccountRestriction[]>
+  > = {};
   for (const exchange of exchanges) {
     const exchangeDetails = await getExchangeWireDetailsInTx(
       tx,
       exchange.baseUrl,
     );
-    // 1. exchange has same currency
-    if (exchangeDetails?.currency !== req.instructedAmount.currency) {
+    // Exchange has same currency
+    if (exchangeDetails?.currency !== req.currency) {
       logger.shouldLogTrace() &&
         logger.trace(`skipping ${exchange.baseUrl} due to currency mismatch`);
       continue;
     }
 
-    // 2. Exchange supports wire method (only for pay/deposit)
+    // Exchange supports wire method (only for pay/deposit)
     if (req.restrictWireMethod) {
-      const wire = findMatchingWire(
+      const wireMatch = findMatchingWire(
         req.restrictWireMethod,
         req.depositPaytoUri,
         exchangeDetails,
       );
-      if (!wire) {
+      if (!wireMatch) {
         if (logger.shouldLogTrace()) {
           logger.trace(
             `skipping ${exchange.baseUrl} due to missing wire info mismatch`,
@@ -875,10 +998,14 @@ async function selectPayCandidates(
         }
         continue;
       }
-      wfPerExchange[exchange.baseUrl] = wire.wireFee;
+      if (!wireMatch.ok) {
+        depositRestrictions[exchange.baseUrl] = wireMatch.accountRestrictions;
+        continue;
+      }
+      wfPerExchange[exchange.baseUrl] = wireMatch.wireFee;
     }
 
-    // 3. exchange is trusted in the exchange list or auditor list
+    // Exchange is trusted in the exchange list or auditor list
     let accepted = checkExchangeAccepted(
       exchangeDetails,
       req.restrictExchanges,
@@ -890,7 +1017,15 @@ async function selectPayCandidates(
       continue;
     }
 
-    // 4. filter coins restricted by age
+    const isInScope = req.restrictScope
+      ? await checkExchangeInScopeTx(tx, exchange.baseUrl, req.restrictScope)
+      : true;
+
+    if (!isInScope) {
+      continue;
+    }
+
+    // Filter coins restricted by age
     let ageLower = 0;
     let ageUpper = AgeRestriction.AGE_UNRESTRICTED;
     if (req.requiredMinimumAge) {
@@ -913,7 +1048,7 @@ async function selectPayCandidates(
 
     let numUsable = 0;
 
-    // 5. save denoms with how many coins are available
+    // Save denoms with how many coins are available
     // FIXME: Check that the individual denomination is audited!
     // FIXME: Should we exclude denominations that are
     // not spendable anymore?
@@ -961,7 +1096,11 @@ async function selectPayCandidates(
       Amounts.cmp(o1.feeDeposit, o2.feeDeposit) ||
       strcmp(o1.denomPubHash, o2.denomPubHash),
   );
-  return [denoms, wfPerExchange];
+  return {
+    coinAvailability: denoms,
+    currentWireFeePerExchange: wfPerExchange,
+    depositRestrictions: depositRestrictions,
+  };
 }
 
 export interface PeerCoinSelectionDetails {
@@ -975,7 +1114,9 @@ export interface PeerCoinSelectionDetails {
   /**
    * How much of the deposit fees is the customer paying?
    */
-  depositFees: AmountJson;
+  customerDepositFees: AmountJson;
+
+  totalDepositFees: AmountJson;
 
   maxExpirationDate: TalerProtocolTimestamp;
 }
@@ -988,7 +1129,9 @@ export interface ProspectivePeerCoinSelectionDetails {
   /**
    * How much of the deposit fees is the customer paying?
    */
-  depositFees: AmountJson;
+  customerDepositFees: AmountJson;
+
+  totalDepositFees: AmountJson;
 
   maxExpirationDate: TalerProtocolTimestamp;
 }
@@ -1005,6 +1148,19 @@ export type SelectPeerCoinsResult =
 export interface PeerCoinSelectionRequest {
   instructedAmount: AmountJson;
 
+  /**
+   * Are deposit fees covered by the counterparty?
+   *
+   * Defaults to false.
+   */
+  feesCoveredByCounterparty?: boolean;
+
+  /**
+   * Restrict the scope of funds that can be spent via the given
+   * scope info.
+   */
+  restrictScope?: ScopeInfo;
+
   /**
    * Instruct the coin selection to repair this coin
    * selection instead of selecting completely new coins.
@@ -1045,17 +1201,21 @@ export async function computeCoinSelMaxExpirationDate(
 }
 
 export function emptyTallyForPeerPayment(
-  instructedAmount: AmountJson,
+  req: PeerCoinSelectionRequest,
 ): CoinSelectionTally {
+  const instructedAmount = req.instructedAmount;
   const currency = instructedAmount.currency;
   const zero = Amounts.zeroOfCurrency(currency);
   return {
     amountPayRemaining: instructedAmount,
     customerDepositFees: zero,
     lastDepositFee: zero,
-    amountDepositFeeLimitRemaining: zero,
+    amountDepositFeeLimitRemaining: req.feesCoveredByCounterparty
+      ? instructedAmount
+      : zero,
     customerWireFees: zero,
     wireFeeCoveredForExchange: new Set(),
+    totalDepositFees: zero,
   };
 }
 
@@ -1088,6 +1248,8 @@ async function internalSelectPeerCoins(
       "denominations",
       "refreshGroups",
       "exchangeDetails",
+      "globalCurrencyAuditors",
+      "globalCurrencyExchanges",
     ]
   >,
   req: PeerCoinSelectionRequest,
@@ -1098,7 +1260,7 @@ async function internalSelectPeerCoins(
   | undefined
 > {
   const candidatesRes = await selectPayCandidates(wex, tx, {
-    instructedAmount: req.instructedAmount,
+    currency: Amounts.currencyOf(req.instructedAmount),
     restrictExchanges: {
       auditors: [],
       exchanges: [
@@ -1111,11 +1273,11 @@ async function internalSelectPeerCoins(
     restrictWireMethod: undefined,
     includePendingCoins,
   });
-  const candidates = candidatesRes[0];
+  const candidates = candidatesRes.coinAvailability;
   if (logger.shouldLogTrace()) {
     logger.trace(`peer payment candidate coins: ${j2s(candidates)}`);
   }
-  const tally = emptyTallyForPeerPayment(req.instructedAmount);
+  const tally = emptyTallyForPeerPayment(req);
   const resCoins: SelectedCoin[] = [];
 
   await maybeRepairCoinSelection(wex, tx, req.repair ?? [], resCoins, tally, {
@@ -1157,6 +1319,8 @@ export async function selectPeerCoinsInTx(
       "denominations",
       "refreshGroups",
       "exchangeDetails",
+      "globalCurrencyExchanges",
+      "globalCurrencyAuditors",
     ]
   >,
   req: PeerCoinSelectionRequest,
@@ -1165,7 +1329,9 @@ export async function selectPeerCoinsInTx(
   if (Amounts.isZero(instructedAmount)) {
     // Other parts of the code assume that we have at least
     // one coin to spend.
-    throw new Error("peer-to-peer payment with amount of zero not supported");
+    throw new Error(
+      "coin selection for peer-to-peer payment with amount of zero not supported",
+    );
   }
 
   const exchanges = await tx.exchanges.iter().toArray();
@@ -1178,6 +1344,19 @@ export async function selectPeerCoinsInTx(
     if (!exchWire) {
       continue;
     }
+    const isInScope = req.restrictScope
+      ? await checkExchangeInScopeTx(tx, exch.baseUrl, req.restrictScope)
+      : true;
+    if (!isInScope) {
+      continue;
+    }
+    if (
+      req.restrictScope &&
+      req.restrictScope.type === ScopeType.Exchange &&
+      req.restrictScope.url !== exch.baseUrl
+    ) {
+      continue;
+    }
     const globalFees = getGlobalFees(exchWire);
     if (!globalFees) {
       continue;
@@ -1215,7 +1394,8 @@ export async function selectPeerCoinsInTx(
           type: "prospective",
           result: {
             prospectiveCoins,
-            depositFees: prospectiveAvRes.tally.customerDepositFees,
+            customerDepositFees: prospectiveAvRes.tally.customerDepositFees,
+            totalDepositFees: prospectiveAvRes.tally.totalDepositFees,
             exchangeBaseUrl: exch.baseUrl,
             maxExpirationDate,
           },
@@ -1239,7 +1419,8 @@ export async function selectPeerCoinsInTx(
         type: "success",
         result: {
           coins: r.coins,
-          depositFees: Amounts.parseOrThrow(r.customerDepositFees),
+          customerDepositFees: Amounts.parseOrThrow(r.customerDepositFees),
+          totalDepositFees: Amounts.parseOrThrow(r.totalDepositFees),
           exchangeBaseUrl: exch.baseUrl,
           maxExpirationDate,
         },
@@ -1277,6 +1458,8 @@ export async function selectPeerCoins(
         "denominations",
         "refreshGroups",
         "exchangeDetails",
+        "globalCurrencyExchanges",
+        "globalCurrencyAuditors",
       ],
     },
     async (tx): Promise<SelectPeerCoinsResult> => {
@@ -1284,3 +1467,198 @@ export async function selectPeerCoins(
     },
   );
 }
+
+function getMaxDepositAmountForAvailableCoins(
+  req: GetMaxDepositAmountRequest,
+  candidateRes: PayCoinCandidates,
+): GetMaxDepositAmountResponse {
+  const wireFeeCoveredForExchange = new Set<string>();
+
+  let amountEffective = Amounts.zeroOfCurrency(req.currency);
+  let fees = Amounts.zeroOfCurrency(req.currency);
+
+  for (const cc of candidateRes.coinAvailability) {
+    if (!wireFeeCoveredForExchange.has(cc.exchangeBaseUrl)) {
+      const wireFee =
+        candidateRes.currentWireFeePerExchange[cc.exchangeBaseUrl];
+      // Wire fee can be null if max deposit amount is computed
+      // without restricting the wire method.
+      if (wireFee != null) {
+        fees = Amounts.add(fees, wireFee).amount;
+      }
+      wireFeeCoveredForExchange.add(cc.exchangeBaseUrl);
+    }
+
+    amountEffective = Amounts.add(
+      amountEffective,
+      Amounts.mult(cc.value, cc.numAvailable).amount,
+    ).amount;
+
+    fees = Amounts.add(
+      fees,
+      Amounts.mult(cc.feeDeposit, cc.numAvailable).amount,
+    ).amount;
+  }
+
+  return {
+    effectiveAmount: Amounts.stringify(amountEffective),
+    rawAmount: Amounts.stringify(Amounts.sub(amountEffective, fees).amount),
+  };
+}
+
+/**
+ * Only used for unit testing getMaxDepositAmountForAvailableCoins.
+ */
+export const testing_getMaxDepositAmountForAvailableCoins =
+  getMaxDepositAmountForAvailableCoins;
+
+export async function getMaxDepositAmount(
+  wex: WalletExecutionContext,
+  req: GetMaxDepositAmountRequest,
+): Promise<GetMaxDepositAmountResponse> {
+  logger.trace(`getting max deposit amount for: ${j2s(req)}`);
+  return await wex.db.runReadOnlyTx(
+    {
+      storeNames: [
+        "exchanges",
+        "coinAvailability",
+        "denominations",
+        "exchangeDetails",
+        "globalCurrencyAuditors",
+        "globalCurrencyExchanges",
+      ],
+    },
+    async (tx): Promise<GetMaxDepositAmountResponse> => {
+      let restrictWireMethod: string | undefined = undefined;
+      if (req.depositPaytoUri) {
+        const p = parsePaytoUri(req.depositPaytoUri);
+        if (!p) {
+          throw Error("invalid payto URI");
+        }
+        restrictWireMethod = p.targetType;
+      }
+      const candidateRes = await selectPayCandidates(wex, tx, {
+        currency: req.currency,
+        restrictExchanges: undefined,
+        restrictWireMethod,
+        restrictScope: req.restrictScope,
+        depositPaytoUri: req.depositPaytoUri,
+        requiredMinimumAge: undefined,
+        includePendingCoins: true,
+      });
+      return getMaxDepositAmountForAvailableCoins(req, candidateRes);
+    },
+  );
+}
+
+function getMaxPeerPushDebitAmountForAvailableCoins(
+  req: GetMaxDepositAmountRequest,
+  exchangeBaseUrl: string,
+  candidateRes: PayCoinCandidates,
+): GetMaxPeerPushDebitAmountResponse {
+  let amountEffective = Amounts.zeroOfCurrency(req.currency);
+  let fees = Amounts.zeroOfCurrency(req.currency);
+
+  for (const cc of candidateRes.coinAvailability) {
+    amountEffective = Amounts.add(
+      amountEffective,
+      Amounts.mult(cc.value, cc.numAvailable).amount,
+    ).amount;
+
+    fees = Amounts.add(
+      fees,
+      Amounts.mult(cc.feeDeposit, cc.numAvailable).amount,
+    ).amount;
+  }
+
+  return {
+    exchangeBaseUrl,
+    effectiveAmount: Amounts.stringify(amountEffective),
+    rawAmount: Amounts.stringify(Amounts.sub(amountEffective, fees).amount),
+  };
+}
+
+export async function getMaxPeerPushDebitAmount(
+  wex: WalletExecutionContext,
+  req: GetMaxPeerPushDebitAmountRequest,
+): Promise<GetMaxPeerPushDebitAmountResponse> {
+  logger.trace(`getting max deposit amount for: ${j2s(req)}`);
+
+  return await wex.db.runReadOnlyTx(
+    {
+      storeNames: [
+        "exchanges",
+        "coinAvailability",
+        "denominations",
+        "exchangeDetails",
+        "globalCurrencyExchanges",
+        "globalCurrencyAuditors",
+      ],
+    },
+    async (tx): Promise<GetMaxPeerPushDebitAmountResponse> => {
+      let result: GetMaxDepositAmountResponse | undefined = undefined;
+      const currency = req.currency;
+      const exchanges = await tx.exchanges.iter().toArray();
+      for (const exch of exchanges) {
+        if (exch.detailsPointer?.currency !== currency) {
+          continue;
+        }
+        const exchWire = await getExchangeWireDetailsInTx(tx, exch.baseUrl);
+        if (!exchWire) {
+          continue;
+        }
+        const isInScope = req.restrictScope
+          ? await checkExchangeInScopeTx(tx, exch.baseUrl, req.restrictScope)
+          : true;
+        if (!isInScope) {
+          continue;
+        }
+        if (
+          req.restrictScope &&
+          req.restrictScope.type === ScopeType.Exchange &&
+          req.restrictScope.url !== exch.baseUrl
+        ) {
+          continue;
+        }
+        const globalFees = getGlobalFees(exchWire);
+        if (!globalFees) {
+          continue;
+        }
+
+        const candidatesRes = await selectPayCandidates(wex, tx, {
+          currency,
+          restrictExchanges: {
+            auditors: [],
+            exchanges: [
+              {
+                exchangeBaseUrl: exchWire.exchangeBaseUrl,
+                exchangePub: exchWire.masterPublicKey,
+              },
+            ],
+          },
+          restrictWireMethod: undefined,
+          includePendingCoins: true,
+        });
+
+        const myExchangeRes = getMaxPeerPushDebitAmountForAvailableCoins(
+          req,
+          exchWire.exchangeBaseUrl,
+          candidatesRes,
+        );
+
+        if (!result) {
+          result = myExchangeRes;
+        } else if (Amounts.cmp(result.rawAmount, myExchangeRes.rawAmount) < 0) {
+          result = myExchangeRes;
+        }
+      }
+      if (!result) {
+        return {
+          effectiveAmount: Amounts.stringify(Amounts.zeroOfCurrency(currency)),
+          rawAmount: Amounts.stringify(Amounts.zeroOfCurrency(currency)),
+        };
+      }
+      return result;
+    },
+  );
+}
