@@ -38,6 +38,8 @@ import {
   TombstoneIdStr,
   Transaction,
   TransactionIdStr,
+  TransactionMajorState,
+  TransactionState,
   WalletNotification,
   assertUnreachable,
   checkDbInvariant,
@@ -68,7 +70,7 @@ import {
 } from "./db.js";
 import { ReadyExchangeSummary } from "./exchanges.js";
 import { createRefreshGroup } from "./refresh.js";
-import { BalanceEffect } from "./transactions.js";
+import { BalanceEffect, applyNotifyTransition } from "./transactions.js";
 import { WalletExecutionContext, getDenomInfo } from "./wallet.js";
 
 const logger = new Logger("operations/common.ts");
@@ -1087,3 +1089,58 @@ export async function cancelableFetch(
     cancellationToken: wex.cancellationToken,
   });
 }
+
+export interface RecordHandle<T> {
+  rec: T | undefined;
+  update(newRec: T | undefined, eff?: BalanceEffect): Promise<void>;
+}
+
+/**
+ * Generic helper for transitioning transactions.
+ */
+export async function getGenericRecordHandle<T>(
+  ctx: TransactionContext,
+  tx: WalletDbReadWriteTransaction<any>,
+  getRec: () => Promise<T | undefined>,
+  storeRec: (r: T) => Promise<void>,
+  deleteRec: () => Promise<void>,
+  toSt: (r: T) => TransactionState,
+  toStId: (r: T) => number,
+  updateMeta: () => Promise<void>,
+): Promise<[T | undefined, RecordHandle<T>]> {
+  const rec = await getRec();
+  let oldTxState: TransactionState;
+  let oldStId: number;
+  if (rec) {
+    oldTxState = toSt(rec);
+    oldStId = toStId(rec);
+  } else {
+    oldTxState = { major: TransactionMajorState.None };
+    oldStId = 0;
+  }
+  const update = async (newRec: T | undefined, eff?: BalanceEffect) => {
+    let newTxState: TransactionState;
+    let newStId: number;
+    if (newRec != null) {
+      newTxState = toSt(newRec);
+      newStId = toStId(newRec);
+      await storeRec(newRec);
+    } else if (rec != null) {
+      newTxState = { major: TransactionMajorState.Deleted };
+      newStId = -1;
+      await deleteRec();
+    } else {
+      // Neither old or new record. Nothing to do.
+      return;
+    }
+    await updateMeta();
+    applyNotifyTransition(tx.notify, ctx.transactionId, {
+      oldTxState,
+      newTxState,
+      balanceEffect: eff ?? BalanceEffect.Any,
+      oldStId,
+      newStId,
+    });
+  };
+  return [rec, { rec, update }];
+}
