@@ -22,38 +22,35 @@
 /**
  * Imports.
  */
-import type { IDBFactory } from "@gnu-taler/idb-bridge";
 // eslint-disable-next-line no-duplicate-imports
 import {
-  AccessStats,
   BridgeIDBFactory,
   MemoryBackend,
   createSqliteBackend,
   shimIndexedDB,
 } from "@gnu-taler/idb-bridge";
-import { createNodeSqlite3Impl } from "@gnu-taler/idb-bridge/node-sqlite3-bindings";
+import { createNodeHelperSqlite3Impl } from "@gnu-taler/idb-bridge/node-helper-sqlite3-impl";
 import {
   Logger,
   SetTimeoutTimerAPI,
   WalletRunConfig,
 } from "@gnu-taler/taler-util";
 import { createPlatformHttpLib } from "@gnu-taler/taler-util/http";
-import * as fs from "fs";
+import * as fs from "node:fs";
 import { NodeThreadCryptoWorkerFactory } from "./crypto/workers/nodeThreadWorker.js";
 import { SynchronousCryptoWorkerFactoryPlain } from "./crypto/workers/synchronousWorkerFactoryPlain.js";
-import { DefaultNodeWalletArgs, makeTempfileId } from "./host-common.js";
-import { Wallet } from "./wallet.js";
+import {
+  DefaultNodeWalletArgs,
+  getSqlite3FilenameFromStoragePath,
+  makeTempfileId,
+} from "./host-common.js";
+import { Wallet, WalletDatabaseImplementation } from "./wallet.js";
 
 const logger = new Logger("host-impl.node.ts");
 
-interface MakeDbResult {
-  idbFactory: BridgeIDBFactory;
-  getStats: () => AccessStats;
-}
-
 async function makeFileDb(
   args: DefaultNodeWalletArgs = {},
-): Promise<MakeDbResult> {
+): Promise<WalletDatabaseImplementation> {
   const myBackend = new MemoryBackend();
   myBackend.enableTracing = false;
   const storagePath = args.persistentStoragePath;
@@ -102,19 +99,27 @@ async function makeFileDb(
   return {
     idbFactory: myBridgeIdbFactory,
     getStats: () => myBackend.accessStats,
+    exportToFile(directory, stem) {
+      throw Error("not supported");
+    },
+    async readBackupJson(path: string): Promise<any> {
+      throw Error("not supported");
+    },
   };
 }
 
 async function makeSqliteDb(
   args: DefaultNodeWalletArgs,
-): Promise<MakeDbResult> {
+): Promise<WalletDatabaseImplementation> {
   if (process.env.TALER_WALLET_DBTRACING) {
     BridgeIDBFactory.enableTracing = true;
   } else {
     BridgeIDBFactory.enableTracing = false;
   }
-  const imp = await createNodeSqlite3Impl();
-  const dbFilename = args.persistentStoragePath ?? ":memory:";
+  const imp = await createNodeHelperSqlite3Impl();
+  const dbFilename = getSqlite3FilenameFromStoragePath(
+    args.persistentStoragePath,
+  );
   logger.info(`using database ${dbFilename}`);
   const myBackend = await createSqliteBackend(imp, {
     filename: dbFilename,
@@ -132,6 +137,16 @@ async function makeSqliteDb(
     getStats() {
       return myBackend.accessStats;
     },
+    async exportToFile(directory, stem) {
+      const path = `${directory}/${stem}.sqlite3`;
+      await myBackend.backupToFile(path);
+      return {
+        path,
+      };
+    },
+    async readBackupJson(path: string): Promise<any> {
+      throw Error("not supported");
+    },
     idbFactory: myBridgeIdbFactory,
   };
 }
@@ -145,7 +160,6 @@ export async function createNativeWalletHost2(
   args: DefaultNodeWalletArgs = {},
 ): Promise<{
   wallet: Wallet;
-  getDbStats: () => AccessStats;
 }> {
   const myHttpFactory = (config: WalletRunConfig) => {
     let myHttpLib;
@@ -160,7 +174,7 @@ export async function createNativeWalletHost2(
     return myHttpLib;
   };
 
-  let dbResp: MakeDbResult;
+  let dbResp: WalletDatabaseImplementation;
 
   if (
     args.persistentStoragePath &&
@@ -173,8 +187,6 @@ export async function createNativeWalletHost2(
     dbResp = await makeSqliteDb(args);
   }
 
-  const myIdbFactory: IDBFactory = dbResp.idbFactory as any as IDBFactory;
-
   shimIndexedDB(dbResp.idbFactory);
 
   let workerFactory;
@@ -203,18 +215,12 @@ export async function createNativeWalletHost2(
 
   const timer = new SetTimeoutTimerAPI();
 
-  const w = await Wallet.create(
-    myIdbFactory,
-    myHttpFactory,
-    timer,
-    workerFactory,
-  );
+  const w = await Wallet.create(dbResp, myHttpFactory, timer, workerFactory);
 
   if (args.notifyHandler) {
     w.addNotificationListener(args.notifyHandler);
   }
   return {
     wallet: w,
-    getDbStats: dbResp.getStats,
   };
 }
