@@ -14,10 +14,9 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
+import * as hostImpl from "#host-impl";
 import { DefaultNodeWalletArgs } from "./host-common.js";
 import { Wallet } from "./index.js";
-import * as hostImpl from "#host-impl";
-import { AccessStats } from "@gnu-taler/idb-bridge";
 
 /**
  * Helpers to initiate a wallet in a host environment.
@@ -30,7 +29,6 @@ export async function createNativeWalletHost2(
   args: DefaultNodeWalletArgs = {},
 ): Promise<{
   wallet: Wallet;
-  getDbStats: () => AccessStats;
 }> {
   return hostImpl.createNativeWalletHost2(args);
 }
