@@ -23,12 +23,9 @@ import {
   Amounts,
   ConvertAmountRequest,
   Duration,
-  GetAmountRequest,
-  GetPlanForOperationRequest,
   TransactionAmountMode,
   TransactionType,
   checkDbInvariant,
-  parsePaytoUri,
   strcmp,
 } from "@gnu-taler/taler-util";
 import { DenominationRecord, timestampProtocolFromDb } from "./db.js";
@@ -85,26 +82,6 @@ interface SelectedCoins {
   refresh?: RefreshChoice;
 }
 
-function getCoinsFilter(req: GetPlanForOperationRequest): CoinsFilter {
-  switch (req.type) {
-    case TransactionType.Withdrawal: {
-      return {
-        exchanges:
-          req.exchangeUrl === undefined ? undefined : [req.exchangeUrl],
-      };
-    }
-    case TransactionType.Deposit: {
-      const payto = parsePaytoUri(req.account);
-      if (!payto) {
-        throw Error(`wrong payto ${req.account}`);
-      }
-      return {
-        wireMethod: payto.targetType,
-      };
-    }
-  }
-}
-
 interface RefreshChoice {
   /**
    * Amount that need to be covered
@@ -141,13 +118,13 @@ interface AvailableCoins {
  * This function is costly (by the database access) but with high chances
  * of being cached
  */
-async function getAvailableDenoms(
+async function getAvailableCoins(
   wex: WalletExecutionContext,
   op: TransactionType,
   currency: string,
   filters: CoinsFilter = {},
 ): Promise<AvailableCoins> {
-  const operationType = getOperationType(TransactionType.Deposit);
+  const operationType = getOperationType(op);
 
   return await wex.db.runReadOnlyTx(
     {
@@ -283,7 +260,10 @@ async function getAvailableDenoms(
               coinAvail.exchangeBaseUrl,
               coinAvail.denomPubHash,
             ]);
-            checkDbInvariant(!!denom, `denomination of a coin is missing hash: ${coinAvail.denomPubHash}`);
+            checkDbInvariant(
+              !!denom,
+              `denomination of a coin is missing hash: ${coinAvail.denomPubHash}`,
+            );
             if (denom.isRevoked || !denom.isOffered) {
               continue;
             }
@@ -354,7 +334,7 @@ export async function convertDepositAmount(
   const amount = Amounts.parseOrThrow(req.amount);
   // const filter = getCoinsFilter(req);
 
-  const denoms = await getAvailableDenoms(
+  const denoms = await getAvailableCoins(
     wex,
     TransactionType.Deposit,
     amount.currency,
@@ -374,6 +354,7 @@ export async function convertDepositAmount(
 
 const LOG_REFRESH = false;
 const LOG_DEPOSIT = false;
+
 export function convertDepositAmountForAvailableCoins(
   denoms: AvailableCoins,
   amount: AmountJson,
@@ -420,9 +401,7 @@ export function convertDepositAmountForAvailableCoins(
   }
 
   const refreshDenoms = rankDenominationForRefresh(denoms.list);
-  /**
-   * FIXME: looking for refresh AFTER selecting greedy is not optimal
-   */
+  // FIXME: looking for refresh AFTER selecting greedy is not optimal
   const refreshCoin = searchBestRefreshCoin(
     depositDenoms,
     refreshDenoms,
@@ -437,7 +416,7 @@ export function convertDepositAmountForAvailableCoins(
       refreshCoin.refreshEffective,
     ).amount;
     const raw = Amounts.sub(effective, fee, refreshCoin.totalFee).amount;
-    //found with change
+    // found with change
     return {
       effective,
       raw,
@@ -449,70 +428,13 @@ export function convertDepositAmountForAvailableCoins(
   return result;
 }
 
-export async function getMaxDepositAmount(
-  wex: WalletExecutionContext,
-  req: GetAmountRequest,
-): Promise<AmountResponse> {
-  // const filter = getCoinsFilter(req);
-
-  const denoms = await getAvailableDenoms(
-    wex,
-    TransactionType.Deposit,
-    req.currency,
-    {},
-  );
-
-  const result = getMaxDepositAmountForAvailableCoins(denoms, req.currency);
-  return {
-    effectiveAmount: Amounts.stringify(result.effective),
-    rawAmount: Amounts.stringify(result.raw),
-  };
-}
-
-export function getMaxDepositAmountForAvailableCoins(
-  denoms: AvailableCoins,
-  currency: string,
-): AmountWithFee {
-  const zero = Amounts.zeroOfCurrency(currency);
-  if (!denoms.list.length) {
-    // no coins in the database
-    return { effective: zero, raw: zero };
-  }
-
-  const result = getTotalEffectiveAndRawForDeposit(
-    denoms.list.map((info) => {
-      return { info, size: info.totalAvailable ?? 0 };
-    }),
-    currency,
-  );
-
-  const wireFee = Object.values(denoms.exchanges)[0]?.wireFee ?? zero;
-  result.raw = Amounts.sub(result.raw, wireFee).amount;
-
-  return result;
-}
-
-export async function convertPeerPushAmount(
-  wex: WalletExecutionContext,
-  req: ConvertAmountRequest,
-): Promise<AmountResponse> {
-  throw Error("to be implemented after 1.0");
-}
-
-export async function getMaxPeerPushAmount(
-  wex: WalletExecutionContext,
-  req: GetAmountRequest,
-): Promise<AmountResponse> {
-  throw Error("to be implemented after 1.0");
-}
-
 export async function convertWithdrawalAmount(
   wex: WalletExecutionContext,
   req: ConvertAmountRequest,
 ): Promise<AmountResponse> {
   const amount = Amounts.parseOrThrow(req.amount);
 
-  const denoms = await getAvailableDenoms(
+  const denoms = await getAvailableCoins(
     wex,
     TransactionType.Withdrawal,
     amount.currency,
@@ -553,14 +475,6 @@ export function convertWithdrawalAmountFromAvailableCoins(
  * *****************************************************
  */
 
-/**
- *
- * @param depositDenoms
- * @param refreshDenoms
- * @param amount
- * @param mode
- * @returns
- */
 function searchBestRefreshCoin(
   depositDenoms: SelectableElement[],
   refreshDenoms: Record<string, SelectableElement[]>,
@@ -643,18 +557,13 @@ function searchBestRefreshCoin(
 
 /**
  * Returns a copy of the list sorted for the best denom to withdraw first
- *
- * @param denoms
- * @returns
  */
 function rankDenominationForWithdrawals(
   denoms: CoinInfo[],
   mode: TransactionAmountMode,
 ): SelectableElement[] {
   const copyList = [...denoms];
-  /**
-   * Rank coins
-   */
+  /// Rank coins
   copyList.sort((d1, d2) => {
     // the best coin to use is
     // 1.- the one that contrib more and pay less fee
@@ -681,8 +590,8 @@ function rankDenominationForWithdrawals(
   return copyList.map((info) => {
     switch (mode) {
       case TransactionAmountMode.Effective: {
-        //if the user instructed "effective" then we need to selected
-        //greedy total coin value
+        // if the user instructed "effective" then we need to selected
+        // greedy total coin value
         return {
           info,
           value: info.value,
@@ -690,8 +599,8 @@ function rankDenominationForWithdrawals(
         };
       }
       case TransactionAmountMode.Raw: {
-        //if the user instructed "raw" then we need to selected
-        //greedy total coin raw amount (without fee)
+        // if the user instructed "raw" then we need to selected
+        // greedy total coin raw amount (without fee)
         return {
           info,
           value: Amounts.add(info.value, info.denomWithdraw).amount,
@@ -713,17 +622,15 @@ function rankDenominationForDeposit(
   mode: TransactionAmountMode,
 ): SelectableElement[] {
   const copyList = [...denoms];
-  /**
-   * Rank coins
-   */
+  // Rank coins
   copyList.sort((d1, d2) => {
     // the best coin to use is
     // 1.- the one that contrib more and pay less fee
     // 2.- it takes more time before expires
 
-    //different exchanges may have different wireFee
-    //ranking should take the relative contribution in the exchange
-    //which is (value - denomFee / fixedFee)
+    // different exchanges may have different wireFee
+    // ranking should take the relative contribution in the exchange
+    // which is (value - denomFee / fixedFee)
     const rate1 = Amounts.isZero(d1.denomDeposit)
       ? Number.MIN_SAFE_INTEGER
       : Amounts.divmod(d1.value, d1.denomDeposit).quotient;
@@ -742,8 +649,8 @@ function rankDenominationForDeposit(
   return copyList.map((info) => {
     switch (mode) {
       case TransactionAmountMode.Effective: {
-        //if the user instructed "effective" then we need to selected
-        //greedy total coin value
+        // if the user instructed "effective" then we need to selected
+        // greedy total coin value
         return {
           info,
           value: info.value,
@@ -751,8 +658,8 @@ function rankDenominationForDeposit(
         };
       }
       case TransactionAmountMode.Raw: {
-        //if the user instructed "raw" then we need to selected
-        //greedy total coin raw amount (without fee)
+        // if the user instructed "raw" then we need to selected
+        // greedy total coin raw amount (without fee)
         return {
           info,
           value: Amounts.sub(info.value, info.denomDeposit).amount,
@@ -765,9 +672,6 @@ function rankDenominationForDeposit(
 
 /**
  * Returns a copy of the list sorted for the best denom to withdraw first
- *
- * @param denoms
- * @returns
  */
 function rankDenominationForRefresh(
   denoms: CoinInfo[],
@@ -817,7 +721,7 @@ function selectGreedyCoins(
       break iterateDenoms;
     }
 
-    //use Amounts.divmod instead of iterate
+    // use Amounts.divmod instead of iterate
     const div = Amounts.divmod(left, denom.value);
     const size = Math.min(div.quotient, denom.total);
     if (size > 0) {
@@ -829,7 +733,7 @@ function selectGreedyCoins(
       denom.total = denom.total - size;
     }
 
-    //go next denom
+    // go next denom
     denomIdx++;
   }
 
@@ -839,7 +743,7 @@ function selectGreedyCoins(
 type AmountWithFee = { raw: AmountJson; effective: AmountJson };
 type AmountAndRefresh = AmountWithFee & { refresh?: RefreshChoice };
 
-export function getTotalEffectiveAndRawForDeposit(
+function getTotalEffectiveAndRawForDeposit(
   list: { info: CoinInfo; size: number }[],
   currency: string,
 ): AmountWithFee {
