@@ -169,8 +169,7 @@ export class PeerPullCreditTransactionContext implements TransactionContext {
           this.wex,
           withdrawalGroupId,
         );
-        const res = await withdrawalCtx.deleteTransactionInTx(tx);
-        notifs.push(...res.notifs);
+        await withdrawalCtx.deleteTransactionInTx(tx);
       }
     });
   }
