@@ -304,8 +304,7 @@ export class PeerPushCreditTransactionContext implements TransactionContext {
           this.wex,
           withdrawalGroupId,
         );
-        const res = await withdrawalCtx.deleteTransactionInTx(tx);
-        notifs.push(...res.notifs);
+        await withdrawalCtx.deleteTransactionInTx(tx);
       }
     });
   }
