@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2019-2024 Taler Systems S.A.
+ (C) 2019-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -26,12 +26,14 @@ import {
   AgeCommitment,
   AgeRestriction,
   AmountJson,
+  AmountLike,
   Amounts,
-  amountToPretty,
   assertUnreachable,
+  BlindedDenominationSignature,
   checkDbInvariant,
   codecForCoinHistoryResponse,
   codecForExchangeMeltResponse,
+  codecForExchangeRevealMeltResponseV2,
   codecForExchangeRevealResponse,
   CoinPublicKeyString,
   CoinRefreshRequest,
@@ -41,8 +43,11 @@ import {
   Duration,
   encodeCrock,
   ExchangeMeltRequest,
+  ExchangeMeltRequestV2,
   ExchangeProtocolVersion,
   ExchangeRefreshRevealRequest,
+  ExchangeRefreshRevealRequestV2,
+  ExchangeRefundRequest,
   fnutil,
   ForceRefreshRequest,
   getErrorDetailFromException,
@@ -50,9 +55,11 @@ import {
   HashCodeString,
   HttpStatusCode,
   j2s,
+  LibtoolVersion,
   Logger,
   makeErrorDetail,
   NotificationType,
+  RefreshPlanchetInfo,
   RefreshReason,
   TalerErrorCode,
   TalerErrorDetail,
@@ -73,6 +80,7 @@ import {
   throwUnexpectedRequestError,
 } from "@gnu-taler/taler-util/http";
 import {
+  cancelableFetch,
   constructTaskIdentifier,
   genericWaitForState,
   makeCoinsVisible,
@@ -80,7 +88,6 @@ import {
   TaskIdStr,
   TaskRunResult,
   TaskRunResultType,
-  TombstoneTag,
   TransactionContext,
   TransitionResult,
   TransitionResultType,
@@ -110,19 +117,22 @@ import {
   WalletDbStoresArr,
 } from "./db.js";
 import { selectWithdrawalDenominations } from "./denomSelection.js";
+import { fetchFreshExchange, getScopeForAllExchanges } from "./exchanges.js";
 import {
+  applyNotifyTransition,
+  BalanceEffect,
   constructTransactionIdentifier,
   isUnsuccessfulTransaction,
-  notifyTransition,
-  TransitionInfo,
 } from "./transactions.js";
 import {
   EXCHANGE_COINS_LOCK,
   getDenomInfo,
   WalletExecutionContext,
 } from "./wallet.js";
-import { getCandidateWithdrawalDenomsTx } from "./withdraw.js";
-import { getScopeForAllExchanges } from "./exchanges.js";
+import { getWithdrawableDenomsTx } from "./withdraw.js";
+
+/** Maximum number of new coins. */
+const maxRefreshSessionSize = 64;
 
 const logger = new Logger("refresh.ts");
 
@@ -187,7 +197,13 @@ export class RefreshTransactionContext implements TransactionContext {
     return {
       type: TransactionType.Refresh,
       txState,
-      scopes: await getScopeForAllExchanges(tx, !refreshGroupRecord.infoPerExchange? []: Object.keys(refreshGroupRecord.infoPerExchange)),
+      stId: refreshGroupRecord.operationStatus,
+      scopes: await getScopeForAllExchanges(
+        tx,
+        !refreshGroupRecord.infoPerExchange
+          ? []
+          : Object.keys(refreshGroupRecord.infoPerExchange),
+      ),
       txActions: computeRefreshTransactionActions(refreshGroupRecord),
       refreshReason: refreshGroupRecord.reason,
       amountEffective: isUnsuccessfulTransaction(txState)
@@ -204,6 +220,7 @@ export class RefreshTransactionContext implements TransactionContext {
         tag: TransactionType.Refresh,
         refreshGroupId: refreshGroupRecord.refreshGroupId,
       }),
+      failReason: refreshGroupRecord.failReason,
       ...(ort?.lastError ? { error: ort.lastError } : {}),
     };
   }
@@ -227,7 +244,7 @@ export class RefreshTransactionContext implements TransactionContext {
         ]
       >,
     ) => Promise<TransitionResult<RefreshGroupRecord>>,
-  ): Promise<TransitionInfo | undefined> {
+  ): Promise<boolean> {
     const baseStores = [
       "refreshGroups" as const,
       "transactionsMeta" as const,
@@ -238,17 +255,20 @@ export class RefreshTransactionContext implements TransactionContext {
     let stores = opts.extraStores
       ? [...baseStores, ...opts.extraStores]
       : baseStores;
-    const transitionInfo = await this.wex.db.runReadWriteTx(
+    return await this.wex.db.runReadWriteTx(
       { storeNames: stores },
       async (tx) => {
         const wgRec = await tx.refreshGroups.get(this.refreshGroupId);
+        let oldStId: number;
         let oldTxState: TransactionState;
         if (wgRec) {
           oldTxState = computeRefreshTransactionState(wgRec);
+          oldStId = wgRec.operationStatus;
         } else {
           oldTxState = {
             major: TransactionMajorState.None,
           };
+          oldStId = 0;
         }
         const res = await f(wgRec, tx);
         switch (res.type) {
@@ -256,44 +276,81 @@ export class RefreshTransactionContext implements TransactionContext {
             await tx.refreshGroups.put(res.rec);
             await this.updateTransactionMeta(tx);
             const newTxState = computeRefreshTransactionState(res.rec);
-            return {
+            applyNotifyTransition(tx.notify, this.transactionId, {
               oldTxState,
               newTxState,
-            };
+              balanceEffect: BalanceEffect.PreserveUserVisible,
+              oldStId,
+              newStId: res.rec.operationStatus,
+            });
+            return true;
           }
           case TransitionResultType.Delete:
             await tx.refreshGroups.delete(this.refreshGroupId);
             await this.updateTransactionMeta(tx);
-            return {
+            applyNotifyTransition(tx.notify, this.transactionId, {
               oldTxState,
               newTxState: {
                 major: TransactionMajorState.None,
               },
-            };
+              // Deletion will affect balance
+              balanceEffect: BalanceEffect.Any,
+              newStId: -1,
+              oldStId,
+            });
+            return true;
           default:
-            return undefined;
+            return false;
         }
       },
     );
-    notifyTransition(this.wex, this.transactionId, transitionInfo);
-    return transitionInfo;
   }
 
   async deleteTransaction(): Promise<void> {
-    await this.transition(
+    const res = await this.wex.db.runReadWriteTx(
       {
-        extraStores: ["tombstones"],
+        storeNames: ["refreshGroups", "refreshSessions", "tombstones"],
       },
-      async (rec, tx) => {
-        if (!rec) {
-          return TransitionResult.stay();
-        }
-        await tx.tombstones.put({
-          id: TombstoneTag.DeleteRefreshGroup + ":" + this.refreshGroupId,
-        });
-        return TransitionResult.delete();
+      async (tx) => {
+        return this.deleteTransactionInTx(tx);
       },
     );
+    for (const notif of res.notifs) {
+      this.wex.ws.notify(notif);
+    }
+  }
+
+  async deleteTransactionInTx(
+    tx: WalletDbReadWriteTransaction<
+      ["refreshGroups", "refreshSessions", "tombstones"]
+    >,
+  ): Promise<{ notifs: WalletNotification[] }> {
+    const notifs: WalletNotification[] = [];
+    const rg = await tx.refreshGroups.get(this.refreshGroupId);
+    if (!rg) {
+      logger.warn(
+        `unable to delete transaction ${this.transactionId}, not found`,
+      );
+      return { notifs };
+    }
+    const oldTxState = computeRefreshTransactionState(rg);
+    const sessions = await tx.refreshSessions.indexes.byRefreshGroupId.getAll(
+      rg.refreshGroupId,
+    );
+    for (const s of sessions) {
+      await tx.refreshSessions.delete([s.refreshGroupId, s.coinIndex]);
+    }
+    await tx.refreshGroups.delete(rg.refreshGroupId);
+    notifs.push({
+      type: NotificationType.TransactionStateTransition,
+      transactionId: this.transactionId,
+      oldTxState,
+      newTxState: {
+        major: TransactionMajorState.Deleted,
+      },
+      newStId: -1,
+    });
+    return { notifs };
   }
 
   async suspendTransaction(): Promise<void> {
@@ -341,7 +398,7 @@ export class RefreshTransactionContext implements TransactionContext {
     });
   }
 
-  async failTransaction(): Promise<void> {
+  async failTransaction(reason?: TalerErrorDetail): Promise<void> {
     await this.transition({}, async (rec, tx) => {
       if (!rec) {
         return TransitionResult.stay();
@@ -353,6 +410,7 @@ export class RefreshTransactionContext implements TransactionContext {
         case RefreshOperationStatus.Pending:
         case RefreshOperationStatus.Suspended: {
           rec.operationStatus = RefreshOperationStatus.Failed;
+          rec.failReason = reason;
           return TransitionResult.transition(rec);
         }
         default:
@@ -368,26 +426,19 @@ export async function getTotalRefreshCost(
   refreshedDenom: DenominationInfo,
   amountLeft: AmountJson,
 ): Promise<AmountJson> {
-  const cacheKey = `denom=${refreshedDenom.exchangeBaseUrl}/${
-    refreshedDenom.denomPubHash
-  };left=${Amounts.stringify(amountLeft)}`;
-  const cacheRes = wex.ws.refreshCostCache.get(cacheKey);
-  if (cacheRes) {
-    return cacheRes;
-  }
-  const allDenoms = await getCandidateWithdrawalDenomsTx(
-    wex,
-    tx,
-    refreshedDenom.exchangeBaseUrl,
-    Amounts.currencyOf(amountLeft),
-  );
-  const res = getTotalRefreshCostInternal(
-    allDenoms,
-    refreshedDenom,
+  const { exchangeBaseUrl, denomPubHash } = refreshedDenom;
+  const key = `denom=${exchangeBaseUrl}/${denomPubHash};left=${Amounts.stringify(
     amountLeft,
-  );
-  wex.ws.refreshCostCache.put(cacheKey, res);
-  return res;
+  )}`;
+  return wex.ws.refreshCostCache.getOrPut(key, async () => {
+    const allDenoms = await getWithdrawableDenomsTx(
+      wex,
+      tx,
+      exchangeBaseUrl,
+      Amounts.currencyOf(amountLeft),
+    );
+    return getTotalRefreshCostInternal(allDenoms, refreshedDenom, amountLeft);
+  });
 }
 
 /**
@@ -406,16 +457,19 @@ export function getTotalRefreshCostInternal(
   refreshedDenom: DenominationInfo,
   amountLeft: AmountJson,
 ): AmountJson {
+  logger.trace(
+    `computing total refresh cost, denom value ${
+      refreshedDenom.value
+    }, amount left ${Amounts.stringify(amountLeft)}`,
+  );
   const withdrawAmount = Amounts.sub(
     amountLeft,
     refreshedDenom.feeRefresh,
   ).amount;
   const denomMap = Object.fromEntries(denoms.map((x) => [x.denomPubHash, x]));
-  const withdrawDenoms = selectWithdrawalDenominations(
-    withdrawAmount,
-    denoms,
-    false,
-  );
+  const withdrawDenoms = selectWithdrawalDenominations(withdrawAmount, denoms, {
+    limitCoins: maxRefreshSessionSize,
+  });
   const resultingAmount = Amounts.add(
     Amounts.zeroOfCurrency(withdrawAmount.currency),
     ...withdrawDenoms.selectedDenoms.map(
@@ -424,9 +478,9 @@ export function getTotalRefreshCostInternal(
   ).amount;
   const totalCost = Amounts.sub(amountLeft, resultingAmount).amount;
   logger.trace(
-    `total refresh cost for ${amountToPretty(amountLeft)} is ${amountToPretty(
-      totalCost,
-    )}`,
+    `total refresh cost for ${Amounts.toPretty(
+      amountLeft,
+    )} is ${Amounts.toPretty(totalCost)}`,
   );
   return totalCost;
 }
@@ -481,8 +535,6 @@ async function initRefreshSession(
 
   const exchangeBaseUrl = oldCoin.exchangeBaseUrl;
 
-  const sessionSecretSeed = encodeCrock(getRandomBytes(64));
-
   const oldDenom = await getDenomInfo(
     wex,
     tx,
@@ -496,7 +548,7 @@ async function initRefreshSession(
 
   const currency = refreshGroup.currency;
 
-  const availableDenoms = await getCandidateWithdrawalDenomsTx(
+  const availableDenoms = await getWithdrawableDenomsTx(
     wex,
     tx,
     exchangeBaseUrl,
@@ -511,12 +563,14 @@ async function initRefreshSession(
   const newCoinDenoms = selectWithdrawalDenominations(
     availableAmount,
     availableDenoms,
-    wex.ws.config.testing.denomselAllowLate,
+    {
+      limitCoins: maxRefreshSessionSize,
+    },
   );
 
   if (newCoinDenoms.selectedDenoms.length === 0) {
     logger.trace(
-      `not refreshing, available amount ${amountToPretty(
+      `not refreshing, available amount ${Amounts.toPretty(
         availableAmount,
       )} too small`,
     );
@@ -547,7 +601,6 @@ async function initRefreshSession(
     coinIndex,
     refreshGroupId,
     norevealIndex: undefined,
-    sessionSecretSeed: sessionSecretSeed,
     newDenoms: newCoinDenoms.selectedDenoms.map((x) => ({
       count: x.count,
       denomPubHash: x.denomPubHash,
@@ -686,35 +739,51 @@ async function refreshMelt(
     return;
   }
 
-  const { newCoinDenoms, oldCoin, oldDenom, refreshGroup, refreshSession } = d;
-
-  let exchangeProtocolVersion: ExchangeProtocolVersion;
-  switch (d.oldDenom.denomPub.cipher) {
-    case DenomKeyType.Rsa: {
-      exchangeProtocolVersion = ExchangeProtocolVersion.V12;
-      break;
+  // Make sure that we have a seed.
+  if (
+    d.refreshSession.sessionPublicSeed == null &&
+    d.refreshSession.sessionSecretSeed == null
+  ) {
+    const exchange = await fetchFreshExchange(wex, d.oldCoin.exchangeBaseUrl);
+    const exchangeVer = LibtoolVersion.parseVersion(
+      exchange.protocolVersionRange,
+    );
+    checkDbInvariant(!!exchangeVer, "bad exchange version string");
+    const seed = encodeCrock(getRandomBytes(64));
+    const updatedSession = await wex.db.runReadWriteTx(
+      {
+        storeNames: ["refreshSessions"],
+      },
+      async (tx) => {
+        const refreshSession = await tx.refreshSessions.get([
+          refreshGroupId,
+          coinIndex,
+        ]);
+        if (!refreshSession) {
+          return undefined;
+        }
+        if (
+          refreshSession.sessionPublicSeed != null ||
+          refreshSession.sessionSecretSeed != null
+        ) {
+          return refreshSession;
+        }
+        if (exchangeVer.current >= 27) {
+          refreshSession.sessionPublicSeed = seed;
+        } else {
+          refreshSession.sessionSecretSeed = seed;
+        }
+        await tx.refreshSessions.put(refreshSession);
+        return refreshSession;
+      },
+    );
+    if (!updatedSession) {
+      throw Error("Could not update refresh session (concurrent deletion?).");
     }
-    default:
-      throw Error("unsupported key type");
+    d.refreshSession = updatedSession;
   }
 
-  const derived = await wex.cryptoApi.deriveRefreshSession({
-    exchangeProtocolVersion,
-    kappa: 3,
-    meltCoinDenomPubHash: oldCoin.denomPubHash,
-    meltCoinPriv: oldCoin.coinPriv,
-    meltCoinPub: oldCoin.coinPub,
-    feeRefresh: Amounts.parseOrThrow(oldDenom.feeRefresh),
-    meltCoinMaxAge: oldCoin.maxAge,
-    meltCoinAgeCommitmentProof: oldCoin.ageCommitmentProof,
-    newCoinDenoms,
-    sessionSecretSeed: refreshSession.sessionSecretSeed,
-  });
-
-  const reqUrl = new URL(
-    `coins/${oldCoin.coinPub}/melt`,
-    oldCoin.exchangeBaseUrl,
-  );
+  const { newCoinDenoms, oldCoin, oldDenom, refreshGroup, refreshSession } = d;
 
   let maybeAch: HashCodeString | undefined;
   if (oldCoin.ageCommitmentProof) {
@@ -723,88 +792,232 @@ async function refreshMelt(
     );
   }
 
-  const meltReqBody: ExchangeMeltRequest = {
-    coin_pub: oldCoin.coinPub,
-    confirm_sig: derived.confirmSig,
-    denom_pub_hash: oldCoin.denomPubHash,
-    denom_sig: oldCoin.denomSig,
-    rc: derived.hash,
-    value_with_fee: Amounts.stringify(derived.meltValueWithFee),
-    age_commitment_hash: maybeAch,
-  };
+  if (refreshSession.sessionSecretSeed) {
+    // Old legacy melt protocol.
+    let exchangeProtocolVersion: ExchangeProtocolVersion;
+    switch (d.oldDenom.denomPub.cipher) {
+      case DenomKeyType.Rsa: {
+        exchangeProtocolVersion = ExchangeProtocolVersion.V12;
+        break;
+      }
+      default:
+        throw Error("unsupported key type");
+    }
 
-  const resp = await wex.ws.runSequentialized(
-    [EXCHANGE_COINS_LOCK],
-    async () => {
-      return await wex.http.fetch(reqUrl.href, {
-        method: "POST",
-        body: meltReqBody,
-        timeout: getRefreshRequestTimeout(refreshGroup),
-        cancellationToken: wex.cancellationToken,
-      });
-    },
-  );
+    const derived = await wex.cryptoApi.deriveRefreshSession({
+      exchangeProtocolVersion,
+      kappa: 3,
+      meltCoinDenomPubHash: oldCoin.denomPubHash,
+      meltCoinPriv: oldCoin.coinPriv,
+      meltCoinPub: oldCoin.coinPub,
+      feeRefresh: Amounts.parseOrThrow(oldDenom.feeRefresh),
+      meltCoinMaxAge: oldCoin.maxAge,
+      meltCoinAgeCommitmentProof: oldCoin.ageCommitmentProof,
+      newCoinDenoms,
+      sessionSecretSeed: refreshSession.sessionSecretSeed,
+    });
 
-  switch (resp.status) {
-    case HttpStatusCode.NotFound: {
-      const errDetail = await readTalerErrorResponse(resp);
-      await handleRefreshMeltNotFound(ctx, coinIndex, resp, errDetail);
-      return;
+    const reqUrl = new URL(
+      `coins/${oldCoin.coinPub}/melt`,
+      oldCoin.exchangeBaseUrl,
+    );
+
+    const meltReqBody: ExchangeMeltRequest = {
+      coin_pub: oldCoin.coinPub,
+      confirm_sig: derived.confirmSig,
+      denom_pub_hash: oldCoin.denomPubHash,
+      denom_sig: oldCoin.denomSig,
+      rc: derived.hash,
+      value_with_fee: Amounts.stringify(derived.meltValueWithFee),
+      age_commitment_hash: maybeAch,
+    };
+
+    const resp = await wex.ws.runSequentialized(
+      [EXCHANGE_COINS_LOCK],
+      async () => {
+        return await cancelableFetch(wex, reqUrl, {
+          method: "POST",
+          body: meltReqBody,
+          timeout: getRefreshRequestTimeout(refreshGroup),
+        });
+      },
+    );
+
+    switch (resp.status) {
+      case HttpStatusCode.NotFound: {
+        const errDetail = await readTalerErrorResponse(resp);
+        await handleRefreshMeltNotFound(ctx, coinIndex, resp, errDetail);
+        return;
+      }
+      case HttpStatusCode.Gone: {
+        const errDetail = await readTalerErrorResponse(resp);
+        await handleRefreshMeltGone(ctx, coinIndex, errDetail);
+        return;
+      }
+      case HttpStatusCode.Conflict: {
+        const errDetail = await readTalerErrorResponse(resp);
+        await handleRefreshMeltConflict(
+          ctx,
+          refreshGroup,
+          coinIndex,
+          errDetail,
+          derived.meltValueWithFee,
+          oldCoin,
+        );
+        return;
+      }
+      case HttpStatusCode.Ok:
+        break;
+      default: {
+        const errDetail = await readTalerErrorResponse(resp);
+        throwUnexpectedRequestError(resp, errDetail);
+      }
     }
-    case HttpStatusCode.Gone: {
-      const errDetail = await readTalerErrorResponse(resp);
-      await handleRefreshMeltGone(ctx, coinIndex, errDetail);
-      return;
+
+    const meltResponse = await readSuccessResponseJsonOrThrow(
+      resp,
+      codecForExchangeMeltResponse(),
+    );
+
+    const norevealIndex = meltResponse.noreveal_index;
+
+    refreshSession.norevealIndex = norevealIndex;
+
+    await wex.db.runReadWriteTx(
+      { storeNames: ["refreshGroups", "refreshSessions"] },
+      async (tx) => {
+        const rg = await tx.refreshGroups.get(refreshGroupId);
+        if (!rg) {
+          return;
+        }
+        if (rg.timestampFinished) {
+          return;
+        }
+        const rs = await tx.refreshSessions.get([refreshGroupId, coinIndex]);
+        if (!rs) {
+          return;
+        }
+        if (rs.norevealIndex !== undefined) {
+          return;
+        }
+        rs.norevealIndex = norevealIndex;
+        await tx.refreshSessions.put(rs);
+      },
+    );
+  } else if (refreshSession.sessionPublicSeed) {
+    // New melt protocol.
+    const derived = await wex.cryptoApi.deriveRefreshSessionV2({
+      kappa: 3,
+      meltCoinDenomPubHash: oldCoin.denomPubHash,
+      meltCoinPriv: oldCoin.coinPriv,
+      meltCoinPub: oldCoin.coinPub,
+      feeRefresh: Amounts.parseOrThrow(oldDenom.feeRefresh),
+      meltCoinMaxAge: oldCoin.maxAge,
+      meltCoinAgeCommitmentProof: oldCoin.ageCommitmentProof,
+      newCoinDenoms,
+      sessionPublicSeed: refreshSession.sessionPublicSeed,
+    });
+
+    // Wallet stores new denoms run-length encoded,
+    // we need to expand the list of denominations
+    // for the exchange.
+    const newDenomsFlat: string[] = [];
+    for (let i = 0; i < newCoinDenoms.length; i++) {
+      const dsel = newCoinDenoms[i];
+      for (let j = 0; j < dsel.count; j++) {
+        newDenomsFlat.push(dsel.denomPubHash);
+      }
     }
-    case HttpStatusCode.Conflict: {
-      const errDetail = await readTalerErrorResponse(resp);
-      await handleRefreshMeltConflict(
-        ctx,
-        coinIndex,
-        errDetail,
-        derived,
-        oldCoin,
-      );
-      return;
-    }
-    case HttpStatusCode.Ok:
-      break;
-    default: {
-      const errDetail = await readTalerErrorResponse(resp);
-      throwUnexpectedRequestError(resp, errDetail);
+
+    const reqUrl = new URL(`melt`, oldCoin.exchangeBaseUrl);
+    const meltReqBody: ExchangeMeltRequestV2 = {
+      old_coin_pub: oldCoin.coinPub,
+      old_denom_pub_h: oldCoin.denomPubHash,
+      old_denom_sig: oldCoin.denomSig,
+      old_age_commitment_h: maybeAch,
+      refresh_seed: refreshSession.sessionPublicSeed,
+      confirm_sig: derived.confirmSig,
+      coin_evs: derived.planchets.map((x) => x.map((y) => y.coinEv)),
+      denoms_h: newDenomsFlat,
+      value_with_fee: Amounts.stringify(derived.meltValueWithFee),
+    };
+    const resp = await wex.ws.runSequentialized(
+      [EXCHANGE_COINS_LOCK],
+      async () => {
+        return await cancelableFetch(wex, reqUrl, {
+          method: "POST",
+          body: meltReqBody,
+          timeout: getRefreshRequestTimeout(refreshGroup),
+        });
+      },
+    );
+
+    switch (resp.status) {
+      case HttpStatusCode.NotFound: {
+        const errDetail = await readTalerErrorResponse(resp);
+        await handleRefreshMeltNotFound(ctx, coinIndex, resp, errDetail);
+        return;
+      }
+      case HttpStatusCode.Gone: {
+        const errDetail = await readTalerErrorResponse(resp);
+        await handleRefreshMeltGone(ctx, coinIndex, errDetail);
+        return;
+      }
+      case HttpStatusCode.Conflict: {
+        const errDetail = await readTalerErrorResponse(resp);
+        await handleRefreshMeltConflict(
+          ctx,
+          refreshGroup,
+          coinIndex,
+          errDetail,
+          derived.meltValueWithFee,
+          oldCoin,
+        );
+        return;
+      }
+      case HttpStatusCode.Ok:
+        break;
+      default: {
+        const errDetail = await readTalerErrorResponse(resp);
+        throwUnexpectedRequestError(resp, errDetail);
+      }
     }
+
+    const meltResponse = await readSuccessResponseJsonOrThrow(
+      resp,
+      codecForExchangeMeltResponse(),
+    );
+
+    // FIXME: Check exchange's signature.
+
+    const norevealIndex = meltResponse.noreveal_index;
+
+    refreshSession.norevealIndex = norevealIndex;
+
+    await wex.db.runReadWriteTx(
+      { storeNames: ["refreshGroups", "refreshSessions"] },
+      async (tx) => {
+        const rg = await tx.refreshGroups.get(refreshGroupId);
+        if (!rg) {
+          return;
+        }
+        if (rg.timestampFinished) {
+          return;
+        }
+        const rs = await tx.refreshSessions.get([refreshGroupId, coinIndex]);
+        if (!rs) {
+          return;
+        }
+        if (rs.norevealIndex !== undefined) {
+          return;
+        }
+        rs.norevealIndex = norevealIndex;
+        await tx.refreshSessions.put(rs);
+      },
+    );
+  } else {
+    throw Error("unsupported refresh session (neither secret nor public seed)");
   }
-
-  const meltResponse = await readSuccessResponseJsonOrThrow(
-    resp,
-    codecForExchangeMeltResponse(),
-  );
-
-  const norevealIndex = meltResponse.noreveal_index;
-
-  refreshSession.norevealIndex = norevealIndex;
-
-  await wex.db.runReadWriteTx(
-    { storeNames: ["refreshGroups", "refreshSessions"] },
-    async (tx) => {
-      const rg = await tx.refreshGroups.get(refreshGroupId);
-      if (!rg) {
-        return;
-      }
-      if (rg.timestampFinished) {
-        return;
-      }
-      const rs = await tx.refreshSessions.get([refreshGroupId, coinIndex]);
-      if (!rs) {
-        return;
-      }
-      if (rs.norevealIndex !== undefined) {
-        return;
-      }
-      rs.norevealIndex = norevealIndex;
-      await tx.refreshSessions.put(rs);
-    },
-  );
 }
 
 async function handleRefreshMeltGone(
@@ -857,19 +1070,59 @@ async function handleRefreshMeltGone(
 
 async function handleRefreshMeltConflict(
   ctx: RefreshTransactionContext,
+  refreshGroup: RefreshGroupRecord,
   coinIndex: number,
   errDetails: TalerErrorDetail,
-  derived: DerivedRefreshSession,
+  meltValueWithFee: AmountLike,
   oldCoin: CoinRecord,
 ): Promise<void> {
   // Just log for better diagnostics here, error status
   // will be handled later.
   logger.error(
     `melt request for ${Amounts.stringify(
-      derived.meltValueWithFee,
+      meltValueWithFee,
     )} failed in refresh group ${ctx.refreshGroupId} due to conflict`,
   );
 
+  const refundReq = refreshGroup.refundRequests[coinIndex];
+  if (refundReq != null) {
+    const refundUrl = new URL(
+      `coins/${oldCoin.coinPub}/refund`,
+      oldCoin.exchangeBaseUrl,
+    );
+    logger.trace(`Doing deposit in refresh for coin ${coinIndex}`);
+    const httpResp = await cancelableFetch(ctx.wex, refundUrl, {
+      method: "POST",
+      body: refundReq,
+    });
+    switch (httpResp.status) {
+      case HttpStatusCode.Ok:
+        await ctx.wex.db.runReadWriteTx(
+          {
+            storeNames: ["refreshGroups"],
+          },
+          async (tx) => {
+            const rg = await tx.refreshGroups.get(refreshGroup.refreshGroupId);
+            if (!rg || rg.operationStatus != RefreshOperationStatus.Pending) {
+              return;
+            }
+            delete rg.refundRequests[coinIndex];
+            await tx.refreshGroups.put(rg);
+          },
+        );
+        break;
+      default:
+        // FIXME: Store the error somewhere in the DB?
+        logger.warn(
+          `Refund request during refresh failed: ${j2s(
+            readTalerErrorResponse(httpResp),
+          )}`,
+        );
+        break;
+    }
+    return;
+  }
+
   const historySig = await ctx.wex.cryptoApi.signCoinHistoryRequest({
     coinPriv: oldCoin.coinPriv,
     coinPub: oldCoin.coinPub,
@@ -881,12 +1134,10 @@ async function handleRefreshMeltConflict(
     oldCoin.exchangeBaseUrl,
   );
 
-  const historyResp = await ctx.wex.http.fetch(historyUrl.href, {
-    method: "GET",
+  const historyResp = await cancelableFetch(ctx.wex, historyUrl, {
     headers: {
       "Taler-Coin-History-Signature": historySig.sig,
     },
-    cancellationToken: ctx.wex.cancellationToken,
   });
 
   const historyJson = await readSuccessResponseJsonOrThrow(
@@ -1031,7 +1282,7 @@ export async function assembleRefreshRevealRequest(args: {
   const privs = Array.from(derived.transferPrivs);
   privs.splice(norevealIndex, 1);
 
-  const planchets = derived.planchetsForGammas[norevealIndex];
+  const planchets = derived.planchets[norevealIndex];
   if (!planchets) {
     throw Error("refresh index error");
   }
@@ -1159,92 +1410,146 @@ async function refreshReveal(
     norevealIndex,
   } = d;
 
-  let exchangeProtocolVersion: ExchangeProtocolVersion;
-  switch (d.oldDenom.denomPub.cipher) {
-    case DenomKeyType.Rsa: {
-      exchangeProtocolVersion = ExchangeProtocolVersion.V12;
-      break;
+  // Blinded signatures, either from the old or the new reveal protocol.
+  let resEvSigs: BlindedDenominationSignature[];
+  let planchets: RefreshPlanchetInfo[][];
+
+  if (refreshSession.sessionSecretSeed != null) {
+    // Legacy refresh session.
+
+    let exchangeProtocolVersion: ExchangeProtocolVersion;
+    switch (d.oldDenom.denomPub.cipher) {
+      case DenomKeyType.Rsa: {
+        exchangeProtocolVersion = ExchangeProtocolVersion.V12;
+        break;
+      }
+      default:
+        throw Error("unsupported key type");
     }
-    default:
-      throw Error("unsupported key type");
+
+    const derived = await wex.cryptoApi.deriveRefreshSession({
+      exchangeProtocolVersion,
+      kappa: 3,
+      meltCoinDenomPubHash: oldCoin.denomPubHash,
+      meltCoinPriv: oldCoin.coinPriv,
+      meltCoinPub: oldCoin.coinPub,
+      feeRefresh: Amounts.parseOrThrow(oldDenom.feeRefresh),
+      newCoinDenoms,
+      meltCoinMaxAge: oldCoin.maxAge,
+      meltCoinAgeCommitmentProof: oldCoin.ageCommitmentProof,
+      sessionSecretSeed: refreshSession.sessionSecretSeed,
+    });
+
+    const reqUrl = new URL(
+      `refreshes/${derived.hash}/reveal`,
+      oldCoin.exchangeBaseUrl,
+    );
+
+    const req = await assembleRefreshRevealRequest({
+      cryptoApi: wex.cryptoApi,
+      derived,
+      newDenoms: newCoinDenoms,
+      norevealIndex: norevealIndex,
+      oldCoinPriv: oldCoin.coinPriv,
+      oldCoinPub: oldCoin.coinPub,
+      oldAgeCommitment: oldCoin.ageCommitmentProof?.commitment,
+    });
+
+    const resp = await wex.ws.runSequentialized(
+      [EXCHANGE_COINS_LOCK],
+      async () =>
+        cancelableFetch(wex, reqUrl, {
+          body: req,
+          method: "POST",
+          timeout: getRefreshRequestTimeout(refreshGroup),
+        }),
+    );
+
+    switch (resp.status) {
+      case HttpStatusCode.Ok:
+        break;
+      case HttpStatusCode.Conflict:
+      case HttpStatusCode.Gone: {
+        const errDetail = await readTalerErrorResponse(resp);
+        await handleRefreshRevealError(ctx, coinIndex, errDetail);
+        return;
+      }
+      default: {
+        const errDetail = await readTalerErrorResponse(resp);
+        throwUnexpectedRequestError(resp, errDetail);
+      }
+    }
+
+    const reveal = await readSuccessResponseJsonOrThrow(
+      resp,
+      codecForExchangeRevealResponse(),
+    );
+    planchets = derived.planchets;
+    resEvSigs = reveal.ev_sigs.map((x) => x.ev_sig);
+  } else if (refreshSession.sessionPublicSeed != null) {
+    const derived = await wex.cryptoApi.deriveRefreshSessionV2({
+      kappa: 3,
+      meltCoinDenomPubHash: oldCoin.denomPubHash,
+      meltCoinPriv: oldCoin.coinPriv,
+      meltCoinPub: oldCoin.coinPub,
+      sessionPublicSeed: refreshSession.sessionPublicSeed,
+      feeRefresh: Amounts.parseOrThrow(oldDenom.feeRefresh),
+      newCoinDenoms,
+      meltCoinMaxAge: oldCoin.maxAge,
+      meltCoinAgeCommitmentProof: oldCoin.ageCommitmentProof,
+    });
+    const req: ExchangeRefreshRevealRequestV2 = {
+      rc: derived.hash,
+      signatures: derived.signatures.filter((v, i) => i != norevealIndex),
+      age_commitment: oldCoin.ageCommitmentProof?.commitment?.publicKeys,
+    };
+    const reqUrl = new URL(`reveal-melt`, oldCoin.exchangeBaseUrl);
+    const resp = await wex.ws.runSequentialized(
+      [EXCHANGE_COINS_LOCK],
+      async () =>
+        cancelableFetch(wex, reqUrl, {
+          body: req,
+          method: "POST",
+          timeout: getRefreshRequestTimeout(refreshGroup),
+        }),
+    );
+
+    switch (resp.status) {
+      case HttpStatusCode.Ok:
+        break;
+      case HttpStatusCode.Conflict:
+      case HttpStatusCode.Gone: {
+        const errDetail = await readTalerErrorResponse(resp);
+        await handleRefreshRevealError(ctx, coinIndex, errDetail);
+        return;
+      }
+      default: {
+        const errDetail = await readTalerErrorResponse(resp);
+        throwUnexpectedRequestError(resp, errDetail);
+      }
+    }
+
+    const reveal = await readSuccessResponseJsonOrThrow(
+      resp,
+      codecForExchangeRevealMeltResponseV2(),
+    );
+    resEvSigs = reveal.ev_sigs;
+    planchets = derived.planchets;
+  } else {
+    throw Error("refresh session not supported");
   }
 
-  const derived = await wex.cryptoApi.deriveRefreshSession({
-    exchangeProtocolVersion,
-    kappa: 3,
-    meltCoinDenomPubHash: oldCoin.denomPubHash,
-    meltCoinPriv: oldCoin.coinPriv,
-    meltCoinPub: oldCoin.coinPub,
-    feeRefresh: Amounts.parseOrThrow(oldDenom.feeRefresh),
-    newCoinDenoms,
-    meltCoinMaxAge: oldCoin.maxAge,
-    meltCoinAgeCommitmentProof: oldCoin.ageCommitmentProof,
-    sessionSecretSeed: refreshSession.sessionSecretSeed,
-  });
-
-  const reqUrl = new URL(
-    `refreshes/${derived.hash}/reveal`,
-    oldCoin.exchangeBaseUrl,
-  );
-
-  const req = await assembleRefreshRevealRequest({
-    cryptoApi: wex.cryptoApi,
-    derived,
-    newDenoms: newCoinDenoms,
-    norevealIndex: norevealIndex,
-    oldCoinPriv: oldCoin.coinPriv,
-    oldCoinPub: oldCoin.coinPub,
-    oldAgeCommitment: oldCoin.ageCommitmentProof?.commitment,
-  });
-
-  const resp = await wex.ws.runSequentialized(
-    [EXCHANGE_COINS_LOCK],
-    async () => {
-      return await wex.http.fetch(reqUrl.href, {
-        body: req,
-        method: "POST",
-        timeout: getRefreshRequestTimeout(refreshGroup),
-        cancellationToken: wex.cancellationToken,
-      });
-    },
-  );
-
-  switch (resp.status) {
-    case HttpStatusCode.Ok:
-      break;
-    case HttpStatusCode.Conflict:
-    case HttpStatusCode.Gone: {
-      const errDetail = await readTalerErrorResponse(resp);
-      await handleRefreshRevealError(ctx, coinIndex, errDetail);
-      return;
-    }
-    default: {
-      const errDetail = await readTalerErrorResponse(resp);
-      throwUnexpectedRequestError(resp, errDetail);
-    }
-  }
-
-  const reveal = await readSuccessResponseJsonOrThrow(
-    resp,
-    codecForExchangeRevealResponse(),
-  );
-
   const coins: CoinRecord[] = [];
 
-  const transactionId = constructTransactionIdentifier({
-    tag: TransactionType.Refresh,
-    refreshGroupId,
-  });
-
   for (let i = 0; i < refreshSession.newDenoms.length; i++) {
     const ncd = newCoinDenoms[i];
     for (let j = 0; j < refreshSession.newDenoms[i].count; j++) {
       const newCoinIndex = coins.length;
-      const pc = derived.planchetsForGammas[norevealIndex][newCoinIndex];
+      const pc = planchets[norevealIndex][newCoinIndex];
       if (ncd.denomPub.cipher !== DenomKeyType.Rsa) {
         throw Error("cipher unsupported");
       }
-      const evSig = reveal.ev_sigs[newCoinIndex].ev_sig;
+      const evSig = resEvSigs[newCoinIndex];
       const denomSig = await wex.cryptoApi.unblindDenominationSignature({
         planchet: {
           blindingKey: pc.blindingKey,
@@ -1265,7 +1570,7 @@ async function refreshReveal(
           refreshGroupId,
           oldCoinPub: refreshGroup.oldCoinPubs[coinIndex],
         },
-        sourceTransactionId: transactionId,
+        sourceTransactionId: ctx.transactionId,
         coinEvHash: pc.coinEvHash,
         maxAge: pc.maxAge,
         ageCommitmentProof: pc.ageCommitmentProof,
@@ -1449,7 +1754,7 @@ export async function processRefreshGroup(
   // We've processed all refresh session and can now update the
   // status of the whole refresh group.
 
-  const transitionInfo = await wex.db.runReadWriteTx(
+  const didTransition: boolean = await wex.db.runReadWriteTx(
     {
       storeNames: [
         "coins",
@@ -1461,15 +1766,16 @@ export async function processRefreshGroup(
     async (tx) => {
       const rg = await tx.refreshGroups.get(refreshGroupId);
       if (!rg) {
-        return;
+        return false;
       }
       switch (rg.operationStatus) {
         case RefreshOperationStatus.Pending:
           break;
         default:
-          return undefined;
+          return false;
       }
       const oldTxState = computeRefreshTransactionState(rg);
+      const oldStId = rg.operationStatus;
       const allFinal = fnutil.all(
         rg.statusPerCoin,
         (x) =>
@@ -1495,17 +1801,23 @@ export async function processRefreshGroup(
         await tx.refreshGroups.put(rg);
         await ctx.updateTransactionMeta(tx);
         const newTxState = computeRefreshTransactionState(rg);
-        return {
+        applyNotifyTransition(tx.notify, ctx.transactionId, {
           oldTxState,
           newTxState,
-        };
+          balanceEffect:
+            rg.operationStatus === RefreshOperationStatus.Failed
+              ? BalanceEffect.Any
+              : BalanceEffect.PreserveUserVisible,
+          oldStId,
+          newStId: rg.operationStatus,
+        });
+        return true;
       }
-      return undefined;
+      return false;
     },
   );
 
-  if (transitionInfo) {
-    notifyTransition(wex, ctx.transactionId, transitionInfo);
+  if (didTransition) {
     return TaskRunResult.progress();
   }
 
@@ -1709,7 +2021,6 @@ async function applyRefreshToOldCoins(
 
 export interface CreateRefreshGroupResult {
   refreshGroupId: string;
-  notifications: WalletNotification[];
 }
 
 /**
@@ -1748,8 +2059,23 @@ export async function createRefreshGroup(
 
   const estimatedOutputPerCoin = outInfo.outputPerCoin;
 
+  if (logger.shouldLogTrace()) {
+    logger.trace(
+      `creating refresh group, inputs ${j2s(oldCoinPubs.map((x) => x.amount))}`,
+    );
+  }
+
   await applyRefreshToOldCoins(wex, tx, oldCoinPubs, refreshGroupId);
 
+  const refundRequests: { [n: number]: ExchangeRefundRequest } = {};
+
+  for (let i = 0; i < oldCoinPubs.length; i++) {
+    const req = oldCoinPubs[i].refundRequest;
+    if (req != null) {
+      refundRequests[i] = req;
+    }
+  }
+
   const refreshGroup: RefreshGroupRecord = {
     operationStatus: RefreshOperationStatus.Pending,
     currency,
@@ -1763,6 +2089,7 @@ export async function createRefreshGroup(
     expectedOutputPerCoin: estimatedOutputPerCoin.map((x) =>
       Amounts.stringify(x),
     ),
+    refundRequests,
     infoPerExchange: outInfo.perExchangeInfo,
     timestampCreated: timestampPreciseToDb(TalerPreciseTimestamp.now()),
   };
@@ -1793,18 +2120,18 @@ export async function createRefreshGroup(
   // group to the DB, the shepherd will give up.
   wex.taskScheduler.startShepherdTask(ctx.taskId);
 
+  tx.notify({
+    type: NotificationType.TransactionStateTransition,
+    transactionId: ctx.transactionId,
+    oldTxState: {
+      major: TransactionMajorState.None,
+    },
+    newTxState,
+    newStId: 0,
+  });
+
   return {
     refreshGroupId,
-    notifications: [
-      {
-        type: NotificationType.TransactionStateTransition,
-        transactionId: ctx.transactionId,
-        oldTxState: {
-          major: TransactionMajorState.None,
-        },
-        newTxState,
-      },
-    ],
   };
 }
 
@@ -1921,10 +2248,6 @@ export async function forceRefresh(
     },
   );
 
-  for (const notif of res.notifications) {
-    wex.ws.notify(notif);
-  }
-
   return {
     refreshGroupId: res.refreshGroupId,
   };
