@@ -66,6 +66,7 @@ import {
 import {
   computeDenomLossTransactionStatus,
   processExchangeKyc,
+  processTaskExchangeAutoRefresh,
   updateExchangeFromUrlHandler,
 } from "./exchanges.js";
 import {
@@ -129,6 +130,7 @@ function taskGivesLiveness(taskId: string): boolean {
   switch (parsedTaskId.tag) {
     case PendingTaskType.Backup:
     case PendingTaskType.ExchangeUpdate:
+    case PendingTaskType.ExchangeAutoRefresh:
     case PendingTaskType.ExchangeWalletKyc:
       return false;
     case PendingTaskType.Deposit:
@@ -208,7 +210,9 @@ export class TaskSchedulerImpl implements TaskScheduler {
     }
     this.isRunning = true;
     try {
+      logger.info(`loading shepherd tasks from database`);
       await this.loadTasksFromDb();
+      logger.info(`done loading shepherd tasks from database`);
     } catch (e) {
       this.isRunning = false;
       throw e;
@@ -271,15 +275,15 @@ export class TaskSchedulerImpl implements TaskScheduler {
    * Mostly useful to interrupt all waits when time-travelling.
    */
   async reload(): Promise<void> {
+    logger.info("reloading all shepherd tasks");
     await this.ensureRunning();
     const tasksIds = [...this.sheps.keys()];
-    logger.info(`reloading shepherd with ${tasksIds.length} tasks`);
+    logger.info(`reloading shepherd with ${tasksIds.length} existing tasks`);
     for (const taskId of tasksIds) {
       this.stopShepherdTask(taskId);
+      this.sheps.delete(taskId);
     }
-    for (const taskId of tasksIds) {
-      this.startShepherdTask(taskId);
-    }
+    await this.loadTasksFromDb();
   }
 
   private async internalStartShepherdTask(taskId: TaskIdStr): Promise<void> {
@@ -353,11 +357,11 @@ export class TaskSchedulerImpl implements TaskScheduler {
       await info.cts.token.racePromise(this.ws.timerGroup.resolveAfter(delay));
     } catch (e) {
       if (e instanceof CancellationToken.CancellationError) {
-        logger.info(
+        logger.trace(
           `waiting for ${taskId} interrupted: ${e.message} ${j2s(e.reason)}`,
         );
       } else {
-        logger.info(`waiting for ${taskId} interrupted: ${e}`);
+        logger.trace(`waiting for ${taskId} interrupted: ${e}`);
       }
     }
   }
@@ -370,8 +374,7 @@ export class TaskSchedulerImpl implements TaskScheduler {
       if (this.ws.stopped) {
         logger.trace(`Shepherd for ${taskId} stopping as wallet is stopped`);
         return;
-      }
-      if (info.cts.token.isCancelled) {
+      } else if (info.cts.token.isCancelled) {
         logger.trace(`Shepherd for ${taskId} got cancelled`);
         return;
       }
@@ -396,32 +399,25 @@ export class TaskSchedulerImpl implements TaskScheduler {
       try {
         res = await callOperationHandlerForTaskId(wex, taskId);
       } catch (e) {
-        const errorDetail = getErrorDetailFromException(e);
-        logger.trace(
-          `Shepherd error ${taskId} saving response ${j2s(errorDetail)}`,
-        );
         res = {
           type: TaskRunResultType.Error,
-          errorDetail,
+          errorDetail: getErrorDetailFromException(e),
         };
       }
-      if (info.cts.token.isCancelled) {
-        logger.trace(`task ${taskId} cancelled, not processing result`);
-        return;
-      }
       if (this.ws.stopped) {
         logger.trace("wallet stopped, not processing result");
         return;
+      } else if (info.cts.token.isCancelled) {
+        logger.trace(`task ${taskId} cancelled, not processing result`);
+        return;
       }
+      logger.trace(`Shepherd for ${taskId} got result ${res.type}`);
       wex.oc.observe({
         type: ObservabilityEventType.ShepherdTaskResult,
         resultType: res.type,
       });
       switch (res.type) {
         case TaskRunResultType.Error: {
-          logger.trace(
-            `Shepherd for ${taskId} got error result: ${j2s(res.errorDetail)}`,
-          );
           const retryRecord = await storePendingTaskError(
             this.ws,
             taskId,
@@ -429,12 +425,11 @@ export class TaskSchedulerImpl implements TaskScheduler {
           );
           const t = timestampAbsoluteFromDb(retryRecord.retryInfo.nextRetry);
           const delay = AbsoluteTime.remaining(t);
-          logger.trace(`Waiting for ${delay.d_ms} ms`);
+          logger.trace(`Stored error for ${taskId}, waiting for ${delay.d_ms} ms`);
           await this.wait(taskId, info, delay);
           break;
         }
         case TaskRunResultType.Backoff: {
-          logger.trace(`Shepherd for ${taskId} got backoff result.`);
           const retryRecord = await storePendingTaskPending(this.ws, taskId);
           const t = timestampAbsoluteFromDb(retryRecord.retryInfo.nextRetry);
           const delay = AbsoluteTime.remaining(t);
@@ -443,14 +438,10 @@ export class TaskSchedulerImpl implements TaskScheduler {
           break;
         }
         case TaskRunResultType.Progress: {
-          logger.trace(
-            `Shepherd for ${taskId} got progress result, re-running immediately.`,
-          );
           await storeTaskProgress(this.ws, taskId);
           break;
         }
         case TaskRunResultType.ScheduleLater: {
-          logger.trace(`Shepherd for ${taskId} got schedule-later result.`);
           const retryRecord = await storePendingTaskPending(
             this.ws,
             taskId,
@@ -463,7 +454,6 @@ export class TaskSchedulerImpl implements TaskScheduler {
           break;
         }
         case TaskRunResultType.Finished:
-          logger.trace(`Shepherd for ${taskId} got finished result.`);
           await storePendingTaskFinished(this.ws, taskId);
           return;
         case TaskRunResultType.LongpollReturnedPending: {
@@ -484,7 +474,6 @@ export class TaskSchedulerImpl implements TaskScheduler {
           break;
         }
         case TaskRunResultType.NetworkRequired: {
-          logger.trace(`Shepherd for ${taskId} got network-required result.`);
           await storePendingTaskPending(this.ws, taskId);
           const delay = Duration.getForever();
           logger.trace(`Not retrying task until network is restored.`);
@@ -657,6 +646,8 @@ async function callOperationHandlerForTaskId(
   switch (pending.tag) {
     case PendingTaskType.ExchangeUpdate:
       return await updateExchangeFromUrlHandler(wex, pending.exchangeBaseUrl);
+    case PendingTaskType.ExchangeAutoRefresh:
+      return await processTaskExchangeAutoRefresh(wex, pending.exchangeBaseUrl);
     case PendingTaskType.Refresh:
       return await processRefreshGroup(wex, pending.refreshGroupId);
     case PendingTaskType.Withdraw:
@@ -706,6 +697,7 @@ async function taskToRetryNotification(
   switch (parsedTaskId.tag) {
     case PendingTaskType.ExchangeUpdate:
     case PendingTaskType.ExchangeWalletKyc:
+    case PendingTaskType.ExchangeAutoRefresh:
       return makeExchangeRetryNotification(ws, tx, pendingTaskId, e);
     case PendingTaskType.PeerPullCredit:
     case PendingTaskType.PeerPullDebit:
@@ -735,13 +727,12 @@ async function getTransactionState(
       "peerPullDebit",
       "peerPushDebit",
       "peerPushCredit",
-      "rewards",
       "refreshGroups",
       "denomLossEvents",
     ]
   >,
   transactionId: string,
-): Promise<TransactionState | undefined> {
+): Promise<{ txState: TransactionState; stId: number } | undefined> {
   const parsedTxId = parseTransactionIdentifier(transactionId);
   if (!parsedTxId) {
     throw Error("invalid tx identifier");
@@ -752,7 +743,10 @@ async function getTransactionState(
       if (!rec) {
         return undefined;
       }
-      return computeDepositTransactionStatus(rec);
+      return {
+        txState: computeDepositTransactionStatus(rec),
+        stId: rec.operationStatus,
+      };
     }
     case TransactionType.InternalWithdrawal:
     case TransactionType.Withdrawal: {
@@ -760,56 +754,80 @@ async function getTransactionState(
       if (!rec) {
         return undefined;
       }
-      return computeWithdrawalTransactionStatus(rec);
+      return {
+        txState: computeWithdrawalTransactionStatus(rec),
+        stId: rec.status,
+      };
     }
     case TransactionType.Payment: {
       const rec = await tx.purchases.get(parsedTxId.proposalId);
       if (!rec) {
         return;
       }
-      return computePayMerchantTransactionState(rec);
+      return {
+        txState: computePayMerchantTransactionState(rec),
+        stId: rec.purchaseStatus,
+      };
     }
     case TransactionType.Refund: {
       const rec = await tx.refundGroups.get(parsedTxId.refundGroupId);
       if (!rec) {
         return undefined;
       }
-      return computeRefundTransactionState(rec);
+      return {
+        txState: computeRefundTransactionState(rec),
+        stId: rec.status,
+      };
     }
     case TransactionType.PeerPullCredit: {
       const rec = await tx.peerPullCredit.get(parsedTxId.pursePub);
       if (!rec) {
         return undefined;
       }
-      return computePeerPullCreditTransactionState(rec);
+      return {
+        txState: computePeerPullCreditTransactionState(rec),
+        stId: rec.status,
+      };
     }
     case TransactionType.PeerPullDebit: {
       const rec = await tx.peerPullDebit.get(parsedTxId.peerPullDebitId);
       if (!rec) {
         return undefined;
       }
-      return computePeerPullDebitTransactionState(rec);
+      return {
+        txState: computePeerPullDebitTransactionState(rec),
+        stId: rec.status,
+      };
     }
     case TransactionType.PeerPushCredit: {
       const rec = await tx.peerPushCredit.get(parsedTxId.peerPushCreditId);
       if (!rec) {
         return undefined;
       }
-      return computePeerPushCreditTransactionState(rec);
+      return {
+        txState: computePeerPushCreditTransactionState(rec),
+        stId: rec.status,
+      };
     }
     case TransactionType.PeerPushDebit: {
       const rec = await tx.peerPushDebit.get(parsedTxId.pursePub);
       if (!rec) {
         return undefined;
       }
-      return computePeerPushDebitTransactionState(rec);
+      return {
+        txState: computePeerPushDebitTransactionState(rec),
+        stId: rec.status,
+      };
     }
     case TransactionType.Refresh: {
       const rec = await tx.refreshGroups.get(parsedTxId.refreshGroupId);
       if (!rec) {
         return undefined;
       }
-      return computeRefreshTransactionState(rec);
+      return {
+        txState: computeRefreshTransactionState(rec),
+        stId: rec.operationStatus,
+      };
     }
     case TransactionType.Recoup:
       throw Error("not yet supported");
@@ -818,7 +836,10 @@ async function getTransactionState(
       if (!rec) {
         return undefined;
       }
-      return computeDenomLossTransactionStatus(rec);
+      return {
+        txState: computeDenomLossTransactionStatus(rec),
+        stId: rec.status,
+      };
     }
     default:
       assertUnreachable(parsedTxId);
@@ -842,8 +863,9 @@ async function makeTransactionRetryNotification(
   const notif: WalletNotification = {
     type: NotificationType.TransactionStateTransition,
     transactionId: txId,
-    oldTxState: txState,
-    newTxState: txState,
+    oldTxState: txState.txState,
+    newTxState: txState.txState,
+    newStId: txState.stId,
   };
   if (e) {
     notif.errorInfo = {
@@ -860,11 +882,11 @@ async function makeExchangeRetryNotification(
   pendingTaskId: string,
   e: TalerErrorDetail | undefined,
 ): Promise<WalletNotification | undefined> {
-  logger.info("making exchange retry notification");
   const parsedTaskId = parseTaskIdentifier(pendingTaskId);
   switch (parsedTaskId.tag) {
     case PendingTaskType.ExchangeUpdate:
     case PendingTaskType.ExchangeWalletKyc:
+    case PendingTaskType.ExchangeAutoRefresh:
       break;
     default:
       throw Error("invalid task identifier");
@@ -1116,6 +1138,12 @@ export async function getActiveTaskIds(
           });
           res.taskIds.push(taskIdUpdate);
 
+          const taskIdAutoRefresh = constructTaskIdentifier({
+            tag: PendingTaskType.ExchangeAutoRefresh,
+            exchangeBaseUrl: rec.baseUrl,
+          });
+          res.taskIds.push(taskIdAutoRefresh);
+
           const reserveId = rec.currentMergeReserveRowId;
           if (reserveId == null) {
             continue;
