@@ -254,6 +254,9 @@ async function addFiltered(
       // FIXME: This means that in some cases we can return fewer transactions
       // than requested.
       if (!txDetails) {
+        logger.warn(
+          `full transaction details missing for transaction ${mtx.transactionId}`,
+        );
         continue;
       }
       numAdded += 1;
