@@ -61,6 +61,7 @@ import {
   DeleteDiscountRequest,
   DeleteExchangeRequest,
   DeleteStoredBackupRequest,
+  DeleteSubscriptionRequest,
   DenominationInfo,
   Duration,
   EmptyObject,
@@ -98,6 +99,8 @@ import {
   ListDiscountsResponse,
   ListGlobalCurrencyAuditorsResponse,
   ListGlobalCurrencyExchangesResponse,
+  ListSubscriptionsRequest,
+  ListSubscriptionsResponse,
   Logger,
   LongpollQueue,
   NotificationType,
@@ -151,9 +154,11 @@ import {
   codecForAcceptManualWithdrawalRequest,
   codecForAcceptPeerPullPaymentRequest,
   codecForAddBankAccountRequest,
+  codecForAddContactRequest,
   codecForAddExchangeRequest,
   codecForAddGlobalCurrencyAuditorRequest,
   codecForAddGlobalCurrencyExchangeRequest,
+  codecForAddMailboxMessageRequest,
   codecForAny,
   codecForApplyDevExperiment,
   codecForCanonicalizeBaseUrlRequest,
@@ -167,9 +172,12 @@ import {
   codecForConfirmWithdrawalRequestRequest,
   codecForConvertAmountRequest,
   codecForCreateDepositGroupRequest,
+  codecForDeleteContactRequest,
   codecForDeleteDiscountRequest,
   codecForDeleteExchangeRequest,
+  codecForDeleteMailboxMessageRequest,
   codecForDeleteStoredBackupRequest,
+  codecForDeleteSubscriptionRequest,
   codecForDeleteTransactionRequest,
   codecForEmptyObject,
   codecForExportDbToFileRequest,
@@ -204,6 +212,8 @@ import {
   codecForListBankAccounts,
   codecForListDiscountsRequest,
   codecForListExchangesRequest,
+  codecForListSubscriptionsRequest,
+  codecForMailboxConfiguration,
   codecForPrepareBankIntegratedWithdrawalRequest,
   codecForPreparePayRequest,
   codecForPreparePayTemplateRequest,
@@ -217,12 +227,14 @@ import {
   codecForResumeTransaction,
   codecForRetryTransactionRequest,
   codecForRunFixupRequest,
+  codecForSendTalerUriMailboxMessageRequest,
   codecForSetCoinSuspendedRequest,
   codecForSetDonauRequest,
   codecForSetWalletDeviceIdRequest,
   codecForSharePaymentRequest,
   codecForStartExchangeWalletKycRequest,
   codecForStartRefundQueryRequest,
+  codecForString,
   codecForSuspendTransaction,
   codecForTestPayArgs,
   codecForTestingGetDenomStatsRequest,
@@ -279,6 +291,7 @@ import {
   getMaxPeerPushDebitAmount,
 } from "./coinSelection.js";
 import { cancelableFetch } from "./common.js";
+import { addContact, deleteContact, listContacts } from "./contacts.js";
 import { TalerCryptoInterface } from "./crypto/cryptoImplementation.js";
 import {
   CryptoDispatcher,
@@ -332,6 +345,15 @@ import {
   markExchangeUsed,
 } from "./exchanges.js";
 import { convertDepositAmount } from "./instructedAmountConversion.js";
+import {
+  addMailboxMessage,
+  createNewMailbox,
+  deleteMailboxMessage,
+  getMailbox,
+  listMailboxMessages,
+  refreshMailbox,
+  sendTalerUriMessage,
+} from "./mailbox.js";
 import {
   ObservableDbAccess,
   ObservableTaskScheduler,
@@ -388,7 +410,12 @@ import {
   waitUntilRefreshesDone,
   withdrawTestBalance,
 } from "./testing.js";
-import { deleteDiscount, listDiscounts } from "./tokenFamilies.js";
+import {
+  deleteDiscount,
+  deleteSubscription,
+  listDiscounts,
+  listSubscriptions,
+} from "./tokenFamilies.js";
 import {
   abortTransaction,
   deleteTransaction,
@@ -1020,20 +1047,28 @@ async function handleAddExchange(
     );
     exchangeBaseUrl = req.exchangeBaseUrl;
   } else if (req.uri) {
-    if (req.uri.startsWith("http")) {
-      const canonUrl = canonicalizeBaseUrl(req.uri);
-      if (req.uri != canonUrl) {
-        throw Error("exchange base URL must be canonicalized");
-      }
-      exchangeBaseUrl = req.uri;
-    } else if (req.uri.startsWith("taler")) {
+    if (req.uri.startsWith("taler")) {
       const p = parseTalerUri(req.uri);
       if (p?.type !== TalerUriAction.AddExchange) {
         throw Error("invalid taler://add-exchange/ URI");
       }
       exchangeBaseUrl = p.exchangeBaseUrl;
+    } else if (req.allowCompletion) {
+      const completeRes = await handleCompleteExchangeBaseUrl(wex, {
+        url: req.uri,
+      });
+      if (completeRes.status != "ok") {
+        throw TalerError.fromUncheckedDetail(completeRes.error);
+      }
+      exchangeBaseUrl = completeRes.completion;
+    } else if (req.uri.startsWith("http")) {
+      const canonUrl = canonicalizeBaseUrl(req.uri);
+      if (req.uri != canonUrl) {
+        throw Error("exchange base URL must be canonicalized");
+      }
+      exchangeBaseUrl = req.uri;
     } else {
-      throw Error("AddExchangeRequest.uri must be http(s) or taler URI");
+      throw Error("AddExchangeRequest.uri must be http(s):// or taler://");
     }
   } else {
     throw Error(
@@ -1041,16 +1076,6 @@ async function handleAddExchange(
     );
   }
 
-  if (req.allowCompletion) {
-    const completeRes = await handleCompleteExchangeBaseUrl(wex, {
-      url: exchangeBaseUrl,
-    });
-    if (completeRes.status != "ok") {
-      throw TalerError.fromUncheckedDetail(completeRes.error);
-    }
-    exchangeBaseUrl = completeRes.completion;
-  }
-
   await fetchFreshExchange(wex, exchangeBaseUrl, {});
   // Exchange has been explicitly added upon user request.
   // Thus, we mark it as "used".
@@ -1292,6 +1317,24 @@ async function handleDeleteDiscount(
   return await deleteDiscount(wex, req.tokenFamilyHash);
 }
 
+async function handleListSubscriptions(
+  wex: WalletExecutionContext,
+  req: ListSubscriptionsRequest,
+): Promise<ListSubscriptionsResponse> {
+  return await listSubscriptions(
+    wex,
+    req.tokenIssuePubHash,
+    req.merchantBaseUrl,
+  );
+}
+
+async function handleDeleteSubscription(
+  wex: WalletExecutionContext,
+  req: DeleteSubscriptionRequest,
+): Promise<EmptyObject> {
+  return await deleteSubscription(wex, req.tokenFamilyHash);
+}
+
 async function handleAbortTransaction(
   wex: WalletExecutionContext,
   req: AbortTransactionRequest,
@@ -1434,6 +1477,14 @@ async function handleGetDepositWireTypes(
           if (!parsedPayto) {
             continue;
           }
+          let preferredEntryType: "iban" | "bban" | undefined = undefined;
+          if (parsedPayto.targetType === "iban") {
+            if (det.currency === "HUF") {
+              preferredEntryType = "bban";
+            } else {
+              preferredEntryType = "iban";
+            }
+          }
           if (
             parsedPayto.isKnown &&
             parsedPayto.targetType === "x-taler-bank"
@@ -1904,6 +1955,46 @@ const handlers: { [T in WalletApiOperation]: HandlerWithValidator<T> } = {
     codec: codecForSetDonauRequest(),
     handler: handleSetDonau,
   },
+  [WalletApiOperation.AddContact]: {
+    codec: codecForAddContactRequest(),
+    handler: addContact,
+  },
+  [WalletApiOperation.DeleteContact]: {
+    codec: codecForDeleteContactRequest(),
+    handler: deleteContact,
+  },
+  [WalletApiOperation.GetContacts]: {
+    codec: codecForEmptyObject(),
+    handler: listContacts,
+  },
+  [WalletApiOperation.InitializeMailbox]: {
+    codec: codecForString(),
+    handler: createNewMailbox,
+  },
+  [WalletApiOperation.GetMailbox]: {
+    codec: codecForString(),
+    handler: getMailbox,
+  },
+  [WalletApiOperation.SendTalerUriMailboxMessage]: {
+    codec: codecForSendTalerUriMailboxMessageRequest(),
+    handler: sendTalerUriMessage,
+  },
+  [WalletApiOperation.AddMailboxMessage]: {
+    codec: codecForAddMailboxMessageRequest(),
+    handler: addMailboxMessage,
+  },
+  [WalletApiOperation.DeleteMailboxMessage]: {
+    codec: codecForDeleteMailboxMessageRequest(),
+    handler: deleteMailboxMessage,
+  },
+  [WalletApiOperation.GetMailboxMessages]: {
+    codec: codecForEmptyObject(),
+    handler: listMailboxMessages,
+  },
+  [WalletApiOperation.RefreshMailbox]: {
+    codec: codecForMailboxConfiguration(),
+    handler: refreshMailbox,
+  },
   [WalletApiOperation.TestingGetDbStats]: {
     codec: codecForEmptyObject(),
     handler: async (wex) => {
@@ -2149,6 +2240,14 @@ const handlers: { [T in WalletApiOperation]: HandlerWithValidator<T> } = {
     codec: codecForDeleteDiscountRequest(),
     handler: handleDeleteDiscount,
   },
+  [WalletApiOperation.ListSubscriptions]: {
+    codec: codecForListSubscriptionsRequest(),
+    handler: handleListSubscriptions,
+  },
+  [WalletApiOperation.DeleteSubscription]: {
+    codec: codecForDeleteSubscriptionRequest(),
+    handler: handleDeleteSubscription,
+  },
   [WalletApiOperation.SuspendTransaction]: {
     codec: codecForSuspendTransaction(),
     handler: handleSuspendTransaction,
