@@ -49,7 +49,11 @@ import {
   testWithFdold,
   testWithGv,
   testWithLocal,
-} from './wallet-qjs-tests.js';
+} from "./wallet-qjs-tests.js";
+
+import * as talerUtil from "@gnu-taler/taler-util";
+import * as talerUtilHttp from "@gnu-taler/taler-util/http";
+import * as talerWalletCore from "@gnu-taler/taler-wallet-core";
 
 setGlobalLogLevelFromString("trace");
 
@@ -92,7 +96,6 @@ class NativeWalletMessageHandler {
           notifyHandler: async (notification: WalletNotification) => {
             sendNativeMessage({ type: "notification", payload: notification });
           },
-          persistentStoragePath: args.persistentStoragePath,
           httpLib: this.httpLib,
           cryptoWorkerType: args.cryptoWorkerType,
           ...args,
@@ -106,11 +109,16 @@ class NativeWalletMessageHandler {
           enableNativeLogging();
         }
 
-        const resp = await wR.wallet.handleCoreApiRequest("initWallet", "native-init", {
-          config: args.config ?? {},
-        });
+        const resp = await wR.wallet.handleCoreApiRequest(
+          "initWallet",
+          "native-init",
+          {
+            config: args.config ?? {},
+          },
+        );
 
-        let initResponse: any = resp.type == "response" ? resp.result : resp.error;
+        let initResponse: any =
+          resp.type == "response" ? resp.result : resp.error;
 
         this.wp.resolve(wR.wallet);
 
@@ -205,7 +213,7 @@ export function installNativeWalletListener(): void {
     }
 
     const id = msg.id;
-    logger.info(`native listener: got request for ${operation} (${id})`);
+    logger.trace(`native listener: got request for ${operation} (${id})`);
 
     const startTimeMs = performanceNow();
     let respMsg: CoreApiResponse;
@@ -214,18 +222,6 @@ export function installNativeWalletListener(): void {
       if (msg.operation.startsWith("anastasis")) {
         // Entry point for Anastasis
         respMsg = await handleAnastasisRequest(operation, id, msg.args ?? {});
-      } else if (msg.operation === "testing-dangerously-eval") {
-        // Eval code, used only for testing. No client may rely on this.
-        logger.info(`evaluating ${msg.args.jscode}`);
-        const f = new Function(msg.args.jscode);
-        f();
-
-        respMsg = {
-          type: "response",
-          result: {},
-          operation: "testing-dangerously-eval",
-          id: msg.id,
-        };
       } else {
         // Entry point for wallet-core
         respMsg = await handler.handleMessage(operation, id, msg.args ?? {});
@@ -244,8 +240,8 @@ export function installNativeWalletListener(): void {
       Number((endTimeMs - startTimeMs) / 1000n / 1000n),
     );
 
-    logger.info(
-      `native listener: sending back ${respMsg.type} message for operation ${operation} (${id}) after ${requestDurationMs} ms`,
+    logger.trace(
+      `native listener: sending ${respMsg.type} for ${operation} (${id}) after ${requestDurationMs} ms`,
     );
     sendNativeMessage(respMsg);
   };
@@ -269,3 +265,10 @@ globalThis.testReduceAction = reduceAction;
 globalThis.testDiscoverPolicies = discoverPolicies;
 // @ts-ignore
 globalThis.testWithFdold = testWithFdold;
+
+// @ts-ignore
+globalThis.talerModules = {
+  talerUtil,
+  talerWalletCore,
+  talerUtilHttp,
+};
