@@ -20,24 +20,28 @@ import {
   AmountString,
   parsePaytoUri,
   PaytoUriIBAN,
+  PaytoUriTaler,
   PaytoUriTalerBank,
+  PaytoUriTalerHttp,
   PaytoUriUnknown,
   segwitMinAmount,
   stringifyPaytoUri,
   TranslatedString,
   WithdrawalExchangeAccountDetails,
 } from "@gnu-taler/taler-util";
+import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { ComponentChildren, Fragment, h, VNode } from "preact";
 import { useEffect, useRef, useState } from "preact/hooks";
+import { useBackendContext } from "../context/backend.js";
+import { useAsyncAsHook } from "../hooks/useAsyncAsHook.js";
 import { Button } from "../mui/Button.js";
+import { SafeHandler } from "../mui/handlers.js";
 import { CopiedIcon, CopyIcon } from "../svg/index.js";
 import { Amount } from "./Amount.js";
+import { ShowBanksForPaytoPopup } from "./ShowBanksForPaytoPopup.js";
+import { ShowQRsForPaytoPopup } from "./ShowQRsForPaytoPopup.js";
 import { ButtonBox, TooltipLeft, WarningBox } from "./styled/index.js";
-import { useAsyncAsHook } from "../hooks/useAsyncAsHook.js";
-import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
-import { useBackendContext } from "../context/backend.js";
-import { QR } from "./QR.js";
 
 export interface BankDetailsProps {
   subject: string;
@@ -131,6 +135,45 @@ export function BankDetailsByPaytoType({
     );
   }
 
+  if (payto.isKnown && payto.targetType === "ethereum") {
+    const min = segwitMinAmount(amount.currency);
+    const copyContent = `${payto.targetPath} ${Amounts.stringifyValue(amount)}`;
+    return (
+      <Frame
+        title={i18n.str`Ethereum transfer details`}
+        accounts={accounts}
+        updateIndex={setIndex}
+        currentIndex={index}
+        defaultCurrency={amount.currency}
+      >
+        <p>
+          <i18n.Translate>
+            You need to wire to the service provider account.
+          </i18n.Translate>
+        </p>
+
+        <table>
+          <tr>
+            <td>
+              <div>
+                {payto.targetPath} <Amount value={amount} hideCurrency /> ETH
+              </div>
+            </td>
+            <td></td>
+            <td>
+              <CopyButton getContent={() => copyContent} />
+            </td>
+          </tr>
+        </table>
+        <p>
+          <i18n.Translate>
+            Make sure the amount show {Amounts.stringifyValue(amount)} ETH
+          </i18n.Translate>
+        </p>
+      </Frame>
+    );
+  }
+
   return (
     <Frame
       title={i18n.str`Bank transfer details`}
@@ -149,20 +192,33 @@ function IBANAccountInfoTable({
   subject,
 }: {
   subject: string;
-  payto: PaytoUriUnknown | PaytoUriIBAN | PaytoUriTalerBank;
+  payto:
+  | PaytoUriUnknown
+  | PaytoUriIBAN
+  | PaytoUriTalerBank
+  | PaytoUriTalerHttp
+  | PaytoUriTaler;
 }) {
   const { i18n } = useTranslationContext();
   const api = useBackendContext();
+  const [showBanks, setShowBanks] = useState(false);
+  const [showQrs, setShowQrs] = useState(false);
 
-  const hook = useAsyncAsHook(
-    () =>
-      api.wallet.call(WalletApiOperation.GetQrCodesForPayto, {
+  const hook = useAsyncAsHook(async () => {
+    const qrs = await api.wallet.call(WalletApiOperation.GetQrCodesForPayto, {
+      paytoUri: stringifyPaytoUri(payto),
+    });
+    const banks = await api.wallet.call(
+      WalletApiOperation.GetBankingChoicesForPayto,
+      {
         paytoUri: stringifyPaytoUri(payto),
-      }),
-    [],
-  );
+      },
+    );
+    return { qrs, banks };
+  }, []);
 
-  const qrCodes = !hook || hook.hasError ? [] : hook.response.codes;
+  const qrCodes = !hook || hook.hasError ? [] : hook.response.qrs.codes;
+  const banksSites = !hook || hook.hasError ? [] : hook.response.banks.choices;
 
   const accountPart = !payto.isKnown ? (
     <Fragment>
@@ -180,11 +236,23 @@ function IBANAccountInfoTable({
       ) : undefined}
       <Row name={i18n.str`IBAN`} value={payto.iban} />
     </Fragment>
+  ) : payto.targetType === "taler-reserve" ||
+    payto.targetType === "taler-reserve-http" ? (
+    <Fragment>
+      <Row name={i18n.str`Exchange`} value={payto.exchange} />
+      <Row name={i18n.str`Reserve Pub`} value={payto.reservePub} />
+    </Fragment>
   ) : undefined;
 
-  const receiver =
+  const receiverName =
     payto.params["receiver-name"] || payto.params["receiver"] || undefined;
 
+  const receiverPostalCode =
+    payto.params["receiver-postal-code"] || undefined;
+
+  const receiverTown =
+    payto.params["receiver-town"] || undefined;
+
   return (
     <table>
       <tbody>
@@ -198,7 +266,7 @@ function IBANAccountInfoTable({
             </i18n.Translate>
           </td>
         </tr>
-        <Row name={i18n.str`Subject`} value={subject} literal />
+        <Row name={i18n.str`Subject`} value={`Taler ${subject}`} literal />
 
         <tr>
           <td colSpan={3}>
@@ -212,8 +280,14 @@ function IBANAccountInfoTable({
           </td>
         </tr>
         {accountPart}
-        {receiver ? (
-          <Row name={i18n.str`Receiver name`} value={receiver} />
+        {receiverName ? (
+          <Row name={i18n.str`Receiver name`} value={receiverName} />
+        ) : undefined}
+        {receiverPostalCode ? (
+          <Row name={i18n.str`Receiver postal code`} value={receiverPostalCode} />
+        ) : undefined}
+        {receiverTown ? (
+          <Row name={i18n.str`Receiver town`} value={receiverTown} />
         ) : undefined}
 
         <tr>
@@ -252,7 +326,7 @@ function IBANAccountInfoTable({
         </tr>
 
         <tr>
-          <td colSpan={2} width="100%" style={{ wordBreak: "break-all" }}>
+          <td colSpan={2} width="100%">
             <i18n.Translate>
               Alternative if your bank already supports PayTo URI, you can use
               this{" "}
@@ -272,15 +346,57 @@ function IBANAccountInfoTable({
           </td>
         </tr>
 
-        {qrCodes.map((qr, idx) => {
-          return (
-            <tr  key={idx}>
-              <td colSpan={3} width="100%">
-                <QR text={qr.qrContent} />
-              </td>
-            </tr>
-          );
-        })}
+        {banksSites.length < 1 ? undefined : (
+          <Fragment>
+            <div>
+              <a
+                href="#"
+                onClick={(e) => {
+                  setShowBanks(true);
+                  e.preventDefault();
+                }}
+              >
+                <i18n.Translate>
+                  Continue with banking app or website
+                </i18n.Translate>
+              </a>
+            </div>
+
+            {showBanks ? (
+              <ShowBanksForPaytoPopup
+                banks={banksSites}
+                onClose={{
+                  onClick: (async () =>
+                    setShowBanks(false)) as SafeHandler<void>,
+                }}
+              />
+            ) : undefined}
+          </Fragment>
+        )}
+
+        {qrCodes.length < 1 ? undefined : (
+          <Fragment>
+            <div>
+              <a
+                href="#"
+                onClick={(e) => {
+                  setShowQrs(true);
+                  e.preventDefault();
+                }}
+              >
+                <i18n.Translate>Show QR code</i18n.Translate>
+              </a>
+            </div>
+            {showQrs ? (
+              <ShowQRsForPaytoPopup
+                qrs={qrCodes}
+                onClose={{
+                  onClick: (async () => setShowQrs(false)) as SafeHandler<void>,
+                }}
+              />
+            ) : undefined}
+          </Fragment>
+        )}
       </tbody>
     </table>
   );
