@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2022 Taler Systems S.A.
+ (C) 2022-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -25,12 +25,12 @@ import {
   TransactionMajorState,
   DenomLossEventType,
   parsePaytoUri,
+  assertUnreachable,
 } from "@gnu-taler/taler-util";
 import { Fragment, h, VNode } from "preact";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Avatar } from "../mui/Avatar.js";
 import { Pages } from "../NavigationBar.js";
-import { assertUnreachable } from "../utils/index.js";
 import {
   Column,
   ExtraLargeText,
@@ -50,10 +50,10 @@ export function HistoryItem(props: { tx: Transaction }): VNode {
   switch (tx.type) {
     case TransactionType.Withdrawal:
       //withdrawal that has not been confirmed are hidden
-      if (!tx.exchangeBaseUrl) return <Fragment />
+      if (!tx.exchangeBaseUrl) return <Fragment />;
       return (
         <Layout
-          id={tx.transactionId}
+          href={Pages.balanceTransaction({ tid: tx.transactionId })}
           amount={tx.amountEffective}
           debitCreditIndicator={"credit"}
           title={new URL(tx.exchangeBaseUrl).hostname}
@@ -77,7 +77,7 @@ export function HistoryItem(props: { tx: Transaction }): VNode {
     case TransactionType.InternalWithdrawal:
       return (
         <Layout
-          id={tx.transactionId}
+          href={Pages.balanceTransaction({ tid: tx.transactionId })}
           amount={tx.amountEffective}
           debitCreditIndicator={"credit"}
           title={new URL(tx.exchangeBaseUrl).hostname}
@@ -101,7 +101,7 @@ export function HistoryItem(props: { tx: Transaction }): VNode {
     case TransactionType.Payment:
       return (
         <Layout
-          id={tx.transactionId}
+          href={Pages.balanceTransaction({ tid: tx.transactionId })}
           amount={tx.amountEffective}
           debitCreditIndicator={"debit"}
           title={tx.info.merchant.name}
@@ -119,7 +119,7 @@ export function HistoryItem(props: { tx: Transaction }): VNode {
     case TransactionType.Refund:
       return (
         <Layout
-          id={tx.transactionId}
+          href={Pages.balanceTransaction({ tid: tx.transactionId })}
           amount={tx.amountEffective}
           debitCreditIndicator={"credit"}
           subtitle={tx.paymentInfo ? tx.paymentInfo.summary : undefined} //FIXME: DD37 wallet-core is not returning this value
@@ -141,7 +141,7 @@ export function HistoryItem(props: { tx: Transaction }): VNode {
     case TransactionType.Refresh:
       return (
         <Layout
-          id={tx.transactionId}
+          href={Pages.balanceTransaction({ tid: tx.transactionId })}
           amount={tx.amountEffective}
           debitCreditIndicator={"credit"}
           title={"Refresh"}
@@ -155,13 +155,15 @@ export function HistoryItem(props: { tx: Transaction }): VNode {
           }
         />
       );
-    case TransactionType.Deposit:{
+    case TransactionType.Deposit: {
       const payto = parsePaytoUri(tx.targetPaytoUri);
-      const title = payto === undefined || !payto.isKnown ? tx.targetPaytoUri : 
-        payto.params["receiver-name"] ;
+      const title =
+        payto === undefined || !payto.isKnown
+          ? tx.targetPaytoUri
+          : payto.params["receiver-name"];
       return (
         <Layout
-          id={tx.transactionId}
+          href={Pages.balanceTransaction({ tid: tx.transactionId })}
           amount={tx.amountEffective}
           debitCreditIndicator={"debit"}
           title={title}
@@ -179,7 +181,7 @@ export function HistoryItem(props: { tx: Transaction }): VNode {
     case TransactionType.PeerPullCredit:
       return (
         <Layout
-          id={tx.transactionId}
+          href={Pages.balanceTransaction({ tid: tx.transactionId })}
           amount={tx.amountEffective}
           debitCreditIndicator={"credit"}
           title={tx.info.summary || "Invoice"}
@@ -196,7 +198,7 @@ export function HistoryItem(props: { tx: Transaction }): VNode {
     case TransactionType.PeerPullDebit:
       return (
         <Layout
-          id={tx.transactionId}
+          href={Pages.balanceTransaction({ tid: tx.transactionId })}
           amount={tx.amountEffective}
           debitCreditIndicator={"debit"}
           title={tx.info.summary || "Invoice"}
@@ -213,7 +215,7 @@ export function HistoryItem(props: { tx: Transaction }): VNode {
     case TransactionType.PeerPushCredit:
       return (
         <Layout
-          id={tx.transactionId}
+          href={Pages.balanceTransaction({ tid: tx.transactionId })}
           amount={tx.amountEffective}
           debitCreditIndicator={"credit"}
           title={tx.info.summary || "Transfer"}
@@ -230,7 +232,7 @@ export function HistoryItem(props: { tx: Transaction }): VNode {
     case TransactionType.PeerPushDebit:
       return (
         <Layout
-          id={tx.transactionId}
+          href={Pages.balanceTransaction({ tid: tx.transactionId })}
           amount={tx.amountEffective}
           debitCreditIndicator={"debit"}
           title={tx.info.summary || "Transfer"}
@@ -249,7 +251,7 @@ export function HistoryItem(props: { tx: Transaction }): VNode {
         case DenomLossEventType.DenomExpired: {
           return (
             <Layout
-              id={tx.transactionId}
+              href={Pages.balanceTransaction({ tid: tx.transactionId })}
               amount={tx.amountEffective}
               debitCreditIndicator={"debit"}
               title={i18n.str`Denomination expired`}
@@ -263,7 +265,7 @@ export function HistoryItem(props: { tx: Transaction }): VNode {
         case DenomLossEventType.DenomVanished: {
           return (
             <Layout
-              id={tx.transactionId}
+              href={Pages.balanceTransaction({ tid: tx.transactionId })}
               amount={tx.amountEffective}
               debitCreditIndicator={"debit"}
               title={i18n.str`Denomination vanished`}
@@ -277,7 +279,7 @@ export function HistoryItem(props: { tx: Transaction }): VNode {
         case DenomLossEventType.DenomUnoffered: {
           return (
             <Layout
-              id={tx.transactionId}
+              href={Pages.balanceTransaction({ tid: tx.transactionId })}
               amount={tx.amountEffective}
               debitCreditIndicator={"debit"}
               title={i18n.str`Denomination unoffered`}
@@ -306,7 +308,7 @@ function Layout(props: LayoutProps): VNode {
   const { i18n } = useTranslationContext();
   return (
     <HistoryRow
-      href={Pages.balanceTransaction({ tid: props.id })}
+      href={props.href}
       style={{
         backgroundColor:
           props.currentState === TransactionMajorState.Pending ||
@@ -363,10 +365,10 @@ interface LayoutProps {
   timestamp: AbsoluteTime;
   title: string;
   subtitle?: string;
-  id: string;
   iconPath: string;
   currentState: TransactionMajorState;
   description?: string;
+  href: string;
 }
 
 interface TransactionAmountProps {
