@@ -52,7 +52,7 @@ const Body = styled.div`
 
 export function Modal({ title, children, onClose }: Props): VNode {
   return (
-    <div style={{ top: 0, position: "fixed", width: "100%", height: "100%" }}>
+    <div style={{ top: 0, left: 0, position: "fixed", width: "100%", height: "100%",zIndex:100 }}>
       <FullSize onClick={onClose?.onClick}>
         <div
           onClick={(e) => e.stopPropagation()}
