@@ -14,18 +14,18 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { AmountJson, AmountString } from "@gnu-taler/taler-util";
+import { AmountJson, AmountString, PaytoUri, ScopeInfo } from "@gnu-taler/taler-util";
 import { ErrorAlertView } from "../../components/CurrentAlerts.js";
 import { Loading } from "../../components/Loading.js";
 import { ErrorAlert } from "../../context/alert.js";
-import { ButtonHandler } from "../../mui/handlers.js";
+import { AmountFieldHandler, ButtonHandler } from "../../mui/handlers.js";
 import { compose, StateViewMap } from "../../utils/index.js";
 import { useComponentState } from "./state.js";
 import { ReadyView } from "./views.js";
 
 export interface Props {
-  talerDepositUri: string | undefined;
-  amountStr: AmountString | undefined;
+  account: PaytoUri;
+  scope: ScopeInfo;
   cancel: () => Promise<void>;
   onSuccess: (tx: string) => Promise<void>;
 }
@@ -44,7 +44,9 @@ export namespace State {
   export interface Ready {
     status: "ready";
     error: undefined;
+    amount: AmountFieldHandler;
     fee: AmountJson;
+    account: PaytoUri;
     cost: AmountJson;
     effective: AmountJson;
     confirm: ButtonHandler;
