@@ -35,7 +35,7 @@ export function useComponentState({
   const amountStr = Amounts.stringify(amount);
 
   const hook = useAsyncAsHook(async () => {
-    const deposit = await api.wallet.call(WalletApiOperation.PrepareDeposit, {
+    const deposit = await api.wallet.call(WalletApiOperation.CheckDeposit, {
       amount: amountStr,
       depositPaytoUri: stringifyPaytoUri(account),
     });
@@ -76,7 +76,8 @@ export function useComponentState({
       error: Amounts.isZero(amount) ? "Can't be zero" : undefined,
     },
     confirm: {
-      onClick: pushAlertOnError(doDeposit),
+      onClick: Amounts.isZero(toBeReceived) ? undefined : pushAlertOnError(doDeposit),
+
     },
     fee: Amounts.sub(toBeReceived, debitAmount)
       .amount,
