@@ -33,6 +33,13 @@ export const Ready = tests.createExample(ReadyView, {
     value: 1,
     fraction: 0,
   },
+  amount: {
+    value: {
+      currency: "ARS",
+      value: 1,
+      fraction: 0,  
+    }
+  },
   expiration: {
     value: "2/12/12",
   },
