@@ -27,10 +27,12 @@ import {
   InvoiceCreationDetails,
 } from "../../wallet/Transaction.js";
 import { State } from "./index.js";
+import { AmountField } from "../../components/AmountField.js";
 
 export function ReadyView({
   exchangeUrl,
   subject,
+  amount,
   expiration,
   create,
   toBeReceived,
@@ -86,6 +88,13 @@ export function ReadyView({
           kind="neutral"
           big
         />
+        <p>
+          <AmountField 
+            label={i18n.str`Amount`}
+            handler={amount}
+            required
+          />
+        </p>
         <p>
           <TextField
             label="Subject"
