@@ -19,12 +19,9 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import {
-  AbsoluteTime,
-  PreparePayResult,
-  PreparePayResultType,
-} from "@gnu-taler/taler-util";
+import { AbsoluteTime, TransactionIdStr } from "@gnu-taler/taler-util";
 import * as tests from "@gnu-taler/web-util/testing";
+import { State } from "../Payment/index.js";
 import { ReadyView } from "./views.js";
 
 export default {
@@ -32,25 +29,31 @@ export default {
 };
 
 export const Ready = tests.createExample(ReadyView, {
+  status: "ready" as const,
   effective: {
     currency: "ARS",
     value: 1,
     fraction: 0,
   },
-  raw: {
+  amount: {
     currency: "ARS",
     value: 1,
     fraction: 0,
   },
   summary: "some subject",
-  uri: "taler://pay/merchant.ar/123",
-  payStatus: {
-    status: PreparePayResultType.PaymentPossible,
-    amountEffective: "ARS:1",
-  } as PreparePayResult,
+
   expiration: AbsoluteTime.fromMilliseconds(
     new Date().getTime() + 1000 * 60 * 60,
   ),
-  accept: {},
-  cancel: {},
-});
+  cancel: async () => {},
+  goToWalletManualWithdraw: async () => {},
+  choices: undefined,
+  error: undefined,
+  merchant: undefined,
+  minimum_age: undefined,
+  payHandler: {
+    onClick: undefined,
+  },
+  shareUri: "asdasd",
+  transactionId: "1" as TransactionIdStr,
+} satisfies State.Ready);
