@@ -14,6 +14,7 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
+import { AbsoluteTime, Duration } from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { Fragment, h, VNode } from "preact";
 import { Part } from "../../components/Part.js";
@@ -23,19 +24,19 @@ import {
   getAmountWithFee,
   InvoicePaymentDetails,
 } from "../../wallet/Transaction.js";
-import { State } from "./index.js";
-import { AbsoluteTime, Duration } from "@gnu-taler/taler-util";
+import { PaymentStates } from "../Payment/views.js";
+
+const inFiveMinutes = AbsoluteTime.addDuration(
+  AbsoluteTime.now(),
+  Duration.fromSpec({ minutes: 5 }),
+);
 
 export function ReadyView(
-  state: State.Ready | State.NoBalanceForCurrency | State.NoEnoughBalance,
+  state: PaymentStates,
 ): VNode {
   const { i18n } = useTranslationContext();
-  const { summary, effective, raw, expiration, uri, status, payStatus } = state;
+  const { summary, effective, amount, expiration } = state;
 
-  const inFiveMinutes = AbsoluteTime.addDuration(
-    AbsoluteTime.now(),
-    Duration.fromSpec({ minutes: 5 }),
-  );
   const willExpireSoon =
     expiration && AbsoluteTime.cmp(expiration, inFiveMinutes) === -1;
 
@@ -43,14 +44,16 @@ export function ReadyView(
     <Fragment>
       <section style={{ textAlign: "left" }}>
         <Part title={i18n.str`Subject`} text={<div>{summary}</div>} />
-        <Part
-          title={i18n.str`Details`}
-          text={
-            <InvoicePaymentDetails
-              amount={getAmountWithFee(effective, raw, "debit")}
-            />
-          }
-        />
+        {!effective ? undefined :
+          <Part
+            title={i18n.str`Details`}
+            text={
+              <InvoicePaymentDetails
+                amount={getAmountWithFee(effective, amount, "debit")}
+              />
+            }
+          />
+        }
         {willExpireSoon && (
           <Part
             title={i18n.str`Expires at`}
@@ -59,13 +62,9 @@ export function ReadyView(
           />
         )}
       </section>
-      <PaymentButtons
-        amount={effective}
-        payStatus={payStatus}
-        uri={uri}
-        payHandler={status === "ready" ? state.accept : undefined}
-        goToWalletManualWithdraw={state.goToWalletManualWithdraw}
-      />
+
+      <PaymentButtons paymentState={state} />
+
     </Fragment>
   );
 }
