@@ -15,11 +15,12 @@
  */
 
 import {
+  AbsoluteTime,
   AmountJson,
-  PreparePayResult,
-  PreparePayResultAlreadyConfirmed,
-  PreparePayResultInsufficientBalance,
-  PreparePayResultPaymentPossible,
+  MerchantContractChoice,
+  MerchantInfo,
+  PaymentInsufficientBalanceDetails,
+  TransactionIdStr
 } from "@gnu-taler/taler-util";
 import { ErrorAlertView } from "../../components/CurrentAlerts.js";
 import { Loading } from "../../components/Loading.js";
@@ -27,7 +28,7 @@ import { ErrorAlert } from "../../context/alert.js";
 import { ButtonHandler } from "../../mui/handlers.js";
 import { compose, StateViewMap } from "../../utils/index.js";
 import { useComponentState } from "./state.js";
-import { BaseView } from "./views.js";
+import { BaseView, PaymentStates } from "./views.js";
 
 export interface Props {
   talerPayUri: string;
@@ -39,16 +40,15 @@ export interface Props {
 export type State =
   | State.Loading
   | State.LoadingUriError
-  | State.Ready
-  | State.NoEnoughBalance
-  | State.NoBalanceForCurrency
-  | State.Confirmed;
+  | PaymentStates;
 
 export namespace State {
+  
   export interface Loading {
     status: "loading";
     error: undefined;
   }
+
   export interface LoadingUriError {
     status: "error";
     error: ErrorAlert;
@@ -56,39 +56,51 @@ export namespace State {
 
   interface BaseInfo {
     amount: AmountJson;
-    uri: string;
+    summary: string;
+    minimum_age: number | undefined,
+    expiration: AbsoluteTime;
+    merchant: MerchantInfo | undefined;
+    transactionId: TransactionIdStr;
+    shareUri: string;
     error: undefined;
     goToWalletManualWithdraw: (amount?: string) => Promise<void>;
     cancel: () => Promise<void>;
   }
-  export interface NoBalanceForCurrency extends BaseInfo {
-    status: "no-balance-for-currency";
-    payStatus: PreparePayResult;
-    balance: undefined;
-  }
+
   export interface NoEnoughBalance extends BaseInfo {
     status: "no-enough-balance";
-    payStatus: PreparePayResultInsufficientBalance;
-    balance: AmountJson;
+    balanceDetails: PaymentInsufficientBalanceDetails | undefined;
+    effective: undefined;
+    choices: undefined | {
+      list: MerchantContractChoice[]
+      index: number;
+      select: (d: number) => void;
+    },
   }
+
   export interface Ready extends BaseInfo {
     status: "ready";
-    payStatus: PreparePayResultPaymentPossible;
     payHandler: ButtonHandler;
-    balance: AmountJson;
+    effective: AmountJson;
+    choices: undefined | {
+      list: MerchantContractChoice[]
+      index: number;
+      select: (d: number) => void;
+    },
   }
 
   export interface Confirmed extends BaseInfo {
     status: "confirmed";
-    payStatus: PreparePayResultAlreadyConfirmed;
-    balance: AmountJson;
+    effective: undefined;
+    message?: string;
+    paid: boolean;
   }
+
 }
 
 const viewMapping: StateViewMap<State> = {
   loading: Loading,
   error: ErrorAlertView,
-  "no-balance-for-currency": BaseView,
   "no-enough-balance": BaseView,
   confirmed: BaseView,
   ready: BaseView,
