@@ -47,8 +47,8 @@ export interface PropsFromURI {
 
 export interface PropsFromParams {
   talerExchangeWithdrawUri: string | undefined;
-  scope: ScopeInfo;
-  amount: string | undefined;
+  scope: ScopeInfo | undefined;
+  amount: AmountJson | undefined;
   cancel: () => Promise<void>;
   onSuccess: (txid: string) => Promise<void>;
 }
