@@ -14,7 +14,10 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { ExchangeListItem } from "@gnu-taler/taler-util";
+import {
+  ExchangeListItem,
+  ScopeInfo
+} from "@gnu-taler/taler-util";
 import { useState } from "preact/hooks";
 import { useAlertContext } from "../context/alert.js";
 import { ButtonHandler } from "../mui/handlers.js";
@@ -39,13 +42,13 @@ export namespace State {
     onSelection: (url: string) => Promise<void>;
     onCancel: () => Promise<void>;
     list: ExchangeListItem[];
-    currency: string;
+    // scope: ScopeInfo;
     initialValue: string;
   }
 }
 
 interface Props {
-  currency: string;
+  scope: ScopeInfo;
   //there is a preference for the default at the initial state
   defaultExchange?: string;
   //list of exchanges
@@ -53,7 +56,7 @@ interface Props {
 }
 
 export function useSelectedExchange({
-  currency,
+  scope,
   defaultExchange,
   list,
 }: Props): State {
@@ -67,18 +70,8 @@ export function useSelectedExchange({
     return {
       status: "no-exchange-found",
       error: undefined,
-      currency,
-      defaultExchange,
-    };
-  }
-
-  const exchangesWithThisCurrency = list.filter((e) => e.currency === currency);
-  if (!exchangesWithThisCurrency.length) {
-    // there should be at least one exchange for this currency
-    return {
-      status: "no-exchange-found",
-      error: undefined,
-      currency,
+      // scope,
+      currency: scope.currency,
       defaultExchange,
     };
   }
@@ -87,12 +80,12 @@ export function useSelectedExchange({
     const currentExchange =
       selectedExchange ??
       defaultExchange ??
-      exchangesWithThisCurrency[0].exchangeBaseUrl;
+      list[0].exchangeBaseUrl;
     return {
       status: "selecting-exchange",
       error: undefined,
-      list: exchangesWithThisCurrency,
-      currency,
+      list: list,
+      // scope,
       initialValue: currentExchange,
       onSelection: async (exchangeBaseUrl: string) => {
         setIsSelecting(false);
@@ -136,6 +129,6 @@ export function useSelectedExchange({
     doSelect: {
       onClick: pushAlertOnError(async () => setIsSelecting(true)),
     },
-    selected: exchangesWithThisCurrency[0],
+    selected: list[0],
   };
 }
