@@ -371,7 +371,11 @@ function ButtonBase({
     );
   }
   return (
-    <button onClick={doClick} class={classNames} {...rest}>
+    <button onClick={(e) => {
+      e.preventDefault();
+      e.stopPropagation();
+      doClick();
+    }} class={classNames} {...rest}>
       {children}
     </button>
   );
