@@ -22,7 +22,7 @@ import { Colors } from "../style.js";
 export interface Props {
   color: Colors;
   disabled: boolean;
-  error?: string;
+  error?: string | Error;
   focused: boolean;
   fullWidth: boolean;
   hiddenLabel: boolean;
@@ -124,7 +124,7 @@ export interface FCCProps {
   // setAdornedStart,
   color: Colors;
   disabled: boolean;
-  error: string | undefined;
+  error: string | undefined | Error;
   filled: boolean;
   focused: boolean;
   fullWidth: boolean;
