@@ -27,7 +27,7 @@ export interface Props {
   defaultValue?: string;
   disabled?: boolean;
   disableUnderline?: boolean;
-  error?: string;
+  error?: string | Error;
   fullWidth?: boolean;
   id?: string;
   margin?: "dense" | "normal" | "none";
@@ -89,9 +89,9 @@ const filledRootStyle = css`
   border-top-left-radius: ${theme.shape.borderRadius}px;
   border-top-right-radius: ${theme.shape.borderRadius}px;
   transition: ${theme.transitions.create("background-color", {
-    duration: theme.transitions.duration.shorter,
-    easing: theme.transitions.easing.easeOut,
-  })};
+  duration: theme.transitions.duration.shorter,
+  easing: theme.transitions.easing.easeOut,
+})};
   // when is not disabled underline
   &:hover {
     background-color: ${backgroundColorHover};
@@ -124,9 +124,9 @@ const underlineStyle = css`
     right: 0px;
     transform: scaleX(0);
     transition: ${theme.transitions.create("transform", {
-      duration: theme.transitions.duration.shorter,
-      easing: theme.transitions.easing.easeOut,
-    })};
+  duration: theme.transitions.duration.shorter,
+  easing: theme.transitions.easing.easeOut,
+})};
     pointer-events: none;
   }
   &[data-focused]:after {
@@ -139,8 +139,8 @@ const underlineStyle = css`
   &:before {
     border-bottom: 1px solid
       ${theme.palette.mode === "light"
-        ? "rgba(0, 0, 0, 0.42)"
-        : "rgba(255, 255, 255, 0.7)"};
+    ? "rgba(0, 0, 0, 0.42)"
+    : "rgba(255, 255, 255, 0.7)"};
     left: 0px;
     bottom: 0px;
     right: 0px;
@@ -156,8 +156,8 @@ const underlineStyle = css`
     @media (hover: none) {
       border-bottom: 1px solid
         ${theme.palette.mode === "light"
-          ? "rgba(0, 0, 0, 0.42)"
-          : "rgba(255, 255, 255, 0.7)"};
+    ? "rgba(0, 0, 0, 0.42)"
+    : "rgba(255, 255, 255, 0.7)"};
     }
   }
   &[data-disabled]:before {
