@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2022 Taler Systems S.A.
+ (C) 2022-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -18,12 +18,12 @@ import {
   canonicalizeBaseUrl,
   Codec,
   codecForSyncTermsOfServiceResponse,
+  assertUnreachable
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import { useEffect, useState } from "preact/hooks";
 import { useAlertContext } from "../../context/alert.js";
 import { useBackendContext } from "../../context/backend.js";
-import { assertUnreachable } from "../../utils/index.js";
 import { Props, State } from "./index.js";
 
 type UrlState<T> = UrlOk<T> | UrlError;
