@@ -23,6 +23,7 @@ import {
   ExchangeEntryStatus,
   ExchangeTosStatus,
   ExchangeUpdateStatus,
+  ScopeType,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
 import * as tests from "@gnu-taler/web-util/testing";
@@ -48,7 +49,11 @@ describe("AddExchange states", () => {
           {
             exchangeBaseUrl: "http://exchange.local/",
             ageRestrictionOptions: [],
-            scopeInfo: undefined,
+            scopeInfo: {
+              currency: "ARS",
+              type: ScopeType.Exchange,
+              url: "http://exchange.local/",
+            },
             masterPub: "123qwe123",
             currency: "ARS",
             exchangeEntryStatus: ExchangeEntryStatus.Ephemeral,
@@ -56,6 +61,8 @@ describe("AddExchange states", () => {
             exchangeUpdateStatus: ExchangeUpdateStatus.UnavailableUpdate,
             paytoUris: [],
             lastUpdateTimestamp: undefined,
+            noFees: false,
+            peerPaymentsDisabled: false,
           },
         ],
       },
