@@ -25,6 +25,7 @@ import {
   ScopeInfo,
   TalerUri,
   TalerUriAction,
+  TransactionIdStr,
   TranslatedString,
   parsePaytoUri,
   parseScopeInfoShort,
@@ -82,6 +83,8 @@ import CloseIcon from "../svg/close_24px.inline.svg";
 import { AddBackupProviderPage } from "./AddBackupProvider/index.js";
 import { AddExchange } from "./AddExchange/index.js";
 import { ConfirmAddExchangeView } from "./AddExchange/views.js";
+import { AddContact } from "./AddContact/index.js";
+import { ConfirmAddContactView } from "./AddContact/views.js";
 import { BackupPage } from "./BackupPage.js";
 import { DepositPage } from "./DepositPage/index.js";
 import { DestinationSelectionPage } from "./DestinationSelection/index.js";
@@ -92,9 +95,11 @@ import { NotificationsPage } from "./Notifications/index.js";
 import { ProviderDetailPage } from "./ProviderDetailPage.js";
 import { QrReaderPage } from "./QrReader.js";
 import { SettingsPage } from "./Settings.js";
+import { ContactsPage } from "./Contacts.js";
 import { SupportedBanksForAccount } from "./SupportedBanksForAccount.js";
 import { TransactionPage } from "./Transaction.js";
 import { WelcomePage } from "./Welcome.js";
+import { MailboxPage } from "./Mailbox.js";
 
 export function Application(): VNode {
   const { i18n } = useTranslationContext();
@@ -153,6 +158,44 @@ export function Application(): VNode {
               </WalletTemplate>
             )}
           />
+          <Route
+          path={Pages.contacts.pattern}
+            component={({ tid }: { tid?: string }) => (
+              <WalletTemplate
+                goToTransaction={redirectToTxInfo}
+                goToURL={redirectToURL}
+              >
+                <ContactsPage
+                  onMessageSent={() => {
+                      return redirectTo(Pages.contacts({}));
+                    }
+                  }
+                  tid={tid}
+                  />
+              </WalletTemplate>
+            )}
+          />
+          <Route
+          path={Pages.mailbox}
+            component={() => (
+              <WalletTemplate
+                goToTransaction={redirectToTxInfo}
+                goToURL={redirectToURL}
+              >
+                <MailboxPage
+                  onTalerUri={(talerActionUrl: string) => {
+                    redirectTo(
+                      Pages.defaultCta({
+                        uri: encodeCrockForURI(
+                          talerActionUrl
+                        ),
+                      }),
+                    );
+                  }}
+                />
+              </WalletTemplate>
+            )}
+          />
           <Route
             path={Pages.notifications}
             component={() => (
@@ -165,13 +208,22 @@ export function Application(): VNode {
            * SETTINGS
            */}
           <Route
-            path={Pages.settingsExchangeAdd.pattern}
+          path={Pages.settingsExchangeAdd.pattern}
             component={() => (
               <WalletTemplate goToURL={redirectToURL}>
-                <AddExchange onBack={() => redirectTo(Pages.balance)} />
+              <AddExchange onBack={() => redirectTo(Pages.balance)} />
               </WalletTemplate>
             )}
           />
+          <Route
+            path={Pages.contactsAdd}
+            component={() => (
+              <WalletTemplate goToURL={redirectToURL}>
+                <AddContact onBack={() => redirectTo(Pages.contacts({}))} />
+              </WalletTemplate>
+            )}
+          />
+
 
           <Route
             path={Pages.balanceHistory.pattern}
@@ -386,6 +438,9 @@ export function Application(): VNode {
                       }),
                     )
                   }
+                  onForwardToContact={(tid: TransactionIdStr) =>
+                    redirectTo(Pages.contacts({tid}))
+                  }
                 />
               </WalletTemplate>
             )}
@@ -821,6 +876,39 @@ export function Application(): VNode {
               );
             }}
           />
+          <Route
+            path={Pages.ctaAddContact}
+            component={({ talerUri }: { talerUri: string }) => {
+              const tUri = parseTalerUri(
+                decodeCrockFromURI(talerUri).toLowerCase(),
+              );
+              const contact =
+                tUri?.type === TalerUriAction.AddContact
+                  ? {
+                    alias: tUri.alias,
+                    aliasType: tUri.aliasType,
+                    mailboxBaseUri: tUri.mailboxBaseUri,
+                    mailboxAddress: tUri.mailboxIdentity,
+                    source: tUri.sourceBaseUrl,
+                  }
+                  : undefined;
+              if (!contact) {
+                redirectTo(Pages.balanceHistory({}));
+                return <div>invalid url {talerUri}</div>;
+              }
+              return (
+                <CallToActionTemplate title={i18n.str`Add contact`}>
+                  <ConfirmAddContactView
+                    contact={contact}
+                    status="confirm"
+                    error={undefined}
+                    onCancel={() => redirectTo(Pages.balanceHistory({}))}
+                    onConfirm={() => redirectTo(Pages.contacts({}))}
+                  />
+                </CallToActionTemplate>
+              );
+            }}
+          />
           <Route
             path={Pages.paytoBanks.pattern}
             component={({ payto }: { payto: string }) => {
