@@ -14,14 +14,16 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { AmountJson, KnownBankAccountsInfo, PaytoUri, ScopeInfo } from "@gnu-taler/taler-util";
+import {
+  WalletBankAccountInfo,
+  PaytoUri,
+  ScopeInfo,
+  PaytoString,
+} from "@gnu-taler/taler-util";
 import { ErrorAlertView } from "../../components/CurrentAlerts.js";
 import { Loading } from "../../components/Loading.js";
 import { ErrorAlert } from "../../context/alert.js";
-import {
-  AmountFieldHandler,
-  ButtonHandler
-} from "../../mui/handlers.js";
+import { ButtonHandler } from "../../mui/handlers.js";
 import { StateViewMap, compose } from "../../utils/index.js";
 import { useComponentState } from "./state.js";
 import { ReadyView, SelectCurrencyView } from "./views.js";
@@ -31,15 +33,15 @@ export type Props = PropsGet | PropsSend;
 interface PropsGet {
   type: "get";
   scope?: ScopeInfo;
-  goToWalletManualWithdraw: (s:ScopeInfo) => void;
-  goToWalletWalletInvoice: (s:ScopeInfo) => void;
+  goToWalletManualWithdraw: (s: ScopeInfo) => void;
+  goToWalletWalletInvoice: (s: ScopeInfo) => void;
 }
 interface PropsSend {
   type: "send";
   scope: ScopeInfo;
-  goToWalletKnownBankDeposit: (s:ScopeInfo, p: PaytoUri) => void;
-  goToWalletNewBankDeposit: (s:ScopeInfo) => void;
-  goToWalletWalletSend: (s:ScopeInfo) => void;
+  goToWalletKnownBankDeposit: (s: ScopeInfo, p: PaytoString) => void;
+  goToWalletNewBankDeposit: (s: ScopeInfo) => void;
+  goToWalletWalletSend: (s: ScopeInfo) => void;
 }
 
 export type State =
@@ -70,14 +72,13 @@ export namespace State {
     status: "ready";
     error: undefined;
     type: Props["type"];
-    onSelectAccount: (p:PaytoUri) => void;
-    previous: KnownBankAccountsInfo[];
+    onSelectAccount: (p: string) => void;
+    previous: WalletBankAccountInfo[];
     goToBank: ButtonHandler;
     goToWallet: ButtonHandler;
   }
 }
 
-
 const viewMapping: StateViewMap<State> = {
   loading: Loading,
   error: ErrorAlertView,
