@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2022 Taler Systems S.A.
+ (C) 2022-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -14,7 +14,12 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { KnownBankAccountsInfo, PaytoUri, stringifyPaytoUri } from "@gnu-taler/taler-util";
+import {
+  WalletBankAccountInfo,
+  parsePaytoUri,
+  stringifyPaytoUri,
+  assertUnreachable
+} from "@gnu-taler/taler-util";
 import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { styled } from "@linaria/react";
 import { Fragment, h, VNode } from "preact";
@@ -31,7 +36,6 @@ import { Paper } from "../../mui/Paper.js";
 import { Pages } from "../../NavigationBar.js";
 import arrowIcon from "../../svg/chevron-down.inline.svg";
 import bankIcon from "../../svg/ri-bank-line.inline.svg";
-import { assertUnreachable } from "../../utils/index.js";
 import { State } from "./index.js";
 
 export function SelectCurrencyView({
@@ -250,7 +254,10 @@ export function ReadyGetView({
           <Grid item xs={1}>
             <Paper style={{ padding: 8 }}>
               <p>
-                <i18n.Translate>From a <pre style={{display:"inline"}}>taler://</pre> URI or QR code</i18n.Translate>
+                <i18n.Translate>
+                  From a <pre style={{ display: "inline" }}>taler://</pre> URI
+                  or QR code
+                </i18n.Translate>
               </p>
               <a href={Pages.qr}>
                 <i18n.Translate>Enter URI here</i18n.Translate>
@@ -291,7 +298,7 @@ export function ReadySendView({
                         <RowExample
                           info={info}
                           onClick={() => {
-                            onSelectAccount(info.uri)
+                            onSelectAccount(info.paytoUri);
                           }}
                           // disabled={!amountHandler.onInput}
                         />
@@ -324,6 +331,19 @@ export function ReadySendView({
               </Button>
             </Paper>
           </Grid>
+          <Grid item xs={1}>
+            <Paper style={{ padding: 8 }}>
+              <p>
+                <i18n.Translate>
+                  To a <pre style={{ display: "inline" }}>taler://</pre> URI or
+                  QR code
+                </i18n.Translate>
+              </p>
+              <a href={Pages.qr}>
+                <i18n.Translate>Enter URI here</i18n.Translate>
+              </a>
+            </Paper>
+          </Grid>
         </Grid>
       </Grid>
     </Container>
@@ -333,31 +353,28 @@ export function ReadySendView({
 function RowExample({
   info,
   disabled,
-  onClick
+  onClick,
 }: {
-  info: KnownBankAccountsInfo;
+  info: WalletBankAccountInfo;
   disabled?: boolean;
   onClick?: () => void;
 }): VNode {
-
-  
   return (
     <MediaExample data-disabled={disabled} onClick={onClick}>
       <MediaLeft>
         <CircleDiv>
-            <SvgIcon
-              title={info.alias}
-              dangerouslySetInnerHTML={{
-                __html: bankIcon,
-              }}
-              color="currentColor"
-            />
-          
+          <SvgIcon
+            title={info.label}
+            dangerouslySetInnerHTML={{
+              __html: bankIcon,
+            }}
+            color="currentColor"
+          />
         </CircleDiv>
       </MediaLeft>
       <MediaBody>
-        <span>{info.alias}</span>
-        <LightText>{describeAccount(info.uri)}</LightText>
+        <span>{info.label}</span>
+        <LightText>{describeAccount(info.paytoUri)}</LightText>
       </MediaBody>
       <MediaRight>
         <SvgIcon
@@ -371,20 +388,32 @@ function RowExample({
   );
 }
 
-
-function describeAccount(p: PaytoUri): string {
+function describeAccount(paytoUri: string): string {
+  const p = parsePaytoUri(paytoUri);
+  if (!p) {
+    return `(invalid)`;
+  }
   if (!p.isKnown) {
-    return stringifyPaytoUri(p)
+    return stringifyPaytoUri(p);
   }
   switch (p.targetType) {
     case "iban": {
-      return p.iban
+      return p.iban;
+    }
+    case "taler-reserve-http": {
+      return `${p.reservePub}@${p.exchange}`;
+    }
+    case "taler-reserve": {
+      return `${p.reservePub}@${p.exchange}`;
     }
     case "x-taler-bank": {
-      return `${p.host}/${p.account}`
+      return `${p.account}@${p.host}`;
     }
     case "bitcoin": {
-      return `${p.address}`
+      return `${p.address}`;
+    }
+    case "ethereum": {
+      return `${p.address}`;
     }
   }
-}
\ No newline at end of file
+}
