@@ -19,10 +19,9 @@
  * @author Sebastian Javier Marchano (sebasjm)
  */
 
-import { PendingTaskType, TaskId } from "@gnu-taler/taler-wallet-core";
+import { AbsoluteTime } from "@gnu-taler/taler-util";
 import * as tests from "@gnu-taler/web-util/testing";
-import { View as TestedComponent } from "./DeveloperPage.js";
-import { AbsoluteTime, PendingIdStr } from "@gnu-taler/taler-util";
+import { DeveloperPage as TestedComponent } from "./DeveloperPage.js";
 
 export default {
   title: "developer",
@@ -36,8 +35,8 @@ export const AllOff = tests.createExample(TestedComponent, {
   onDownloadDatabase: async () => "this is the content of the database",
   operations: [
     {
-      id: " " as TaskId,
-      type: PendingTaskType.ExchangeUpdate,
+      id: " ",
+      type: "exchange-update",
       exchangeBaseUrl: "http://exchange.url.",
       givesLifeness: false,
       lastError: undefined,
