@@ -21,13 +21,17 @@
 
 import * as tests from "@gnu-taler/web-util/testing";
 import { ComparingView, ReadyView, NoExchangesView } from "./views.js";
+import { ScopeType } from "@gnu-taler/taler-util";
 
 export default {
   title: "select exchange",
 };
 
 export const NoExchangeFound = tests.createExample(NoExchangesView, {
-  currency: "USD",
+  // scope: {
+  //   currency: "USD",
+  //   type: ScopeType.Global,
+  // },
   defaultExchange: "https://exchange.taler.ar",
 });
 
@@ -253,7 +257,7 @@ function timelineExample() {
         },
       },
     ],
-    refresh: [
+  refresh: [
       {
         group: "0.1",
         from: {
