@@ -39,8 +39,8 @@ import {
   TransactionWithdrawal,
   WithdrawalType,
 } from "@gnu-taler/taler-util";
-import { HistoryView as TestedComponent } from "./History.js";
 import * as tests from "@gnu-taler/web-util/testing";
+import { HistoryView as TestedComponent } from "./History.js";
 
 export default {
   title: "history",
@@ -94,7 +94,6 @@ const exampleData = {
     refundPending: undefined,
     totalRefundEffective: "USD:0" as AmountString,
     totalRefundRaw: "USD:0" as AmountString,
-    proposalId: "1EMJJH8EP1NX3XF7733NCYS2DBEJW4Q2KA5KEB37MCQJQ8Q5HMC0",
     status: PaymentStatus.Accepted,
     refundQueryActive: false,
   } as TransactionPayment,
@@ -172,9 +171,9 @@ export const SomeBalanceWithNoTransactions = tests.createExample(
     scope: {
       currency: "Ásd",
       type: ScopeType.Auditor,
-      url: "",
+      url: "http://auditor.taler.ar",
     },
-      balances: [
+    balances: [
       {
         available: "TESTKUDOS:10" as AmountString,
         flags: [],
@@ -185,11 +184,10 @@ export const SomeBalanceWithNoTransactions = tests.createExample(
         scopeInfo: {
           currency: "Ásd",
           type: ScopeType.Auditor,
-          url: "",
+          url: "http://auditor.taler.ar",
         },
       },
     ],
-    
   },
 );
 
@@ -200,7 +198,7 @@ export const OneSimpleTransaction = tests.createExample(TestedComponent, {
   scope: {
     currency: "Ásd",
     type: ScopeType.Auditor,
-    url: "",
+    url: "http://auditor.taler.ar",
   },
   balances: [
     {
@@ -213,11 +211,11 @@ export const OneSimpleTransaction = tests.createExample(TestedComponent, {
       scopeInfo: {
         currency: "Ásd",
         type: ScopeType.Auditor,
-        url: "",
+         url: "http://auditor.taler.ar",
+
       },
     },
   ],
-  
 });
 
 export const TwoTransactionsAndZeroBalance = tests.createExample(
@@ -229,9 +227,10 @@ export const TwoTransactionsAndZeroBalance = tests.createExample(
     scope: {
       currency: "Ásd",
       type: ScopeType.Auditor,
-      url: "",
+       url: "http://auditor.taler.ar",
+
     },
-      balances: [
+    balances: [
       {
         flags: [],
         available: "USD:0" as AmountString,
@@ -242,11 +241,11 @@ export const TwoTransactionsAndZeroBalance = tests.createExample(
         scopeInfo: {
           currency: "Ásd",
           type: ScopeType.Auditor,
-          url: "",
+           url: "http://auditor.taler.ar",
+
         },
       },
     ],
-    
   },
 );
 
@@ -264,7 +263,8 @@ export const OneTransactionPending = tests.createExample(TestedComponent, {
   scope: {
     currency: "Ásd",
     type: ScopeType.Auditor,
-    url: "",
+     url: "http://auditor.taler.ar",
+
   },
   balances: [
     {
@@ -277,11 +277,11 @@ export const OneTransactionPending = tests.createExample(TestedComponent, {
       scopeInfo: {
         currency: "Ásd",
         type: ScopeType.Auditor,
-        url: "",
+         url: "http://auditor.taler.ar",
+
       },
     },
   ],
-  
 });
 
 export const SomeTransactions = tests.createExample(TestedComponent, {
@@ -306,7 +306,8 @@ export const SomeTransactions = tests.createExample(TestedComponent, {
   scope: {
     currency: "Ásd",
     type: ScopeType.Auditor,
-    url: "",
+     url: "http://auditor.taler.ar",
+
   },
   balances: [
     {
@@ -319,11 +320,11 @@ export const SomeTransactions = tests.createExample(TestedComponent, {
       scopeInfo: {
         currency: "Ásd",
         type: ScopeType.Auditor,
-        url: "",
+         url: "http://auditor.taler.ar",
+
       },
     },
   ],
-  
 });
 
 export const SomeTransactionsInDifferentStates = tests.createExample(
@@ -407,9 +408,10 @@ export const SomeTransactionsInDifferentStates = tests.createExample(
     scope: {
       currency: "Ásd",
       type: ScopeType.Auditor,
-      url: "",
+       url: "http://auditor.taler.ar",
+
     },
-      balances: [
+    balances: [
       {
         flags: [],
         available: "USD:10" as AmountString,
@@ -420,11 +422,11 @@ export const SomeTransactionsInDifferentStates = tests.createExample(
         scopeInfo: {
           currency: "Ásd",
           type: ScopeType.Auditor,
-          url: "",
+           url: "http://auditor.taler.ar",
+
         },
       },
     ],
-    
   },
 );
 
@@ -445,9 +447,10 @@ export const SomeTransactionsWithTwoCurrencies = tests.createExample(
     scope: {
       currency: "Ásd",
       type: ScopeType.Auditor,
-      url: "",
+       url: "http://auditor.taler.ar",
+
     },
-      balances: [
+    balances: [
       {
         flags: [],
         available: "USD:0" as AmountString,
@@ -458,7 +461,8 @@ export const SomeTransactionsWithTwoCurrencies = tests.createExample(
         scopeInfo: {
           currency: "Ásd",
           type: ScopeType.Auditor,
-          url: "",
+           url: "http://auditor.taler.ar",
+
         },
       },
       {
@@ -471,11 +475,11 @@ export const SomeTransactionsWithTwoCurrencies = tests.createExample(
         scopeInfo: {
           currency: "Ásd",
           type: ScopeType.Auditor,
-          url: "",
+           url: "http://auditor.taler.ar",
+
         },
       },
     ],
-    
   },
 );
 
@@ -486,7 +490,8 @@ export const FiveOfficialCurrencies = tests.createExample(TestedComponent, {
   scope: {
     currency: "Ásd",
     type: ScopeType.Auditor,
-    url: "",
+     url: "http://auditor.taler.ar",
+
   },
   balances: [
     {
@@ -499,7 +504,8 @@ export const FiveOfficialCurrencies = tests.createExample(TestedComponent, {
       scopeInfo: {
         currency: "Ásd",
         type: ScopeType.Auditor,
-        url: "",
+         url: "http://auditor.taler.ar",
+
       },
     },
     {
@@ -512,7 +518,8 @@ export const FiveOfficialCurrencies = tests.createExample(TestedComponent, {
       scopeInfo: {
         currency: "Ásd",
         type: ScopeType.Auditor,
-        url: "",
+         url: "http://auditor.taler.ar",
+
       },
     },
     {
@@ -525,7 +532,8 @@ export const FiveOfficialCurrencies = tests.createExample(TestedComponent, {
       scopeInfo: {
         currency: "Ásd",
         type: ScopeType.Auditor,
-        url: "",
+         url: "http://auditor.taler.ar",
+
       },
     },
     {
@@ -538,7 +546,8 @@ export const FiveOfficialCurrencies = tests.createExample(TestedComponent, {
       scopeInfo: {
         currency: "Ásd",
         type: ScopeType.Auditor,
-        url: "",
+         url: "http://auditor.taler.ar",
+
       },
     },
     {
@@ -551,11 +560,11 @@ export const FiveOfficialCurrencies = tests.createExample(TestedComponent, {
       scopeInfo: {
         currency: "Ásd",
         type: ScopeType.Auditor,
-        url: "",
+         url: "http://auditor.taler.ar",
+
       },
     },
   ],
-  
 });
 
 export const FiveOfficialCurrenciesWithHighValue = tests.createExample(
@@ -567,9 +576,10 @@ export const FiveOfficialCurrenciesWithHighValue = tests.createExample(
     scope: {
       currency: "Ásd",
       type: ScopeType.Auditor,
-      url: "",
+       url: "http://auditor.taler.ar",
+
     },
-      balances: [
+    balances: [
       {
         flags: [],
         available: "USD:881001321230000" as AmountString,
@@ -580,7 +590,8 @@ export const FiveOfficialCurrenciesWithHighValue = tests.createExample(
         scopeInfo: {
           currency: "Ásd",
           type: ScopeType.Auditor,
-          url: "",
+           url: "http://auditor.taler.ar",
+
         },
       },
       {
@@ -593,7 +604,8 @@ export const FiveOfficialCurrenciesWithHighValue = tests.createExample(
         scopeInfo: {
           currency: "Ásd",
           type: ScopeType.Auditor,
-          url: "",
+           url: "http://auditor.taler.ar",
+
         },
       },
       {
@@ -605,7 +617,8 @@ export const FiveOfficialCurrenciesWithHighValue = tests.createExample(
         scopeInfo: {
           currency: "Ásd",
           type: ScopeType.Auditor,
-          url: "",
+           url: "http://auditor.taler.ar",
+
         },
         requiresUserInput: false,
       },
@@ -619,7 +632,8 @@ export const FiveOfficialCurrenciesWithHighValue = tests.createExample(
         scopeInfo: {
           currency: "Ásd",
           type: ScopeType.Auditor,
-          url: "",
+           url: "http://auditor.taler.ar",
+
         },
       },
       {
@@ -632,11 +646,11 @@ export const FiveOfficialCurrenciesWithHighValue = tests.createExample(
         scopeInfo: {
           currency: "Ásd",
           type: ScopeType.Auditor,
-          url: "",
+           url: "http://auditor.taler.ar",
+
         },
       },
     ],
-    
   },
 );
 
@@ -652,7 +666,8 @@ export const PeerToPeer = tests.createExample(TestedComponent, {
   scope: {
     currency: "Ásd",
     type: ScopeType.Auditor,
-    url: "",
+     url: "http://auditor.taler.ar",
+
   },
   balances: [
     {
@@ -665,7 +680,8 @@ export const PeerToPeer = tests.createExample(TestedComponent, {
       scopeInfo: {
         currency: "Ásd",
         type: ScopeType.Auditor,
-        url: "",
+         url: "http://auditor.taler.ar",
+
       },
     },
   ],
