@@ -21,7 +21,9 @@ import {
   ScopeInfo,
   ScopeType,
   Transaction,
+  URL,
   WalletBalance,
+  parseScopeInfoShort,
   stringifyScopeInfoShort,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
@@ -44,11 +46,11 @@ import { useBackendContext } from "../context/backend.js";
 import { useAsyncAsHook } from "../hooks/useAsyncAsHook.js";
 import { useSettings } from "../hooks/useSettings.js";
 import { Button } from "../mui/Button.js";
+import { TextField } from "../mui/TextField.js";
+import { TextFieldHandler } from "../mui/handlers.js";
 import { NoBalanceHelp } from "../popup/NoBalanceHelp.js";
 import DownloadIcon from "../svg/download_24px.inline.svg";
 import UploadIcon from "../svg/upload_24px.inline.svg";
-import { TextField } from "../mui/TextField.js";
-import { TextFieldHandler } from "../mui/handlers.js";
 
 interface Props {
   scope?: ScopeInfo;
@@ -71,13 +73,10 @@ export function HistoryPage({
   const state = useAsyncAsHook(async () => {
     const b = await api.wallet.call(WalletApiOperation.GetBalances, {});
     const balances = b.balances;
-    // const balance =
-    //   b.balances.length > 0 ? b.balances[balanceIndex] : undefined;
-    const tx = await api.wallet.call(WalletApiOperation.GetTransactions, {
+    const tx = await api.wallet.call(WalletApiOperation.GetTransactionsV2, {
       scopeInfo: showSearch ? undefined : selectedScope,
-      sort: "descending",
       includeRefreshes: settings.showRefeshTransactions,
-      search,
+      limit: -100
     });
     return { balances, transactions: tx.transactions };
   }, [selectedScope, search]);
@@ -110,7 +109,9 @@ export function HistoryPage({
     return (
       <NoBalanceHelp
         goToWalletManualWithdraw={{
-          onClick: pushAlertOnError(() => goToWalletManualWithdraw(selectedScope)),
+          onClick: pushAlertOnError(() =>
+            goToWalletManualWithdraw(selectedScope),
+          ),
         }}
       />
     );
@@ -184,7 +185,11 @@ export function HistoryView({
   );
   const balance = balances[balanceIndex];
   if (!balance) {
-    return <div>unknown scope</div>;
+    return (
+      <div>
+        <i18n.Translate>unknown scope</i18n.Translate>
+      </div>
+    );
   }
 
   const available = balance
@@ -193,6 +198,19 @@ export function HistoryView({
 
   const datesWithTransaction: string[] = Object.keys(transactionsByDate);
 
+  const groupByScope = balances.reduce(
+    (prev, cur) => {
+      const key = stringifyScopeInfoShort(cur.scopeInfo);
+      const value = prev[key] ?? [];
+      value.push(cur);
+      prev[key] = value;
+      return prev;
+    },
+    {} as Record<string, WalletBalance[]>,
+  );
+  // scopes.sort((a, b) => {
+  //   return a[1]
+  // });
   return (
     <Fragment>
       <section>
@@ -227,7 +245,9 @@ export function HistoryView({
             )}
           </div>
           <div style={{ display: "flex", flexDirection: "column" }}>
-            <h3 style={{ marginBottom: 0 }}>Balance</h3>
+            <h3 style={{ marginBottom: 0 }}>
+              <i18n.Translate>Balance</i18n.Translate>
+            </h3>
             <div
               style={{
                 width: "fit-content",
@@ -237,6 +257,12 @@ export function HistoryView({
               {balances.length === 1 ? (
                 <CenteredText style={{ fontSize: "x-large", margin: 8 }}>
                   {balance.scopeInfo.currency}
+                  <div style={{ fontSize: "small", color: "grey" }}>
+                    {balance.scopeInfo.type === ScopeType.Exchange ||
+                    balance.scopeInfo.type === ScopeType.Auditor
+                      ? balance.scopeInfo.url
+                      : undefined}
+                  </div>
                 </CenteredText>
               ) : (
                 <NiceSelect style={{ flexDirection: "column" }}>
@@ -244,17 +270,47 @@ export function HistoryView({
                     style={{
                       fontSize: "x-large",
                     }}
-                    value={balanceIndex}
+                    value={scopeStr}
                     onChange={(e) => {
-                      const bIdx = Number.parseInt(e.currentTarget.value, 10);
-                      changeScope(balances[bIdx].scopeInfo);
+                      const value = e.currentTarget.value;
+                      const sc = parseScopeInfoShort(value);
+                      if (!sc) {
+                        console.error("no scope for", value);
+                        return;
+                      }
+                      changeScope(sc);
                     }}
                   >
-                    {balances.map((entry, index) => {
+                    {Object.keys(groupByScope).map((key, si) => {
+                      const entry = groupByScope[key][0];
+                      const st = stringifyScopeInfoShort(entry.scopeInfo);
+                      const sc = entry.scopeInfo;
+                      if (sc.type === ScopeType.Global) {
+                        return (
+                          <option value={st} key={st}>
+                            <b>{sc.currency}</b>
+                          </option>
+                        );
+                      }
+
                       return (
-                        <option value={index} key={entry.scopeInfo.currency}>
-                          {entry.scopeInfo.currency}
-                        </option>
+                        <Fragment>
+                          {groupByScope[key].map((ex) => {
+                            const st = stringifyScopeInfoShort(ex.scopeInfo);
+                            if (ex.scopeInfo.type === ScopeType.Global) {
+                              return <Fragment />;
+                            }
+                            return (
+                              <optgroup
+                                label={new URL(ex.scopeInfo.url).hostname}
+                              >
+                                <option value={st} key={st}>
+                                  {ex.scopeInfo.currency}
+                                </option>
+                              </optgroup>
+                            );
+                          })}
+                        </Fragment>
                       );
                     })}
                   </select>
