@@ -15,81 +15,118 @@
  */
 
 import {
-  KnownBankAccountsInfo,
   parsePaytoUri,
+  PaytoString,
   stringifyPaytoUri,
+  WalletBankAccountInfo,
 } from "@gnu-taler/taler-util";
 import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
+import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { useState } from "preact/hooks";
 import { alertFromError, useAlertContext } from "../../context/alert.js";
 import { useBackendContext } from "../../context/backend.js";
-import { useTranslationContext } from "@gnu-taler/web-util/browser";
 import { useAsyncAsHook } from "../../hooks/useAsyncAsHook.js";
 import { AccountByType, Props, State } from "./index.js";
-import { useSettings } from "../../hooks/useSettings.js";
 
 export function useComponentState({
   scope,
-  onAccountAdded,
+  onAccountSelected,
   onCancel,
 }: Props): State {
   const api = useBackendContext();
   const { pushAlertOnError } = useAlertContext();
   const { i18n } = useTranslationContext();
+
   const hook = useAsyncAsHook(() =>
-    api.wallet.call(WalletApiOperation.ListKnownBankAccounts, { currency: scope.currency }),
+    api.wallet.call(WalletApiOperation.ListBankAccounts, {
+      currency: scope.currency,
+    }),
   );
-  const accountType: Record<string, string> = {
-    iban: "IBAN",
-  };
-  const [settings] = useSettings();
-  if (settings.extendedAccountTypes) {
-    accountType["bitcoin"] = "Bitcoin";
-    accountType["x-taler-bank"] = "Taler Bank";
-  }
 
-  const [payto, setPayto] = useState("");
-  const [alias, setAlias] = useState("");
-  const [type, setType] = useState("iban");
+  const hook2 = useAsyncAsHook(() =>
+    api.wallet.call(WalletApiOperation.GetDepositWireTypesForCurrency, {
+      currency: scope.currency,
+      scopeInfo: scope,
+    }),
+  );
 
-  if (!hook) {
+  if (!hook || !hook2) {
     return {
       status: "loading",
       error: undefined,
     };
   }
+
   if (hook.hasError) {
     return {
       status: "error",
       error: alertFromError(
         i18n,
         i18n.str`Could not load known bank accounts`,
-        hook),
+        hook,
+      ),
     };
   }
 
+  if (hook2.hasError) {
+    return {
+      status: "error",
+      error: alertFromError(
+        i18n,
+        i18n.str`Could not load supported wire methods`,
+        hook2,
+      ),
+    };
+  }
+
+  if (hook2.response.wireTypes.length === 0) {
+    return {
+      status: "error",
+      error: {
+        type: "error",
+        message: i18n.str`No wire methods supported for this currency`,
+        description: i18n.str``,
+        cause: new Error("something"),
+        context: {},
+      },
+    };
+  }
+
+  const [payto, setPayto] = useState("");
+  const [label, setLabel] = useState("");
+  const [type, setType] = useState(hook2.response.wireTypes[0]);
+
+  const accountType: Record<string, string> = {};
+  hook2.response.wireTypes.forEach((t) => {
+    if (t === "iban") {
+      accountType[t] = "IBAN";
+    } else if (t === "x-taler-bank") {
+      accountType[t] = "x-taler-bank";
+    } else if (t === "bitcoin") {
+      accountType[t] = "Bitcoin";
+    }
+  });
+
   const uri = parsePaytoUri(payto);
   const found =
-    hook.response.accounts.findIndex(
-      (a) => stringifyPaytoUri(a.uri) === payto,
-    ) !== -1;
+    hook.response.accounts.findIndex((a) => a.paytoUri === payto) !== -1;
 
   async function addAccount(): Promise<void> {
     if (!uri || found) return;
 
     const normalizedPayto = stringifyPaytoUri(uri);
-    await api.wallet.call(WalletApiOperation.AddKnownBankAccounts, {
-      alias,
-      currency: scope.currency,
-      payto: normalizedPayto,
+    await api.wallet.call(WalletApiOperation.AddBankAccount, {
+      label,
+      currencies: [scope.currency],
+      paytoUri: normalizedPayto,
     });
-    onAccountAdded(uri);
+    onAccountSelected(normalizedPayto);
   }
 
   const paytoUriError = found ? "that account is already present" : undefined;
 
   const unableToAdd =
-    !type || !alias || paytoUriError !== undefined || uri === undefined;
+    !type || !label || paytoUriError !== undefined || uri === undefined;
 
   const accountByType: AccountByType = {
     iban: [],
@@ -98,13 +135,14 @@ export function useComponentState({
   };
 
   hook.response.accounts.forEach((acc) => {
-    accountByType[acc.uri.targetType].push(acc);
+    const p = parsePaytoUri(acc.paytoUri)!;
+    accountByType[p.targetType].push(acc);
   });
 
-  async function deleteAccount(account: KnownBankAccountsInfo): Promise<void> {
-    const payto = stringifyPaytoUri(account.uri);
-    await api.wallet.call(WalletApiOperation.ForgetKnownBankAccounts, {
-      payto,
+  async function deleteAccount(account: WalletBankAccountInfo): Promise<void> {
+    const payto = account.paytoUri;
+    await api.wallet.call(WalletApiOperation.ForgetBankAccount, {
+      bankAccountId: account.bankAccountId,
     });
     hook?.retry();
   }
@@ -112,7 +150,7 @@ export function useComponentState({
   return {
     status: "ready",
     error: undefined,
-    currency:scope.currency,
+    currency: scope.currency,
     accountType: {
       list: accountType,
       value: type,
@@ -120,10 +158,13 @@ export function useComponentState({
         setType(v);
       }),
     },
+    selectAccount: async (w) => {
+      onAccountSelected(w.paytoUri as PaytoString); // trust in the wallet that the payto is correct
+    },
     alias: {
-      value: alias,
+      value: label,
       onInput: pushAlertOnError(async (v) => {
-        setAlias(v);
+        setLabel(v);
       }),
     },
     uri: {
