@@ -1,6 +1,6 @@
 /*
  This file is part of GNU Taler
- (C) 2022 Taler Systems S.A.
+ (C) 2022-2025 Taler Systems S.A.
 
  GNU Taler is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
@@ -18,6 +18,7 @@ import {
   AbsoluteTime,
   AttentionInfo,
   AttentionType,
+  assertUnreachable
 } from "@gnu-taler/taler-util";
 import { Fragment, h, VNode } from "preact";
 import {
@@ -33,7 +34,6 @@ import { Avatar } from "../../mui/Avatar.js";
 import { Button } from "../../mui/Button.js";
 import { Grid } from "../../mui/Grid.js";
 import { Pages } from "../../NavigationBar.js";
-import { assertUnreachable } from "../../utils/index.js";
 import { State } from "./index.js";
 
 const term = 1000 * 60 * 60 * 24;
