@@ -44,7 +44,7 @@ import {
   WithdrawalType,
 } from "@gnu-taler/taler-util";
 import * as tests from "@gnu-taler/web-util/testing";
-import beer from "../../static-dev/beer.png";
+import beer from "../../static-stories/beer.png";
 import { TransactionView as TestedComponent } from "./Transaction.js";
 
 export default {
@@ -70,12 +70,14 @@ const commonTransaction: TransactionCommon = {
   timestamp: TalerProtocolTimestamp.now(),
   transactionId: "txn:deposit:12" as TransactionIdStr,
   type: TransactionType.Deposit,
+  stId: 1,
+
 } as Omit<
   Omit<Omit<TransactionCommon, "extendedStatus">, "frozen">,
   "pending"
 > as TransactionCommon;
 
-import merchantIcon from "../../static-dev/merchant-icon.jpeg";
+import merchantIcon from "../../static-stories/merchant-icon.jpeg";
 
 const exampleData = {
   withdraw: {
@@ -117,7 +119,6 @@ const exampleData = {
     refundPending: undefined,
     totalRefundEffective: "KUDOS:0" as AmountString,
     totalRefundRaw: "KUDOS:0" as AmountString,
-    proposalId: "1EMJJH8EP1NX3XF7733NCYS2DBEJW4Q2KA5KEB37MCQJQ8Q5HMC0",
     status: PaymentStatus.Accepted,
     refundQueryActive: false,
   } as TransactionPayment,
@@ -156,6 +157,7 @@ const exampleData = {
     ...commonTransaction,
     type: TransactionType.PeerPushCredit,
     info: {
+      iconId: "",
       expiration: {
         t_s: new Date().getTime() / 1000 + 2 * 60 * 60,
       },
@@ -171,6 +173,7 @@ const exampleData = {
     talerUri:
       "taler://pay-push/exchange.taler.ar/HS585JK0QCXHJ8Z8QWZA3EBAY5WY7XNC1RR2MHJXSH2Z4WP0YPJ0",
     info: {
+      iconId: "",
       expiration: {
         t_s: new Date().getTime() / 1000 + 2 * 60 * 60,
       },
@@ -185,6 +188,7 @@ const exampleData = {
     talerUri:
       "taler://pay-push/exchange.taler.ar/HS585JK0QCXHJ8Z8QWZA3EBAY5WY7XNC1RR2MHJXSH2Z4WP0YPJ0",
     info: {
+      iconId: "",
       expiration: {
         t_s: new Date().getTime() / 1000 + 2 * 60 * 60,
       },
@@ -197,7 +201,8 @@ const exampleData = {
   pull_debit: {
     ...commonTransaction,
     type: TransactionType.PeerPullDebit,
-    info: {
+    info: {      iconId: "",
+
       expiration: {
         t_s: new Date().getTime() / 1000 + 2 * 60 * 60,
       },
