@@ -80,9 +80,14 @@ import refreshIcon from "../svg/refresh_24px.inline.svg";
 interface Props {
   tid: string;
   goToWalletHistory: (scope?: ScopeInfo) => Promise<void>;
+  onForwardToContact?: (id: TransactionIdStr) => Promise<void>;
 }
 
-export function TransactionPage({ tid, goToWalletHistory }: Props): VNode {
+export function TransactionPage({
+  tid,
+  goToWalletHistory,
+  onForwardToContact,
+}: Props): VNode {
   const transactionId = tid as TransactionIdStr; //FIXME: validate
   const { i18n } = useTranslationContext();
   const api = useBackendContext();
@@ -166,6 +171,7 @@ export function TransactionPage({ tid, goToWalletHistory }: Props): VNode {
         });
       }}
       onBack={() => goToWalletHistory(txScope)}
+      onForwardToContact={onForwardToContact}
     />
   );
 }
@@ -180,6 +186,7 @@ export interface WalletTransactionProps {
   onRetry: () => Promise<void>;
   onRefund: (id: TransactionIdStr) => Promise<void>;
   onBack: () => Promise<void>;
+  onForwardToContact?: (id: TransactionIdStr) => Promise<void>;
 }
 
 const PurchaseDetailsTable = styled.table`
@@ -488,6 +495,7 @@ export function TransactionView({
   onRetry,
   onRefund,
   onCancel,
+  onForwardToContact,
 }: WalletTransactionProps): VNode {
   const { i18n } = useTranslationContext();
   const { safely } = useAlertContext();
@@ -963,7 +971,7 @@ export function TransactionView({
           !transaction.error && (
             <Part
               title={i18n.str`URI`}
-              text={<ShowQrWithCopy text={transaction.talerUri} startOpen />}
+              text={<ShowQrWithCopy text={transaction.talerUri} startOpen  onForwardToContact={() => { onForwardToContact && onForwardToContact(transaction.transactionId)} }/>}
               kind="neutral"
             />
           )}
@@ -1059,7 +1067,7 @@ export function TransactionView({
         {transaction.talerUri && (
           <Part
             title={i18n.str`URI`}
-            text={<ShowQrWithCopy text={transaction.talerUri} startOpen />}
+            text={<ShowQrWithCopy text={transaction.talerUri} startOpen onForwardToContact={() => { onForwardToContact && onForwardToContact(transaction.transactionId)} } />}
             kind="neutral"
           />
         )}
@@ -1993,9 +2001,11 @@ function NicePayto({ payto }: { payto: PaytoUri }): VNode {
 function ShowQrWithCopy({
   text,
   startOpen,
+  onForwardToContact,
 }: {
   text: string;
   startOpen?: boolean;
+  onForwardToContact?: () => void;
 }): VNode {
   const [showing, setShowing] = useState(startOpen);
   const { i18n } = useTranslationContext();
@@ -2015,6 +2025,12 @@ function ShowQrWithCopy({
         <Button onClick={toggle as SafeHandler<void>}>
           <i18n.Translate>hide qr</i18n.Translate>
         </Button>
+        <EnabledBySettings name="p2p_aliases">
+        {(onForwardToContact) && (
+          <Button onClick={onForwardToContact as SafeHandler<void>}>
+            <i18n.Translate>forward to contact</i18n.Translate>
+          </Button>)}
+        </EnabledBySettings>
       </div>
     );
   }
@@ -2027,6 +2043,12 @@ function ShowQrWithCopy({
       <Button onClick={toggle as SafeHandler<void>}>
         <i18n.Translate>show qr</i18n.Translate>
       </Button>
+      <EnabledBySettings name="p2p_aliases">
+      {(onForwardToContact) && (
+        <Button onClick={onForwardToContact as SafeHandler<void>}>
+          <i18n.Translate>forward to contact</i18n.Translate>
+        </Button>)}
+      </EnabledBySettings>
     </div>
   );
 }
