@@ -21,16 +21,15 @@ import {
   assertUnreachable,
 } from "@gnu-taler/taler-util";
 import { Fragment, VNode, h } from "preact";
-import { useCommonPreference } from "../context/common-preferences.js";
-import { useTranslationContext } from "../index.browser.js";
+import { useCommonPreferences, useTranslationContext } from "../index.browser.js";
 import { Attention } from "./Attention.js";
 
 export function DebugInfo({ error }: { error: any }): VNode {
   const { i18n } = useTranslationContext();
-  const { showDebugInfo, toggle } = useCommonPreference();
+  const [{ showDebugInfo }, update] = useCommonPreferences();
   return (
     <div class="text-[grey]">
-      <button onClick={toggle}>
+      <button onClick={() => update("showDebugInfo", !showDebugInfo)}>
         {!showDebugInfo ? (
           <i18n.Translate>Show more information</i18n.Translate>
         ) : (
