@@ -14,7 +14,14 @@
  GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
  */
 
-import { ChallengerHttpClient, ObservabilityEvent, TalerAuthenticationHttpClient, TalerBankConversionHttpClient, TalerCoreBankHttpClient, TalerExchangeHttpClient, TalerMerchantInstanceHttpClient, TalerMerchantManagementHttpClient } from "@gnu-taler/taler-util";
+import {
+  ChallengerHttpClient,
+  ObservabilityEvent,
+  TalerBankConversionHttpClient,
+  TalerCoreBankHttpClient,
+  TalerExchangeHttpClient,
+  TalerMerchantManagementHttpClient,
+} from "@gnu-taler/taler-util";
 
 type Listener<Event> = (e: Event) => void;
 type Unsuscriber = () => void;
@@ -23,11 +30,11 @@ export type Subscriber<Event> = (fn: Listener<Event>) => Unsuscriber;
 export class ActiviyTracker<Event> {
   private observers = new Array<Listener<Event>>();
   constructor() {
-    this.notify = this.notify.bind(this)
-    this.subscribe = this.subscribe.bind(this)
+    this.notify = this.notify.bind(this);
+    this.subscribe = this.subscribe.bind(this);
   }
   notify(data: Event): void {
-    this.observers.forEach((observer) => observer(data))
+    this.observers.forEach((observer) => observer(data));
   }
   subscribe(func: Listener<Event>): Unsuscriber {
     this.observers.push(func);
@@ -49,14 +56,13 @@ export class ActiviyTracker<Event> {
 export interface APIClient<T, C> {
   getRemoteConfig(): Promise<C>;
   VERSION: string;
-  lib: T,
+  lib: T;
   onActivity: Subscriber<ObservabilityEvent>;
   cancelRequest(id: string): void;
 }
 
 export interface MerchantLib {
   instance: TalerMerchantManagementHttpClient;
-  authenticate: TalerAuthenticationHttpClient;
   subInstanceApi: (instanceId: string) => MerchantLib;
 }
 
@@ -67,10 +73,10 @@ export interface ExchangeLib {
 export interface BankLib {
   bank: TalerCoreBankHttpClient;
   conversion: TalerBankConversionHttpClient;
-  auth: (user: string) => TalerAuthenticationHttpClient;
+  conversionForUser(username:string): TalerBankConversionHttpClient;
+  conversionForClass(classId: number): TalerBankConversionHttpClient;
 }
 
 export interface ChallengerLib {
   challenger: ChallengerHttpClient;
 }
-
