@@ -16,15 +16,14 @@
 
 import {
   CacheEvictor,
-  TalerCorebankConfigResponse,
   LibtoolVersion,
   ObservabilityEvent,
   ObservableHttpClientLibrary,
-  TalerAuthenticationHttpClient,
   TalerBankConversionCacheEviction,
   TalerBankConversionHttpClient,
   TalerCoreBankCacheEviction,
   TalerCoreBankHttpClient,
+  TalerCorebankApi,
   TalerError,
 } from "@gnu-taler/taler-util";
 import {
@@ -46,7 +45,7 @@ import { useTranslationContext } from "./translation.js";
 
 export type BankContextType = {
   url: URL;
-  config: TalerCorebankConfigResponse;
+  config: TalerCorebankApi.TalerCorebankConfigResponse;
   lib: BankLib;
   hints: VersionHint[];
   onActivity: Subscriber<ObservabilityEvent>;
@@ -88,7 +87,7 @@ export const BankApiProvider = ({
   frameOnError: FunctionComponent<{ children: ComponentChildren }>;
 }): VNode => {
   const [checked, setChecked] =
-    useState<ConfigResult<TalerCorebankConfigResponse>>();
+    useState<ConfigResult<TalerCorebankApi.TalerCorebankConfigResponse>>();
   const { i18n } = useTranslationContext();
 
   const { getRemoteConfig, VERSION, lib, cancelRequest, onActivity } =
@@ -135,7 +134,7 @@ export const BankApiProvider = ({
   }
   if (checked.type === "error") {
     return h(frameOnError, {
-      children: h(ErrorLoading, { error: checked.error, showDetail: true }),
+      children: h(ErrorLoading, { error: checked.error }),
     });
   }
   if (checked.type === "incompatible") {
@@ -165,7 +164,7 @@ export const BankApiProvider = ({
 function buildBankApiClient(
   url: URL,
   evictors: Evictors,
-): APIClient<BankLib, TalerCorebankConfigResponse> {
+): APIClient<BankLib, TalerCorebankApi.TalerCorebankConfigResponse> {
   const httpFetch = new BrowserFetchHttpLib({
     enableThrottling: true,
     requireTls: false,
@@ -183,27 +182,41 @@ function buildBankApiClient(
     httpLib,
     evictors.conversion,
   );
-  const auth = (user: string) =>
-    new TalerAuthenticationHttpClient(
-      bank.getAuthenticationAPI(user).href,
-      httpLib,
-    );
 
-  async function getRemoteConfig(): Promise<TalerCorebankConfigResponse> {
+  async function getRemoteConfig(): Promise<TalerCorebankApi.TalerCorebankConfigResponse> {
     const resp = await bank.getConfig();
     if (resp.type === "fail") {
-      throw TalerError.fromUncheckedDetail(resp.detail);
+      if (resp.detail) {
+        throw TalerError.fromUncheckedDetail(resp.detail);
+      } else {
+        throw TalerError.fromException(
+          new Error("failed to get bank remote config"),
+        );
+      }
     }
     return resp.body;
   }
 
   return {
     getRemoteConfig,
-    VERSION: bank.PROTOCOL_VERSION,
+    VERSION: TalerCoreBankHttpClient.PROTOCOL_VERSION,
     lib: {
       bank,
       conversion,
-      auth,
+      conversionForClass(classId) {
+        return new TalerBankConversionHttpClient(
+          bank.getConversionInfoAPIForClass(classId).href,
+          httpLib,
+          evictors.conversion,
+        );
+      },
+      conversionForUser(username) {
+        return new TalerBankConversionHttpClient(
+          bank.getConversionInfoAPIForUser(username).href,
+          httpLib,
+          evictors.conversion,
+        );
+      },
     },
     onActivity: tracker.subscribe,
     cancelRequest: httpLib.cancelRequest,
