@@ -10,13 +10,21 @@ import {
   getYear,
   isSameDay,
   isSameMonth,
+  isValid,
+  setYear,
   startOfDay,
   startOfMonth,
   startOfWeek,
+  subYears,
 } from "date-fns";
 import { VNode, h } from "preact";
-import { useState } from "preact/hooks";
+import { useEffect, useRef, useState } from "preact/hooks";
 import { useTranslationContext } from "../index.browser.js";
+import { composeRef, saveRef } from "../components/utils.js";
+
+const THIS_MONTH = getMonth(new Date());
+const THIS_YEAR = getYear(new Date());
+const TODAY = startOfDay(new Date());
 
 export function Calendar({
   value,
@@ -25,11 +33,21 @@ export function Calendar({
   value: AbsoluteTime | undefined;
   onChange: (v: AbsoluteTime) => void;
 }): VNode {
-  const today = startOfDay(new Date());
-  const selected = !value ? today : new Date(AbsoluteTime.toStampMs(value));
+  const selectedMaybeInvalid = !value
+    ? TODAY
+    : new Date(AbsoluteTime.toStampMs(value));
+  const selected = isValid(selectedMaybeInvalid) ? selectedMaybeInvalid : TODAY;
   const [showingDate, setShowingDate] = useState(selected);
-  const month = getMonth(showingDate);
-  const year = getYear(showingDate);
+  const m = getMonth(showingDate);
+  const y = getYear(showingDate);
+  const month = Number.isNaN(m) ? THIS_MONTH : m;
+  const year = Number.isNaN(y) ? THIS_YEAR : y;
+  const input = useRef<HTMLInputElement>();
+  useEffect(() => {
+    if (!input.current) return;
+    if (input.current === document.activeElement) return;
+    input.current.value = !year ? "" : String(year);
+  }, [year]);
 
   const start = startOfWeek(startOfMonth(showingDate));
   const end = endOfWeek(endOfMonth(showingDate));
@@ -73,7 +91,24 @@ export function Calendar({
             />
           </svg>
         </button>
-        <div class="flex-auto text-sm font-semibold">{year}</div>
+        <div class="flex-auto text-sm font-semibold">
+          <input
+            type="text"
+            ref={composeRef(saveRef(input))}
+            onChange={(e) => {
+              const text = e.currentTarget.value;
+              const num = Number.parseInt(text, 10);
+
+              if (Number.isSafeInteger(num) && num > 0) {
+                const nextYear = setYear(showingDate, num);
+
+                if (isValid(nextYear)) {
+                  setShowingDate(nextYear);
+                }
+              }
+            }}
+          />
+        </div>
         <button
           type="button"
           class="flex px-4 flex-none items-center justify-center p-1.5 text-gray-400 hover:text-gray-500 ring-2 round-sm"
@@ -157,7 +192,7 @@ export function Calendar({
               type="button"
               key={idx}
               data-month={isSameMonth(current, showingDate)}
-              data-today={isSameDay(current, today)}
+              data-today={isSameDay(current, TODAY)}
               data-selected={isSameDay(current, selected)}
               onClick={() => {
                 onChange(AbsoluteTime.fromStampMs(current.getTime()));
